#!/usr/local/bin/python2.4
# -*- coding: utf-8 -*-

'''
VirtualSpectrum produces simulated peak list for various liquid and solid
state NMR experiments. See README file
'''

import numpy
from numpy import log, exp, sqrt, take, hstack, vstack, ones, zeros, empty,\
argmax, argmin, arange, linspace, pi, array, transpose, cos, sin, prod,\
average, sum, dot
from numpy.random import normal as randNorm
from numpy.random import rand
from scipy.optimize.anneal import anneal
from random import normalvariate
from random import randint
import string, os, math
##import mmLib.Structure as struc
##import mmLib.FileIO as fileio
##import mmLib.AtomMath as AtomMath
from util import *
##from jakob.protstat import *
from pylab import show, scatter, plot, clf, contour, imshow, rot90, subplot, title, hist
import Bio.PDB
import scipy.optimize.optimize as opt
import time
import sys
from sets import Set
##from protStat12 import DSSPData_probabilistic, get_probdssp_dict_APSSP2, getconvdct, getmethyls, getambgroups, Parser
aa31dict={'CYS': 'C', 'GLN': 'Q', 'ILE': 'I', 'SER': 'S', 'VAL': 'V', 'MET': 'M', 'ASN': 'N', 'PRO': 'P', 'LYS': 'K', 'THR': 'T', 'PHE': 'F', 'ALA': 'A', 'HIS': 'H', 'GLY': 'G', 'ASP': 'D', 'LEU': 'L', 'ARG': 'R', 'TRP': 'W', 'GLU': 'E', 'TYR': 'Y'}
aa13dict={'A': 'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE', 'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y': 'TYR'}
aa1sorted=aa13dict.keys();aa1sorted.sort();#introduce ordering

#GLOBAL CONSTANTS:
SILENT=0
SYMM=None
SHOWON=False
SHOWON2=False
SQRT_2PI=2.506628
DEBUG=0

def calc_distance(at1,at2):
    return sqrt(sum((at1.position-at2.position)**2))

def getrandomcoils():
    buf=initfil2('randcoilCenter.txt')
    dct={}
    for lin in buf[1:]:
      aa=lin[0]
      dct[aa]={}
      shifts=[eval(x) for x in lin[1:]]
      for i,atn in enumerate(['CA','CB','C','HA','H','N']):
        if shifts[i]>0.0:
          dct[aa][atn]=shifts[i]
    return dct

def getncorrs():
    buf=initfil2('neighborCorrs.txt')
    dct={'p':{},'s':{}}
    for lin in buf:
	atn=lin[0]
	direc=lin[1]
	if not atn in dct[direc]:dct[direc][atn]={}
	aan=lin[2]
	corr=eval(lin[3])
	dct[direc][atn][aan]=corr
	##std=eval(lin[4])
    return dct

def getTot(parser,getambhb=True,gethbdb=True,getSE=False):
    peakline=parser.search('beginning',verb=False)
    num=eval(parser.buffer[peakline-1][6])
    ##lin,col=parser.search('E(poten)=',incr=0,doReturn2=True)
    ##Epot=eval(parser.getCurrent(column=col+1))
    epot=parser.getNext('E(poten)=')
    hbdb=0.0;se=0.0
    if epot==None:
        if gethbdb:
            return -1,None,None,None
        else:return -1,None,None
    Epot=eval(epot)
    darr2d=eval(parser.getNext('darr2d='))
    if gethbdb:
        hbdb=eval(parser.getNext('HBDB='))
    if getSE:
        se=eval(parser.getNext('NOE='))
    if getambhb:
        ambhb=eval(parser.getNext('ambhb='))
        ##print Epot,hbdb,darr2d,ambhb
        return num,Epot-darr2d-ambhb,hbdb,se
    else: return num,Epot-darr2d,hbdb,se

def getsoftviol(dv,dExp=2,rSwitch=0.5,asympSlope=0.5,softExp=1.0,thr=0.5):
    trunc=clip(dv,0,rSwitch)
    hi=clip(dv,rSwitch,10000)
    ishi=dv>rSwitch
    r=rSwitch
    b=(asympSlope-dExp*(r**(dExp-1)))*r**(softExp+1)
    a=-b*r**-softExp-asympSlope*r
    return a+trunc**dExp+asympSlope*hi+b*hi**-softExp#+ishi*0.25

##t=numpy.arange(0,2.0,0.01)
##for t in [0.4,0.6]:
##    y=getsoftviol(t)
##    print y
##plot(t,y)
##show()

#global functions:
def heigh2LevelCurve(A,*args):
   '''nivea-kurven i et "contour-plot" for en cross peak'''
   K,hlev,lw0=args
   return K*lw0*sqrt(log(A/hlev)) 

def pOr(p1,p2):
    return 1.0-(1.0-p1)*(1.0-p2)

def showdata(xlst,ylst,cl,f,args=(),lws=None):
   if len(xlst)>0:
       Xlst=array(xlst)
       Ylst=array(ylst)
       w1,w2=Xlst.transpose() 
       ##scatter(w1*-1.0, w2*-1.0, s=f(Ylst,*args), c=cl, alpha=0.75,linewidths=lws,faceted=False)
       ##scatter(w2*-1.0, w1*-1.0, s=f(Ylst,*args), c=cl, alpha=0.75,linewidths=lws,faceted=False)
       scatter(w2*-1.0, w1*-1.0, s=f(Ylst,*args), c=cl, alpha=0.75,faceted=False)

def loren1(x,mu,g):
    return 1/(1+((x-mu)/g)**2)

def lorenNd(A,x,mu,g):
    prod=A
    for i in range(len(mu)):
       prod*=loren1(x[i],mu[i],g[i])
    return prod

def gaus1(x,mu,g):
    return exp(-0.5*(x-mu)**2/(g**2))

def agaus(x,g):
    return exp(-0.5*x**2/(g**2))

def gaus2(x,y,mu,g):
    return gaus1(x,mu[0],g[0])*gaus1(y,mu[1],g[1])

def gausNd(A,x,mu,g):
    prod=A
    for i in range(len(mu)):
       prod=prod*gaus1(x[i],mu[i],g[i])
    return prod
      
def lorenNdGrad(A,x,mu,g):
    prod=A*pi
    grad=numpy.ones(len(mu))*prod
    f=numpy.zeros(len(mu))
    for i in range(len(mu)):
       f[i]=loren1(x[i],mu[i],g[i])
       grad[i]=-2.0*(x[i]-mu[i])/g[i]/((1+((x[i]-mu[i])/g[i])**2)**2)
    for i in range(len(mu)):
     for j in range(len(mu)):
      if i<>j:grad[i]*=f[j]
    return grad

def gausNdGrad(A,x,mu,g):
    prod=A
    grad=numpy.zeros(len(mu))
    for i in range(len(mu)):
       prod=prod*gaus1(x[i],mu[i],g[i])
       grad[i]=-1.0*(x[i]-mu[i])/(g[i])**2	
    return prod*grad

def GetSumGradGausNd(x,*args):
   vc,sigGausCurves,lw=args
   sum=0
   for i in sigGausCurves:
      p,h=vc.list[i],vc.vallst[i]
      sum=sum-gausNdGrad(h,x,p,lw)
   return sum

def GetSumGausNd(x,*args):
   vc,sigGausCurves,lw=args
   sum=0
   for i in sigGausCurves:
      p,h=vc.list[i],vc.vallst[i]
      sum=sum-gausNd(h,x,p,lw)
   return sum

def chid2chnum(c):
    #s='ABCDEFGHIJKLMN'
    s='ABDEGH'
    return s.index(c)
    
def CalcVolfacCum(lr,lsig2,rmin=4.5,lsigint=1.0,n=6.0):
    ##print ('sdf',rmin),log(rmin),numpy.log(rmin),log(2.0),lr,lsig2,lsigint
##    if lsig2==0:return 0.0
    z=(log(rmin)-lr)/(sqrt(lsig2+(lsigint/n)**2))
    return comStandardNormal(z)

def entrop(r,printing=0):
    sum=0
    a=(1-r)/r
    for j in range(1,17):
        if printing:print a*r**j,log(a*r**j)
        sum=sum+r**j*(log(a)+j*log(r))
    return -1.0*sum*a

def dist2pScaled(co,p,s):
    sum=0.0
    for i in range(len(co)):
        sum=sum+(co[i]-p[i])**2/(s[0][i]**2+s[1][i]**2)
    ##print co,p,sum
    return sum

def getType(ats,pair=(0,1)):
    diff=abs(string.atoi(ats[pair[0]].fragment_id)-string.atoi(ats[pair[1]].fragment_id))
    return diff

def dist2p(s,t,noargs):
    sum=0
    for i in range(len(s)):
        sum=sum+(s[i]-t[i])**2
    return sum

def distMetric3(u,v,s):
   U=numpy.array(u)
   V=numpy.array(v)
   S=numpy.array(s)
   return math.sqrt(numpy.dot((U-V)/S,(U-V)/S))

def distMetricVec(U,V,S=None):
    if S==None:S=ones(len(U))
    return math.sqrt(dot((U-V)/S,(U-V)/S))

def aboveDiag(U):
    return U[1]>U[0]

def distMetricSym(U,v,S=None):
    V=take(v,[1,0])#invert
    if S==None:S=ones(len(U))
    return math.sqrt(dot((U-V)/S,(U-V)/S)/len(U))

def distMetricVec2(U,V,S=None):
    if S==None:S=ones(len(U))
    return math.sqrt(dot((U-V)/S,(U-V)/S)/len(U))

def distMetricTup(u,v,S=None):
    if S==None:S=numpy.ones(len(u))
    U=numpy.array(u)
    V=numpy.array(v)
    return math.sqrt(numpy.dot((U-V)/S,(U-V)/S))

def distMetricTup2(u,v,S=None):
    if S==None:S=numpy.ones(len(u))
    U=numpy.array(u)
    V=numpy.array(v)
    return math.sqrt(numpy.dot((U-V)/S,(U-V)/S)/len(U))

def diffSingArrays(s,t,noargs):
    return abs(s[0]-t[0])

def newvol(vol):
    if vol==0:return -999
    else:return vol/100000000

def logNormal(x,lmu,lsig):
    const=x*lsig*math.sqrt(2.0*pi)
    return numpy.exp(-0.5*((numpy.log(x)-lmu)/lsig)**2)/const

def powerlaw(param, r):
    return param[0]*r**param[1]

def linlaw(param, r):
    return param[0]*r+param[1]

def loglaw(param,t):
    return log(param[0])+param[1]*log(t)

def getfunc(k,p):
    def func(x):
        return (1.0+k*x**2)**-p#*math.exp(-0.5*(x-0.1)**2)
    
    return func

def getfunc2(n):
    return getfunc(n/(n-1),n/2)

def G(x):
    return (1.0+1.0*x**2)**-1.0    

def Bprob(dev,n):
    g=getfunc2(n)
    normK=romb.romberg(g,(-10.0,10.0),accuracy=0.01)
    return g(dev)#/normK

#functions defined on an array of the class Atom
def getAsFormat(ats,sep=''):
   if ats==None:return '?-?-?'
   elif isinstance(ats,str):return ats
   rns,ris,rcs=[],[],[]
   for j in range(len(ats)):
       ##print ats[j]
       ##print ats[j].res_name
       ris.append(ats[j].fragment_id)
       try:rns.append(aa31dict[ats[j].res_name])
       except KeyError:
           ##print 'trying one letter:',ats[j].res_name
           rns.append(ats[j].res_name)
       rcs.append(ats[j].name)
   frd=abs(eval(ris[0])-eval(ris[1]))
   if len(ats)==2:
       if ris[0]==ris[1]:#2d
           line='%6s-%-6s'%(rns[0]+ris[0]+sep+rcs[0],rcs[1])
       else:
           line='%6s-%-6s'%(rns[0]+ris[0]+sep+rcs[0],rns[1]+ris[1]+sep+rcs[1])
   elif len(ats)==3:
       if ris[0]==ris[1]:#3d
           ##line='%6s-%-6s%-6s'%(rns[0]+ris[0]+rcs[0],rcs[1],rcs[2])
           line='%6s-%-6s'%(rns[0]+ris[0]+sep+rcs[0],rcs[1])
       else:
           ##line='%6s-%-6s%-6s'%(rns[0]+ris[0]+rcs[0],rns[1]+ris[1]+rcs[1],rns[2]+ris[2]+rcs[2])
           line='%6s-%-6s'%(rns[0]+ris[0]+sep+rcs[0],rns[1]+ris[1]+sep+rcs[1])
       if ris[1]==ris[2]:
           line=''+line.strip()+'-'+rcs[2]
       else:
           line=''+line.strip()+'-'+rns[2]+ris[2]+sep+rcs[2]
   return line

def getCCpairdct():
    buf=initfil2('ccpairs.txt')
    dct={}
    for lin in buf:
	resn1=lin[0]
	if not resn1 in dct:dct[resn1]={}
	atn1=lin[1]
	atn2=lin[2]
	if not atn1 in dct[resn1]:dct[resn1][atn1]={}
	dct[resn1][atn1][atn2]=eval(lin[3])
	if not atn2 in dct[resn1]:dct[resn1][atn2]={}
	dct[resn1][atn2][atn1]=eval(lin[3])
    return dct

def getAssignmentKey(ats):
   key=[]
   for j in range(len(ats)):
       key.append((ats[j].fragment_id,ats[j].name))
   return key


#functions defined on an array of the class Experiment
def getVolFunc(explst):
    print 'entering function definition (once?)'
    Gs=zeros((len(explst),3,4-1))*0.0#allocation for 4d experiments
    ds=(1.7,2.6,3.4)
    rminis=[]
    for i in range(len(explst)):
        expi=explst[i]
        rmini=expi.getMaxRmin()
        print i,rmini
        rminis.append(rmini)
        for j in range(3):
            for k in range(len(rmini)):
                Gs[i,j,k]=CalcVolfacCum(log(ds[j]),0.15,rmin=rmini[k],n=6.0)

    print 'Her er Gs:',Gs
        
    def f(i,ds):
      G=1.0
      ##ds.sort()
      print i,ds
      if explst[i].multi in [[1],[1,1]]:#TODO generaliser...
        ds.sort()
      ds=ds[:explst[i].dimension-1]
      for k in range(len(ds)):
       if explst[i].multi[k]>0:
        d=ds[k]
        if 1.0<d<=2.1:G*=Gs[i,0,k]#1-bond
        elif 2.1<d<=2.7:G*=Gs[i,1,k]#2-bond
        elif 3.0<d<=3.7:G*=Gs[i,2,k]#3-bond
        else:G*=CalcVolfacCum(log(d),0.15,rmin=rminis[i][k],n=6.0)
      return G
    
    return f

def testDuplicates(explst):
    explst[0].stripPeaks([explst[1]],slice=1,sym=1,tol=0.5)
    for peak in explst[0].peakData.peaklist:
	if peak.strip:
	   co=peak.shifts
	   ats=peak.names
	   if co[1]<45:
	     if peak.virtFlag in [0,3]:
	      if ats[1].name<>'CA':
                dist=round(AtomMath.calc_distance(ats[0],ats[1]),3)
		if dist<3.1:
		  print co,dist,ats[1].res_name+ats[0].fragment_id,ats[1].temp_factor,ats[1].name,peak.virtFlag

def testDuplicates2(explst):
    rat0k5=explst[0].stripPeaks([explst[1]],sym=1,tol=0.5)
    rat1  =explst[0].stripPeaks([explst[1]],sym=1,tol=1.0)
    return rat0k5,rat1

def createShiftDBA(filename):
   buf=initfil2(filename)
   dba={}
   for lst in buf:
      if lst<>[]:
        elem=lst[2]
        ##if elem in ['C']
        ##if elem in ['N','C']:
        if elem in ['N','C','H']:
           name=lst[0]
           at=lst[1]
           avg=string.atof(lst[6])
           std=string.atof(lst[7])
           if not dba.has_key(name):dba[name]={}
           dba[name][at]=(avg,std)
   return dba

def repos(vec,inds,dtype=int):
    rvec=zeros(len(inds),dtype=dtype)
    rvec[inds]=vec
    return rvec

def getValidDBA(validMat):
    dba={}
    for i in range(len(validMat)):
        validassigns,dists,inds=validMat[i]
        ##print validMat[i]
        for j in range(len(validassigns)):
            vec=validassigns[j];distvec=dists[j]
            ##rvec=repos(vec,inds)
            rvec=take(vec,inds)#representative combination
            tup=tuple(rvec)
            if not dba.has_key(tup):dba[tup]=[[],[]]
            dba[tup][0].append(inds)
            dba[tup][1].append(distvec)
    return dba

def getconvdct(rev=False,format='bmr',fromxplor=False,tocsy=False):
    if tocsy:
      buf2=initfil2('nonTOCSY.txt')
      buf = initfil('nonTOCSY.txt')
      buf2+=initfil2('nonTOCSY13C.txt')
      buf += initfil('nonTOCSY13C.txt')
    else:
      buf2=initfil2('conversions.txt')
      buf = initfil('conversions.txt')
    parser=Parser(buf2)
    parser.search('Residue','atoms',positions=[1],positions2=[2],verb=True)
    i0=parser.start
    buf=buf[i0+1:];buf2=buf2[i0+1:]
    dct={}
    for i,lin2 in enumerate(buf2):
	if len(lin2)<>0:
	    lin=buf[i]
	    ##print lin
	    ##print lin2
	    lint=string.split(lin,'\t')
	    ##print lint,len(lint)
	    resn=lin[0]
	    if resn not in dct:dct[resn]={}
	    bmratn=lin2[1]
	    pos=len(bmratn)+3
	    tc=lin[pos:pos+2].count('\t')
	    ##print '---',lin[pos:]
	    if tc==2:pdbatn  =lin2[2]
	    elif tc==1:pdbatn=lin2[3]
	    xploratn=lint[7]
	    if xploratn=='':xploratn=pdbatn
	    ##print pdbatn,bmratn
	    if format=='xplor':bmratn=xploratn
	    elif format=='DIANA':bmratn=lint[10][:-1]#excluding the trailing newline char
	    if rev:
	     if fromxplor:
	      dct[resn][xploratn]=bmratn
	     else:
	      dct[resn][pdbatn]=bmratn
	    else:
	      dct[resn][bmratn]=pdbatn
    if format=='DIANA':
	print 'NOTE: using modyfied GLY HA name conventions!'
	dct['G']['HA2']='1HA'
	dct['G']['HA3']='2HA'
    return dct

def getambgroups(rev=False,colnum=3):
    buf=initfil2('ambgroups.txt')
    dct={}
    aa1s=aa13dict.keys()
    for aa in aa1s:dct[aa]={}
    for i in range(len(buf)/2):
	    lins=[buf[2*i+j] for j in range(2)]
	    resn=lins[0][0]
	    pdbatns=[lin[colnum] for lin in lins]
	    print resn,pdbatns,
	    comnam=''
	    for k in range(len(pdbatns[0])):
		if pdbatns[0][k]==pdbatns[1][k]:comnam+=pdbatns[0][k]
	    print comnam
	    if rev:
	      dct[resn][comnam]=tuple(pdbatns)
	    else:
	      dct[resn][tuple(pdbatns)]=comnam
    print dct.keys()
    return dct

def getmethyls(colnum=2):
    buf=initfil2('methyls.txt')
    dct={}
    aa1s=aa13dict.keys()
    for aa in aa1s:dct[aa]={}
    for i in range(len(buf)/3):
	    lins=[buf[3*i+j] for j in range(3)]
	    resn=lins[0][0]
	    pdbatns=[lin[colnum] for lin in lins]
	    print 'tjekmet',resn,pdbatns,
	    comnam=''
	    for k in range(len(pdbatns[0])):
		if pdbatns[0][k]==pdbatns[1][k]:comnam+=pdbatns[0][k]
	    print comnam
	    dct[resn][tuple(pdbatns)]=comnam
    print dct.keys()
    return dct

class ShiftDict(dict):

    rps=[('HD1','HD2'),('HE1','HE2'),('CD1','CD2'),('CE1','CE2')]

    def __init__(self):
        dict.__init__(self)
        self.counts={'H':0,'C':0,'N':0}

    def show(self):
	ress=self.keys()
	ress.sort(lambda x,y: cmp(eval(x),eval(y)))
	for res in ress:
	    ats=self[res].keys()
	    ats.sort()
	    for at in ats:
	      vals=self[res][at]
	      if len(vals)==3:#meant for shiftX
		sh,err,rl3=vals
		print '%3s %-s %-4s %8.4f %s'%(res,rl3,at,sh,err)
	      else:
		if len(vals)==4:
		    sh,err,rl3,ambc=vals
		    ambat='----'
		elif len(vals)==5:
		    sh,err,rl3,ambc,ambat=vals
		else:print 'WARNING: illegal definition',vals
		print '%3s %-4s %3s %-3s %-4s %6.2f %s'%(res,at,rl3,ambc,ambat,sh,err)

    def defineambiguities(self):
	for res in self:
	    amb2={'C':[],'H':[],'N':[]}
	    amb3={'C':[],'H':[],'N':[]}
	    for at in self[res]:
		sh,err,rl3,ambc=self[res][at]
		elem=getelem(at)
		if ambc=='2':
		   amb2[elem].append(at)
		elif ambc=='3':
		   amb3[elem].append(at)
	    ##print res,amb2,amb3
	    for elem in 'CHN':
		#geminal ambiguities
		if amb2[elem]<>[]:
		   if len(amb2[elem])==2:
			for i,at in enumerate(amb2[elem]):
			    self[res][at].append(amb2[elem][1-i])#the other atom
		   else:
			#find pairs...
			##print 'This is the hard part'
			olmat=[]
			for i,ati in enumerate(amb2[elem]):
			    ollst=[]
			    for j in range(len(amb2[elem])):
			      if i<>j:
			        atj=amb2[elem][j]
			        olij=get_str_overlap(ati,atj)
				ollst.append(olij)
			      else:
				ollst.append(0)
			    maxol=max(ollst)
			    j=ollst.index(maxol)
			    olmat.append([maxol,j])
			    ##print 'pair:',ati,amb2[elem][j]
		        if len(amb2[elem])%2==1:
			  print 'WARNING: uneven number of ambiguous assignments within res',amb2[elem]
			  minol=min(olmat)
			  minoli=olmat.index(minol)
			  ##print amb2[elem][minoli]
			  olmat[minoli][0]=-1
			for i,ati in enumerate(amb2[elem]):
			  maxol,j=olmat[i]
			  if maxol>1:
			    self[res][ati].append(amb2[elem][j])#the other atom? - uniquely?
			  if maxol<0:
			    self[res][ati][3]='2.1'
		#aromatic symmetric ambiguities
		if amb3[elem]<>[]:
		   if len(amb3[elem])==1:
			arat=amb3[elem][0]
			vals=self[res][arat]
			arres=vals[2]
			if arres in ['PHE','TYR']:
			    #add the mirror atom shift
			    match=False
			    for p in self.rps:
				for n in range(2):
				    if arat==p[n]:
					vals[3]=1#unambig.
					self[res][p[1-n]]=vals[:]
					self[res][p[1-n]].append(arat)
					vals.append(p[1-n])
					break
			    if not match:
				print 'WARNING: found no matching aromatic atom',arat
			else:
			  print 'WARNING: no pairs defined for',arres,arat
		   else:
		      for arat in amb3[elem]:
			vals=self[res][arat]
			arres=vals[2]
			if arres in ['PHE','TYR']:
			    for p in self.rps:
				for n in range(2):
				    if arat==p[n]:
				      ##print 'found match:',p
				      if not (p[1-n] in self[res]) or\
				      self[res][p[1-n]][0]==vals[0]:#no entry or same chemical shifts
					vals[3]=1#unambig.
					if not (p[1-n] in self[res]):
					  self[res][p[1-n]]=vals[:]
					else:self[res][p[1-n]][3]=1
					self[res][p[1-n]].append(arat)
					##print 'mirroring',arres,arat,p[1-n],vals
				      elif p[1-n] in amb3[elem]:#different shifts
					vals.append(p[1-n])
					##print arres,arat,p[1-n],'is truly ambiguous'
			    
		
def get_str_overlap(s,t):
    ol=0
    for i in range(min(len(s),len(t))):
	if s[i]==t[i]:
	    if s[i] in string.digits:
		ol+=0.1
	    else:
		ol+=1.0
    return ol

def getelem(at):
    if at in 'HCN':return at
    if at[0] in 'HCN':return at[0]
    if 'C' in at:return 'C'
    if 'N' in at:return 'N'
    if 'H' in at:return 'H'

def getShiftDBA(start,end,buf,chnum,idict,includeLabel=True,includeAmb=True):
    dba=ShiftDict()
    for lst in buf[start:end]:
        ambc=lst[idict['Chem_shift_ambiguity_code']]
        ##if ambc in ['1','.']:
        if ambc in ['1','2','3','.']:#unamb, geminal, arom (sym degenerate)
            val=lst[idict['Chem_shift_value_error']]
            if val=='.':std=None
            else:std=string.atof(val)
            ##if std==None or std<=0.2:
            if std==None or std<0.6:
                elem=lst[idict['Atom_type']]
                res=lst[idict['Residue_seq_code']]
                rl3=lst[idict['Residue_label']]
                at= lst[idict['Atom_name']]
                avg=string.atof(lst[idict['Chem_shift_value']])
                if not dba.has_key(res):dba[res]={}
                if includeLabel:
		  if includeAmb:
                    dba[res][at]=[avg,std,rl3,ambc]
		  else:
                    dba[res][at]=[avg,std,rl3]
                else:
		  if includeAmb:
                    dba[res][at]=[avg,std,'X',ambc]
		  else:
                    dba[res][at]=[avg,std,'X']
                dba.counts[elem]+=1
		##print res,rl3,at,avg,ambc
		##print dba[res][at]
	    else:print 'skipping',std
    dba.show()
    if includeAmb:
	dba.defineambiguities()
	dba.show()
    return dba  

class Parser:

    def __init__(self,buf):
        self.buffer=buf
        self.start=0
        self.numlines=len(buf)
        self.terminated=False

    def search(self,s='',s2=None,stopstr=None,stop=-1,numhits=1,changeStart=True,
               doReturn=True,positions=None,positions2=None,incr=1,verb=False):
        hits=0
        for k in range(self.start,len(self.buffer)):
            match=False
            if stopstr=='':
                if self.buffer[k]==[]:return None
            if s=='':#to nearest new-line
                if self.buffer[k]==[]:match=True
            elif s=='noneblank':
                if len(self.buffer[k])>0:
                    match=True
            if s<>'' or stopstr<>None:
                if positions==None:searchlist=range(len(self.buffer[k]))
                else:searchlist=positions
                for j in searchlist:
                  if len(self.buffer[k])>j:
                    if self.buffer[k][j]==s:
                        if verb:print self.buffer[k]
                        if s2==None:match=True
                        else:
                            if positions2==None:searchlist2=range(len(self.buffer[k]))
                            else:searchlist2=positions2
                            for j2 in searchlist2:
                                if self.buffer[k][j2]==s2:
                                    match=True
                    if self.buffer[k][j]==stopstr:return None
            if match:
                if verb:print 'matching'
                if verb:print self.buffer[k]
                if changeStart:self.start=k+incr
                hits+=1
                if hits==numhits:
                    if doReturn:return k+incr
                    break
        self.terminated=True

    def getExcerpt(self,end=None):
	if end==None:end=self.numlines
	return self.buffer[self.start:end]

    def getCurrent(self):
	return self.buffer[self.start]

    def findShiftData(self,ch,verb=True):
        self.search(s="_Saveframe_category",s2='assigned_chemical_shifts',
                    positions=[0],positions2=[1])
	print 'finding NMR shifts for chain',ch
        if verb:print self.start
        li,mi=None,None
        while mi==None:
            li=self.search(s='loop_')
            mi=self.search(s='_Chem_shift_value',stopstr='')
            if verb:print li,mi
            if self.terminated:
                print 'WARNING: no _Chem_shift_value entry found'
                return ShiftDict()#empty
        pos=mi-li-1
        idict={}
        self.search()
        for i in range(self.start-li-1):
            if verb:print '**',i,self.buffer[li+i]
            key=self.buffer[li+i][0][1:]
            idict[key]=i
        if verb:print idict
        startI=self.search('noneblank',incr=0)
        endI=self.search(incr=0)
        return getShiftDBA(startI,endI,self.buffer,ch,idict)


class Prob(float):
    
    def __new__(cls,x):
        if x>1 or x<0:raise ValueError
        else: return float.__new__(cls,x)
        
    def __and__(self,other):
        return Prob(self*other)
    
    def __iand__(self,other):
        return self&other
    
    def __or__(self,other):
        return Prob(1.0-(1.0-self)*(1.0-other))
    
    def __ior__(self,other):
	return self|other

    def __invert__(self):
        return Prob(1.0-self)
    
    def __div__(self,other):
	return Prob(float.__div__(self,other))

    def __idiv__(self,other):
	return self/other

    def sub(self,other):
        return self&(~other)

    def allAnd(ps):
	if len(ps)==0:return Prob(1.0)
	elif len(ps)==1:return ps[0]
	else: P=ps[0]
	for p in ps[1:]:P=P&p
	return P

    allAnd = staticmethod(allAnd)


class PickData:
    #observed non-shifted peaks:
    ObsSingle=0#red
    ObsMerged=3#magenta
    #"shifted" peaks
    ObsShifted=1#black
    ObsMergedShifted=2#blue

    def __init__(self,FinalPeakPos,PeakHeights,PeakAssignments,exp0,lws):
       #[plst,p2lst,p3lst,p4lst],[hlst,h2lst,h3lst,h4lst],[aslst,as2lst,as3lst,as4lst]
       self.obsPeaks=len(FinalPeakPos[self.ObsSingle])+\
                      len(FinalPeakPos[self.ObsMerged])
       self.shiftedPeaks=len(FinalPeakPos[self.ObsShifted]+\
                             FinalPeakPos[self.ObsMergedShifted])
       print self.obsPeaks,'observed unshifted peaks'
       print self.shiftedPeaks,'observed shifted peaks'
       print self.obsPeaks+self.shiftedPeaks,'total'
       self.FinalPeakPos=FinalPeakPos
       self.PeakHeights=PeakHeights
       self.PeakAssignments=PeakAssignments
       self.peakList=[]
       for i in range(4):
           for j in range(len(FinalPeakPos[i])):
               peak=Peak_sparky_Virtual(i,FinalPeakPos[i][j],PeakHeights[i][j],exp0,
                                            assign=PeakAssignments[i][j],lws=lws)
               self.peakList.append(peak)
       self.allFinalPeakPos=FinalPeakPos[0]+FinalPeakPos[1]+FinalPeakPos[2]+FinalPeakPos[3]
       self.allFinalPeakHeights=PeakHeights[0]+PeakHeights[1]+PeakHeights[2]+PeakHeights[3]
       vc=VoxelContainer()
       vc.vwidth=exp0.lineWidth*2.01*ones(exp0.dimension)
       vc.fillContainer(self.allFinalPeakPos)
       numNoisePeaks=int(exp0.pN*len(self.allFinalPeakPos))
       print 'adding',numNoisePeaks, 'noise peaks'
       for i in range(numNoisePeaks):
           self.addNoisePeak(len(self.allFinalPeakPos),exp0,vc)

    def writePeakList(self,exp0,filename):
        file=open(filename,'w')
        ##file1=open(filename+'_sparky','w')
        ##file2=open(filename+'_sparky2','w')
        for i in range(len(self.peakList)):
          peak=self.peakList[i]
          if peak.names==None:
               if exp0.dimension==2:
                  line='     ?-?     '
               else:
                  line='    ?-?-?    '
               dist=0.0
          elif 'input' in exp0.inpmapexp:
	       line=peak.names
	       dist=0.0
          else:
               ats=peak.names
               ##dist=AtomMath.calc_distance(ats[-2],ats[-1])
               dist=sqrt(sum((ats[-2].position-ats[-1].position)**2))
               rns,ris,rcs=[],[],[]
               for j in range(exp0.dimension):
                   ris.append(ats[j].fragment_id)
                   rns.append(aa31dict[ats[j].res_name])
                   rcs.append(ats[j].name)
               frd=abs(eval(ris[0])-eval(ris[1]))
               if exp0.dimension==2:
                   if ris[0]==ris[1]:#2d
                       line='%6s-%-6s'%(rns[0]+ris[0]+rcs[0],rcs[1])
                   else:
                       line='%6s-%-6s'%(rns[0]+ris[0]+rcs[0],rns[1]+ris[1]+rcs[1])
               elif exp0.dimension==3:
                   if ris[0]==ris[1]:#3d
                       ##line='%6s-%-6s%-6s'%(rns[0]+ris[0]+rcs[0],rcs[1],rcs[2])
                       line='%6s-%-6s'%(rns[0]+ris[0]+rcs[0],rcs[1])
                   else:
                       ##line='%6s-%-6s%-6s'%(rns[0]+ris[0]+rcs[0],rns[1]+ris[1]+rcs[1],rns[2]+ris[2]+rcs[2])
                       line='%6s-%-6s'%(rns[0]+ris[0]+rcs[0],rns[1]+ris[1]+rcs[1])
                   if ris[1]==ris[2]:
                       line=''+line.strip()+'-'+rcs[2]
                   else:
                       line=''+line.strip()+'-'+rns[2]+ris[2]+rcs[2]
                   ##print line
               else:
                   line='%6s'%(rns[0]+ris[0]+rcs[0])
                   for i in range(1,exp0.dimension):
                       if ris[i-1]==ris[i]:
                           line=line.strip()+'-%-6s'%(rcs[i])
                       else:
                           line=line.strip()+'-%-6s'%(ris[i]+rcs[i])
                   print line
          if exp0.dimension>3:
              line2='%-16s'%(line)
              for i in range(exp0.dimension):
                  line2=line2+'%8.2f'%(peak.shifts[i])
              line2=line2+'%9.2f %9.2f %9.2f %9.2f '%(peak.volumes,peak.rms,
                                                  peak.fitheight,peak.height)
              for i in range(exp0.dimension):
                  line2=line2+'%7.2f'%(peak.lws[i])
              line2=line2+'%7.2f'%(dist)
          elif exp0.dimension==2:
              ##tup=(line,peak.shifts[0],peak.shifts[1],peak.volumes,peak.rms,
              ##        peak.fitheight,peak.height,peak.lws[0],peak.lws[1],dist)
              tup=(line,peak.shifts[0],peak.shifts[1],peak.volumes,dist)
              ##print tup
              ##line2='%16s%8.3f%8.3f%8.1f %8.1f %8.3f %8.3f %8.1f %8.1f %8.3f '%tup
              line2='%16s%8.3f%8.3f%8.1f %8.3f '%tup
          elif exp0.dimension==3:
              ##tup=(line,peak.shifts[0],peak.shifts[1],peak.shifts[2],peak.volumes,peak.rms,
              ##        peak.fitheight,peak.height,peak.lws[0],peak.lws[1],peak.lws[2],dist)
              tup=(line,peak.shifts[0],peak.shifts[1],peak.shifts[2],peak.volumes,dist)
              ##print tup
              ##line2='%16s%8.3f%8.3f%8.3f%8.1f%8.1f%9.2f%9.2f%8.1f%8.1f%8.1f%8.3f '%tup
              line2='%16s %8.3f %8.3f %8.3f %8.1f %8.3f '%tup
	  if peak.allAssigns<>None:
		for ats in peak.allAssigns:line2=line2+' '+getAsFormat(ats,sep='_')
          file.write(line2+'\n')
##          file1.write(lineSparky+'\n')
##          file2.write(lineSparky2+'\n')
        file.close()
        ##file2.close()
        ##file1.close()
                   
    def addNoisePeak(self,num,exp0,vc):
        co=[]
        lh=max(-0.5,normalvariate(0,0.3))
        hi=1.65*exp(lh)
        for n in range(exp0.dimension):
            i=randint(0,num-1)
            ei=normalvariate(0,exp0.lineWidth)
            pi=self.allFinalPeakPos[i][n]
            co.append(pi+ei)
        dlst=vc.findNearestFromCoord(co,distMetricVec,None,maxd=exp0.lineWidth)
        if len(dlst)==0:
            ##print co,hi
            peak=Peak_sparky_Virtual(4,co,hi,exp0)
            self.peakList.append(peak)
            self.allFinalPeakPos.append(co)
            self.allFinalPeakHeights.append(hi)
            #TODO: excl hvis exp0.get_pFalseNeg(co)>0
        else:
            print 'trying again'
            self.addNoisePeak(num,exp0,vc)
            

    def visualize(self,lw,dim):
         print self.FinalPeakPos[0][0]
         print self.PeakHeights[0][0]
         if dim==3:
             cols='rkbm'
             for i in range(4):
               if len(self.FinalPeakPos[i])>0:
                 vc=VoxelContainer()
                 vc.fillContainer(self.FinalPeakPos[i])
                 vc.vallst=self.PeakHeights[i]
                 print vc.vwidth
                 vc.putSlices()
                 cosa,hia,posa=vc.getRawPeaks(120,4,distMetricVec,None)
                 if len(cosa)>0:
                     showdata(cosa,hia*agaus(posa,lw),
                             cols[i],heigh2LevelCurve,args=(80,0.75,lw))
         else:
             showdata(self.FinalPeakPos[self.ObsSingle],self.PeakHeights[self.ObsSingle],
                      'r',heigh2LevelCurve,args=(600,0.75,lw))
             showdata(self.FinalPeakPos[self.ObsShifted],self.PeakHeights[self.ObsShifted],
                      'g',heigh2LevelCurve,args=(600,0.75,lw))
             showdata(self.FinalPeakPos[self.ObsMergedShifted],self.PeakHeights[self.ObsMergedShifted],
                      'g',heigh2LevelCurve,args=(600,0.75,lw))
             showdata(self.FinalPeakPos[self.ObsMerged],self.PeakHeights[self.ObsMerged],
		      'r',heigh2LevelCurve,args=(600,0.75,lw))
             ##show();1/0


class SimplePickData(PickData):

    def __init__(self,vspec):
	self.peakList=[]
	self.allFinalPeakPos=[]
	self.allFinalPeakHeights=[]
	for i,co in enumerate(vspec.list):
    	  peak=Peak_sparky_Virtual(i,list(co),vspec.vallst[i],vspec.experiment,[vspec.atlst[i]])
	  self.peakList.append(peak)
	  self.allFinalPeakPos.append(co)
	  self.allFinalPeakHeights.append(vspec.vallst[i])


class VirtSpectrum(VoxelContainer):
    lineWidths=[]

    def getSpectrumHeight(self,X,lws):
       sigGausCurves=[]
       nnlst=self.findNearestFromCoord(X,distMetric3,lws,list='y')
       if nnlst==[]:return 0.0
       val=-1.0*GetSumGausNd(X,self,nnlst,lws)
       return val

    def showContourPlot(self,xr,yr,s,z=None):
       lws=self.lineWidths
       X=numpy.arange(xr[0],xr[1],s[0])
       Y=numpy.arange(yr[0],yr[1],s[1])
       Z=numpy.empty(((yr[1]-yr[0])/s[1],(xr[1]-xr[0])/s[0])) 
       print len(X),len(Y),len(Z),len(Z[0])
       if len(X)>len(Z[0]):X=X[:-1]
       if len(Y)>len(Z):Y=Y[:-1]
       maxh=0
       for i in range(int((yr[1]-yr[0])/s[1])):
          for j in range(int((xr[1]-xr[0])/s[0])):
            x=xr[0]+s[0]*j
            y=yr[0]+s[1]*i
            if z==None:tup=(x,y)
            else:tup=(z,x,y)
            h=self.getSpectrumHeight(tup,lws)
            if h>maxh:maxh=h
            Z[i][j]=h
            #print Z[i][j],
       #print Z
       ##contour(X*-1.0,Y*-1.0,Z,georange(1.0,maxh,1.5))   
       contour(Y*-1.0,X*-1.0,transpose(Z),georange(1.0,maxh,1.5))   

    def getPeakPos(self):
       peakPos=VoxelContainer()
       peakPos.vwidth=self.lineWidths*6.0
       peaklst=[]
       Vallst=[]
       signns=[]
       print len(self.list)
       for i in range(len(self.list)):
         #print i,
         p,h=self.list[i],self.vallst[i]
         if abs(p[0]-p[1])/self.lineWidths[0]>3:
          nnlst=self.findNearestNeighbours(i)
          sigGausCurves=[i]
          sumreloh=0
          signn=[]
          for ni in nnlst:
           if ni<>i:
            np,nh=self.list[ni],self.vallst[ni]
            reloh=gausNd(nh,p,np,self.lineWidths)/h
            if reloh>0.001:
               sigGausCurves.append(ni)
               sumreloh=sumreloh+reloh
               signn.append((reloh,ni))
            #print ni,np,nh,reloh
          #print len(sigGausCurves),sumreloh
          if sumreloh<5.0 or len(sigGausCurves)<15:
             signn.sort()
             signn.reverse()
	     print signn
             ##print p,sigGausCurves
             sol=opt.fmin_bfgs(GetSumGausNd,p,fprime=GetSumGradGausNd,gtol=0.001,\
                          args=(self,sigGausCurves,self.lineWidths),disp=0)
##             sol=opt.fmin(GetSumGausNd,p,xtol=1e-2,ftol=5e-4,\
##                         args=(self,sigGausCurves,self.lineWidths),disp=0)
             #print sol
             val=GetSumGausNd(sol,self,sigGausCurves,self.lineWidths)
             newsignn=[(1.0,i)]
             dlst=self.findNearestFromCoord(sol,distMetricVec,None,maxd=self.lineWidths[0]*3)
             h0=gausNd(h,sol,p,self.lineWidths)
             for x in dlst:
               if x[2]<>i:
                 px,hx=self.list[x[2]],self.vallst[x[2]]
                 reloh=gausNd(hx,sol,px,self.lineWidths)/h0
                 if reloh>0.01:newsignn.append((reloh,x[2]))
##             cdiff=distMetricTup(p,sol)
##             if cdiff>0.5:print 'differs:',cdiff,p,sol
             newsignn.sort()
             newsignn.reverse()
             peaklst.append(sol)
             Vallst.append(-val)
             allsignn=[(1.0,i)]+signn
             ##signns.append(allsignn)
             signns.append(newsignn)
             ##print distMetricVec(p,sol),
             #print i,len(sigGausCurves),sumreloh
             #print '------------------'
       peakPos.fillContainer(peaklst)
       peakPos.vallst=Vallst
       peakPos.signns=signns
       print len(peaklst),'raw peak-pick positions'
       self.peakPos=peakPos
       return peakPos

    def showRawPickedPeaks(self):
       lw=self.lineWidths[0]
       if self.dimension==3:
           self.peakPos.putSlices(vwidth=10)
           cosa,hia,posa=self.peakPos.getRawPeaks(120,5,distMetricVec,None)
           ##print len(cosa),cosa
           showdata(cosa,hia*agaus(posa,lw),
                     'g',heigh2LevelCurve,args=(40,0.75,lw))
       else: showdata(self.peakPos.list,self.peakPos.vallst,'g',heigh2LevelCurve,args=(40,0.75,lw))

    def getFinalPeakPosH(self):
       plst,hlst=[],[]
       aslst=[]
       p2lst,h2lst=[],[]
       p3lst,h3lst=[],[]
       p4lst,h4lst=[],[]
       as4lst=[]
       as2lst=[];as3lst=[]
       ##rh2lst=[];rh4lst=[]
       for i in range(len(self.peakPos.list)):
         if not i in self.PeakClusterContainer.i2c.keys():
           "single-cluster"
           signn=self.peakPos.signns[i]
           ##print 'results:',signn
           ##print getAsFormat(self.atlst[signn[0][1]])
           fac=0.0
           if len(signn)>1:
               ##print getAsFormat(self.atlst[signn[1][1]])
               fac=signn[1][0]/signn[0][0]
               ##print fac
           if fac<0.1:#"red" peak
               thispos=self.peakPos.list[i]
               plst.append(thispos)
               hlst.append(self.peakPos.vallst[i])
               aslst.append([self.atlst[signn[0][1]]])#the true assignment
##               ats=self.atlst[signn[0][1]]
##               shifts=ats[0].shift[0],ats[1].shift[0]
##               cdiff=distMetricTup(thispos,shifts)
##               if cdiff>0.5:
##                   print cdiff,fac,thispos,shifts
##                   print signn
##                   print ats[0]
##                   print ats[1]
           else:#"black" peak
               p2lst.append(self.peakPos.list[i])
               h2lst.append(self.peakPos.vallst[i])
               allass=[]
               for i in range(len(signn)):
		 print signn[i][0]
		 if signn[i][0]>=0.1 or i==0:#added 28/11 2013
                   allass.append(self.atlst[signn[i][1]])#one of the assignments
               as2lst.append(allass)
               ##rh2lst.append(allass)

       for c in self.PeakClusterContainer.c2i.keys():
            "cluster"
            sum=numpy.zeros(len(self.peakPos.list[0]))
            sumh=0
            asdba={}
            for j in self.PeakClusterContainer.c2i[c]:
               sum=sum+self.peakPos.list[j]
               sumh=sumh+self.peakPos.vallst[j]
               signnj=self.peakPos.signns[j]
               for reloh,ni in signnj:
                   key=getAsFormat(self.atlst[ni])
                   if not asdba.has_key(key):
                       asdba[key]=[0,self.atlst[ni]]
                   asdba[key][0]+=reloh
            meanpos=sum/len(self.PeakClusterContainer.c2i[c])
            meanh=sumh/len(self.PeakClusterContainer.c2i[c])
            ##print asdba
            vals=asdba.values()
            vals.reverse()
            vals.sort()
            vals.reverse()
            fac=vals[1][0]/vals[0][0]
            if fac<0.1:#"magenta" peak
               p4lst.append(meanpos)
               h4lst.append(meanh)
               as4lst.append([vals[0][1]])#the true assignment
            else:#"blue" peak
               p3lst.append(meanpos)
               h3lst.append(meanh)
               allass=[]
               for i in range(len(vals)):
		 if vals[i][0]>=0.1 or i==0:
                   allass.append(vals[i][1])#one of the assignments
	       ##print allass
	       ##print asdba,1/0
               as3lst.append(allass)

       exp0=self.experiment
       pnums=[len(pxlst) for pxlst in (plst,p2lst,p3lst,p4lst)]
       frac=(pnums[0]+pnums[3]+0.0)/(pnums[0]+pnums[1]+pnums[2]+pnums[3])
       print 'statspeaks:',exp0.name,self.lineWidths,frac,pnums
       #order is rkbm
       self.pickData=PickData([plst,p2lst,p3lst,p4lst],
                              [hlst,h2lst,h3lst,h4lst],
                              [aslst,as2lst,as3lst,as4lst],self.experiment,self.lineWidths)

    def getObservedPeaks(self):
       self.PeakClusterContainer=getClusters(self.peakPos,distMetric3,self.lineWidths,0.5)
       self.getFinalPeakPosH()
##       del PeakClusterContainer.i2c
##       del PeakClusterContainer.c2i
##       del spectrum3d,peakPos,PeakClusterContainer
       ##return FinalPeakPos,PeakHeights,PeakAssignments,numpeaks,obsPeak,newPeak

    def setallRawPeak2Obs(self):#initialiserer simplere PickData
	self.pickData=SimplePickData(self)

    def getRawPeaks(self,val,dist,axis=0):
        dlst=self.slices[axis].findNearestFromCoord([val],distMetricVec,None,maxd=dist)
        print dlst[:3],len(dlst)
        coslst=[];hilst=[];poslst=[]
        for x in dlst:
            coslst.append(self.slices[axis].vallst2[x[2]])
            hilst.append(self.slices[axis].vallst[x[2]])
            poslst.append(x[0])
        return array(coslst),array(hilst),array(poslst)

    def defineData(self,lst,hlst,atomlst,exp0):
       ##print len(lst[0])
       lw=exp0.lineWidth
       self.experiment=exp0
       ##self.vwidth=8*lw*ones(len(lst[0]))
       ##for tup in lst:print tup
       ##self.lineWidths=ones(len(lst[0]))*lw
       self.lineWidths=exp0.lineWidths
       print 'lineWidths',self.lineWidths
       ##SHOWON=self.experiment.name=='DipCor'
       ##SHOWON2=self.experiment.name=='DipCor'
       self.vwidth=6.0*self.lineWidths
       self.fillContainer(lst)
       self.vallst=hlst
       self.atlst=atomlst
       if SHOWON:# and self.dimension==2:
            if self.dimension==3:
                self.putSlices()
                cosa,hia,posa=self.getRawPeaks(120,4)
                showdata(cosa,hia*agaus(posa,lw),
                         'y',heigh2LevelCurve,args=(300,0.75,lw))
            else:showdata(lst,hlst,'y',heigh2LevelCurve,args=(300,0.75,lw))
            ##show()
       ##if SHOWON2:
       if exp0.docontour:
	    ctl=exp0.ctlimits
	    lws=self.lineWidths
            ##self.showContourPlot((15,30),(50,60),(0.1,0.1))
            ##self.showContourPlot((39,70),(24,50),(0.2,0.2))
            ##self.showContourPlot((0.5,5.5),(7.0,10.0),(0.01,0.01))
            ##self.showContourPlot((3.0,4.7),(7.0,9.3),(0.01,0.01))
            ##self.showContourPlot((105.0,132.0),(7.0,10.0),(0.05,0.01))
            ##self.showContourPlot((7.0,9.3),(3.0,4.6),(0.003,0.003))
            ##self.showContourPlot((3.0,4.6),(7.0,9.3),(0.003,0.003))
            ##self.showContourPlot((20,40),(170,183),(0.1,0.1))
            ##self.showContourPlot((35,70),(168,185),(0.2,0.2),z=120)
            ##self.showContourPlot((169,185),(5,70),(0.2,0.2),z=55)
            ##self.showContourPlot((169,185),(5,70),(0.2,0.2),z=120)
            self.showContourPlot((ctl[2],ctl[3]),(ctl[0],ctl[1]),(lws[0]/5,lws[1]/5))
            show()
       ##print self.container.keys()
       print '.....'
       self.getPeakPos()
       print 'Analytical peakpicking...done'
       if False:##SHOWON:# and len(lst[0])==2:
           self.showRawPickedPeaks()
           ##show();1/0
       self.getObservedPeaks()#initialiserer PickData
       ##self.setallRawPeak2Obs()#initialiserer simplere PickData
       ##self.addNoisePeaks()

    def writeRawPeakList(self):
	for i,ats in enumerate(self.atlst):
	    print '%6.2f %6.2f'%tuple(-array(self.list[i])),
	    print getAsFormat(ats)

def getFac(at,Bref=10.0, Bpow=0.0, Garom=1.0):
    if Bpow<=0.0:result=1.0
    else: 
      B=at.temp_factor
      if B<=1.0:
	##print 'WARNING: small (zero? / NMR?) B temp_factor',at
	B=10.0
      result=(Bref/B)**Bpow
    if Garom<>1.0:
      sh=at.shift[0]
      if 85<sh<160 and at.element=='C':
        result*=Garom#empiric correction for aromatic "spin-diffusion"
    return result

class pdbVoxelContainer(VoxelContainer):

    def fillpdbvc(self,pdb,distcut,nuc='C',perm=None):
       self.pdb=pdb
       clst=[]
       shlst=[]
       atreflst=[]
       ##for ch in range(len(pdb)):
       ##  for i in range(len(pdb[ch])): 
       ##    for j in range(len(pdb[ch][i])):
       ##      at=pdb[ch][i][j]
       for at in pdb.get_atoms():
             if getelem(at.name)==nuc:
               c=at.coord
               sh=at.shift
               if perm<>None and sh<>None:sh=sh+perm[ch][i][j].shift
               if sh<>None:#OK?
                   clst.append(c)
                   shlst.append(at.shift)
                   atreflst.append(_Atom(at))
		   ##print 'appended shift:',c,at.shift,at
       self.vwidth=distcut*2.01*numpy.ones(3)
       self.fillContainer(clst)
       self.shlst=shlst
       self.atreflst=atreflst

    def getcopy(self,distcut):
       pdbvc=pdbVoxelContainer()
       pdbvc.pdb=self.pdb
       pdbvc.vwidth=distcut*2.01*numpy.ones(3)
       pdbvc.fillContainer(self.list[:])
       pdbvc.shlst=self.shlst[:]
       pdbvc.atreflst=self.atreflst
       return pdbvc

class Experiment:
    #defaults
    ##lineWidth=0.30
    name=''
    dimension=0
    nucnames=[]
    atnames=[]
    order=[]
    multi=0
    dict={}
    distCut=None
    SN=10.0
    fromStructure=0
    structure=0
    rangeType=''
    skipDim=[]
    corrections=[0.0,0.0]
    ShiftDampsElem={'N':0.5,'C':0.3,'H':1.0}
    pE_H=0.5;pFN=0.10;pFNcorr=0.0;pDiff=0.0;pN=0.1;pC=0.1
    pH_H0=1.0;pE1_H=0.5
    e0=0.3;eFac=0.333
    overrideGlobSh=0
    strip=0.0
    fixbonds=None
    exclude=[]
    e1=0.0
    diagCut=0.0
    subn=None
    multitype='NOESY'
    Bref=10.0
    shpert=0.0
    onebondCH=[]

    def putVars(self,im,suffix=''):
        print '---------- '+suffix+' --------------'
        print self.name
        self.shiftDamps=numpy.ones(self.dimension)*1.0
        self.shiftTols=numpy.zeros(self.dimension)*1.0
        self.shiftstds=numpy.ones(self.dimension)*1.0
        self.globShifts=numpy.zeros(self.dimension)#0th order corr.
        self.gsh1s=numpy.zeros(self.dimension)*1.0#1st order corr.
        self.rmins=numpy.zeros(self.dimension-1)*1.0#Nonsens
        self.hypType=0
        self.rdips=numpy.zeros(self.dimension-1)*1.0
        self.maxIs=numpy.zeros(self.dimension-1)*1.0
        self.distCut=numpy.ones(self.dimension-1)*1000.0
	print self.onebondCH
	for I in self.onebondCH:
           self.distCut[I]=1.9#through bond
           self.maxIs[I]=0.99
           self.rmins[I]=9.99
        self.mindistCut=numpy.zeros(self.dimension-1)*1000.0
        self.resInclude=99999
        self.pLR=0.0
        self.noArom=False
	self.noiselevel=1.0
	self.commonTol=None
	self.restrictpairs=False
	self.ccpairdct={}
	self.bounds=[]
	self.intscale=0.4
	self.sws=[None for _ in range(self.dimension)]
        if im.has_key('skiparomatic'):
            self.noArom=eval(im['skiparomatic'])
        if im.has_key('overrideAllTols'):
            self.commonTol=eval(im['overrideAllTols'])
        if im.has_key('overrideglobalshift'):
            self.overrideGlobSh=eval(im['overrideglobalshift'])
        if im.has_key('overrideHypType'):
            self.hypType=eval(im['overrideHypType'])
            im['hypType']=im['overrideHypType']
        if im['specvars'+suffix].has_key('hypType'):
            self.hypType=eval(im['specvars'+suffix]['hypType'])
            if self.hypType==1:self.rangeType='intra'
            elif self.hypType==0:self.rangeType='long-range'
        else:
            if self.rangeType=='intra':
                self.hypType=1
        print 'self.hypType:',self.hypType
        print 'self.rangeType:',self.rangeType
        if im['specvars'+suffix].has_key('strip'):
            self.pStrip=eval(im['specvars'+suffix]['strip'])
        if im['specvars'+suffix].has_key('noiselevel'):
            self.noiselevel=eval(im['specvars'+suffix]['noiselevel'])
        if im['specvars'+suffix].has_key('resinclude'):
            self.resInclude=eval(im['specvars'+suffix]['resinclude'])
        if im['specvars'+suffix].has_key('plr'):
            self.pLR=eval(im['specvars'+suffix]['plr'])
        if im['specvars'+suffix].has_key('structure'):
            self.fromStructure=eval(im['specvars'+suffix]['structure'])
            if im.has_key('overrideFromStruc'):
                self.fromStructure=eval(im['overrideFromStruc'])
	    ##if not self.fromStructure:
	    print self.name,self.fromStructure,self.multi
            if not self.fromStructure:
              if im['specvars'+suffix].has_key('restrictpairs'):
		self.restrictpairs=True
        for i in range(self.dimension):
            dkey='shiftdamp'+str(i+1)
            if im['specvars'+suffix].has_key(dkey):
                self.shiftDamps[i]=eval(im['specvars'+suffix][dkey])
            else:
                nucname=self.nucnames[i]
                self.shiftDamps[i]=self.ShiftDampsElem[nucname]
            skey='shifttol'+str(i+1)
	    if self.commonTol<>None:self.shiftTols[i]=self.commonTol
	    elif im['specvars'+suffix].has_key(skey):
                self.shiftTols[i]=eval(im['specvars'+suffix][skey])
            else:
                self.shiftTols[i]=self.shiftDamps[i]*2.0
            skey='shiftstd'+str(i+1)
            if im['specvars'+suffix].has_key(skey):
                self.shiftstds[i]=eval(im['specvars'+suffix][skey])
            swkey='shsws'+str(i+1)
            if im['specvars'+suffix].has_key(swkey):
                swsi=im['specvars'+suffix][swkey]
		print 'using user defined spectral widths',swsi
                self.sws[i]=(eval(swsi[0]),eval(swsi[1]))
                print self.sws
            if not self.overrideGlobSh:
                gskey='globshift'+str(i+1)
                if im['specvars'+suffix].has_key(gskey):
                    self.globShifts[i]=eval(im['specvars'+suffix][gskey])
                gskey1='gsh1st'+str(i+1)
                if im['specvars'+suffix].has_key(gskey1):
                    self.gsh1s[i]=eval(im['specvars'+suffix][gskey1])
            if i<self.dimension-1:
                rmkey='rmin'+str(i+1)
                if im['specvars'+suffix].has_key(rmkey):
                    self.rmins[i]=eval(im['specvars'+suffix][rmkey])
                rdkey='rdip'+str(i+1)
                if im['specvars'+suffix].has_key(rdkey):
                    self.rdips[i]=eval(im['specvars'+suffix][rdkey])
                maxkey='maxI'+str(i+1)
                if im['specvars'+suffix].has_key(maxkey):
                    self.maxIs[i]=eval(im['specvars'+suffix][maxkey])
                rckey='rcut'+str(i+1)
                if im['specvars'+suffix].has_key(rckey):
                    self.distCut[i]=eval(im['specvars'+suffix][rckey])
                rckey='rmincut'+str(i+1)
                if im['specvars'+suffix].has_key(rckey):
                    self.mindistCut[i]=eval(im['specvars'+suffix][rckey])
        if im['specvars'+suffix].has_key('pE_H'):
            self.pE_H=eval(im['specvars'+suffix]['pE_H'])
        if eval(im['numexperiments'])>1 and self.hypType==0:
            if im['specvars'+suffix].has_key('pE1_H'):
                self.pE1_H=eval(im['specvars'+suffix]['pE1_H'])
            self.pE2_H=CalcVolfacCum(log(2.6),0.15,rmin=self.rmins[0],n=6.0)/\
                     CalcVolfacCum(log(1.7),0.15,rmin=self.rmins[0],n=6.0)*\
                     self.pE1_H
            self.pEn_H=CalcVolfacCum(log(min(4,self.distCut[0])),0.15,rmin=self.rmins[0],n=6.0)/\
                     CalcVolfacCum(log(1.7),0.15,rmin=self.rmins[0],n=6.0)*\
                     self.pE1_H
            print self.pE1_H,self.pE2_H,self.pEn_H
            if im['specvars'+suffix].has_key('pE2_H'):
                self.pE2_H=eval(im['specvars'+suffix]['pE2_H'])
            if im['specvars'+suffix].has_key('pEn_H'):
                self.pEn_H=eval(im['specvars'+suffix]['pEn_H'])
            else:rmin0=4.5
	    print self.rmins
            if rmin0<>self.rmins[-1]:
                fac=CalcVolfacCum(log(1.7),0.15,rmin=self.rmins[0],n=6.0)
		if len(self.rmins)>1:
                     fac+=CalcVolfacCum(log(rmin0),0.15,rmin=self.rmins[1],n=6.0)*2
                print 'fac:',fac
                self.pH_H0=fac#this is the default
                print 'self.pH_H0:',self.pH_H0
		if len(self.rmins)>1:#TODO: TJEK!
                    self.pE0_H=CalcVolfacCum(log(rmin0),0.15,rmin=self.rmins[1],n=6.0)
                    print 'self.pE0_H',self.pE0_H
            if im['specvars'+suffix].has_key('pH_H0'):
                self.pH_H0=eval(im['specvars'+suffix]['pH_H0'])
                print 'self.pH_H0:',self.pH_H0
                self.pE0_H=self.pH_H0/2/CalcVolfacCum(log(1.7),0.15,rmin=self.rmins[0],n=6.0)
                print 'self.pE0_H',self.pE0_H
            self.norm1=CalcVolfacCum(log(1.7),0.15,rmin=self.rmins[0],n=6.0)
            self.PH1_H0=self.pH_H0/CalcVolfacCum(log(1.7),0.15,rmin=self.rmins[0],n=6.0)
        if im['specvars'+suffix].has_key('pFN'):
            self.pFN=eval(im['specvars'+suffix]['pFN'])
	    print 'setting prob false negative (pFN) to',self.pFN
        if im['specvars'+suffix].has_key('pFNcorr'):
            self.pFNcorr=eval(im['specvars'+suffix]['pFNcorr'])
        if im['specvars'+suffix].has_key('pDiff'):
            self.pDiff=eval(im['specvars'+suffix]['pDiff'])
        if im['specvars'+suffix].has_key('pN'):
            self.pN=eval(im['specvars'+suffix]['pN'])
        if im.has_key('rminH'):rmin0=eval(im['rminH'])
        if im['specvars'+suffix].has_key('lineWidth'):
            self.lineWidth=eval(im['specvars'+suffix]['lineWidth'])
        if im.has_key('overridelw'):
            self.lineWidth=eval(im['overridelw'])
        if im['specvars'+suffix].has_key('SN'):
            self.SN=eval(im['specvars'+suffix]['SN'])
            if im.has_key('multSN'):
                self.SN*=eval(im['multSN'])
        self.inpmapexp=im['specvars'+suffix]
        if im['specvars'+suffix].has_key('F0s'):
          F0sdat=im['specvars'+suffix]['F0s']
          if isinstance(F0sdat,str):
	    same=eval(F0sdat)
	    self.F0s=[same,same,same]
          else:
	    self.F0s=[eval(F0) for F0 in F0sdat]
        if 'intscale' in self.inpmapexp:self.intscale=eval(self.inpmapexp['intscale'])
        if 'Bref' in self.inpmapexp:self.Bref=eval(self.inpmapexp['Bref'])
        if 'Bpow1' in self.inpmapexp:self.Bpow1=eval(self.inpmapexp['Bpow1'])
        if 'Bpow2' in self.inpmapexp:self.Bpow2=eval(self.inpmapexp['Bpow2'])
        if 'Garom' in self.inpmapexp:self.Garom=eval(self.inpmapexp['Garom'])
        if 'shpert' in self.inpmapexp:self.shpert=eval(self.inpmapexp['shpert'])
	self.errorvars=[0.3,0.1,0.75,0.333,0.8]
	for n,varnam in enumerate(['e_0','emin','h_0','klw','m_0']):
          if varnam in self.inpmapexp:self.errorvars[n]=eval(self.inpmapexp[varnam])
	self.e0=self.errorvars[0]
	self.e1=self.errorvars[1]
	self.eFac=self.errorvars[3]
        self.setGexpect()            
        self.setAll_lineWidths(im['state'])
	self.docontour=False
	if 'contour' in im:
	  ctnum=im['contour']
	  if ctnum==suffix:
	    self.docontour=True
	    self.ctlimits=[eval(x) for x in im['ctlimits']]
	    if 'plottype' in im:plottype=im['plottype']
	    else:plottype='gaus'
	    if plottype[:3] in ['lor','Lor']:
		gausNd=lorenNd
		gausNdGrad=lorenNdGrad
		global gausNd
		global gausNdGrad
        print self.shiftDamps
        print self.shiftTols
        print self.globShifts
        print self.G_expects
	print 'above is G_expects'

    def setAll_lineWidths(self,state):
	if 'lineWidths' in self.inpmapexp:
	    self.lineWidths=array([eval(lw) for lw in self.inpmapexp['lineWidths']])
	    return
	else:self.lineWidths=ones(self.dimension)*self.lineWidth
	if state[:3]=='liq':
	  for j in range(self.dimension):
	    if self.nucnames[j]<>'H':self.lineWidths[j]*=10.0

    def setatnords(self):
	self.atnords=set([])
	for i in range(self.dimension):
	  if self.atnames[i]<>None:
	    if self.order[i]==None:ordi=self.exporder[i]
	    else:ordi=self.order[i]
	    self.atnords.add((self.atnames[i],ordi))

    def setGexpect(self):
        dkBonds=(1.7,2.6,3.4)
        Gs=ones((self.dimension-1,3))*1.0
        for n in range(self.dimension-1):
          if self.multi[n]<>0:
            for k in range(len(dkBonds)):
                Gs[n,k]=CalcVolfacCum(log(dkBonds[k]),0.15,
                                      rmin=self.rmins[n],n=6.0)
        self.G_expects=Gs

    def getGexpt(self,ats,j):
	try:conPris=self.getConPris(None)
	except AttributeError:conPris=None
        G=1.0
        Gs=self.G_expects
        for i in range(len(ats)-1):
            if self.multi[i]<>0:
	      if self.fixbonds==None:
                if self.fromStructure:
                  d=AtomMath.calc_distance(ats[i],ats[i+1])
                  if 1.0<d<=2.1:G*=Gs[i,0]#1-bond
                  elif 2.1<d<=2.7:G*=Gs[i,1]#2-bond
                  elif 3.0<d<=3.7:G*=Gs[i,2]#3-bond
                  else:G*=CalcVolfacCum(log(d),0.15,rmin=self.rmins[i],n=6.0)
		else:
		  cp=conPris[i]
		  if cp<>None:G*=cp.prob(ats[i],ats[i+1])
        return G

    def getNormCs(self,xstd,mindist=1.53,elr=0.15,minstd=0.25):
        shnormc=1.0
        vnormc=1.0
	shnormcs=[]
        for k in range(self.dimension):
            shnormc=shnormc/math.sqrt(2*pi*(self.shiftDamps[k]**2+xstd**2))##+\
                                   ##(self.shiftstds[k]*minstd)**2)#hvad for hypType==2?
                                   ##(self.shiftstds[k]*1.0   )**2)#hvad for hypType==2?
            shnormcs.append(1/math.sqrt(2*pi*(self.shiftstds[k]*self.shiftDamps[k]**2+xstd**2)))
            if k<self.dimension-1 and self.multi[k]>0:
                vnormc=vnormc*CalcVolfacCum\
                          (log(mindist),elr**2,rmin=self.rmins[k],lsigint=1.0,n=6.0)
	print 'shnormc',shnormc
	print 'shnormcs',shnormcs
        self.shnormc=shnormc
        self.shnormcs=shnormcs
        self.vnormc=vnormc

    def getMaxRmin(self):
        return self.rmins

    def getShFac(self,ass,mode):
        '''Bruger pdf og metode paa ass'''
        facprod=1.0
	pkmu,pksd=ass.peakShifts,ass.peakDevs
        ##print ass.stdevs
        ##print 'peak mu,sd',pkmu[0],pksd[0]
	shfacvec=[]
        for i in range(self.dimension):
            if mode in ('normal','update'):
                pdfi=ass.atoms[i].resAs.pdf
                print ass.atoms[i].resAs.max
                ##print 'pdfi',pdfi
                shfaci=pdfi.calcIntNorm(pkmu[i],pksd[i])
                ##shfaci=min(1.0,shfaci/(self.shnormc**(1.0/self.dimension)))#TODO: check constant!
                ##shfaci=min(1.0,shfaci/self.shnormcs[i])
		if shfaci>self.refshfacs[i]:
		  print 'increasing refshfacs',i,shfaci,self.refshfacs[i],self.name,self.itnum,ass
		  self.refshfacs[i]=shfaci
                shfaci=min(1.0,shfaci/self.refshfacs[i])
            elif mode=='pdf':
		##shfaci=ass.shfacs[i]
		shfaci=min(1.0,ass.shfacs[i]/self.refshfacs[i])
		#NOTE: shfaci must be "raw" non-normalized value
            print 'shfaci:',shfaci
	    shfacvec.append(shfaci)
            facprod*=shfaci
        print
        ##print 'final shFac:',facprod
        ass.setShFac(facprod)
        ass.setProjGifi(shfacvec)

    def getVolFac(self,ass,Rmins,vnormc):
        ##TODO:tjek metode...
        volfac=1.0##/vnormc
        ##print Rmins
        maxvolfac=0.0
        maxpos=0
        for k in range(len(ass.distances)):
            distsk=ass.distances[k]
            volfacs=zeros(self.dimension-1)
            for i in range(self.dimension-1):
                if self.multi[i]<>0:
                    if self.fromStructure:
                        modelLD=log(distsk[i])
                        modelSig=0.15
                        volfacs[i]=CalcVolfacCum(modelLD,modelSig**2,rmin=Rmins[i],n=6.0)
                    else:
                        if isinstance(distsk[i],LRProb):
                            volfacs[i]=distsk[i].prob
			    print 'LRPROB:',[distsk[i]]
                        else:
                            volfacs[i]=distsk[i]#a probability
		else:volfacs[i]=0.97
            volfac*=prod(volfacs)
            ##if volfac>maxvolfac:maxvolfac=volfac;maxpos=k;maxvolfacs=volfacs
        if self.fromStructure:volfac/=vnormc
        ##print maxvolfac
        print volfac
##        ass.setVolFac(maxvolfac,maxvolfacs,k) FEJL???!!!
        ##ass.setVolFac(min(1.0,maxvolfac),maxvolfacs,maxpos)
        ass.setVolFac(min(1.0,volfac),volfacs,-1)
        ass.setOrderAtoms()

    def getFactors(self,ass,Rmins,vnormc,updvec,mode):
        if updvec[0]:
            self.getVolFac(ass,Rmins,vnormc)
        if updvec[1]:
            self.getShFac(ass,mode)

    def showProj(self,i,j,col='r'):
        lst=self.peakData.peakvc.list
        print lst[0]
        A=array(lst)*-1.0
        scatter(A[:,i],A[:,j],c=col,s=100,faceted=False,alpha=0.75)

    def getDistCuts(self,peak):
        return self.distCut

    def getResIncl(self,peak):
	return self.resInclude

    def getConPris(self,peak):
	return self.conPriors

    def getGeneralizedDist(self,ati,atj,dimnum,conPris,pairdct={},bounds=None,ResIncl=9999):
        if self.fromStructure:
            ##if self.resInclude<=4:
            if ResIncl<=4:
                rtype=getType((ati,atj))
                if rtype<=ResIncl:
                    return AtomMath.calc_distance(ati,atj)
                else: return 9999.0
            else: return AtomMath.calc_distance(ati,atj)
        else:
	    ##print conPris,dimnum
	    if self.fixbonds<>None:#bonded (HSQC) type
		fb=self.fixbonds[dimnum]-1
		rtype=getType((ati,atj))
		print 'TJEKFB:',fb,rtype,(ati[2],atj[2])
		if rtype>0:return 0.0
		if abs(ati[2]-atj[2])==fb:
		   print 'accepting bonded pair',fb,ati,atj
		   return 1.0#ati[2] is the pointer num
		else:return 0.0
            cp=conPris[dimnum]
            if ResIncl<=4:
                rtype=getType((ati,atj))
                if rtype<=ResIncl:
		    if ati==atj:return 0.0
		    ##print 'this time using',ati,atj,pairdct.keys(),bounds
                    return cp.prob(ati,atj,pairdct=pairdct,bounds=bounds)
                else: return 0.0
            else: return cp.prob(ati,atj)

    def acceptGeneralizedDist(self,dist,cut):
        if self.fromStructure:
            return dist<cut
        else:
            #dist is a probability
            if isinstance(dist,LRProb):
                return dist.prob>cut
            else:
                return dist>cut
        #dist is a probability cut is a minProb

    def getValidAssigns(self,posasmat,DistCuts,sideInds,conPris,ResIncl):
	if self.restrictpairs:pdct=self.ccpairdct
	else:pdct={}
        print 'DistCuts:',DistCuts,ResIncl,pdct.keys(),self.name,self.bounds
        print 'sideInds:',sideInds
        for k in range(len(posasmat)):
            print len(posasmat[k]),
        print
        allvalid=[]
        for inds in sideInds:
            validassigns=[]
            if not SILENT:print inds
            for i in range(len(posasmat[inds.index(0)])):
                ##validassigns.append([posasmat[inds[0]][i]])
                validassigns.append([i])
            if not SILENT:print validassigns
            for k in range(self.dimension-1):
		if len(self.bounds)==0:boundsk=None
		else:boundsk=self.bounds[k]
                newvalidassigns=[]
                newdistances=[]
                ik=inds.index(k);ik1=inds.index(k+1)
                for i in range(len(validassigns)):
                    ##ass1di=validassigns[i][-1]
                    ass1di=posasmat[ik][validassigns[i][-1]]
                    ati=ass1di.atom
                    fri=string.atoi(ati.fragment_id)
                    for j in range(len(posasmat[ik1])):
                        #print i,j
                        ass1dj=posasmat[ik1][j]##.deep_copy()
                        atj=ass1dj.atom
                        frj=string.atoi(atj.fragment_id)
                        if self.multi[k]==0:#through-bond transfer
                            ##if fri+self.order[k]==frj+self.order[k+1]:
			    acceptorder=False
			    if isinstance(self.order[k+1],int) and fri-self.order[k]==frj-self.order[k+1]:acceptorder=True
			    if isinstance(self.order[k+1],tuple):
				print 'checking order for',self.order[k+1]
				for ordn in self.order[k+1]:
				  if fri-self.order[k]==frj-ordn:acceptorder=True
			    if acceptorder:
                                #TODO: TJEK: er det det rigtige atom?
                                newvalidassigns.append(validassigns[i]+[j])
                                if k==0:newdistances.append([None])
                                else:newdistances.append(distances[i]+[None])
                                ##print 'TJEK',k,fri,frj
                        else:#through-space transfer
			    print 'this time using',k,ati,atj,pdct.keys(),boundsk
                            dist=self.getGeneralizedDist(ati,atj,k,conPris,pdct,boundsk,ResIncl)
                            if self.acceptGeneralizedDist(dist,DistCuts[k]):
                                if k==0:
                                    newdistances.append([dist])
                                    newvalidassigns.append(validassigns[i]+[j])
                                else:
                                    lst=validassigns[i]+[j]
                                    atlst=[posasmat[inds.index(n)][lst[n]].atom for n in range(len(lst))]
                                    if not _Atom.isMultipleLongRange(atlst):
                                        newdistances.append(distances[i]+[dist])
                                        newvalidassigns.append(lst)
                validassigns=newvalidassigns
                distances=newdistances
                ##print validassigns
##                if k==self.dimension-3:
                ##for tup in validassigns:
                ##    for n in range(len(tup)):
                ##        ass1dj=posasmat[inds.index(n)][tup[n]]
                ##        print ass1dj.atom,
                ##    print '--',
                ##print
##            if len(validassigns)==0:
##                return [([],[],[]) for inds in sideInds]
                #no valid assignments (no valid in one dim-combination)
            allvalid.append((validassigns,distances,inds))
        print 'allvalid:',allvalid
##        print len(allvalid[0][0]),'valid assignments'
        return allvalid

    def acceptPeak(self,peak):
       if peak.hErrProb>0.95:return False
       if not self.noArom:return True
       for sh in peak.shifts:
           if 90<sh<160:return False
       return True

    def isalllongrange(self):
	for lr in self.multi:
	  if not lr:return False
	return True

    def modify_data(self,other,data):
	if self.dimension<3 or other.dimension<3:return
	if len(data)<2:return
	if 's' in data.keys():
	  print 'no modification',self.name,other.name,data.keys()
	  return
	print 'modifying data',self.name,other.name,data.keys()
	nearest={};nearestrev={};uniq=[];uniqrev=[]
	axes=data.keys()
	tups=[data[ax] for ax in axes]
        #nearest,nearestrev,uniq,uniqrev,labels,plabels
	nearests=[tup[0] for tup in tups]
	labs =[tup[-2] for tup in tups]
	plabs=[tup[-1] for tup in tups]
	#nearest[I][J]=(d2,cnt,aliresid)
	if len(data)==2:#generalize?
	  clabs=set(labs[0])|set(labs[1])
	  print 'clabs:',self.name,other.name,axes,len(clabs),len(labs[0]),len(labs[1]),clabs
	  for i in nearests[0]:
	    nearest[i]={}
	    if i in nearests[1]:
	      shared=set(nearests[0][i].keys())&set(nearests[1][i].keys())
	      print 'shared:',self.name,other.name,axes,i,len(shared),shared
	      for j in shared:
		newd2=nearests[0][i][j][0]+nearests[1][i][j][0]
		aty0 =nearests[0][i][j][2]
		aty1 =nearests[1][i][j][2]
		if aty0==aty1:aliresid=aty0
		else:aliresid='na'
		nearest[i][j]=(newd2,2,aliresid)
		if not j in nearestrev:nearestrev[j]={}
		nearestrev[j][i]=(newd2,2,aliresid)
	elif len(data)>2 and other.name==self.name and self.dimension==len(data):# f.eks 2 x 3d CCC
	  clabs=set(labs[0])|set(labs[1])|set(labs[2])
	  print 'clabs:',self.name,other.name,axes,len(clabs),len(labs[0]),len(labs[1]),len(labs[2]),clabs
	  for i in nearests[0]:
	    nearest[i]={}
	    allin=True
	    for k in range(len(nearests)):
	      if not i in nearests[k]:allin=False
	    if allin:
	      nkis=[nk[i] for nk in nearests]
	      uni=allUnion([set(nki.keys()) for nki in nkis])
	      for j in uni:
		nums=sum([j in nki for nki in nkis])
		if nums>=self.dimension-1:
		  newd2=0.0;aliset=set([])
		  for k,nki in enumerate(nkis):
		    if j in nki:
		      newd2+=nki[j][0]
		      aliset.add(nki[j][2])
		  if len(aliset)>1:aliresid='na'
		  else:aliresid=aliset.pop()
		  nearest[i][j]=(newd2,self.dimension-1,aliresid)
 	for i in nearest:
	  its=nearest[i].items()
	  its.sort(lambda x, y: cmp(x[1],y[1]))
	  bestener=9.999
	  if len(its)>0:bestener=its[0][1][0]
	  print 'newnearest',self.name,other.name,i,len(its),round(bestener,3),its
	  if len(its)==1:
	    uniq.append(i)
	    if its[0][1][2]=='na':print 'warning unique false new link! (fwd)',i,its[0],bestener,self.expnum,other.expnum
	    else:print 'good unique new link! (fwd)',i,its[0],bestener,self.expnum,other.expnum
 	for i in nearestrev:
	  its=nearestrev[i].items()
	  its.sort(lambda x, y: cmp(x[1],y[1]))
	  bestener=9.999
	  if len(its)>0:bestener=its[0][1][0]
	  print 'newnearest',self.name,other.name,i,len(its),round(bestener,3),its
	  if len(its)==1:
	    uniqrev.append(i)
	    if its[0][1][2]=='na':print 'warning unique false new link! (rev)',i,its[0],bestener,self.expnum,other.expnum
	    else:print 'good unique new link! (rev)',i,its[0],bestener,self.expnum,other.expnum
 	data.clear()
	data['c']=nearest,nearestrev,uniq,uniqrev,clabs,clabs
	  

    def getAlignedPeaks(self,other,outfile,hasncaco=True,allatnords=None,delta=0):
       print 'aligning peaks..............',self.name,other.name
       vcs0= self.peakData.peakvc.slices
       vcs1=other.peakData.peakvc.slices
       ddct={}
       ##print len(vcs0)
       ##print len(vcs1)
       ord0=[]
       for n in range(self.dimension):
	if self.order[n]<>None:ord0.append(self.order[n])
	else:ord0.append(self.exporder[n])#expected order
       ord1=[]
       for n in range(other.dimension):
	##print other.order
	if other.order[n]<>None:ord1.append(other.order[n])
	else:ord1.append(other.exporder[n])#expected order
       ##for i in range(len(vcs0)):print vcs0[i],len(vcs0[i].list)
       ##for i in range(len(vcs1)):print vcs1[i],len(vcs1[i].list)
       cx0=[]
       spec0={}
       for n in range(len(vcs0)):
	    if self.atnames[n]==None:cx0.append(n)
	    else:
		##spec0[self.atnames[n]]=n
		spec0[(self.atnames[n],self.order[n])]=n
		if self.nucnames[n]=='C':cx0.append(n)
       cx1=[]
       spec1={}
       for k in range(len(vcs1)):
	     if other.atnames[k]==None:cx1.append(k)
	     else:
		##spec1[other.atnames[k]]=k
		spec1[(other.atnames[k],other.order[k]+delta)]=k
		if other.nucnames[k]=='C':cx1.append(k)
       specdct={}
       pspecdct={}
       for k in spec0:#k e.g. CA
	   if k in spec1:specdct[k]=({},spec0[k],spec1[k]) #e.g. {} 1 2
	   elif spec0[k] in cx0 and self.expnum<>other.expnum:
	     for ax in cx1:
	       if not ax in spec1.values() and ord1[ax]+delta==k[1]:
		print 'possible? (fwd)',delta,self.name,other.name,k,spec0[k],ax
	   	##pspecdct[k]=({},spec0[k],ax)
	   	pspecdct[(k,(spec0[k],ax))]=({},spec0[k],ax)
       for k in spec1:
	   if k in spec0:pass
	   elif spec1[k] in cx1 and self.expnum<>other.expnum:
	     for ax in cx0:
	       if not ax in spec0.values() and ord0[ax]==k[1]:
		print 'possible? (rev)',delta,self.name,other.name,k,spec1[k],ax
	   	##pspecdct[k]=({},ax,spec1[k])
	   	pspecdct[(k,(ax,spec1[k]))]=({},ax,spec1[k])
       remlst=[]
       for n in cx0:
	   for k in cx1:
		if ({},n,k) in specdct.values():
		   print 'removing key',self.name,other.name,n,k
		   remlst.append((n,k))
       for pair in remlst:
	   cx0.remove(pair[0])
	   cx1.remove(pair[1])
       if (self.name=='NCACX' or other.name=='NCACX') and hasncaco and not self.isalllongrange() and not other.isalllongrange():
	print 'skipping possib extra axes',delta,self.name,other.name,pspecdct##more unique with NCACO
	pspecdct={}
       if len(specdct)+len(pspecdct)>=min(self.dimension,other.dimension) and not self.isalllongrange() and not other.isalllongrange() and not self.name==other.name:
	print 'NOTE! too many axes:',delta,self.name,other.name,len(specdct),len(pspecdct),specdct,pspecdct
	if len(specdct)==min(self.dimension,other.dimension)-1:pspecdct={}
	else:
	  print 'still a problem!...',pspecdct
	  iscb=None;hasfull=[]
	  for key in pspecdct.keys():
	    if key[0][0]=='CB':iscb=key
	    sself=self.atnords.copy()
	    sself.add((key[0]))
	    sother=other.atnords.copy()
	    sother.add((key[0]))
	    if None in self.atnames and sself in allatnords and len(sself)==self.dimension:
	      print 'axes allready present as experiment (self)',key,sself,self.name
	      hasfull.append(key)
	    elif None in other.atnames and sother in allatnords and len(sother)==other.dimension:
	      print 'axes allready present as experiment (other)',key,sother,other.name
	      hasfull.append(key)
	  if iscb<>None:
	    print 'CB takes priority - removing other',iscb
	    for key in pspecdct.keys():
	      if key<>iscb:pspecdct.pop(key)
	  elif len(hasfull)>0:
	    print 'degenerate axis (or more) - removing other',len(hasfull),hasfull
	    for key in hasfull:pspecdct.pop(key)
	  else:
	    print 'still no key popped - popping C=O',pspecdct
	    for key in pspecdct.keys():
	      if key[0][0]=='C':pspecdct.pop(key)
       ##specdct.update(pspecdct)
       results={};usegen=False
       if len(pspecdct)>0 and delta==0 and not (self.isalllongrange() or other.isalllongrange()):
	print 'NOTE two options',self.name,other.name,specdct.keys(),cx0,cx1,pspecdct
	bothspecdcts=[specdct.copy()];axpairs=None
	specdct.update(pspecdct)
	bothspecdcts.append(specdct)
       else:
	specdct.update(pspecdct)
	bothspecdcts=[specdct]
	axpairs=set([v[1:] for v in specdct.values()])
	alidct={}
	if (self.isalllongrange() or other.isalllongrange()) and delta==0:
         for n in cx0:
	   for k in cx1:
	      ##if k>=n or self.expnum<>other.expnum:
	       if not (n,k) in axpairs and ord0[n]==0:
		alidct[(n,k)]=({},n,k)#k is int
		usegen=True
	        print 'general alignment',self.name,other.name,delta,(n,k)
       avelw=(self.lineWidth+other.lineWidth)/2
       for bnum,specdct in enumerate(bothspecdcts):#in former case there are two options
	print 'using',delta,bnum,self.name,other.name,specdct.keys(),axpairs,cx0,cx1,pspecdct
	##nearest={}
	if (len(cx0)==0 or len(specdct)==0) and usegen==False:return None
	for i in range(len(self.peakData.peakvc.list)):
	   peaki=self.peakData.peaklist[i]
	   co=self.peakData.peakvc.list[i]
	   print '*******',i,co,peaki.assSet,
	   ##nearest[i]={}
	   cnt=0
	   sumprob=0.0
	   maxprob=0.0
	   maxtrue=None
	   if False:
            asd=self.allassigns[i]
            if asd.maxIndex<>None:
             ass=asd.assigns[asd.maxIndex][1]
            else:
             ass=asd.asMax2
	   #find alignment on special axes
           for key in specdct:
	       specdctk,k1,k2=specdct[key]
	       print key,k1,k2,
               dlst=vcs1[k2].findNearestFromCoord([co[k1]],distMetricVec,None,maxd=avelw*1.5)##2)
	       print len(dlst)
	       for elem in dlst:
		 d,c,num=elem
		 if num<>i or self.expnum<>other.expnum:
		   peak=other.peakData.peaklist[num]
		   specdctk[(i,num)]=d
		   trueali,aliat=isTrueAlign(peaki.assSet,peak.assSet)
		   print 'align spec?: %7.4f %7.3f %3d %1d'%(d,c[0],num,trueali),peak.assSet
		   ##prob=exp(-d/0.2)
	   	   ##print 'align %2d %6.3f %7.3f'%(len(dlst),d,exp(asd.entropy)),trueali
	   #find alignment on generic CX axis
           if self.isalllongrange() or other.isalllongrange() and delta==0:
	    for n in cx0:
	     for k in cx1:
	      if k>=n or self.expnum<>other.expnum:
	       if not (n,k) in axpairs and ord0[n]==0:
	        ##print 'general alignment',self.name,other.name,delta,(n,k)
	        alipeaks=alidct[(n,k)][0]
                dlst=vcs1[k].findNearestFromCoord([co[n]],distMetricVec,None,maxd=avelw*1.5)##2)
	        ##print len(dlst)
	        for elem in dlst:
		 d,c,num=elem
		 if num<>i or self.expnum<>other.expnum:
		   usegen=True
		   peak=other.peakData.peaklist[num]
		   alipeaks[(i,num)]=d
		   trueali,aliat=isTrueAlign(peaki.assSet,peak.assSet)
		   print 'align cx?: %7.4f %7.3f %3d %1d'%(d,c[0],num,trueali),peak.assSet
		   cnt+=1
	sets=[set(v[0].keys()) for v in specdct.values()]#all the (i,num) in specdctk
	allali=allIntersect(sets)##|allUnion(gensets)
	combali=allUnion(sets)
	if usegen:
	 gensets=[set(v[0].keys()) for v in alidct.values()]
	 combgen=allUnion(gensets)
	else:combgen=set([])
	print 'allali',self.name,other.name,delta,len(sets),len(allali),len(combali),len(combgen)#list of (i,num)
	if len(bothspecdcts)>1:
	  ax1='su'[bnum]
	  results[ax1]=self.fetchAlignedPeaks(other,outfile,delta,allali,specdct,pspecdct,ax0=ax1)
       #ned-for bnum,specdct in enumerate(bothspecdcts):
       if len(results)>0:return results
       if self.isalllongrange() or other.isalllongrange():
	 if delta==0:
	   allali=combali
	   keydct={}
	   for tup in specdct:
	     atnord,axpair=tup
  	     ax0=axpair[0]
	     if not ax0 in keydct:keydct[ax0]=[]
	     keydct[ax0].append(tup)
	   print 'using "combali"',delta,self.name,other.name,keydct
	   for ax0 in keydct:
	     allali=allUnion([set(specdct[tup][0].keys()) for tup in keydct[ax0]])
	     results[ax0]=self.fetchAlignedPeaks(other,outfile,delta,allali,specdct,pspecdct,keydct,ax0)
	   if usegen:
	     keydct={}
	     for axpair in alidct:
		ax0=axpair[0]
		if not ax0 in keydct:keydct[ax0]=[]
		keydct[ax0].append(axpair)
	     ##results.append(self.fetchAlignedPeaks(other,outfile,delta,combgen,alidct,alidct))
	     print 'using general:',delta,self.name,other.name,keydct
	     for ax0 in keydct:
	       allali=allUnion([set(alidct[axpair][0].keys()) for axpair in keydct[ax0]])
	       results[ax0]=self.fetchAlignedPeaks(other,outfile,delta,allali,alidct,alidct,keydct,ax0,mode='gen')
	   return results
	 else:return None
       else:
  	 return {'u':self.fetchAlignedPeaks(other,outfile,delta,allali,specdct,pspecdct)}
    #enddef getAlignedPeaks

    def fetchAlignedPeaks(self,other,outfile,delta,allali,specdct,pspecdct,keydct=None,ax0=None,mode='spec'):
       ##final={}
       avelw=(self.lineWidth+other.lineWidth)/2
       bools=[]
       nearestrev={}
       if mode=='spec':
	 ##if ax0<>None:
	 if isinstance(ax0,int):
	   specname=self.atnames[ax0]
	   if specname==None:
	     specname=other.atnames[ax0]
	 else:
           if delta==0:specname='N'
           else:specname=specdct.keys()[0][0]
       else:specname=None
       print 'specname is:',specname,self.name,other.name,ax0,mode
       nearest={}
       for pair in allali:
	   peaks=(self.peakData.peaklist[pair[0]],other.peakData.peaklist[pair[1]])
           ##ep=self.allassigns[pair[0]].entropy*other.allassigns[pair[1]].entropy
	   assSets=[peak.assSet for peak in peaks]
	   trueali,aliat=isTrueAlign(*assSets)
	   nali,naliat=isTrueAlign(assSets[0],assSets[1],specname=specname)
	   bools.append(trueali)
	   d2=0.0
	   cnt=0
	   print 'assSets:',assSets
	   print trueali,aliat,nali,naliat
	   ##print dir(naliat)
	   if nali:aliresid=naliat[0]
	   else:   aliresid='na'
	   ##for v in specdct.values():
	   if isinstance(ax0,int):ataxpairs=keydct[ax0]
	   else:ataxpairs=specdct.keys()
	   ##print ataxpairs
	   for key in ataxpairs:
	      v=specdct[key]
	      ##print v
	      if pair in v[0]:
		cnt+=1
		d=v[0][pair]
		##print round(d,4),
		##d2+=((d/0.2)**2)
		d2+=((d/avelw)**2)
	   I=pair[0];J=pair[1]
	   ##nearest[I][J]=(d2,trueali,naliat)
	   ##nearest[I][J]=(d2,aliresid)
	   if not I in nearest:nearest[I]={}
	   nearest[I][J]=(d2,cnt,aliresid)
	   if self.expnum<>other.expnum:
	     if not J in nearestrev:nearestrev[J]={}
	     ##nearestrev[J][I]=(d2,trueali,naliat)
	     ##nearestrev[J][I]=(d2,aliresid)
	     nearestrev[J][I]=(d2,cnt,aliresid)
	   if False:##for alipeaks in alidct.values():
	      if pair in alipeaks:
		cnt+=1
		d=alipeaks[pair]
		##print round(d,4),
		d2+=((d/0.2)**2)
	   ##print
	   ##if cnt>1:print 'multi!',cnt
	   lin1='%1d %1d %1d %4d %4d %1d '%(self.expnum,other.expnum,delta,pair[0],pair[1],cnt)
	   ##lin2='%6.4f %8.3f %d'%(exp(-sqrt(d2)),ep,trueali)
	   ##lin2='%6.4f %d'%(exp(-sqrt(d2)),trueali)
	   lin2='%6.4f %d %2s '%(d2,trueali,aliresid)
	   lin2+='%s %s'%tuple(assSets)
	   ##print 'PAIR:',lin1,lin2
	   outfile.write(lin1)
	   outfile.write(lin2+'\n')
	   ##print assSets
	   ##final[pair]=(cnt*exp(-sqrt(d2)),ep,trueali)
       uniq=[];uniqrev=[]
       for i in nearest:
	 its=nearest[i].items()
	 its.sort(lambda x, y: cmp(x[1],y[1]))
	 print 'nearest',self.name,other.name,i,len(its),its,avelw
	 if len(its)==1:
	    uniq.append(i)
	    if its[0][1][1]=='na':print 'warning unique false link! (fwd)',i,its[0],self.expnum,other.expnum,delta,pspecdct.keys()
       if self.expnum<>other.expnum:
        for i in nearestrev:
	 its=nearestrev[i].items()
	 its.sort(lambda x, y: cmp(x[1],y[1]))
	 print 'nearestrev',self.name,other.name,i,len(its),its
	 if len(its)==1:
	    uniqrev.append(i)
	    if its[0][1][1]=='na':print 'warning unique false link! (rev)',i,its[0],self.expnum,other.expnum,delta,pspecdct.keys()
       print 'peaks aligned...',ax0,delta,self.name,other.name,self.expnum,other.expnum,
       print '%4d %4d %3d %3d'%(bools.count(True),bools.count(False),len(uniq),len(uniqrev)),
       labels=[(k,specdct[k][1:]) for k in specdct]
       plabels=[(k,pspecdct[k][1:]) for k in pspecdct]
       print labels,plabels
       return nearest,nearestrev,uniq,uniqrev,labels,plabels

    

    #below are procedures for virtual experiments
    def getIntensityDipCor(self,d,dimnum=0,rType=0):
       if d==0.0:return 1.0
       if d>self.distCut[dimnum] or d<self.mindistCut[dimnum]:return 0.0
       if self.SN<>None:
           if self.rdips[dimnum]==0:
	     if self.resInclude<rType:
	       print 'excluding pair',d,rType,self.resInclude
	       return 0.0
	     else:
               I0=self.maxIs[dimnum]
               r2=self.rmins[dimnum]
               b=max(-log(I0),0)
               C=0.1/I0*r2**6
               Int=exp(-b)*(1-exp(-C*(d**-6)))
	       print 'testinten',I0,r2,b,C,Int
               perm=self.seed[int(d*100000)%1000]
               #pseudo-random for each different atom pair
               result=Int*exp(perm*self.intscale)##0.4)
               if rType==1:result*=self.F0s[0]
               elif 1<rType<=4:result*=self.F0s[1]
               elif rType>4:result*=self.F0s[2]
               return result
##           lInt=log(Int)
##           lni=normalvariate(lInt,0.4)
##           return exp(lni)##*self.SN
       else:
           if d<=self.distCut[dimnum]:return 1.0
           else: return 0.0

    def getPeaksFromInput(self,buf):
	lst=[];hlst=[];atlst=[]
	for lin in buf:
	  lab=lin[0]
	  shs=[eval(x) for x in lin[1:1+self.dimension]]
	  hi=eval(lin[1+self.dimension])
	  lst.append(shs)
	  hlst.append(hi)
	  atlst.append(lab)##None)
	return lst,hlst,atlst

    def getPeak(self,ch,nuc,pdb):
        pass

    def getPeaks(self,pdb,perm=None,scales={}):
       lst=[];hlst=[];atlst=[]
       if self.multi[-1]<>0:#sidste tranfer er gennem rum
           if self.order[-1]<>None:orddiff=self.order[-1]-self.order[0]
           if self.exporder[-1]<>None:orddiff=self.exporder[-1]-self.order[0]
           elif self.porder[1]<>None:orddiff=self.order[1]-self.order[0]
           else:orddiff=0
           print self.skipDim
           'NCOCX or NCACX or NCACO'
	   if self.multitype in ['NOESY','DARR']:#through space, not via bonds as in TOCSY
             pdbvc=pdbVoxelContainer()
             pdbvc.fillpdbvc(pdb,self.distCut[-1],self.nucnames[-1],perm=perm)
	   elif self.multitype=='TOCSY':
	     tocsyconvdct=getconvdct(rev=True,fromxplor=False,tocsy=True)
           print 'getting the peaks in each residue-step',self.name
	   print self.order,self.atnames,orddiff
           for ch in pdb[0]:
              for resi in ch:
	       nuc=resi.id[1]
               print ch,nuc
               print '----------'
               if nuc+orddiff>=0 and nuc+orddiff<len(ch):
                   try:at0=_Atom(resi[self.atnames[0]])
		   except KeyError:
			print 'WARNING:no atom',nuc,self.atnames[0]
			at0=None
                   try:at1=_Atom(ch[nuc+self.order[1]][self.atnames[1]])
		   except KeyError:
			print 'WARNING:no atom',nuc+self.order[1],self.atnames[1]
			at1=None
                   if at0<>None and at1<>None and at0.shift<>None and at1.shift<>None:
                       print ch,nuc
                       print at0
                       print at1
                       ##fac=(getFac(at0)**2)*sqrt(getFac(at1))
                       fac= getFac(at0,self.Bref,self.Bpow1,self.Garom)
                       fac*=getFac(at1,self.Bref,self.Bpow2,self.Garom)
                       c=at1.position
		       at1sub=None
		       if at1.element=='N' and self.subn<>None:
			atnsub,ordsub=self.subn
			at1sub=_Atom(ch[nuc+self.order[1]+ordsub][atnsub])
			c=at1sub.position
			print 'position substituted',at1sub
		       if self.multitype=='TOCSY':
                        nextres=ch[nuc+self.order[2]]
			aai=aa31dict[nextres.resname]
			print aai,nextres
			for ati in nextres:
			 if getelem(ati.name)==self.nucnames[-1] and ati.name not in ('H','HN','C'):
			  if aai in tocsyconvdct and ati.name in tocsyconvdct[aai]:
	      		    print 'atom not related to HN by TOCSY',aai,ati.name,ati.shift
			  elif ati.shift<>None:
                            shvec=(at0.shift[0],at1.shift[0],ati.shift[0])
                            if not self.get_pFalseNeg(shvec,addDim=len(self.skipDim)):
                              at2=_Atom(ati)
                              lst.append(tuple([shi+normalvariate(0,self.shpert) for shi in shvec]))
                              atlst.append((at0,at1,at2))
                              hlst.append(getFac(at2,self.Bref,self.Bpow1,self.Garom)*self.SN)
                              print lst[-1],atlst[-1],hlst[-1]
		       elif self.multitype=='NOESY':
                        dlst=pdbvc.findNearestFromCoord\
                             (c,distMetricVec,None,maxd=self.distCut[-1])
                        if len(dlst)>1:
                           ##print len(dlst)
                           startI=1
                           if len(self.skipDim)>0:startI=0
                           for i in range(0,len(dlst)):
                               num=dlst[i][2]
                               d,ati=dlst[i][0],pdbvc.atreflst[num]
                               if ati.shift<>None:
                                 if self.atnames[2]==None or ati.name==self.atnames[2]:
                                   shvec=(at0.shift[0],at1.shift[0],pdbvc.shlst[num][0])
                                   ##print shvec
                                   if not self.get_pFalseNeg(shvec,addDim=len(self.skipDim)):
				       if at1sub==None:rtype=getType((at1,ati))
				       else:rtype=getType((at1sub,ati))
                                       inten=self.getIntensityDipCor(d,dimnum=1-len(self.skipDim),rType=rtype)
                                       print 'ati: ',ati,d,inten
                                       inten*=(fac*getFac(ati,self.Bref,self.Bpow2,self.Garom)*self.SN)
                                       if inten>1:
                                           if self.skipDim==[1]:#TODO: generaliser
                                              lst.append((at0.shift[0]+normalvariate(0,self.shpert),
                                                          pdbvc.shlst[num][0]+normalvariate(0,self.shpert)))
                                              hlst.append(inten)
                                              atlst.append((at0,pdbvc.atreflst[num]))
                                           elif self.skipDim==[0]:#TODO: generaliser
                                              lst.append((at1.shift[0]+normalvariate(0,self.shpert),
                                                          pdbvc.shlst[num][0]+normalvariate(0,self.shpert)))
                                              hlst.append(inten)
                                              atlst.append((at1,pdbvc.atreflst[num]))
                                           else:
                                              lst.append((at0.shift[0]+normalvariate(0,self.shpert),
                                                          at1.shift[0]+normalvariate(0,self.shpert),
                                                          pdbvc.shlst[num][0]+normalvariate(0,self.shpert)))
                                              hlst.append(inten)
                                              atlst.append((at0,at1,pdbvc.atreflst[num]))
               
           #self.order[0] boere vaere 0
       elif prod(self.multi)==0:## in [[0],[0,0],[0,0,0]]:
	    print self.atnames
	    split=None
	    bpows=self.Bpow1,self.Bpow2,self.Bpow2,self.Bpow2
            for ch in pdb[0]:
                for resi in ch:
		    nuc=resi.id[1]
                    lstn=[];atlstn=[]
                    fac=1.0
                    for j in range(self.dimension):
                       name,ord=self.atnames[j],self.order[j]
                       print nuc,ord
		       try:
                          resx=ch[nuc+ord]
		       except KeyError:
			    print 'WARNING: no index',nuc+ord,name
			    resx=None;break
		       if resx<>None:
		        if '/' in name:
			    split=j
			    names=string.split(name,'/')
			    print 'more atoms',names
			    ats=[];atlstx=[];facs=[];lstx=[]
			    for k in range(len(names)):
			     try:
                              at=ch[nuc+ord][names[k]]
			      shk=at.shift
			      if shk<>None:
			       lstx.append(shk[0]+normalvariate(0,self.shpert))
			       atlstx.append(_Atom(at))
			       facs.append(getFac(atlstx[-1],self.Bref,bpows[j],self.Garom))
			     except KeyError:print 'no key: (OK)',nuc+ord,names[k]
			    print 'tjekmore:',ats,atlstx,facs,lstx
		        else:
		         try:
                          at=ch[nuc+ord][name]
                          sh=at.shift
                          if sh<>None:
                              fac*=getFac(at,self.Bref,bpows[j],self.Garom)
                              lstn.append(sh[0]+normalvariate(0,self.shpert))
                              at=ch[nuc+ord][name]
                              atlstn.append(_Atom(at))
                          else:
			    print 'no shifts for:',nuc+ord,name,at
			    break
			 except KeyError:
			    print 'WARNING: no key',nuc+ord,name
			    break
                    inten=fac*self.SN
		    minlen=self.dimension
		    if split<>None:minlen-=1
                    if len(atlstn)>=minlen and inten>1 and j==self.dimension-1:
		     if split==None:
                      if self.get_pFalseNeg(lstn,addDim=len(self.skipDim)):
			print 'peak skipped due to excluded shifts (PRO? / arom?):',nuc,lstn
		      else:
                        lst.append(tuple(lstn))
                        atlst.append(tuple(atlstn))
                        hlst.append(inten)
		     else:
		      for k,shk in enumerate(lstx):
                       if self.get_pFalseNeg(lstn+[shk],addDim=len(self.skipDim)):
			print 'peak skipped due to excluded shifts (PRO? / arom?):',nuc,lstn+[shk]
		       else:
                        lst.append(tuple(lstn+[shk]))
                        atlst.append(tuple(atlstn+[atlstx[k]]))
                        hlst.append(inten*facs[k])
       return lst,hlst,atlst

    def getPeaks2(self,pdb,perm=None):#skal jeg bruge perm til noget?
       #Dette bruges naar 1. transfer er afstands-afhaengig
       #put carbon koordinater i en 3d voxelcontainer
       pdbvc=pdbVoxelContainer()
       ##pdbvc.fillpdbvc(pdb,self.distCut[0],nuc='H',perm=perm)
       pdbvc.fillpdbvc(pdb,self.distCut[0],nuc=self.nucnames[1],perm=perm)
       if self.nucnames[0]<>self.nucnames[1]:
	 pdbvc0=pdbVoxelContainer()
	 pdbvc0.fillpdbvc(pdb,self.distCut[0],nuc=self.nucnames[0],perm=perm)
       else:pdbvc0=pdbvc
       #Dette bruges naar 2. transfer er afstands-afhaengig
       if self.dimension==3:#f.eks 3D DipCor
           ##pdbvc2=pdbvc.getcopy(self.distCut[1])
           #pdbvc2 kan evt. have en finere grid-indeling ~ kortere afstande korreleres
	   pdbvc2=pdbVoxelContainer()
	   pdbvc2.fillpdbvc(pdb,self.distCut[1],nuc=self.nucnames[2],perm=perm)
       #get the peaks based on the structure
       lst=[]
       hlst=[]
       atlst=[]
       for i in range(len(pdbvc0.list)):
        if self.atnames[0]==None or self.atnames[0]==pdbvc.atreflst[i].name:
         ##if sel==None or sel[0]==None or shlst[i]>sel[0][0] and shlst[i]<sel[0][1]:
          ati=pdbvc0.atreflst[i]
	  print ati
          if self.nucnames[0]==self.nucnames[1]:searchjs=pdbvc.findNearestNeighbours(i)
	  else:
		dlst=pdbvc.findNearestFromCoord(ati.position,distMetricVec,None,maxd=self.distCut[0])
                searchjs=[tup[2] for tup in dlst]
	  for j in searchjs:
            atj=pdbvc.atreflst[j]
            d=calc_distance(ati,atj)
            inten=self.getIntensityDipCor(d,rType=getType((ati,atj)))
            fac= getFac(ati,self.Bref,self.Bpow1,self.Garom)
            fac*=getFac(atj,self.Bref,self.Bpow2,self.Garom)
            inten=inten*fac
            print 'check vars:',ati,atj,d,inten,fac
            ##if inten>1:print d,inten
            if inten*self.SN>=1.0 and pdbvc0.shlst[i]<>None and pdbvc.shlst[j]<>None:
                 if self.dimension==2:
                        shvec=(pdbvc0.shlst[i][0],pdbvc.shlst[j][0])#2d
                        if not self.get_pFalseNeg(shvec):
                          lst.append((shvec[0]+normalvariate(0,self.shpert),
                                      shvec[1]+normalvariate(0,self.shpert)))
                          hlst.append(inten*self.SN)
                          atlst.append((ati,atj))
                 else:#3d
                  for k in pdbvc2.findNearestNeighbours(j):#OK if nucnames[1]==nucnames[2] as for C13HSQC_NOESY
                    if True:##i<>k and i<>j and j<>k:#shlst[i]<shlst[j] and shlst[j]<shlst[k] and i<>k:
                      shvec=(pdbvc0.shlst[i][0],pdbvc.shlst[j][0],pdbvc2.shlst[k][0])#3d
                      if not self.get_pFalseNeg(shvec):
                        atk=pdbvc2.atreflst[k]
                        d2=calc_distance(atj,atk)
                        inten2=inten*self.getIntensityDipCor(d2,dimnum=1,rType=getType((atj,atk)))#*sqrt((getFac(atk)))
            		inten2*=getFac(atk,self.Bref,self.Bpow2,self.Garom)
			print atk,d2,inten2
                        if inten2*self.SN>=1.0:
                          lst.append(tuple([shx+normalvariate(0,self.shpert) for shx in shvec]))
                          hlst.append(inten2*self.SN)
                          atlst.append((ati,atj,atk))
                        ##else:print i,j,k,d,d2,inten2*self.SN
       return lst,hlst,atlst


    def getVirtualPeakList(self,pdb,scales={}):
       print self.name
       if 'input' in self.inpmapexp:
	 print 'using model peaks from:',self.inpmapexp['input']
	 buf=initfil2(self.inpmapexp['input'])
         lst,hlst,atomlist=self.getPeaksFromInput(buf)
       #TODO: gen-implementer getPeaks metoderne til een generelt som i base.getnearest
       else:
	if self.multi[0]==0:#1. transfer er entydig (f.eks. gennem binding)
	    splitord=None
	    for j,ord in enumerate(self.order):
	      if isinstance(ord,tuple):splitord=j
	    if splitord==None:
              lst,hlst,atomlist=self.getPeaks(pdb,scales=scales)
	    else:
              lst,hlst,atomlist=[],[],[]
	      tupord=self.order[splitord]
	      for ord in tupord:
		self.order[splitord]=ord
                lstx,hlstx,atomlistx=self.getPeaks(pdb,scales=scales)
                lst+=lstx;hlst+=hlstx;atomlist+=atomlistx
            print lst[0]
            print atomlist[0][0].name
            print len(lst)
	else:
          lst,hlst,atomlist=self.getPeaks2(pdb)
          print lst[0];print hlst[0];print atomlist[0][0]
          print len(lst)
       if 1:
          virtSpec=VirtSpectrum()
          virtSpec.defineData(lst,hlst,atomlist,self)
          ##virtSpec.writeRawPeakList()
          ##show()
          ##if SHOWON:# and self.dimension==2:
          if False:##self.docontour:# and self.dimension==2:
              virtSpec.pickData.visualize(max(virtSpec.lineWidths),self.dimension)
              show()
          if SHOWON or SHOWON2:
	      pylab.axis((-9.3,-7.0,-4.6,-3.0))
              show()
              clf()
       ##save_dba2(virtSpec,'virtSpec.pck')
       return virtSpec
       ##return FinalPeakPos, PeakHeights, PeakAssignments,obsFrac

    def writePeakList(self,ext):
       file=open(self.name+ext,'w')
       for i in range(len(self.list)):
          line1=''
          line2=' '
          for j in range(self.dimension):
            line1=line1+'%8.2f 0.0'%(self.list[i][j])
          if self.aslst[i]==None:
           for j in range(self.dimension):
            line2=line2+'iX:N-'
          else:
           for j in range(self.dimension):
            if SYMM<>None:
              line2=line2+self.aslst[i][j].chain_id+self.aslst[i][j].fragment_id\
              +self.aslst[i][j].res_name+':'+self.aslst[i][j].name+'-'
            else:
              line2=line2+self.aslst[i][j].fragment_id\
              +self.aslst[i][j].res_name+':'+self.aslst[i][j].name+'-'
          hlin='%10.2f'%(self.vallst[i])
          ilin='%4d'%(i)
          #print line1+hlin+line2
          file.write(ilin+line1+hlin+line2[:-1]+'\n')
       file.close()
       
    def writePeakList2(self,ext):
       file=open(self.name+ext,'w')
       for i,peak in enumerate(self.peakData.peaklist):
	  ##print dir(peak)
	  ##print peak.assSet
	  ##print peak.maxAssSet
	  ##print 'peakline',i,
	  if len(peak.assSet)==0:
	    file.write('%13s '%'?-?')	
	  for atoms in peak.assSet:
	    atstr=getAsFormat(atoms)
	    ##print atstr,
	    file.write('%13s '%atstr)
	  ##print peak.volumes,peak.shifts
	  file.write(('%7.3f '*self.dimension)%tuple(peak.shifts))
	  file.write('%4.1f\n'%peak.volumes)
	  print peak.volumes,peak.shifts

    def writePeakListAssigned(self,ext):
       file=open(self.name+ext,'w')
       file1=open(self.name+'_sparky'+ext,'w')
       file2=open(self.name+'_sparky2'+ext,'w')
       C=self.corrections
       for i,peak in enumerate(self.peakData.peaklist):
	  print dir(peak)
	  print peak.assSet
	  print peak.maxAssSet
          asd=self.allassigns[i]
          if asd.assigns==None:
               prob=0
               if self.dimension==2:
                  line='     ?-?     '
          else:
               ass=asd.assigns[asd.maxIndex][1];prob=asd.maxPpost
               rns,ris,rcs=[],[],[]
               for j in range(self.dimension):
                   ris.append(ass.atoms[j].fragment_id)
                   rns.append(aa31dict[ass.atoms[j].res_name])
                   rcs.append(ass.atoms[j].name)
               frd=abs(eval(ris[0])-eval(ris[1]))
               if self.dimension==2:
                   if ris[0]==ris[1]:#2d
                       line='%6s-%-6s'%(rns[0]+ris[0]+rcs[0],rcs[1])
                   else:
                       line='%6s-%-6s'%(rns[0]+ris[0]+rcs[0],rns[1]+ris[1]+rcs[1])
          ##X=ass.coords+ass.shiftDiffSigned#globally shifted
          X=peak.shifts#globally shifted
          G=self.peakData.globalshift
          G1=self.peakData.gsh1s#1st order corrections
          line2='%20s%7.3f%11.3f%13.1f%13.1f%13.3f%13.3f%12.1f%11.1f%9.3f'%\
                 (line,X[0],
                  X[1],peak.volumes,peak.rms,
                  peak.fitheight,peak.height,peak.lws[0],peak.lws[1],prob)
##                     (line[:-1],X[0]-(C[0][0]*X[0]+C[0][1]),
##                      X[1]-(C[1][0]*X[1]+C[1][1]),peak.volumes,peak.rms,
##                      peak.fitheight,peak.height,peak.lws[0],peak.lws[1],prob)
          lineSparky='%20s%7.3f%11.3f%9.4f'%\
                      (line,X[0]-G[0]-G1[0]*X[0],
                                 X[1]-G[1]-G1[1]*X[1],prob)
          lineSparky2='%20s%7.3f%11.3f%9.4f'%\
                      (line,X[0]-G[0]-G1[0]*X[0],
                                 X[1]-G[1]-G1[1]*X[1],frd)
##                          (line,X[0]-G[0]-(C[0][0]*X[0]+C[0][1]),
##                           X[1]-G[1]-(C[1][0]*X[1]+C[1][1]),prob)
          ##print line2
##	      hlin='%10.2f'%(self.vallst[i])
##	      ilin='%4d'%(i)
##	      #print line1+hlin+line2
          file.write(line2+'\n')
          file1.write(lineSparky+'\n')
          file2.write(lineSparky2+'\n')
       file.close()
       file2.close()
       file1.close()

    def get_pFalseNeg(self,shvec,addDim=0,symm=False):
	if symm==True:
	    return self.get_pFalseNeg(shvec,addDim,False) and \
                   self.get_pFalseNeg((shvec[1],shvec[0]),addDim,False)
        for regmat in self.exclude:
            for n in range(self.dimension+addDim):
                if shvec[n]<regmat[n][0] or shvec[n]>regmat[n][1]:break
                #still here:
                if n==self.dimension-1+addDim:return 1.0
        for i in range(self.dimension+addDim):
            for j in range(i+1,self.dimension+addDim):
                if abs(shvec[i]-shvec[j])<self.diagCut:return 1.0
        return 0.0

    def stripPeaks(self,explst0,pair=(0,1),sym=2,tol=0.5,slice=None,doReturn=True):
        stripped=0
	if slice<>None:
          for j in range(len(explst0)):
            explst0[j].peakData.peakvc.putSlices(comp=1)
	    for s in explst0[j].peakData.peakvc.slices:
		print len(s.list),
	    print
	print 'starting stripping...'
	A=numpy.transpose(array(self.peakData.peakvc.list))
	B=numpy.transpose(array(explst0[0].peakData.peakvc.list))
	##pylab.scatter(A[0],A[1],alpha=0.5,faceted=False,c='b')
	##pylab.scatter(B[0],B[1],alpha=0.5,faceted=False,c='r')
	##show()
        for i in range(len(self.peakData.peaklist)):
          peak=self.peakData.peaklist[i]
          thisstrip=None
          for k in range(sym):
            shs=[]
            for n in range(len(pair)):
                shs.append(peak.shifts[pair[n+k*((-1)**n)]])
            for j in range(len(explst0)):
                if slice<>None:vcj=explst0[j].peakData.peakvc.slices[slice]
                else:vcj=explst0[j].peakData.peakvc
		##vcj.list=array(vcj.list)
                dlst=vcj.findNearestFromCoord(shs,distMetricTup2,None)
		##print 'shs',shs,dlst,len(vcj.list)
		##if len(dlst)>0:print 'dlst:',dlst
                if len(dlst)==0 or dlst[0][0]>tol:
                    if thisstrip<>0:thisstrip=1
                else:
		    thisstrip=0
		    print 'match:',shs,dlst[0]
          if peak.strip<>1:#peak er ikke allerede stripped
              ##peak.strip=thisstrip
              if thisstrip==1:stripped=stripped+1
        print stripped, 'stripped peaks'
	num=len(self.peakData.peaklist)*1.0
	rat=(num-stripped)/num
	print rat,'peaks stripped'
	if doReturn:return rat

    def putConPriors(explst,conpri,im):
	pairdct=getCCpairdct()
        for exp0 in explst:
            assert isinstance(exp0,Experiment)
            ##if not exp0.fromStructure:
            if not exp0.fromStructure and 1 in exp0.multi and exp0.fixbonds==None:#at least one through space
                exp0.conPriors=[]
		print 'connum for exp:',exp0.name,exp0.connum
                for i,n in enumerate(exp0.connum):#connum is a list!
                    if n<>None:
                        exp0.conPriors.append(conpri[n])
                        print 'conpriormin:',exp0.name,n,exp0.rmins[i],conpri[n]
			if conpri[n].name=='simple':
			    print 'setting lrpob',exp0.name,exp0.pLR
			    conpri[n].setLRprob(exp0.pLR)
                        conpri[n].rmin=exp0.rmins[i]
      			##print conpri[n].prob(25,25,'C','CB')
                    else:exp0.conPriors.append(None)
                if exp0.restrictpairs:
		    exp0.ccpairdct=pairdct
		    for n in range(len(exp0.distCut)):
			exp0.bounds.append((exp0.distCut[n],exp0.mindistCut[n]))
		print 'bounds:',exp0.bounds
                print exp0.distCut
                exp0.distCut=numpy.ones(len(exp0.distCut))*eval(im['pcut'])
                print exp0.distCut

    putConPriors = staticmethod(putConPriors)
    
    def chooseAssigns(self,fac=10,thr=0.002,reject=0,maxnum=100,show=0,show2=0,
                      errmax=0.5,shtrunc=0.00,minmaxvolprob=0.0):
        newAllAssigns=[]
        Mat=zeros((70,70))
        for pnum,peak in enumerate(self.peakData.peaklist):
            asd=peak.assignData
            asd.numAccepted=0
            asd.zeroNum=0
            if asd.assigns<>None and len(asd.assigns)>0:
                if asd.use in [1,2]:
                  if not peak.restraintContainInformation(False,errmax):
                    finalassigns=[]
                    print 'restraint contain no information'
                  else:
                    finalassigns=asd.chooseAssigns(\
                        fac,thr,reject,maxnum,show,show2,
                        errmax,shtrunc,pnum,peak.shifts,peak.minexp,
                        assMat=Mat,minmaxvolprob=minmaxvolprob)
                newAllAssigns.append(finalassigns)
            else:
                newAllAssigns.append([])
		print 'WARNING: no assignments for this peak',pnum
##        print 'See the count matrix:'
##        print Mat
##        D=rot90(Mat)
##        title('count matrix')
##        imshow(D,interpolation='nearest')
##        pylab.show()
##        clf()
        return newAllAssigns,Mat


class Generic_Experiment(Experiment):

    def setVars(self,spv):
	self.name=spv['spectype']
	self.dimension=string.atoi(spv['dimension'])
	self.nucnames=spv['nucnames']
	self.atnames=spv['atnames']
	for i in range(self.dimension):
	   if self.atnames[i]=='None':self.atnames[i]=None#any atom (CX)
	   ##else:print 'atn:',(i,self.atnames[i])
	self.order=[]
	self.multi=[]
	self.exporder=[]
	for i in range(self.dimension):
	   ordi=spv['order'][i]
	   if ordi=='None':self.order.append(None)#through space
	   else:
	     if '/' in ordi:
		splitord=string.split(ordi,'/')
		print splitord
		self.order.append(tuple([string.atoi(ordx) for ordx in splitord]))
	     else:
		self.order.append(string.atoi(ordi))#0 or -1
	   ##if 'exporder' in spv:
	   ##  self.exporder.append(string.atoi(spv['exporder'][i]))
	   ##else:self.exporder.append(self.order[-1])
	for i in range(self.dimension-1):
	   self.multi.append(string.atoi(spv['multi'][i]))
	self.multitype='NOESY'
	if len(self.exporder)==0:self.exporder=self.order
	if 'multitype' in spv:self.multitype=spv['multitype']
	print 'generic experiment intialized using:',
	print self.name,self.nucnames,self.atnames,self.order,self.multi,self.exporder

class NMR3DcccDipCorAli(Experiment):
        name='DipCor3Ali'
        dimension=3
        nucnames=['C','C','C']
        atnames=[None,None,None]
##        #selective=[[0.0,100.0],[0.0,100.0],None]
##        selective=[[0.0,100.0],None,None]
##	spectralWidths=[[4.5,70.0],[4.5,70.0],[-20,200]]
##	exclude=([((75,200),(0,200))],[((75,160),(0,200))])
        exclude=[((75,200),(0,200),(0,200)),
                 ((0,200),(75,200),(0,200)),
                 ((0,200),(0,200),(75,160)),
                 ((0,200),(32,48),(0,200))]
        multi=[1,1]
	order=[None,None,None]
	exporder=[0,0,0]
        ##distCut=[100,100]
        diagCut=5.0
	e1=0.15

class NMRccDipCor(Experiment):
	name='DipCor'
	dimension=2
	nucnames=['C','C']
	atnames=[None,None]
	multi=[1]
	order=[None,None]
	exporder=[0,0]
	##distCut=[100.0]
##	exclude=[((75,200),(0,200)),((75,170),(0,200)),((0,200),(75,170))] # exclude aromatic
##	exclude=[((75,200),(0,200)),((75,170),(0,200)),((0,200),(75,200))] # exclude also carbonyl
	##exclude=[((75,200),(0,200)),((75,170),(0,200))]#aromatic re-included
	exclude=[((162,200),(0,200))]
##	excludeBoth=[((75,160),(0,200))]
	exclpt2=[1]
	diagCut=3.0
	##diagCut=5.0
	e1=0

class n15hsqc_noesy(Experiment):
        name='N15HSQC_NOESY'
        dimension=3
        ##nucnames=['H','N','H']
        ##atnames=[None,'N','H']
        nucnames=['N','H','H']
        atnames=['N','H',None]
        multi=[0,1]
        order=[0,0,None]
        rangeType='long-range'
	exclude=[]
	diagCut=0.1
	e1=0.002

class c13hsqc_noesy(Experiment):
        name='C13HSQC_NOESY'
        dimension=3
        ##nucnames=['H','N','H']
        ##atnames=[None,'N','H']
        nucnames=['C','H','H']
        atnames=[None,None,None]
        multi=[1,1]
        order=[0,0,None]
        rangeType='long-range'
	exclude=[]
	diagCut=0.1
	e1=0.002
        onebondCH=[0]

class NOESY(Experiment):
	name='NOESY'
	dimension=2
	nucnames=['H','H']
	atnames=[None,None]
	multi=[1]
	order=[None,None]
	exporder=[0,0]
	diagCut=0.1
	e1=0
	exclude=[]

class HNOESY(NOESY):
	name='HNOESY'
	atnames=['H',None]

class ncacx(Experiment):
        name='NCACX'
        dimension=3
        nucnames=['N','C','C']
        atnames=['N','CA',None]
        multi=[0,1]
        order=[0,0,None]
        exporder=[0,0,0]#expected order
        rangeType='intra'
        ##exclude=[((100,150),(40,80),(80,165)),
        exclude=[((100,150),(40,80),(80,200)),
                 ((133,150),(40,80),(0,200))]
        #C', aromatic and PRO
	diagCut=3.0
	e1=0.1

class ncoicx(ncacx):
        name='NCOiCX'
        atnames=['N','C',None]
        exclude=[((100,150),(165,200),(80,200)),
                 ((133,150),(165,200),(0,200))]

class NcaCX(ncacx):
        name='NcaCX'
        dimension=2
        skipDim=[1]
        diagCut=-1.0

class ncocx(Experiment):
        name='NCOCX'
        dimension=3
        nucnames=['N','C','C']
        atnames=['N','C',None]
        multi=[0,1]
        order=[0,-1,None]
        exporder=[0,-1,-1]#expected order
        rangeType='intra'
        exclude=[((100,150),(165,200),(80,200)),
                 ((133,150),(165,200),(0,200))]
        #C', aromatic and PRO
	diagCut=3.0
	e1=0.1

class ncapcx(ncocx):
        name='NCApCX'
        atnames=['N','CA',None]
        exclude=[((100,150),(40,80),(80,200)),
                 ((133,150),(40,80),(0,200))]
	#TAKEBACK to exclude C=0 as CX (could be side chain C=O)

class n15hsqc_backbone(Experiment):
        name='N15HSQCbb'
        dimension=2
        nucnames=['N','H']
        atnames=['N','H']#was HN
        multi=[0]
        order=[0,0]
        rangeType='intra'
	exclude=[]
	e1=0.0
	diagCut=0.0
        onebondCH=[0]

class n15hsqc(n15hsqc_backbone):
        name='N15HSQC'
        atnames=[None,None]
        multi=[1]
        order=[0,0]
        rangeType='intra'

class cahn(Experiment):
        name='CAHN'
        dimension=2
        nucnames=['C','H']
        atnames=['CA','H']
        multi=[0]
        order=[0,0]
        rangeType='intra'

class hnca(Experiment):
        name='HNCA'
        dimension=3
        nucnames=['H','N','C']
        atnames=['H','N','CA']
        multi=[0,0]
        order=[0,0,(0,-1)]
        rangeType='intra'#or preceding CA

class hncac(hnca):
        name='HNCAc'#only intra-residue
        order=[0,0,0]

class hncap(hnca):
        name='HNCAp'#only preceding residue CA
        order=[0,0,-1]

class hco(Experiment):
	name='HCO'
        dimension=2
        nucnames=['H','C']
        atnames=['H','C']
        multi=[0]
        order=[0,-1]
        rangeType='intra'#only preceding CO

class hncb(Experiment):
        name='HNCB'#HNCACB experiment using only negative peaks
        dimension=3
        nucnames=['H','N','C']
        atnames=['H','N','CB']
        multi=[0,0]
        order=[0,0,(0,-1)]
        rangeType='intra'#or preceding CB

class hncbc(hncb):
        name='HNCBc'#only intra-residue
        order=[0,0,0]

class hncbp(hncb):
        name='HNCBp'#only preceding residue CB
        order=[0,0,-1]

class hnco(hncbp):
        name='HNCO'#only preceding residue CO
        atnames=['H','N','C']

class c13hsqc(Experiment):
        name='C13HSQC'
        dimension=2
        nucnames=['C','H']
        atnames=[None,None]
        multi=[1]#not allways same atom
        order=[0,0]
        rangeType='intra'
	fixbonds=[1]#not None
        onebondCH=[0]

class c13hsqcb2(c13hsqc):
        name='C13HSQCb2'
	fixbonds=[2]#not None
	#CH peaks due to 2J (C-C-H)

class n15hsqc_tocsy(Experiment):
        name='N15HSQC_TOCSY'
        dimension=3
        nucnames=['H','N','H']
        atnames=[None,'N','H']
        multi=[0,0]
        order=[0,0,0]
        rangeType='intra'
        onebondCH=[0]

class ncaco(Experiment):
        #through-space transfer to C'
        name='NCACO'
        dimension=3
        nucnames=['N','C','C']
        atnames=['N','CA','C']
        multi=[0,0]
        order=[0,0,0]
        ##multi=[0,0]
        ##order=[0,0,0]
        rangeType='intra'
        exclude=[((100,150),(40,80),(0,160)),
                 ((133,150),(40,80),(0,200))]
        #C_ali, aromatic and PRO
	diagCut=3.0
	e1=0.1

class conca(Experiment):
        name='CONCA'
        dimension=3
        nucnames=['C','N','C']
        atnames=['C','N','CA']
        multi=[0,0]
        order=[-1,0,0]
        rangeType='intra'
        ##exclude=[]
        exclude=[((0,200),(133,150),(40,80))]
	diagCut=0.0
	e1=0.1

class concx(conca):
        name='CONCX'
        dimension=3
        nucnames=['C','N','C']
        atnames=['C','N',None]
        multi=[0,1]
        order=[-1,0,None]
        exporder=[-1,0,0]
        exclude=[((150,200),(100,150),(40,80)),
                 ((150,200),(100,150),(80,165)),
                 ((150,200),(133,150),(0,200))]
	subn=('CA',0)

class canco(Experiment):
        name='CANCO'
        dimension=3
        nucnames=['C','N','C']
        atnames=['CA','N','C']
        multi=[0,0]
        order=[0,0,-1]
        rangeType='intra'
        exclude=[]
	diagCut=0.0
	e1=0.1

class cancx(canco):
        name='CANCX'
        atnames=['CA','N',None]
        multi=[0,1]
        order=[0,0,None]
        exporder=[0,0,-1]#expected order
        rangeType='intra'
        exclude=[((40,80),(100,150),(80,200)),
                 ((40,80),(133,150),(0,200))]
	#TAKEBACK to exclude C=0 as CX (could be side chain C=O)
	subn=('C',-1)

class ncacb(Experiment):
        name='NCACB'
        dimension=3
        nucnames=['N','C','C']
        atnames=['N','CA','CB']
        multi=[0,0]
	##distCut=[None,4.0]
        order=[0,0,0]
        rangeType='intra'
        exclude=[]
	diagCut=3.0
	e1=0.1

class ncacon(Experiment):
        name='NCACON'
        dimension=4
        nucnames=['N','C','C','N']
        atnames=['N','CA','C','N']
        multi=[0,0,0]
	##distCut=[None,4.0]
        order=[0,0,0,1]
        rangeType='intra'
        exclude=[((100,150),(40,80),(0,160),(100,150)),
                 ((133,150),(40,80),(0,200),(100,150)),
                 ((100,150),(40,80),(0,200),(133,150))]
        #C_ali, aromatic and PRO
	diagCut=3.0
	e1=0

class ncacbcon(Experiment):
        name='NCACBCON'
        dimension=5
        nucnames=['N','C','C','C','N']
        atnames=['N','CA','CB','C','N']
        multi=[0,0,0,0]
	##distCut=[None,4.0]
        order=[0,0,0,0,1]
        rangeType='intra'
        exclude=[((100,150),(40,80),(40,80),(0,160),(100,150)),
                 ((133,150),(40,80),(40,80),(0,200),(100,150)),
                 ((100,150),(40,80),(40,80),(0,200),(133,150))]
        #C_ali, aromatic and PRO
	diagCut=3.0
	e1=0

class caconcaco(Experiment):
        name='CACONCACO'
        dimension=5
        nucnames=['C','C','N','C','C']
        atnames=['CA','C','N','CA','C']
        multi=[0,0,0,0]
	##distCut=[None,4.0]
        order=[0,0,1,1,1]
        rangeType='intra'
        exclude=[((40,80),(0,160),(100,150),(40,80),(0,200)),
                 ((40,80),(0,200),(100,150),(40,80),(0,160)),
                 ((40,80),(0,200),(133,150),(40,80),(0,200))]
        #C_ali, aromatic and PRO
	diagCut=3.0
	e1=0

class cacbconcacbco(Experiment):
        name='CACBCONCACBCO'
        dimension=7
        nucnames=['C','C','C','N','C','C','C']
        atnames=['CA','CB','C','N','CA','CB','C']
        multi=[0,0,0,0,0,0]
	##distCut=[None,4.0]
        order=[0,0,0,1,1,1,1]
        rangeType='intra'
        exclude=[((40,80),(40,80),(0,160),(100,150),(40,80),(40,80),(0,200)),
                 ((40,80),(40,80),(0,200),(100,150),(40,80),(40,80),(0,160)),
                 ((40,80),(40,80),(0,200),(133,150),(40,80),(40,80),(0,200))]
        #C_ali, aromatic and PRO
	diagCut=3.0
	e1=0



def parser(buf,i,j,s):
    if s=='':
        for k in range(i,len(buf)):
            if buf[k]==[]:return k
    for k in range(i,len(buf)):
        if len(buf[k])>j and buf[k][j]==s:return k
    return k

def getinds(buf,firstnum=0):
    k=parser(buf,firstnum,1,'assigned_chemical_shifts')
    if k==len(buf)-1:return k, k
    k2=parser(buf,k+1,0,'loop_')
    k3=parser(buf,k2+1,0,'loop_')
    k4=parser(buf,k3+1,0,'loop_')
    k5=parser(buf,k4+1,0,'')
    start=k5+1
    end=parser(buf,start,0,'')
    return start,end

def getrerefvalue(shift,atn,pinevars):
	  alias=0.0
	  key='cor'+atn
	  if key in pinevars:
	    print 'using shift correction',key,pinevars[key]
	    shifti=shift+pinevars[key]
	  else:shifti=shift
	  key='shsws'+atn
	  if key in pinevars:
	    swi=pinevars[key][1]-pinevars[key][0]
	    while True:
	      if   pinevars[key][0]>shifti:
	        print 'using shift aliasing (>)',key,pinevars[key],shifti
		shifti+=swi;alias+=swi
	      elif pinevars[key][1]<shifti:
	        print 'using shift aliasing (<)',key,pinevars[key],shifti
		shifti-=swi;alias-=swi
	      else:break
	  return shifti,alias

def getpineShiftData(buf0,minpHN=0.0,minpHNalt=0.01):
    buf=buf0[1:]
    bbatns=['HN','N','C','CA','CB']
    pinevars=getinpmap('pine_config.txt',type='float')
    print 'pinevars',pinevars
    dba=ShiftDict()#class defined in protstat
    print 'getting probabilistic resonance backbone assignments from pine'
    for lst in buf:
      pHN=string.atof(lst[2])
      if pHN>=minpHN:#maaske trunkeres til 0.9?
	resi=lst[0]
	resn=lst[1]
	shifts=[string.atof(lst[i]) for i in range(3,8)]
	for i,x in enumerate(shifts):
	 if x<>0.00:
	  atn=bbatns[i]
	  if not resi in dba:dba[resi]={}
	  shifti,alias=getrerefvalue(shifts[i],atn,pinevars)
	  #TODO:store alias together with shift
	  dba[resi][atn]=[shifti,pHN]#pHN is strictly a comb prob for H and N
	  print 'added shift',resi,resn,atn,pHN,shifti
	prev=str(string.atoi(resi)-1)
	if prev in dba:
	  for carb in ['CA','CB','C']:
	   if carb in dba[prev]:
	    print 'overriding prob for preceding carbon',pHN,dba[prev][carb][1],carb,prev
	    dba[prev][carb][1]=max(pHN,dba[prev][carb][1])#overriding prob for preceding C/CA/CB
	for j in range(3):
	  pHNalt=string.atof(lst[8+3*j])
	  if pHNalt>=minpHNalt:
	    shifts=[string.atof(lst[i]) for i in (8+3*j+1,8+3*j+2)]
	    print '  adding alternative contributions',j,pHNalt,shifts
	    for i,x in enumerate(shifts):
		##atn='HN'[i]
		atn=['HN','N'][i]
	        shiftix,aliasx=getrerefvalue(shifts[i],atn,pinevars)
		dba[resi][atn]+=[shiftix,pHNalt]
    return dba

def getShiftDBAsparky(buf,chnum='',includeLabel=True,includeAmb=False,format='Xplor',waitlst=[],dba=''):
    ambdct=getambgroups(rev=True,colnum=1);redo=False
    if len(waitlst)==0:
        dba=ShiftDict()
    else:
	print 'redoing ambiguous assignments...',len(waitlst);redo=True
        ats=[tup[0] for tup in waitlst]
        buf=[tup[1] for tup in waitlst];waitlst=[]
    for i,lst in enumerate(buf):
     if len(lst)>2:
      if redo:at=ats[i]
      else:at=lst[-5]
      if at[-1]<>'?':
       if len(lst)==6:
	 group=lst[0];res=group[1:];rl1=group[0]
       elif len(lst)==7:
         rl1=lst[0]
	 res=lst[1]
       print res,rl1
       if rl1 in aa13dict.keys() and res.isdigit():
	rl3=aa13dict[rl1]
	elem=lst[-4][-1]
	if at[0]=='Q':
	  if elem+at[1:] in ambdct[group[0]]:
	    at=ambdct[group[0]][elem+at[1:]]#tuple
	elif at[-1]=='#':
	  if at[:-1] in ambdct[group[0]]:
	    waitlst.append(('Q'+at[1:-1],lst))#f.ex. QB
	  else:
	    waitlst.append((at[:-1],lst))
	avg=string.atof(lst[-3])#was 3
	##if elem=='C':avg+=1.8#might TAKEBACK
	##elif elem=='N':avg+=1.0#might TAKEBACK
	shiftstd=eval(lst[-2])
	count=string.atoi(lst[-1])
	if at[-1]<>'#' and res.isdigit():
         dba.counts[elem]+=1
	 if count==1:shiftstd=0.005
         if not dba.has_key(res):dba[res]={}
	 if isinstance(at,tuple):
	  for atom in at:
	   if atom in dba[res]:print 'atom already in res',res,atom
	   else:
            dba[res][atom]=[avg,shiftstd,rl3]
	 else:
	   if at in dba[res]:print 'atom already in res',res,at
	   else:
            dba[res][at]=[avg,shiftstd,rl3]
    if len(waitlst)>0:return getShiftDBAsparky(buf,chnum,includeLabel,includeAmb,format,waitlst,dba)
    return dba  

def dbafill(dba,start,end,buf,chnum):
    if len(buf[0])==8:return dbafill2(dba,start,end,buf,chnum)
    if buf[start:end]<>[]:dba[chnum]={}
    for lst in buf[start:end]:
        ##print lst
        elem=lst[5]
        if elem in ['N','C']:#['N','C']:
            res=lst[1]
            at=lst[4]
            avg=string.atof(lst[6])#+normalvariate(0,0.1)
            if lst[7]=='.':std=None
            else:std=string.atof(lst[7])
            if not dba[chnum].has_key(res):dba[chnum][res]={}
            dba[chnum][res][at]=(avg,std)
            #print avg,std
    return dba  

def dbafill2(dba,start,end,buf,chnum):
    if buf[start:end]<>[]:dba[chnum]={}
    for lst in buf[start:end]:
        ##print lst
        elem=lst[4]
##        if elem in ['C']:#['N','C']:
        if elem in ['N','C']:
            res=lst[1]
            at=lst[3]
            avg=string.atof(lst[5])
            if lst[6]=='.':std=None
            else:std=string.atof(lst[6])
            if not dba[chnum].has_key(res):dba[chnum][res]={}
            dba[chnum][res][at]=(avg,std)
    return dba  

def parseBMRfile(filename):
    buf=initfil2(filename)
    dba={}
    first=0
    chnum=0
    while first<len(buf):
        start,end=getinds(buf,firstnum=first)
        dba=dbafill(dba,start,end,buf,chnum)
        first=end+1
        chnum=chnum+1
    if len(dba)==0:return parseBMRfile2(filename)
    return dba

def parseBMRfile2(filename):
    buf=initfil2(filename)
    dba={}
    dba=dbafill2(dba,0,len(buf),buf,0)
    return dba

class Peak:
    #globalshift=[1.69,1.69]
    
    def __init__(self,peakdata,ind,ilst,dim):
        shs=[]
        ass=[]
        lst=peakdata.peakbuffer[ind]
        gsh=peakdata.globalshift
        for i in range(dim):
            sh=string.atof(lst[ilst[i]])+gsh
            shs.append(sh)
            asi=lst[ilst[i+dim]]
            ass.append(asi)
        self.shifts=tuple(shs)
        self.names=tuple(ass)
        self.score=string.atof(lst[0])
        volset=[[],[]]#skal aendres med 3d!!!!!!!!!
        errset=[[],[]]#skal aendres med 3d!!!!!!!!!
        for i in range(2):
            vols=[]
            errs=[]
            for j in range(len(peakdata.volbuffers[0][0])-1):
                vol=string.atof(peakdata.volbuffers[i][ind][j+1])
                vols.append(vol)
                err=string.atof(peakdata.errbuffers[i][ind][j+1])
                errs.append(err)
            volset[i]=vols
            errset[i]=errs
        self.volumes=volset
        self.errors=errset


class SpinSystem:

    def __init__(self,peak,group,varnum=0):
	self.varnum=varnum
	self.rootcoords=peak.shifts
	self.group=group
	self.names=peak.names
	self.aliased=peak.aliased
	##self.peaklist=[]
	self.varclist=[]

    def getDistance(self,peak,damps,mapi=[0,1,2]):
	self.damps=damps
	tot=0.0
	for i,c in enumerate(self.rootcoords):
	 if i<>self.varnum:
	  tot+=(((c-peak.shifts[mapi[i]])/damps[mapi[i]])**2)
	return sqrt(tot/2.0)

    def getpDistance(self,psdct,damps,hndiff=0.0,ndiff=0.0):
	tot=0.0
	if 'N15' in psdct and ('N', '0') in psdct['N15']:
	  tot+=(((psdct['N15'][('N', '0')]-self.rootcoords[1]+self.aliased- ndiff)/damps[1])**2)
	if 'H1' in psdct and ('H', '0') in psdct['H1']:
	  tot+=(((psdct['H1'][('H', '0')] -self.rootcoords[2]+self.aliased-hndiff)/damps[2])**2)
	if 'H1' in psdct and ('HA', '0') in psdct['H1']:
	  tot+=(((psdct['H1'][('HA', '0')] -self.rootcoords[0]+self.aliased-hndiff)/damps[0])**2)
	return sqrt(tot/2.0)

    def addPeak(self,peak):
	##self.peaklist.append(peak)
	self.varclist.append(peak.shifts[self.varnum])

    def addNOESYPeak(self,peak,varnum):
	##self.noesypeaklist.append(peak)
	self.noesyvarclist.append(peak.shifts[varnum])

    def calc_elink(self,prevsys):
	pairdct={'C13':[(('CA','-1'),('CA','0'),0.15,0.02,('CA','na')),
		        (('CB','-1'),('CB','0'),0.25,0.05,('CB','na'))]}
	hdctc=self.hetdct
	hdctp=prevsys.hetdct
	k_least_plausible=1.0#.......?TODO: optimize this value
	kamb=0.2
	trunc=0.3#5
	tot=0.0
	eobs=0.75
	##print 'evaluating link possibility',self.group,prevsys.group
	for elem in pairdct:
	  if elem in hdctc and elem in hdctp:
	    ##print 'elem found',elem
	    for pair in pairdct[elem]:
	      valc=None;valp=None;viol=0.0
	      if pair[0] in hdctc[elem]:
		valc=[hdctc[elem][pair[0]]]#unambig
	      elif pair[4] in hdctc[elem]:
		valc=hdctc[elem][pair[4]]#ambig (list)
		viol+=kamb
	      if pair[1] in hdctp[elem]:
		valp=[hdctp[elem][pair[1]]]
	      elif pair[4] in hdctp[elem]:
		valp=hdctp[elem][pair[4]]
		viol+=kamb
	      if valc==None or valp==None:
	        pass
	        ##tot+=(k_least_plausible/2)
	      else:
		emin=-log(pair[3])# was nviol
	        tot +=viol
		emin-=viol
		if pair[0][0]=='CB' and viol==kamb*2 and len(valc)==1 and len(valp)==1:
		  print 'aligning two single CB-missing peaks - possibly GLY',
		  print valc,valp,self.group,prevsys.group
		  emin/=1.2 #more forgiving (was 2)
		  #generalize forgiving missing peaks
		for cval in valc:
		  for pval in valp:
		    sdiff=(((cval-pval)/pair[2])**2)
		    edev=0.5*max(0.0,sdiff-trunc)-eobs
		    if edev<emin:emin=edev
		    if sdiff<1.0:
		      print 'found matching atom',pair,sdiff,cval,pval,self.group,prevsys.group
		    else:
		      print 'no matching atom',   pair,sdiff,cval,pval,self.group,prevsys.group
		      #max is ca. ...
	        tot+=emin
	  else:return k_least_plausible
	return tot
	
    def calc_elinknoesy(self,prevsys,damppair,asdiff=999):
	tot=0.0
	dampcomb=damppair[0]**2+damppair[1]**2
	##print 'calculating noesy links',self.group,prevsys.group,prevsys.varclist,self.noesyvarclist,
	##print -log(0.3)-log(sqrt(dampcomb)*math.sqrt(2*pi))
	eobs=0.75
	shha=prevsys.hetdct['H1'][('HA','0')]
	shhn=prevsys.hetdct['H1'][('H','0')]
	shhni=self.hetdct['H1'][('H','0')]
	shhai=self.hetdct['H1'][('HA','0')]
	pFalseNeg=0.3
	if asdiff==0:
	  print 'testsh',shha,shhn,shhai,shhni,prevsys.varclist
	  print self.noesyvarclist
	for co in prevsys.varclist:#tocsy connections to 
	  eminco=-log(pFalseNeg)
	  if abs(co-shha)>0.1:
	   for con in self.noesyvarclist:
	     edev=0.5*(((con-co)**2)/(dampcomb))-eobs
	     if edev<eminco:eminco=edev
	   tot+=eminco
	pFalseNeg=0.05
	eminco=-log(pFalseNeg)
	if abs(shha-shhai)>0.2:
	  for con in self.noesyvarclist:
	   edev=0.5*(((con-shha)**2)/(dampcomb))-eobs
	   if edev<eminco:
	     eminco=edev
	     print 'found matching prev HA',con,shha,edev
	  tot+=eminco
	eminco=-log(pFalseNeg)
	if abs(shhn-shhni)>0.7:
	  for con in self.noesyvarclist:
	   edev=0.5*(((con-shhn)**2)/(dampcomb))-eobs
	   if edev<eminco:
	     eminco=edev
	     print 'found matching prev HN',con,shhn,edev
	  tot+=eminco
	return tot

    def removeoverlap(self,lst,cut):
	#TODO: more realistic clustering...
	mindist=cut;mini=None
	for i in range(len(lst)-1):
	  di=lst[i]-lst[i+1]
	  if di<mindist:
	    mindist=di
	    mini=i
	if mini<>None:
	  ave=(lst[mini]+lst[mini+1])/2
	  ##print 'removing',di,mini,lst[mini],ave
	  lst.pop(mini)
	  lst.pop(mini)
	  lst.insert(mini,ave)
	  self.removeoverlap(lst,cut)

    def randomizepos(self,lst,dev):
	for i in range(len(lst)):
	  lst[i]+=normalvariate(0.0,dev)

    def setHetspins(self,dct,mat,ires,seeds=(6,2,8),odds=(20,10,10),cuts=(0.3,0.6)):
	mindist=cuts[1];pair=None
	for i in range(len(mat[0])):
	  for j in range(len(mat[1])):
	    di=abs(mat[0][i]-mat[1][j])
	    if di<mindist:
	      mindist=di
	      pair=(i,j)
	if pair<>None:
	  mat[0].pop(pair[0])
	  mat[1].pop(pair[1])
	for i in range(2):
	  if len(mat[i])>1:
	    di=abs(mat[i][0]-mat[i][1])
	    if di<cuts[i]:
	      ave=(mat[i][0]+mat[i][1])/2
	      mat[i].pop(0)
	      mat[i].pop(0)
	      mat[i].append(ave)
	    if len(mat[i])>1:
	      if ires%odds[i]==seeds[i]:
		n=randint(0,1)
		mat[i].pop(n)
	print mat
	cfixatns=['CA','CB']
	offs=[['0','-1'] for _ in range(2)]
	if ires%odds[2]==seeds[2]:
	  i=randint(0,1)
	  if len(mat[i])>1:offs[i]=['na','na']#equal hights for i,i-1 peaks
	for i,catn in enumerate(cfixatns):
	  if len(mat[i])<2:offs[i]=['na','na']#ambigous i,i-1 peaks
	  for j in range(len(mat[i])):
	    val=mat[i][j]+normalvariate(0.0,cuts[i]/5)
	    if offs[i][j]=='na':
	      if not (catn,'na') in dct['C13']:dct['C13'][(catn,'na')]=[]
	      dct['C13'][(catn,'na')].append(val)#type is list - was float
	    else:
	      dct['C13'][(catn,offs[i][j])]=val

    def removerandom(self):
	if len(self.varclist)>1:
	  n=randint(1,len(self.varclist)-1)
	  self.varclist.pop(n)

    def addrandom(self):
	x=1+rand()*2.5
	self.varclist.append(x)
	self.varclist.sort(lambda x, y: cmp(y,x))

    def addTOCSYspins(self,resi,i,aai,tocsyconvdct,seed=0,odds=20):
	self.varclist=[]
	for at in resi:
          if at.element[0]=='H' and at.shift[0]<>None:
	    if aai in tocsyconvdct and at.name in tocsyconvdct[aai]:
	      pass##print 'atom not related to HN by TOCSY',aai,at.name,at.shift[0]
	    elif at.name<>'H':
	      sh,sdev=at.shift[0]
	      ##print i,at,sh,sdev
	      self.varclist.append(sh)
	self.varclist.sort(lambda x, y: cmp(y,x))
	print 'varclist',i,resi,len(self.varclist),self.varclist
	self.removeoverlap(self.varclist,0.15)
	if i%odds==seed:self.removerandom()
	if i%odds==seed+10:self.addrandom()
	self.randomizepos(self.varclist,0.15/5)
	print 'varclist',i,resi,len(self.varclist),self.varclist

    def addNOESYpeaks(self,resi,i,aai,chain,distthr=5.0):
	lst=[]
	hnat=resi['H']
	hnsh=hnat.shift[0][0]
	for at in chain.iter_atoms():
	  if at.element=='H':
	   if at<>hnat:
	    if at.shift[0]<>None:
	     sh=at.shift[0][0]
	     if abs(hnsh-sh)>0.7:
	      dist=AtomMath.calc_distance(hnat,at)
	      if dist<distthr:lst.append(sh)
	lst.sort(lambda x, y: cmp(y,x))
	print 'before',i,aai,len(lst),lst
        self.removeoverlap(lst,0.15)
	self.randomizepos(lst,0.15/5)
	print 'noesylist',i,aai,len(lst),lst
	self.noesyvarclist=lst

    def addHetspins(self,resi,i,aai,prev):
	print 'adding hetero spins for',i,resi,prev
	##atns=['HA','N','H']
	dct={'C13':{}, 'N15':{}, 'H1':{}}
	##dct['H1'][0]['HA']=self.rootcoords[0]
	dct['H1'][('HA','0')]=self.rootcoords[0]#stupid notation!
	dct['H1'][('H','0')] =self.rootcoords[2]
	dct['N15'][('N','0')]=self.rootcoords[1]
	coshift=prev['C'].shift[0][0]
	dct['C13'][('C','-1')]=coshift
	cfixatns=['CA','CB']
	mat=[]
	for catn in cfixatns:
	  vals=[]
	  for off in ['0','-1']:
	    try:
	      if off=='0':sh=resi[catn].shift[0][0]#TypeError means shift is missing
	      else:       sh=prev[catn].shift[0][0]
	      vals.append(sh)
	    except KeyError:print 'warning: atom not found',catn,i,off,resi,prev
	    ##self['C13'][(catn,off)]=sh
	  mat.append(vals)
	print mat
	self.setHetspins(dct,mat,i)##,seed,odds)
	self.hetdct=dct
	print self.hetdct['C13']

    def calc_etyp(self,resi,i,tocsyconvdct,resp):
	atlst=[]
	aai=aa31dict[resi.res_name]
	for at in resi:
	  if resi.res_name=='PRO':return 19.999#impossible - cannot have an NH root
          if at.element[0]=='H' and at.shift<>None:#shift value is from database
	    if aai in tocsyconvdct and at.name in tocsyconvdct[aai]:
		print 'atom not related to HN by TOCSY',aai,at.name,at.shift
	    ##elif at.name<>'HN':
	    elif at.name<>'H':
	     ##sh,sdev=at.shift
	     if at.shift[1]<>None:
	      sh,sdev=at.shift[1]
	      print i,at,sh,sdev
	      atlst.append((sh,sdev))
	print 'atlst',i,resi,len(atlst),atlst,self.varclist
	pFalsePos=0.02
	pFalseNeg=0.02
	num=len(atlst)+len(self.varclist)+5
	tot=0.0
	eobs=0.75
	for sh,sdev in atlst:
	  eminco=-log(pFalseNeg)##-log(sdev*math.sqrt(2*pi))
	  for co in self.varclist:
	     edev=0.5*(((sh-co)**2)/(sdev**2+self.damps[0]**2))-eobs/2
	     if edev<eminco:eminco=edev
	  tot+=eminco
	for co in self.varclist:
	  eminsh=-log(pFalseNeg)##-log(0.3*math.sqrt(2*pi))
	  for sh,sdev in atlst:
	     edev=0.5*(((sh-co)**2)/(sdev**2+self.damps[0]**2))-eobs/2
	     if edev<eminsh:eminsh=edev
	  tot+=eminsh
	tot/=2
	print self.group,resi,tot
	klp=1.0#.......?TODO: optimize this value
	print '------ hetero nuclear assignments --------',# not using preceding CA/CB
	hdct=self.hetdct#probs below were 0.02 and 0.05
	hetats={'N15':[('N', ['0'],     0.12,0.005)],
		'H1' :[('H', ['0'],     0.03,0.005)]}
	print hdct
	for elem in hetats:
	 for tup in hetats[elem]:
	  atn,offs,hsig,pn=tup
	  co=None
	  for off in offs:
	    if elem in hdct and (atn,off) in hdct[elem]:
	      co=hdct[elem][(atn,off)]
	      viol=-log(pn)##-log(hsig*math.sqrt(2*pi))
	      try:
		##if atn=='H':at=resi['HN']#Xplor
		if off=='-1':at=resp[atn]
		else:at=resi[atn]
		##sh,sdev=at.shift
		sh,sdev=at.shift[1]
		print at,sh,co
		edev=0.5*(((sh-co)**2)/(sdev**2+0.2**2))
		if elem=='C13':edev-=eobs
		print 'matching:',atn,off,co,i,aai,self.group,edev
		tot+=min(edev,viol)
	      except KeyError:
		print 'assignment found but no atom (only CB)',atn,off,co,i,aai,self.group
		tot+=(viol*2)#false positive
	  #endfor off in offs:
	  if co==None:
	    if (atn,'na') in hdct[elem]:
	        viol=-log(pn)
		co=hdct[elem][(atn,'na')]
		edevs=[]
		if len(co)==1:
		  co0=co[0];shs=[]
		  for resna in [resi,resp]:
		    try:
			at=resna[atn]
			sh,sdev=at.shift[1];shs.append(sh)
		  	edevs.append(0.5*(((sh-co0)**2)/(sdev**2+0.2**2)))
		    except KeyError:
		        print 'assignment found (amb.) but no atom (only CB)',atn,co0,i,aai,self.group
		  edevs.sort()
		  if len(edevs)==0:
		    print 'warning: no atom (CB?) found in either res',atn,co0,i,aai,self.group,resi,resp
		    tot+=(viol*2)#false positive
		  else:
		    first=min(viol,edevs[0]-eobs)
		    if len(edevs)==1:
			tot+=first
			print 'first edev:',first,co0,shs
		    else:
			violna=min(viol/2,edevs[1]-eobs)
			tot+=(first+violna)
			print 'first edev and violna:',first,violna,co0,shs
		else:
		  bestedev=2*viol
		  for k in range(2):
		    esum=0.0;shs=[]
		    for rn,resna in enumerate([resi,resp]):
		      try:
			at=resna[atn]
			sh,sdev=at.shift[1]
			shs.append(sh)
			cok=co[(rn+k)%2]
		  	esum+=(0.5*(((sh-cok)**2)/(sdev**2+0.2**2))-eobs)
		      except KeyError:
		        print 'assignment found (amb. double) but no atom (only CB)',atn,co[k],i,aai,k,self.group
		    if esum<bestedev:bestedev=esum
		  print 'bestedev',bestedev,shs,co
		  tot+=bestedev
	  if co==None:#_still_ no assignment found
	    try:at=resi[atn]
	    except KeyError:at=None
	    if at<>None:
		print 'atom found but no assignment',atn,i,aai,self.group
		tot+=klp
	val=tot##/num
	print 'typresult %3s %2d %s %6.3f'%(self.group,i,aai,val)
        return val
    #enddef calc_etyp



class Peak_sparky:
    #globalshift=[1.69,1.69]
    errors=None
    probnoise=None
    rms=None
    lhrat=None
    strip=None
    virtFlag=None
    badlyIntegrated=False
    
    def __init__(self,peakdata,ind,dim):
        shs=[]
        ass=[]
        self.fromheight=0
        lst=peakdata.peakbuffer[ind]
        gsh=peakdata.globalshift
        gsh1=peakdata.gsh1s
	self.aliased=0
        for i in range(dim):
            sh0=string.atof(lst[i+1])
            sh=sh0+gsh[i]#0th order correctiom
            sh=sh+gsh1[i]*sh0#1st order correction
	    if peakdata.sws[i]<>None:
		swsi=peakdata.sws[i]
		swi=swsi[1]-swsi[0]
		print 'testing for aliasing',swsi,swi,i
		while True:
		  if swsi[0]>sh:
		    print 'aliasing shift (>)',sh,swsi
		    sh+=swi;self.aliased+=swi
		  elif swsi[1]<sh:
		    print 'aliasing shift (<)',sh,swsi
		    sh-=swi;self.aliased-=swi
		  else:break
            shs.append(sh)
        self.shifts=tuple(shs)
        self.names=peakdata.peakbuffer[ind][0]
        self.score=None
        self.lws=numpy.ones(dim)*200
        self.fitheight=0
        ##self.volumes=eval(peakdata.peakbuffer[ind][dim+1])
	if len(peakdata.peakbuffer[ind])==dim+1:self.volumes=1.0
        else:self.volumes=abs(eval(peakdata.peakbuffer[ind][dim+1]))
        ##print peakdata.peakbuffer[ind]
        if len(peakdata.peakbuffer[ind])>5:
            if peakdata.peakbuffer[ind][dim+2]=='ga':
                del peakdata.peakbuffer[ind][dim+2]
            #print peakdata.peakbuffer[ind]
            if self.volumes>0.0:
                peakdata.allvols.append(self.volumes)
            else:self.badlyIntegrated=True
##            self.height=string.atof(peakdata.peakbuffer[ind][dim+4])
##            self.fitheight=string.atof(peakdata.peakbuffer[ind][dim+3])
            self.height=abs(string.atof(peakdata.peakbuffer[ind][dim+4]))
            self.fitheight=abs(string.atof(peakdata.peakbuffer[ind][dim+3]))
##            if self.volumes<0.0 and self.height>0.0:
##                self.fromheight=1
            for j in range(dim):
                self.lws[j]=string.atof(peakdata.peakbuffer[ind][dim+5+j])
            self.lhrat=abs(log(self.height/self.fitheight))
            self.rms=max(eval(peakdata.peakbuffer[ind][dim+2]),1.0)
            if self.height>0.0:
                peakdata.allheights.append(self.height)
            peakdata.sumfac=peakdata.sumfac+self.volumes/self.height
            if len(peakdata.peakbuffer[ind])>dim*2+5:#virtual data
                self.mindist=eval(peakdata.peakbuffer[ind][dim*2+5])
            if len(peakdata.peakbuffer[ind])>dim*2+6:#virtual data + model assignments
                self.modelAsslst=peakdata.peakbuffer[ind][dim*2+6:]
                ##print self.modelAsslst
	    ##else:self.modelAsslst=[]
	    #print self.lhrat,log(self.rms)
        else:
	   self.fromheight=1
	   ##self.modelAsslst=[]
        ##print self.shifts,self.volumes
	self.pDiff=0.0

    def setAssignData(self,exp0,i):
        self.assignData=AssignData(self,exp0,i)

    def getSideInds(self):
        return [range(len(self.shifts))]

    def getRmin(self,exp0):
        self.vnormc=exp0.vnormc
        return exp0.rmins

    def getPeakDevs(self,exp0):
        return exp0.shiftDamps

    def getDampings(self,exp0):
        return exp0.shiftDamps

    def reshapeAssigns(self,valdba,posasmat):
        #nu skal dimensionerne byttes rundt for at beholde den gl datastruktur
        assignlst=[]
        ptmat=[]
        atkeys=[]
        for tup in valdba.keys():
            if not SILENT:print 'tup:',tup
            atvec,shvec,stdvec,probvec,ddvec,tdvec,shfvec=[],[],[],[],[],[],[]
            shdiffSgn=[]
            distNDs=[]
            ptlst=[]
            for k in range(len(tup)):
                if not SILENT:print '....',k
                ##ass=ass1dvec[k]
                ass=posasmat[k][tup[k]]
                atk,shk,stdk,probk,diffk,ddk,tdk,shfk=\
                ass.atom,ass.shift,ass.stdev,ass.prob,ass.diff,ass.dampdiff,ass.totdamp,ass.shfaci
                if not SILENT:print atk,shk,stdk,probk,diffk,ddk,tdk,shfk
                try:ptk=atk.pointer
                except AttributeError:ptk=atk.pt
                ptlst.append(ptk)
                pdk=ass.prevdist
                if k>0:distNDs.append(pdk)
                #generaliseres! da distND bliver dist fra den sidste dim!
                dimk=ass.dim
                #print '+++',atk,shk,stdk,diffk,pdk,dimk
                atvec.append(atk)
                shvec.append(shk)
                stdvec.append(stdk)
                probvec.append(probk)
                shdiffSgn.append(diffk)
                ddvec.append(ddk)
                tdvec.append(tdk)
                shfvec.append(shfk)
            atkey=atvec[:]
            atkey.sort()
            atkeys.append(tuple(atkey))
            if not SILENT:print distNDs
            ptlst.sort()
            ptmat.append(ptlst)
            #print '-----------------'
            assND=Assignment(atvec,shvec,stdvec,probvec,distNDs,None,shdiffSgn,ddvec,tdvec,shfvec,valdba[tup])
	    if not SILENT:print assND#enable debugging...
            assignlst.append(assND)
        self.initAssigns=assignlst
        self.ptMat=ptmat
        if isinstance(self,BuildUpPeakVector):
            if not SILENT:print self.dba
            if DEBUG:
                print self.assSet
                Sn=Set(atkeys)
                print Sn
                S_int=Sn&self.assSet
                if len(S_int)==0:print 'Empty intersection'
                else:print S_int
                Sdiff=self.assSet-Sn
                print 'All observed:',self.assSet.issubset(Sn),Sdiff,
                if len(Sdiff)>0:
                    for s in Sdiff:
                        for pair in comb4(len(s)):
                            print calcDist(s[pair[0]],s[pair[1]]),
                print
            ##print '',Sn.issubset(self.assSet)

    def rankInitAssigns(self,ide,exp0,inc,updvec,mode):
        '''method to rank the assignments for each peak'''
        different=0
        if not SILENT:
            ##print '#'*30;print self.shifts
            print '#'*30;print self;print self.shifts
            print 'peak.hErrProb:',self.hErrProb
            try:print self.assSet
            except AttributeError:pass
        Rmins=self.getRmin(exp0)#also sets self.vnormc
        peakDevs=self.getPeakDevs(exp0)
        for ass in self.initAssigns:
	    ass.peakDevs=peakDevs
	    ass.peakShifts=self.shifts
            co=ass.coords
            dlst=exp0.peakData.peakvc.findNearestFromCoord(co,distMetricTup,None)
            ##print dlst
            if 1:##len(dlst)>0:#TODO lave peakvc med passende vwidth...
             ##if inc or stdsize>dampsize or ide==dlst[0][2]:
             if inc or ide==dlst[0][2]:
               #TODO: genimplementere includeall==0...
               #alternativt findes der en anden peak taettere paa
               #dette skal vaere en option, boer ikke benyttes hvis assignments ikke haves! (lambda>gamma)
                ##ass.setResAsVec()
                if not SILENT:print Rmins,self.vnormc,self.shifts,peakDevs
                exp0.getFactors(ass,Rmins,self.vnormc,updvec,mode)
                #also sets ass.volFac and ass.shFac
                ass.calcGiFi()
                if not SILENT:
                    ass.show_assign(exp0.fromStructure)
                    print ide,'----------------------',(ass)
             else:
                 ass.GiFi=0.0;ass.shFac=0.0;ass.volFac;ass.volFacs=zeros(exp0.dimension);ass.maxPos=0
                 if not SILENT:print 'DIFFERENT?',ide,dlst[0][2]
                 if ide<>dlst[0][2]:different=1

    def stripTest(self,exp0,vc,peaklist,tol,pairlist,accepts,pdb=None,shifts=None,**kwargs):
        #TODO: generalize...
        #der skal vaere noget med at tage hoejde for omraader i spektret med meget overlap...
        #...specielt f.eks. NCACX: CACX ses maaske ikke pga overlap
	rev=False
        if 'rev' in kwargs:rev=kwargs['rev']
	print 'running stripTest ********************************'
	##for i,item in enumerate(vc.list): print item##,vc.vallst[i]
        if shifts==None:shs=self.shifts
        else:shs=shifts
        xErr=0.0
        if len(shs)==3:print '%7.2f%7.2f%7.2f'%tuple(shs),self
        elif len(shs)==2:print '%7.2f %7.2f'%tuple(shs),
	if isinstance(self,Peak_sparky_Virtual):
	    print '%18s'%(getAsFormat(self.names)),
	    S0=Set(self.names)
	    S0=Set(self.allAssigns)
	elif isinstance(self,BuildUp):
	    print self.assSet
	else:
            if DEBUG:
                S0=self.assSet
		print S0
                try:print self.modelAsslst,
                except AttributeError:pass
        ##if vc.dimension<len(shs):#hvad ellers?
        if vc.dimension<=len(shs):#hvad ellers?
            for pair in pairlist:
                valid=False;Dist=10
                if pair==pairlist[0]:accept=accepts[0];Tol=tol*0.8
                else:accept=accepts[1];Tol=tol*1.1
                #TODO: accept i inputfil
                shs2=[shs[pair[0]],shs[pair[1]]]
                shs2.sort()
                if exp0.get_pFalseNeg(shs2)>0.9:
		    print 'NOTE: returning False based on get_pFalseNeg',self
		    return False
		invsum=0.0
                dlst=vc.findNearestFromCoord(tuple(shs2),distMetricTup2,None,numb=None)
                if len(dlst)>0:
                    Dist=dlst[0][0]
		    print 'more?:',dlst##,self
		    if rev and Dist<Tol:
			print 'stripping peak (reverse)',Dist
		        self.hErrProb=1.0#actually the peak should be removed
                        return True#strip
                    for i in range(len(dlst)):
                        mpeak=peaklist[dlst[i][2]]
                        minexpi=mpeak.minexp
                        if dlst[i][0]<=Tol*1.5 and minexpi<=accept:
                           valid=True;Dist=dlst[i][0]
			   ##print 'found close peak:',Dist;break
			   print 'found close peak:',Dist
			   invsum+=(1/Dist)
                ##relDist=Dist/Tol
                if invsum<=0.0:relDist=999
		else:relDist=1/invsum/Tol
                if relDist>0.5 and not rev:
                    xErr=comStandardNormal(log(relDist)*3.0)
                    xErr*=0.8#peak blev ikke fundet f.eks pga. overlap
		    print 'peak is not very close to any other peak - upscaling error estimate',relDist
                if len(dlst)==0 or (not valid):
		    if rev:
		        ##self.hErrProb=self.hErrProb
                        return False#DONT strip
                    if 'scc' in kwargs and DEBUG:
                        num,asslst,astr=self.showPeaks(kwargs['explst'],kwargs['scc'])
                        print '-------------------------------'
                        print 'SHIFTS:',shs,shs2,Dist,num,asslst[0],
                        if asslst[0]<>None:
                            at0,at1=asslst[0][pair[0]],asslst[0][pair[1]]
                            if pdb<>None:
                                At0=at0.getAt(pdb);At1=at1.getAt(pdb)
                                print AtomMath.calc_distance(At0,At1),
                            print 'mistake:',at0,at1
                            ##print At0;print At1
                    print 'xErr','%8.3f'%(xErr),'stripping',
                    if DEBUG:
                        if len(dlst)>0:
                          found=False
                          for i in range(len(dlst)):
                            mpeak=peaklist[dlst[i][2]]
                            for s in mpeak.assSet:
                                Sn=Set(s)
                                for S0i in S0:
                                    s0=Set(S0i)
                                    if Sn.issubset(s0):print 'sub',Sn;found=True;break
                                if found:break
                            if found:break
                          if not found:print 'no match'
                        else:print 'no valid'
		    self.hErrProb=pOr(self.hErrProb,xErr)
                    return True#do strip
        elif 'rev' in kwargs and kwargs['rev']:pass
	    ##print 'performing reverse stripping',self
        if xErr>0 and 'scc' in kwargs and DEBUG:
            num,asslst,astr=self.showPeaks(kwargs['explst'],kwargs['scc'])
            print 'xErr',xErr,num,asslst[0]
        else:print 'xErr',xErr
	if xErr<=0.0:
	    print 'downscaling error estimate'
            self.hErrProb*=0.2
	else:
            self.hErrProb=pOr(self.hErrProb,xErr)
	return False


class PeakSimple(Peak_sparky):

    def __init__(self,sh,names):
	self.shifts=sh
	self.names=names
	self.aliased=False


class Peak_sparky_Virtual(Peak_sparky):
    lhrat=0.2
    rms=100
    #hvad betyder det?

    def __init__(self,i,pos,hi,exp0,assign=None,doRandPos=True,lws=None):
        ##print i,pos,hi,assign
	print i,pos,hi,assign
        if doRandPos and i<>4:#4 is an error peak
	    e0,emin,h0,klw,m0=exp0.errorvars
            err0=1.0/max(m0,log(hi/h0))
            ##if i>0:error=error+exp0.e0
            if i in [1,2]:errmin=exp0.e0
	    else:errmin=0.0
	    ##e1+(e0+1.0/max(0.8,log(hi/0.75)))*(eFac*sqrt(dim)*lineWid)
	    for j in range(exp0.dimension):
                rnd=normalvariate(0.0,1.0)
		error=exp0.e1+(errmin+err0)*exp0.eFac*lws[j]
		pos[j]+=rnd*error
	    if False:#old implementation
              if exp0.dimension==2:
                r=rand()
                pos[0]=pos[0]+cos(2*r*pi)*error
                pos[1]=pos[1]+sin(2*r*pi)*error
              elif exp0.dimension==3:
                u=rand()
                v=rand()
                pos[0]=pos[0]+cos(2*u*pi)*sin(v*pi)*error
                pos[1]=pos[1]+sin(2*u*pi)*sin(v*pi)*error
                pos[2]=pos[2]+cos(v*pi)*error
        self.shifts=tuple(pos)
        ##str='%8.2f'*exp0.dimension
        ##print round(error,3),
        ##print str%self.shifts
        if assign==None:
            self.names=assign
            self.mindist=999
            self.maxAssSet=Set([])
            self.assSet=Set([])
	else:
          self.names=assign[0]
	  if not 'input' in exp0.inpmapexp:
            self.mindist=calcDist(self.names[-1],self.names[-2])
            key=list(self.names);key.sort();key=tuple(key)
            self.maxAssSet=getAssignSet3([key])[1]
            self.assSet=getAssignSet3(assign)[1]
	self.allAssigns=assign
        self.score=None
        self.lws=exp0.lineWidth*ones(exp0.dimension)#i ppm!!!!
        self.volumes=hi
        self.fitheight=hi/prod(self.lws)*0.8
        self.height=hi/prod(self.lws)
        self.virtFlag=i

    def TEST(self):
        print 'TESTING:',self.shifts,self.allAssigns,
        if self.names<>None:
            shs=self.names[-1].shift[0],self.names[-2].shift[0]
            shsc=shs[1],shs[0]
            cdiff=min(distMetricTup(self.shifts,shs),distMetricTup(self.shifts,shsc))
            if cdiff>0.5:print 'ERROR?:',cdiff
            else:print
        else:print

class PeakData:    
    globalshift=0.0
    sumfac=0.0

    def __init__(self,im,exp0,suffix='',pdb=None):
      self.peakbuffer=[]
      self.peaklist=[]
      self.allvols=[]
      self.allheights=[]
      noisescale=2.0
      dim=exp0.dimension
      self.globalshift=exp0.globShifts
      self.sws=exp0.sws
      self.gsh1s=exp0.gsh1s
      vc=VoxelContainer()
      ##vc.vwidth=exp0.shiftTols*2.0
      vc.vwidth=exp0.shiftDamps*4.0
      ##pnoiseConstant=eval(im['errorMin'])
      header=2
      if im.has_key('header'):header=string.atoi(im['header'])
      if True:
                if 'infile'+suffix in im:
                  pksname=im['infile'+suffix]
                else:
                  pksname='virt'
                if pksname[:4] in ['virt','Virt','VIRT']:
                    print 'calculating virtual data...'
                    print pdb
                    exp0.globalshift=zeros(exp0.dimension)
                    exp0.gsh1s=zeros(exp0.dimension)
                    self.globalshift=zeros(exp0.dimension)
                    self.gsh1s=zeros(exp0.dimension)
                    virtSpec=exp0.getVirtualPeakList(pdb)
                    self.peakcoords=virtSpec.pickData.allFinalPeakPos
                    virtSpec.pickData.writePeakList(exp0,exp0.name+suffix+'_virt.txt')
                    self.peaklist=virtSpec.pickData.peakList
                    print len(self.peaklist),'peaks in total'
                    self.peakbuffer=None
                    self.virtSpec=virtSpec
                    self.minHeight=1.0
                else:
                    buf=initfil2(pksname)
    ##                self.peakbuffer=buf[:10]+buf[-10:-1]#slet igen
                    self.peakbuffer=buf#was TAKEBACK!
                    for i in range(header):del self.peakbuffer[0]
                    self.peakcoords=[]
                    ##t=numpy.linspace(numpy.min(self.allheights),numpy.max(self.allheights),num=2)
                    ##plot(log(t),log(relfac*t),'k--')
                    ##scatter(log(self.allheights),log(self.allvols));show()
                    for i in range(len(self.peakbuffer)):
                        peak=Peak_sparky(self,i,dim)
                        self.peaklist.append(peak)
                        ##self.peakcoords.append(array(peak.shifts))
                        self.peakcoords.append(peak.shifts)
		    if len(self.allvols)>0:relfac=self.sumfac/len(self.allvols)
		    else:relfac=1.0
                    minvol   =9999999999999
                    minheight=9999999999999
                    for peak in self.peaklist:
                        if peak.fromheight:
                            peak.height=peak.volumes
                            peak.fitheight=peak.volumes
                            peak.volumes=peak.volumes*relfac
                        if 0.0<peak.volumes<minvol:minvol=peak.volumes
                        if 0.0<peak.height<minheight:minheight=peak.height
                    print minvol,minheight
                    self.minHeight=minheight/minvol
                    print self.minHeight,'lll'
		    #normailizing
		    ##print ('noiselevel',exp0.noiselevel)
                    for i in range(len(self.peakbuffer)):
                        if self.peaklist[i].volumes<minvol:
                            self.peaklist[i].volumes=minvol
                        if self.peaklist[i].height<minheight:
                            self.peaklist[i].height=minheight
                        normvol=self.peaklist[i].volumes/minvol
			if exp0.noiselevel==1.0:
			  print 'normalizing using minimum volume'
                          self.peaklist[i].volumes=normvol
                          self.peaklist[i].height=self.peaklist[i].height/minvol
                          self.peaklist[i].fitheight=self.peaklist[i].fitheight/minvol
			else:
			  print 'normalizing using user defined noise level'
                          self.peaklist[i].volumes/=exp0.noiselevel
                          self.peaklist[i].height/=exp0.noiselevel
                          self.peaklist[i].fitheight/=exp0.noiselevel
                        #print self.peaklist[i].height,self.peaklist[i].volumes
                        ##print normvol,pnoise
          ##print self.peakcoords[:2]
      vc.fillContainer(self.peakcoords)
      if 0:#TODO put i inputfil
          cplst=vc.getClosePairs(distMetricTup,None,0.4)
          print cplst
          for x in cplst:
              if x[0]>x[1]:
                  a=numpy.array(self.peaklist[x[0]].shifts)-self.globalshift
                  b=numpy.array(self.peaklist[x[1]].shifts)-self.globalshift
                  print a,b,x[2],x[0],x[1]
      self.peakvc=vc

    def getpreassignedspinsystems(self):
	dct={}
	buf=initfil2('hmbd_prespinsystems.txt')
	##elemcorrs={'N15':-0.1, 'C13':-2.70, 'H1':-0.116}
	elemcorrs={'C13':-2.70}
	for lin in buf:
	  k=lin[0]
	  if k<>'nan':
	    if not k in dct:dct[k]={}
	    elem=lin[1]
	    if not elem in dct[k]:dct[k][elem]={}
	    val=eval(lin[2])
	    if elem in elemcorrs:val-=elemcorrs[elem]
	    atn=lin[3]
	    off=lin[4]
	    if atn=='na':
		if not 'na' in dct[k][elem]:dct[k][elem]['na']=[]
		dct[k][elem]['na'].append(val)
	    else:
	        key=atn,off
		dct[k][elem][key]=val#unique key
	return dct

    def findSpinSystems(self,damps,distthr=2.0):
	prespindct=self.getpreassignedspinsystems()
	dct={}
	for i,peak in enumerate(self.peaklist):
	  if peak.names[0]<>'?':
	    nams=string.split(peak.names,'-')
	    print nams
	    group=nams[0][0]
	    for c in nams[0][1:]:
	      if not c in '0123456789':break
	      group+=c
	    print group
	    if not group in dct:dct[group]=SpinSystem(self.peaklist[i],group)
	ddct={}
	for i,peak in enumerate(self.peaklist):
	    mindist=999;mink=None
	    for k in dct:
		distik=dct[k].getDistance(peak,damps)
		if 0.0<distik<5.0:print 'note smalldist',i,k,distik,dct[k].rootcoords,peak.shifts,peak
		if distik<mindist:
		  mindist=distik
		  mink=k
	    if mink<>None and mindist<distthr:
		print 'bestdist:',i,mink,mindist,dct[mink].rootcoords,peak.shifts##,peak
		dct[mink].addPeak(peak)
		##ddct[i]=mink
	print len(dct),'spin systems in total'
	hndiffs=[];ndiffs=[]
	for k in dct:
	    lstk=dct[k].varclist
	    print 'spinsystem:',k,len(lstk),dct[k].rootcoords,lstk
	    if k[0]=='A':
		psdct=prespindct[k[1:]]
		dct[k].hetdct=psdct
		##dct[k].matchkey=k
		print 'presystem',k,psdct
		if 'N15' in psdct and ('N', '0') in psdct['N15']:
		   ndiffs.append(psdct['N15'][('N', '0')]-dct[k].rootcoords[1]+dct[k].aliased)
		   print 'ndiff:',k,ndiffs[-1]
		if 'H1' in psdct and ('H', '0') in psdct['H1']:
		   hndiffs.append(psdct['H1'][('H', '0')]-dct[k].rootcoords[2])
	    else:
		print 'trying to align',k
	        minpdist=999;minpk=None
		for pk in prespindct:
		   pdistik=dct[k].getpDistance(prespindct[pk],damps,-0.0035,-0.098)
		   if pdistik<minpdist:
		     minpdist=pdistik
		     minpk=pk
	        if minpk<>None and minpdist<distthr*2.0:
		   psdct=prespindct[minpk]
		   keyvals=[psdct['N15'][('N', '0')],psdct['H1'][('H', '0')]]
		   if 'H1' in psdct and ('HA', '0') in psdct['H1']:keyvals.append(psdct['H1'][('HA', '0')])
		   print 'bestpdist:',k,minpk,minpdist,dct[k].rootcoords,keyvals
		   dct[k].hetdct=psdct
	        else:print 'WARNING: no related spin systems',k,minpdist,minpk
	    if len(lstk)==1:
		##peak1=dct[k].peaklist[0]
		print 'obs: single spin system',k,dct[k].rootcoords##,peak1
	print average(hndiffs)
	print average(ndiffs)
	print len(dct.keys()),dct.keys()
	return dct
        #enddef findSpinSystems
	    
    def addNOESYspins(self,damps,dct,distthr=2.0):
	for k in dct:
	    ##dct[k].noesypeaklist=[]
	    dct[k].noesyvarclist=[]
	for i,peak in enumerate(self.peaklist):
	    mindist=999;mink=None
	    for k in dct:
		distik=dct[k].getDistance(peak,damps,mapi=[2,0,1])
		##if 0.0<distik<5.0:print 'note smalldist',i,k,distik,dct[k].rootcoords,peak.shifts,peak
		if distik<mindist:
		  mindist=distik
		  mink=k
	    if mink<>None and mindist<distthr:
		print 'bestnoesydist:',i,mink,mindist,dct[mink].rootcoords,peak.shifts##,peak
		dct[mink].addNOESYPeak(peak,2)
		##ddct[i]=mink
	for k in dct:
	    lstk=dct[k].noesyvarclist;lstk.sort()
	    print 'noesyspinsystem:',k,len(lstk),dct[k].rootcoords,lstk

    def estimateErrorProbs(self,exp0):
      'TODO: laese parametre ind fra eksp.-inputfil'
      X=numpy.zeros(len(self.peaklist))
      Y=numpy.zeros(len(self.peaklist))
      Z=numpy.zeros(len(self.peaklist))
      ##print self.peakvc.list[0]
      for i in range(len(self.peaklist)):
          peak=self.peaklist[i]
          if peak.rms==None:peak.rms=exp(10)
          X[i]=log(peak.rms);Y[i]=log(peak.volumes)
          nnlst=self.peakvc.findNearestFromCoord(\
              peak.shifts,dist2pScaled,
              (exp0.shiftDamps,numpy.zeros(exp0.dimension)),
              maxd=exp0.dimension*((3.0)**2))[1:]
##          print len(nnlst),
          ##print peak.shifts
##          nnlst=self.peakvc.findNearestFromCoord(\
##              peak.shifts,dist2p,None,
##              maxd=exp0.dimension*((1.5)**2))[1:]
          ##print len(nnlst),
          sumh0=0.0
          for x in nnlst:
              h=self.peaklist[x[2]].height
              h0=h*exp(-0.5*x[0]/(0.4**2))
              sumh0=sumh0+h0
          if sumh0<0.5:sumh0
          #print sumh0,
          noiseHeight=math.sqrt((self.minHeight*0.3)**2+sumh0**2)
          heightFac=noiseHeight/peak.height*1.0
          if heightFac>3:hErr=0.9;print 'probably an error:',i
          else:hErr=comStandardNormal(log(heightFac)*1.6)
          Z[i]=hErr
##          peak.hErrProb=min(min(hErr,2*exp0.pN),0.5)+exp0.pLR
          peak.hErrProb=max(min(hErr+exp0.pLR,0.5),0.1*exp0.pN)
	  print 'TJEKNOISE',exp0.name,i,self.minHeight,sumh0,noiseHeight,heightFac,hErr
      print
          ##print X[i],Z[i]
      #scatter(Y,X,alpha=0.5,faceted=False)
      #scatter(X,Z,alpha=0.5,c='g',faceted=False);show()

    def analyzeErrors(self,exp0):
      print len(exp0.signalIDs)
      print len(exp0.noiseIDs)
      Xs=numpy.zeros(len(exp0.signalIDs))
      Xn=numpy.zeros(len(exp0.noiseIDs))
      Ys=numpy.zeros(len(exp0.signalIDs))
      Yn=numpy.zeros(len(exp0.noiseIDs))
      for i in range(len(exp0.signalIDs)):
          ide=exp0.signalIDs[i]
          peak=self.peaklist[ide]
          Xs[i]=peak.hErrProb
          Ys[i]=peak.rms
      for i in range(len(exp0.noiseIDs)):
          ide=exp0.noiseIDs[i]
          peak=self.peaklist[ide]
          Xn[i]=peak.hErrProb
          Yn[i]=peak.rms
      Bins=numpy.linspace(0.025,0.625,num=12)
      scatter(Xs,log(Ys),alpha=0.5,faceted=False)
      scatter(Xn,log(Yn),alpha=0.5,c='r',faceted=False)
      ns,bs,ps=hist(Xs,bins=Bins,width=0.005,align='center')#,normed=1)
      nn,bn,pn=hist(Xn,bins=Bins,width=0.002,align='center')#,normed=1)
      fracs=(numpy.array(nn)*1.0)/(numpy.array(ns)*1.0)
      sumdevs2=0
      for j in range(len(Bins)):
          print Bins[j],fracs[j]
          sumdevs2=sumdevs2+(Bins[j]-fracs[j])**2
      print math.sqrt(sumdevs2/len(Bins))
      show()
      
    def stripPeaks(self,exp0,pairlist=[(1,2)],tol=0.5,pdb=None,**kwargs):
	print 'starting stripping...'
        if 'slice' in kwargs:
            exp0.peakData.peakvc.putSlices(comp=1)
            for s in exp0.peakData.peakvc.slices:
                print len(s.list),
            print
        if not 'accepts' in kwargs:
            accepts=[0]*len(pairlist)
        else:
	    accepts=kwargs['accepts']
	    del kwargs['accepts']
	stripped=0
        for peak in self.peaklist:
            if 'slice' in kwargs:vc=exp0.peakData.peakvc.slices['slice']
            else:vc=exp0.peakData.peakvc
            if isinstance(peak,BuildUpPeakVector):
                thisstrip=peak.stripTest(exp0,vc,exp0.peakData.peaklist,tol,pairlist,accepts,pdb=pdb,
                                         scc=self.scc,explst=self.oldexplst,**kwargs)
            else:
                thisstrip=peak.stripTest(exp0,vc,exp0.peakData.peaklist,tol,pairlist,accepts,pdb=pdb)
            if peak.strip<>1:#peak er ikke allerede stripped
              peak.strip=thisstrip
              #peak skal aldrig fjernes men have evt. en hoej pNoise
              #jeg beholder stripflager men bruger det kun til debugging..
              if thisstrip:stripped+=1
        print stripped, 'stripped peaks (',len(self.peaklist)-stripped,'left)'

    def getNshiftData(self,exp0):
	dct={}
	for i,peak in enumerate(self.peaklist):
	  nind=exp0.nucnames.index('N')
	  nsh=peak.shifts[nind]
	  ##print peak.allAssigns
	  if peak.allAssigns<>None:
	   for ass in peak.allAssigns:
	    resi=ass[nind][0]
	    if not resi in dct:dct[resi]={}
	    if not ass in dct[resi]:dct[resi][ass]={}
	    dct[resi][ass][i]=nsh##,peak.volumes)
	    if len(dct[resi][ass])>1:
		print 'NOTE: more values',exp0.name,resi,ass,dct[resi][ass]
	##print 'ndata:',exp0.name,dct[2]
	return dct

    def analNshiftDatas(self,allndats,explst):
	cdct={}
	allstdns=[]
	for dct in allndats:
	  for resi in dct:
	    if not resi in cdct:cdct[resi]={}
	    cdct[resi].update(dct[resi])#no overlap in different experiments
	resis=cdct.keys();resis.sort()
	for resi in resis:
	 rdct=cdct[resi]
	 if len(rdct)>1:
	  maxnum=max([len(rdct[ass]) for ass in rdct])
	  if maxnum==1:nvals=[rdct[ass].values()[0] for ass in rdct]
	  else:
	    print 'multi',resi,rdct
	    nvals=[];multiass=[]
	    for ass in rdct:
		if len(rdct[ass])==1:nvals.append(rdct[ass].values()[0])
		else:multiass.append(ass)
	    if len(multiass)>1:print 'NOTE: more multiass!',multiass
	    if len(nvals)>0: aven=average(nvals)
	    else:
	      print 'NOTE: no single values for average',rdct
	      allvals=[]
	      for ass in rdct:
		allvals+=rdct[ass].values()
	      aven=average(allvals)
	    print aven,multiass,rdct[multiass[0]]
	    for mass in multiass:
	      diffs=[(abs(aven-rdct[mass][i]),i) for i in rdct[mass]]
	      diffs.sort()
	      nvals.append(rdct[mass][diffs[0][1]])
	    print nvals
	  aven=average(nvals)
	  stdn=std(nvals)
	  allstdns.append(stdn)
	  print 'nstats:',resi,aven,stdn
	print 'avestdn:',explst[0].lineWidth,average(allstdns)

class Cycle:
    def __init__(self,cyclename):
        cycvs=initializer(cyclename)
        cuts=string.split(cycvs[2])
        endcuts=string.split(cycvs[8])
        endcuts2=string.split(cycvs[9])
        volpows=string.split(cycvs[3])
        self.distCutoff=string.atof(cycvs[0])
        self.shiftCutoff=string.atof(cycvs[1])
        self.minGVC=string.atof(cuts[0])
        self.minGVCamb=string.atof(cuts[1])
        self.maxAmb=string.atoi(cuts[2])
        self.dpow=string.atof(volpows[0])
        self.tpow=string.atof(volpows[1])
        self.damping=string.atof(cycvs[4])    
        self.trunc=string.atof(cycvs[5])
        self.gvc0=string.atof(cycvs[6])
        self.numit=string.atof(cycvs[7])
        self.finalShiftCut=string.atof(endcuts[0]),string.atof(endcuts2[0])
        self.finalDistCut=string.atof(endcuts[1]),string.atof(endcuts2[1])
        self.finalTriCut=string.atof(endcuts[2]),string.atof(endcuts2[2])


class ShiftVC:
    def __init__(self,shvc,shlst,atreflst,stdlst):
        self.vc=shvc
        self.list=shlst
        self.atreflist=atreflst
        self.stdlist=stdlst
        self.problist=[]
        for i in range(len(atreflst)):
            self.problist.append([1.0])

    def getnearestList(self,shifts,scut,sdamp,mode,shnormck):
	print 'shifts:',shifts,self.vc.vwidth
        met=diffSingArrays #a metric
        xlst=[]
        nnilst=self.vc.findNearestFromCoord([shifts],met,None,list='y')
        ##nnilst=self.vc.findNearestFromCoord([shifts],dist2pScaled,((sdamp,stdev)),list='y')
        posaslst=[]
        #print sh, nnilst[i]
        maxshfaci=0.0
	M=None
        for I in nnilst:
            i,tagi=I
            #print self.list[I]
            #print self.stdlist[I]
            #print self.problist[I]
            #i=(gamle_i,tag)
            shi=self.list[i][tagi]
            stdi=self.stdlist[i][tagi]
            probi=self.problist[i][tagi]
            ati=self.atreflist[i]
            diff=shifts-shi
            accept=False
            if mode=='normal':
                totdamp=math.sqrt(sdamp**2+stdi**2)
                dampdiff=abs(diff)/totdamp
                if not SILENT:print ati,diff,shi,stdi,totdamp,dampdiff,
                if dampdiff<scut*1.0:#her har scut enheden rel. std!
                    accept=True
                    shfaci=None
		elif not SILENT:print 'neglecting assignment possibility'
            else:
                if not SILENT:print ati,diff,shi,stdi,sdamp,
                pdfi=ati.resAs.pdf
                shfaci=pdfi.calcIntNorm(shifts,sdamp)
                if not SILENT:print shfaci,
                ##shfaci=min(1.0,shfaci/shnormcsq)
                if shfaci/shnormck>0.02:
                    accept=True
                    dampdiff=None;totdamp=None
                ##elif shfaci>maxshfaci:
                if shfaci>maxshfaci:
                    maxshfaci=shfaci
                    M=(ati,shi,stdi,probi,diff,None,None,shfaci)
            if accept:
                ass1d=Assignment1D(ati,shi,stdi,probi,diff,dampdiff,totdamp,shfaci)
                posaslst.append(ass1d)
                if not SILENT:print ati,shi
	    elif not SILENT:print 'neglecting'## assignment possibility'
	print 'best nearest',M,len(posaslst)
        if len(posaslst)==0 and maxshfaci/shnormck>0.005:
            posaslst.append(Assignment1D(M[0],M[1],M[2],M[3],M[4],M[5],M[6],M[7]))
            print 'appending maximum 1d assignment',M
        return posaslst,maxshfaci

    def getnearest(self,shifts,scut,dcut=None):
        met=diffSingArrays #a metric
        nnilst=[[],[]]
        assignlst=[]
        for i in range(len(shifts)):
            sh=shifts[i]
            nnilst[i]=self.vc.findNearestFromCoord([sh],met,None,list='y')
            #print sh, nnilst[i]
        for I in nnilst[0]:
            i,tagi=I
            #print self.list[I]
            #print self.stdlist[I]
            #print self.problist[I]
            #i=(gamle_i,tag)
            shi=self.list[i][tagi]
            stdi=self.stdlist[i][tagi]
            probi=self.problist[i][tagi]
            for J in nnilst[1]:
                j,tagj=J
                shj=self.list[j][tagj]
                stdj=self.stdlist[j][tagj]
                probj=self.problist[j][tagj]
                if abs(shi-shifts[0])<scut and abs(shj-shifts[1])<scut:
                    diff=(shifts[0]-shi)**2+(shifts[1]-shj)**2
                    diffSigned=(shifts[0]-shi,shifts[1]-shj)
                    #print shi,shj,diff,
                    ati,atj=self.atreflist[i],self.atreflist[j]
                    #print ati.fragment_id,ati.name,atj.fragment_id,atj.name
                    dist=AtomMath.calc_distance(ati,atj)
                    #print dist
                    if dist<dcut and probi>0.1 and probj>0.1:
                        #print shi,shj
                        #print ati.fragment_id,ati.name,atj.fragment_id,atj.name
                        #print dist
                        ass=Assignment((ati,atj),(shi,shj),(stdi,stdj),(probi,probj),
                                       dist,diff,diffSigned)
                        assignlst.append(ass)
        return assignlst
        #print '------------------'


class Assignment:
    atoms=[]
    distances=[0.0]
    shiftDiff=1000.0
    shiftDiffSigned=[]
    coords=[]
    gvc=0.0
    GiFi=0.0
    volFac=0.0
    shFac=0.0
    pPost=None
    
    def __init__(self,ats,shifts,stds,probs,dists,diff,diffSigned,ddvec,tdvec,shfacvec,ten):
        if not SILENT:print 'initializing Assignment'
        ordmat,dmat=ten
        ##print ordmat,dmat
        self.atoms=ats
##        self.lrSet=[getType(p) for p in comb3(ats)]
##        self.distances=dists
        self.distances=dmat#is now matrix!!!
        self.shiftDiffSigned=numpy.array(diffSigned)
        self.gvc=0.0
        self.coords=numpy.array(shifts)
        self.stdevs=stds
        self.probs=probs
        self.dampDiffs=ddvec
        self.totDamps=tdvec
        self.shfacs=shfacvec
        self.orderMat=ordmat
        ##print self.orderMat
        if diff==None:
            diff=math.sqrt(numpy.dot(diffSigned,diffSigned))
        self.shiftDiff=diff

    def setResAsVec(self,ptlst,ptlstN):
        self.resAsVec=[]
        for i in range(len(self.atoms)):
             ati=self.atoms[i]
             self.resAsVec.append(ati.resAs)

    def setOrderAtoms(self):
        self.orderedAtoms=[]
        atord=self.orderMat[self.maxPos]
        for i in range(len(atord)):
            self.orderedAtoms.append(self.atoms[atord[i]])

    def setVolFac(self,volfac,volfacs,k):
        self.volFac=volfac
        self.volFacs=volfacs
        self.maxPos=k

    def setShFac(self,shfac):
	##if shfac<0.005:shfac=0.0
        self.shFac=shfac

    def setProjGifi(self,shfacvec):
	self.projGiFis=[self.volFac for j in range(len(shfacvec))]
	for j in range(len(shfacvec)):
	  for k in range(len(shfacvec)):
	    if k<>j: self.projGiFis[j]*=shfacvec[k]

    def setpPost(self,exp0,allnot,*args):
        if exp0.hypType==1:#resonance assignments
            LH=self.volFac
            prH=self.shFac
        else:#her kommer maaske en fejl?
            if exp0.dimension==2 or exp0.multi[0]==0:pVol=exp0.pE_H
            else:pVol=0.5/exp0.norm1*self.volFacs[0]#dvs. stoerre end foer...
            #TODO hvad goeres her???!!!
            if exp0.hypType==0:#LR-conn.
                LH=max(pVol,self.volFac)*self.shFac
                prH=self.volFac
            elif exp0.hypType==2:#restraint assignments? eller fejl?!!!
                LH=max(pVol,self.volFac)
                prH=self.volFac*self.shFac
                #TODO: TJEK om udtrykket giver mening!!!
	if allnot<=0.0:self.pE_H=1.0
	elif self.GiFi>=1.0:self.pE_H=1.0
	else:self.pE_H=1-allnot*(1-LH)/(1-self.GiFi)
        self.pE=1-allnot
        #TODO:max(...) er ikke det helt rigtige?...
        #TODO:generaliser... skal LH,prH,likRat vaere attributter?
	##if len(args)>0:
	if len(args)>0 and args[-1]>0.0:#TODO er det vigtigt at bruge nedenst.???
            pN,pN0,pD,pDiff,fac=args
	    ##print 'tjekvals:',pN,pN0,pD,pDiff,fac
	    ##pE_HnoN=Prob(1-(1-self.pE_H)/(1-exp0.pN))
	    ##pEnoN  =Prob(1-(1-self.pE)/(1-exp0.pN))
	    pE_HnoN=Prob(1-(1-self.pE_H)/(1-pN))
	    pEnoN  =Prob(1-(1-self.pE)/(1-pN))
	    pD_H=(pD/fac)&(pDiff|exp0.pE_H)
	    pU_H=pD_H|exp0.pDiff
	    pU  =pD  |exp0.pDiff
	    pE_Hf=(pE_HnoN&pU_H)|exp0.pC
	    pEf  =(pEnoN&pU)|exp0.pC
	else:
	    pE_Hf=self.pE_H
	    pEf  =self.pE
        self.likRat=float(pE_Hf)/float(pEf)
        if not SILENT:print 'likRat,volfac',self.likRat,self.volFac
        'here is the posterior probability:'
        self.pPost=prH*self.likRat*exp0.pH_H0
        
    def calcGiFi(self):
        self.GiFi=self.shFac*self.volFac
        
    def getLikFrac(self,ret=1,dim=None,exp0=None):
	if self.GiFi>=1.0:allnot=0.0
	elif self.allNot<=0.0:allnot=0.0
	else:allnot=self.allNot/(1-self.GiFi)
        Gpa=self.volFac*allnot;panot=1-allnot
        ##why not: Gpa=self.volFac/(1-allnot) ? can be > 1!
	if dim<>None: Gpa=self.projGiFis[dim]*allnot
        likRat=Gpa/panot
        likFrac=Gpa/(Gpa+panot)
        if ret==1:return likFrac
        elif ret==2:return likFrac,Gpa
        #TODO tjek om det virker!

    def show_assign(self,showDist=True):
##        maxd=max(self.distances[self.maxPos])
##        if isinstance(maxd,LRProb):thismaxd=maxd.prob
##        else:thismaxd=maxd
##        print '%8.5f%6.2f%6.3f'%(self.GiFi,thismaxd,self.shiftDiff)
        if showDist:
            maxd=max(self.distances[self.maxPos])
            print '%8.5f%6.2f%6.3f'%(self.GiFi,maxd,self.shiftDiff)
        else:
            print '%8.5f%6.3f'%(self.GiFi,self.shiftDiff)
        for k in range(len(self.atoms)):
            print self.atoms[k].fragment_id,self.atoms[k].name,
            print '%6.2f%6.2f'%(self.atoms[k].shift[0],self.shiftDiffSigned[k])
        print '%8.5f%8.5f'%(self.volFac,self.shFac)

    def getViolation(self,pdb,thr):
        atlst=[]
        for i in range(2):
            at0=self.atoms[i]
	    ##print at0
            ati=pdb[0][at0.fragment_id][at0.name]
            atlst.append(ati)
        dist=AtomMath.calc_distance(atlst[0],atlst[1])
        print dist,
##        oi=None
##        if atlst[0].name=='O':oi=0
##        elif atlst[1].name=='O':oi=1
##        if oi<>None:
##            ath=pdb[0][atlst[1-oi].fragment_id]['HN']
##            ang=AtomMath.calc_angle(atlst[oi],ath,atlst[1-oi])/pi
##            if ang<0.5:return 25.0
##            atc=pdb[0][atlst[oi].fragment_id]['C']
##            ang=AtomMath.calc_angle(atc,atlst[oi],ath)/pi
##            if ang<0.5:return 25.0
        if dist<=thr:return 0.0
        else:return (dist-thr)**2

    def acceptAsSingle(self,proberr,thr=0.002,errmax=0.1):
##	answ=self.pPost>thr and self.shFac>0.05
	answ=self.pPost>thr and self.shFac>0.10 and proberr<errmax
	print 'accepttest:',self.shFac
	if not answ:
            print 'rejected:',self.pPost,self.shFac,self.shiftDiffSigned
	    self.show_assign(False)
        else:
            print 'accepted:',self.pPost,self.shFac,self.shiftDiffSigned
	    self.show_assign(False)
	#TODO: generaliser med pdf lokale maxima i stedet for shiftDiffSigned
	return answ

    def getAtomPairKey(self,k,i):
        ats=[]
        atord=self.orderMat[k]
        for n in [i,i+1]:
##            ats.append(self.atoms[atord[n]])
            ats.append(self.atoms[atord.index(n)])
	rtype=getType(ats)
	if rtype>4:
	    fris=[ at.fragment_id          for at in ats]
	else:
	    fris=[(at.fragment_id,at.name) for at in ats]
	return tuple(sorted(fris))

    def showDistanceCombinations(self):
	for k in range(len(self.orderMat)):
	    atord=self.orderMat[k]
	    ats=[]
            for n in range(len(atord)):
##                ats.append(self.atoms[atord[n]])
                ats.append(self.atoms[atord.index(n)])
            for n in range(len(atord)-1):
                d=AtomMath.calc_distance(ats[n],ats[n+1])
		print k,n,ats[n].fragment_id,ats[n].name,ats[n+1].fragment_id,ats[n+1].name,d,self.distances[k][n]

    def getRealDist(self,ats):
        return AtomMath.calc_distance(ats[-2],ats[-1])

    def isTrueAssignment_old(self,asslst):
	for ass in asslst:
	    for n,at in enumerate(self.atoms):
		found=True
		if not (at.name       ==ass[n].name and\
		        at.fragment_id==ass[n].fragment_id):
			found=False
			break
		if found==True:return found#else continue searching list
	return False

    def isTrueAssignment(self,peak):
	##print dir(peak)
	atstr=getAsFormat(self.atoms)
	if hasattr(peak,'assSet'):#buildups + virtual peaks
	    print peak.assSet
	    print peak.maxAssSet
	    print atstr,self.atoms
	    return tuple(sorted(self.atoms)) in peak.assSet
	if peak.names[0]=='?':return False
	##print peak.modelAsslst,atstr
	return atstr in peak.modelAsslst


class SimpleAssignment(Assignment):

    def __init__(self,ats,gifi,volfac,projgifi):
        self.atoms=ats
	self.GiFi=gifi
	self.volFac=volfac
	self.projGiFis=[projgifi]


class ReadAssignment(Assignment):

    def __init__(self,ats,asncateg,asnrealdist,asnrType,lik,shiftprob):
	self.assignmentCategory=asncateg
	self.realdist=asnrealdist
	self.rType=asnrType
	self.pPost=lik
	self.atoms=[]
	self.shFac=shiftprob
	for i in range(2):
            resn,fri,name=splitAtom2(ats[i])
	    ##print resn,fri,name
            self.atoms.append(_Atom2(resn,fri,name,name[0]))
	    ##print self.atoms[-1]
        self.orderedAtoms=self.atoms

    def update(self,pdbs,refpdb,rmin):
        if len(pdbs)==0:return None
        ats=self.atoms[-2:]
        fris=[ats[i].fragment_id for i in range(2)]
        names=[ats[i].name for i in range(2)]
        self.pPost=0.0
##        self.atoms=[refpdb[0][fris[i]][names[i]] for i in range(2)]
##        self.orderedAtoms=self.atoms
        for pdb in pdbs.values():
##            print pdb
            ats=[pdb[0][fris[i]][names[i]] for i in range(2)]
            dist=AtomMath.calc_distance(ats[0],ats[1])
            vol=CalcVolfacCum(log(dist),0.0,rmin)
##            if self.rType=='long':print vol,
##            print dist,vol,
            self.pPost+=vol
##        print
        self.pPost/=len(pdbs)
##        if True:##self.rType=='long':
##            print ats[0],
##            print ats[1],
##            print self.pPost

    def getRealDist(self,ats):
        return self.realdist

    def show_assign(self,option=True):
        print self.atoms


class Assignment1D:
    dim=None
    prevdist=None
    
    def __init__(self,at,sh,std,pr,di,dd,td,shf):
        self.atom=at
        self.shift=sh
        self.stdev=std
        self.prob=pr
        self.diff=di
        self.totdamp=td
        self.dampdiff=dd
        self.shfaci=shf

##    def deep_copy(self):
##        at=self.atom
##        sh=self.shift
##        std=self.stdev
##        pr=self.prob
##        di=self.diff
##        td=self.totdamp
##        dd=self.dampdiff
##        new=Assignment1D(at,sh,std,pr,di,dd,td)
##        new.prevdist=self.prevdist
##        new.dim=self.dim
##        return new
##

class AssignData:
    use=0
    maxPpost=0.0
    numAssigns=0
    ambgvc=0.0
    assigns=None
    identifier=None
    asMax=None

    def __init__(self,peak,exp0,ide):
        self.identifier=ide
        self.assigns=[]
        self.pointerMat=peak.ptMat
        self.numAssigns=len(peak.initAssigns)
        self.probNoise=peak.hErrProb
	try:self.pDiff=peak.pDiff
	except AttributeError:self.pDiff=0.0
	#TODO: set peak.pDiff i peak.__init__
        for i in range(len(peak.initAssigns)):
            ass=peak.initAssigns[i]
            ##if not SILENT:print 'GiFi:',ass.GiFi
            self.assigns.append([0.0,ass])
##        self.setAllNot()
##        self.setpPosts(exp0)
##        self.setUse()

    def setAllNot(self):
        sumGiFi=self.probNoise
        allnot=1.0-sumGiFi
        ##probs=zeros(len(self.assigns))
        for i in range(len(self.assigns)):
            ass=self.assigns[i][1]
            if not SILENT:print 'GiFi:',ass.GiFi
            sumGiFi+=ass.GiFi
            ##probs[i]=ass.GiFi
            allnot*=(1-ass.GiFi)
        self.allNot=allnot
        print 'sumGiFi',sumGiFi
        print 'allNot',self.allNot
        if False:##sumGiFi<0.9999 and len(self.assigns)>0:
	    rat=(1-self.probNoise)/(sumGiFi-self.probNoise)
            print 'normalizing...'
            for val,ass in self.assigns:
		##ass.GiFi/=sumGiFi
		ass.GiFi*=rat
            ##self.probNoise/=sumGiFi
	    ##TODO: TJEK skal probNoise opskaleres?
            self.setAllNot()

    def setEntrop(self,normed=True):
        sumGiFi=0.0
	if len(self.assigns)==0:self.entropy=log(999)
	else:
          ##probs=zeros(len(self.assigns))
	  gifis=array([self.assigns[i][1].GiFi for i in range(len(self.assigns))])
          if normed:gifis/=sum(gifis)
	  self.entropy=-sum(gifis*log(gifis))

    def setpPosts(self,exp0,assSet,Smax=None):
        print '__________________________'
        ##pMax=self.probNoise
        pMax=0.0
        print 'probNoise',self.probNoise
	pN=Prob(self.probNoise)
	pN0=Prob(exp0.pN)
        iMax=None;self.asMax=None;self.asMax2=None
        flag='OK'
        if DEBUG and (assSet==None or len(assSet)==0):flag='NOISE'
	if self.pDiff<>None:
	  plst=[]
          for i in range(self.numAssigns):
            ass=self.assigns[i][1]
	    plst.append(~Prob(ass.GiFi)|Prob(self.pDiff))
	  pD=Prob.allAnd(plst)
	self.sum_pPosts=0.0
        for i in range(self.numAssigns):
            ass=self.assigns[i][1]
            ass.allNot=self.allNot
            if self.pDiff<>None:
		ass.setpPost(exp0,self.allNot,pN,pN0,pD,Prob(self.pDiff),plst[i])
	    else:ass.setpPost(exp0,self.allNot)
            if not SILENT:print 'pPost:',ass.pPost
            if ass.pPost>pMax:pMax=ass.pPost;iMax=i;self.asMax=ass
            self.assigns[i][0]=ass.pPost
	    self.sum_pPosts+=ass.pPost
        as0=self.asMax
        if pMax<self.probNoise:
            pMax2=pMax;iMax2=iMax;self.asMax2=self.asMax
            pMax=self.probNoise;iMax=None;self.asMax=None
        if iMax==None:flag+=' assigned noise'
        self.maxPpost=pMax
        self.maxIndex=iMax
        derAsslst,derAssSet=getAssignSet2(self.assigns)
        if len(derAssSet)==0:flag+=' no assignments'
        if flag=='NOISE':#aldrig uden DEBUG
            flag+=' %2d%6.3f%8.5f'%(len(derAssSet),self.maxPpost,self.probNoise)
            maxats=self.asMax.atoms[:];maxats.sort()
            flag+=(' '+getAsFormat(maxats))
            for ass in derAssSet:flag+=(' '+getAsFormat(ass))
        if DEBUG:
            if as0<>None and Smax<>None and flag[:2]=='OK':
                key=list(as0.atoms);key.sort();key=tuple(key)
                if not key in Smax:
                    flag+=(' not max:'+getAsFormat(key))
            cmpSet=derAssSet&assSet
            Sdiff=assSet-derAssSet
            if len(Sdiff)>0:flag+=(' '+str(len(Sdiff))+' are missing')
            print assSet
            print Smax,'(Smax)'
            print cmpSet
            print Sdiff
            if len(cmpSet)==0 and len(assSet)>0:
                flag+=' empty intersection'
            else:
                for ass in cmpSet:print getAsFormat(ass),
                print
        if not SILENT:self.showMaxAssign(exp0.expnum,exp0.fromStructure,exp0.itnum)
        print flag

    def getProjectedAssignments(self,exp0):
	self.projassigns=[]
	for j in range(exp0.dimension):
	  dct={}
          for i in range(self.numAssigns):
            ass=self.assigns[i][1]
	    key=ass.atoms[j]
	    if not key in dct:dct[key]=[]
	    dct[key].append(ass)#store (i,j) to trace back?
	  assplst=[]
	  print dct.keys()
	  for atn in dct:
	    print '%%%%%%%%assignments for atom',j,atn
	    s=1.0; v=1.0; p=1.0
	    for ass in dct[atn]:
		s*=(1-ass.GiFi)
        	v*=(1-ass.volFac)
        	p*=(1-ass.projGiFis[j])
		print ass
	    assp=SimpleAssignment([atn],1-s,1-v,1-p)
            assp.allNot=self.allNot
	    assp.peakDev=ass.peakDevs[j]
            likFrac,Gpa=assp.getLikFrac(ret=2,dim=0)
	    print 'projected assign:',j,atn,1-s,1-v,likFrac,Gpa
	    assplst.append(assp)
	  self.projassigns.append(assplst)

    def setUse(self):
        if self.asMax==None:self.use=2
        if self.probNoise==1-self.allNot:self.use=0
        elif self.use<>2:self.use=1
##	if self.maxPpost<0.00001:self.use=3
##	if self.maxPpost<0.0000001:self.use=3
            
    def showMaxAssign(self,num=-1,showDist=True,itnum=0):
        if self.asMax<>None:
            likfrac=self.asMax.getLikFrac()
            asformat=getAsFormat(self.asMax.atoms)
            print 'pPostMAX: %2d%-4d %1d %8.5f%8.5f%8.5f%20s'%\
                  (num,self.identifier,itnum,self.maxPpost,1-self.allNot,likfrac,asformat)#og tilfoej mere:...
            self.asMax.show_assign(showDist)
        else:
            print 'pPostMAX: %2d%-4d %1d %8.5f%8.5f 0.0 ???'%\
                  (num,self.identifier,itnum,self.probNoise,1-self.allNot)
            print 'peak is probably noise'

    def accept(self,c,thr=0.002,errmax=0.5):
        ##self.assigns must be sorted
        if c==1:
            return self.assigns[0][1].acceptAsSingle(self.probNoise,thr)
        else:
            if c<100 and self.sum_pPosts>thr and self.probNoise<=errmax:
##            if c<100 and self.sum_pPosts>0.002:
                for elem in self.assigns:
                    print 'TESTSHFAC:',elem[1].shFac
                    ##if elem[1].shFac>0.03:
                    if elem[1].shFac>0.01:
                        return True
                return False
            return False

    def getASDlst(filename):
        buf=initfil2(filename)
        parser=Parser(buf)
        peakline=parser.search('peak',verb=False)
	asdlst=[]
        while not parser.terminated:
            name=buf[parser.start-1][1]
            as0num=parser.search('peakAssign',verb=False)
            cos=[]
            notes=[]
            for i in range(peakline,as0num-1):
                lin=buf[i]
                if lin[0][1:]=='bounds':bounds=[eval(lin[j]) for j in (1,2)]
                elif lin[0][1:] in ['fromProtonShift','toProtonShift']:
                    cos.append(eval(lin[1]))
                elif lin[0][1:]=='note':
                    notes.append(lin[1:])
                    if lin[1]=='degeneracy':numAcc=eval(lin[2])
                    elif lin[1] in ['Good','Bad']:
                        categ=lin[1]
                        realdist=eval(lin[3])
                    elif lin[1][:5]=='noise':
                        noiseprob=eval(lin[4])
                        totprob=eval(lin[5])
                    elif lin[1][:4] in ['intr','sequ','medi','long']:
                        rType=lin[1]
            ##print name,round(cos[0],2),round(cos[1],2),round(bounds[0],1),numAcc,categ,realdist,noiseprob,totprob,rType
	    asd=_AssignData(name,cos,bounds,numAcc,categ,realdist,noiseprob,totprob,rType)
            asnstart=as0num-1
            while True:
                lin=buf[asnstart]
                atoms=(lin[2][:-4],lin[3][:-2])
                stop=parser.search('end',verb=False)-1
                for i in range(asnstart+1,stop):
                    lin=buf[i]
                    if lin[0][1:]=='likelihood':lik=eval(lin[1])
                    elif lin[0][1:]=='note':
                        if lin[1] in ['Good','Bad']:
                            asncateg=lin[1]
                            asnrealdist=eval(lin[2])
                        elif lin[1][:4] in ['intr','sequ','medi','long']:
                            asnrType=lin[1]
                        elif lin[1][:5]=='shift':
                            shiftprob=eval(lin[2])
                ##print '---',atoms,asncateg,asnrealdist,asnrType,lik
		asn=ReadAssignment(atoms,asncateg,asnrealdist,asnrType,lik,shiftprob)
		asd.assigns.append([lik,asn])
##                print buf[parser.start],parser.start
                if buf[parser.start][0]=='end':break
                asnstart=parser.start
	    asdlst.append(asd)
            peakline=parser.search('peak',verb=False)
	return asdlst

    getASDlst = staticmethod(getASDlst)

    def chooseAssigns(self,fac,thr,reject,maxnum,show,show2,errmax,
                      shtrunc,pnum,shifts,minexp,minmaxvolprob=0.0,assMat=None):
        if reject==0:mingvc=min(1.0/(fac**2),thr)
        else:mingvc=reject
        sumpp=min(self.sum_pPosts+self.probNoise,1.0)
        maxcount=0
        finalassigns=[]
        self.zeroNum=len(self.assigns)
        num=len(self.assigns)
        maxpp=0.0
        for c in range(num):
            if self.assigns[c][1].shFac<shtrunc:
                print 'shfac truncated:',pnum,c,self.assigns[c][1].shFac
                self.assigns[c][0]=0.0
                self.assigns[c][1].pPost=0.0
            else:
                maxpp=max(maxpp,self.assigns[c][1].pPost)
        self.assigns.sort()
        #WARNING: kan ikke bruges igen til at opdatere assignments
        self.assigns.reverse()
        subtract=0
        for c in range(len(self.assigns)):
                if self.assigns[c][0]==0.0:
                    self.zeroNum=c;break
        if len(self.assigns)==1 or maxpp>1-1.0/fac:
            c=0
            print 'unambiguous',self.maxPpost
            self.assigns[0][1].show_assign(False)
            self.entropy=1.0
        else:
            entropy=0.0
            if self.assigns[0][0]/sumpp<=minmaxvolprob:
              c=0;subtract=1
            else:
              print 'probs:',pnum,
              for c in range(len(self.assigns)):
                p=self.assigns[c][0]
                print p,
                if p>0:
                    lp=log(p)
                    entropy=entropy-p*lp
                if p==0 or maxpp/p>fac or p<mingvc:subtract=1;break
            self.entropy=exp(entropy)
        print
        if c>maxcount:maxcount=c
        if c<=maxnum:
          c+=(1-subtract)
          if self.accept(c,thr,errmax):
            finalassigns=self.assigns[:c]
            self.numAccepted=min(len(self.assigns),c)
          else:self.zeroNum=0
        else:
            finalassigns=[]
            self.numAccepted=0
        print self.zeroNum
        if assMat<>None:
          ##for c in range(self.numAccepted):
          for c in range(self.zeroNum):
            asc=self.assigns[c][1]
            ats=asc.atoms
            resids=sorted([eval(at.fragment_id)-1 for at in ats])
            assMat[resids[0],resids[1]]+=1
        if self.zeroNum==0:
            print 'all assignments truncated:',shifts[0],shifts[1],self.probNoise,minexp,pnum
        return finalassigns

    
class _AssignData(AssignData):

    def __init__(self,name,cos,bounds,numAcc,categ,realdist,
                 noiseprob,totprob,rType):
	self.name=name
	self.coords=cos
	self.bounds=bounds
	self.numAccepted=numAcc
	self.category=categ
	self.realdist=realdist
	self.probNoise=noiseprob
	self.totprob=totprob
	self.rType=rType
	self.assigns=[]

    def update(self,pdbs,refpdb,
               dmaxs=[3.0,4.0,5.0,5.5,6.0,6.5],
               rmins=[2.2,3.2,3.6,4.0,4.4,4.7]):
        minexp=dmaxs.index(self.bounds[0])
        rmin=rmins[minexp]
        self.sum_pPosts=0.0
        print self.rType,rmin,'---------------'
        for tup in self.assigns:
            tup[1].update(pdbs,refpdb,rmin)
            tup[0]=tup[1].pPost
            self.sum_pPosts+=tup[0]
        if minexp<=2:
            self.numAccepted=0
            self.zeroNum=0
        else:
            self.numAccepted=len(self.assigns)
            self.zeroNum=self.numAccepted


class Restraints:
    def __init__(self):
        self.rt0,self.rt1,self.rtm,self.rtlr=0,0,0,0

    def update(self,rtype):
        if rtype>=0:
            if rtype==0:   self.rt0=self.rt0+1
            elif rtype==1: self.rt1=self.rt1+1
            elif rtype<=4: self.rtm=self.rtm+1
            else:         self.rtlr=self.rtlr+1

    def showres(self):
        print '%4d%4d%4d%4d%4d'%(self.rt0,self.rt1,self.rtm,self.rtlr,
                              self.rt0+self.rt1+self.rtm+self.rtlr)

def getMeanSd(s):
    if s[2]==0:return None,None
    elif s[2]==1:return s[0],None
    else:
        mean=s[0]/s[2]
        arg=s[1]/s[2]-mean**2
        if arg<0:
            #print arg
            if arg>-0.001:
                #numerisk instabilitet
                return mean,0.0#TJEK! safe?
        sd=math.sqrt(arg)
        return mean,sd

def calcMeanSd(s):
    sx,sx2=0.0,0.0
    for x in s:sx=sx+x;sx2=sx2+x**2
    return getMeanSd((sx,sx2,len(s)))

def getGeneralizedMeanSd(H0list):
    if H0list==[]:return None,None
    else:
        sum=0
        enum=0
        varsum=0
        for x in H0list:
            sum=sum+x[0]*x[1]
            enum=enum+x[1]
        mean0=sum/enum
        if len(H0list)>1:
            for x in H0list:
                varsum=varsum+((x[0]-mean0)**2)*x[1]
            std0=math.sqrt(varsum/enum)#"sharpening"?
            return mean0,std0
        else: return mean0,0.1#TJEK en konstant

def normalP(x,mu,sig,p,norm=0):
    const=1.0
    if norm==1:const=math.sqrt(2.0*pi)*sig
    ##p=0.25
    ##const=sig/0.17
    ##print 1/const
    #TODO gange gauskurve med en normconst...
    return p*exp(-0.5*((x-mu)/sig)**2)/const+1-p
    ##return (p*exp(-0.5*((x-mu)/sig)**2)+1-p)/const

def sumProb(c,wid,mus,ps):
    t=arange(c-wid,c+wid,0.01)
    #y=Numeric.ones(len(t))
    y=normalP(t,0,2,1.0)
    for i in range(len(mus)):
        #y=y*prob(t,ps[i],mus[i],1)
        y=y*normalP(t,mus[i],0.2,ps[i])
    plot(t,y,'k--')

def sumProb1(c,wid,lst,mean,sigma,sigm0):
    t=arange(c-wid,c+wid,0.01)
    #y=Numeric.ones(len(t))
    y0=normalP(t,mean,sigma,1.0)
    y=ones(len(t))
##    y0=normalP(t,mean,sigma,1.0)
##    lny0=log(y0)
##    entvec0=y0*lny0*(-1.0)
##    entrop0=entvec0.sum()
##    L=array(lst)
##    hist(L[:,0],bins=10)
##    show()
##    lk=None+0
    for i in range(len(lst)):
##        y=y*normalP(t,lst[i][0],sigm0,lst[i][1])
        y=y*normalP(t,lst[i][0],lst[i][4],lst[i][1])
    y=y*y0
    ##lny=log(y)
    area=sum(y)*0.01
##    y=y**1.0/6
    y=y/area
##    area=y.sum()*0.01
    print 'AREA', area
    yargmax=argmax(y)
    maxpoint=yargmax*0.01+c-wid
    print 'MAX:',maxpoint,y[yargmax]
    cols='brgykcm'
##    if y[yargmax]<0.3:
    if entrop>-400:#TODO generaliser
        col=cols[randint(0,6)]
        plot(t,y,col)#+'--')
        ##plot(t,-lny,col+'--')
        scatter([c],[y[len(t)/2]],c='b',faceted=False,alpha=0.5)
        scatter([maxpoint],[y[yargmax]],c='r',faceted=False)
    return y,maxpoint,0,y[yargmax]

def sumProbNew(c,pdf,mat,truemean,sigma,col,doscatter=1,showMarkers=False):
    wid=pdf.width
    res=pdf.resolution
    kms='so^>v<dph8'
    cols='bgrcmykkkyy'
    t=arange(c-wid,c+wid+res,res)
    #y=Numeric.ones(len(t))
    if pdf<>None:y0=pdf.prior
    else:y0=normalP(t,c,sigma,1.0)
    if doscatter:
        plot(t,y0,'k--')
    ##y0w=normalP(t,c,sigma*5,1.0)
    Y=ones(len(t))*1.0
    totlen=0
    for j in range(len(mat)):
        lst,fac=mat[j]
        y=ones(len(t))*fac
        for i in range(len(lst)):
            totlen=totlen+1
	    print 'datapoint:',i,lst[i][0],lst[i][4],lst[i][1],int(lst[i][2]),int(lst[i][3])
            y+=normalP(t,lst[i][0],lst[i][4],1.0)*lst[i][1]
            ##if doscatter:plot(t,y,cols[j%7]+'--')
            if doscatter and showMarkers and lst[i][1]>0.05:
		scatter([lst[i][0]],[lst[i][1]],c=cols[int(lst[i][3])],s=100,marker='s',alpha=0.5,faceted=False)
        Y*=y
    ##Y*=(0.9*y0+0.1*y0w)
    Z=numpy.array([x for x in Y])
    area=sum(Z)*0.01
    Z/=area
    Y*=y0
    area=sum(Y)*0.01
    Y/=area
    i0s=(Y==0.0)
    lnY=log(Y)
    lnY[i0s]=0.0
    ##plot(t,lnY,'r--')
    entrop=sum(lnY*Y*-1.0)*res
    print 'entrop',entrop
    print 'final AREA', area
    yargmax=argmax(Y)
    maxpoint=yargmax*res+c-wid
    refpoint=int((truemean-c-wid)/res)
    print 'MAX:',maxpoint,Y[yargmax]
    cols='brgykcm'
##    if y[yargmax]<0.3:
    if doscatter:# and totlen>2:
        ##col=cols[randint(0,6)]
##        if doscatter==1:
##            col=col+'--'
        ##col='k'
        plot(t,Y,col)
        plot(t,Z,'r--')
        ##plot(t,-lny,col+'--')
        scatter([truemean],[Y[refpoint]],s=100,c='b',faceted=False,alpha=0.5,marker=kms[doscatter%10])
        scatter([maxpoint],[Y[yargmax]],s=100,c='r',faceted=False,marker=kms[doscatter%10])
    return Y,maxpoint,entrop,Y[yargmax]


def sumProb2(c,wid,res,mat,mean,sigma,doscatter=1):
    t=arange(c-wid,c+wid+res,res)
    #y=Numeric.ones(len(t))
    y0=normalP(t,c,sigma,1.0)
    Y=zeros(len(t))
    for j in range(len(mat)):
        y=ones(len(t))
        lst,fac=mat[j]
	fac=1.0#TODO: verificer...
        for i in range(len(lst)):
    ##        y=y*normalP(t,lst[i][0],sigm0,lst[i][1])
            y=y*normalP(t,lst[i][0],lst[i][4],lst[i][1])
        ##y=y*y0
        ##lny=log(y)
        area=sum(y)*res
        y=y/area*fac
        ##if doscatter:plot(t,y,'k--')
        print 'AREA', area
        Y=Y+y
    Y=Y*y0
    area=sum(Y)*0.01
    Y=Y/area
    i0s=(Y==0.0)
    lnY=log(Y)
    lnY[i0s]=0.0
    entrop=sum(lnY*Y*-1.0)*res
    print 'entrop',entrop
    print 'final AREA', area
    yargmax=argmax(Y)
    maxpoint=yargmax*res+c-wid
    print 'MAX:',maxpoint,Y[yargmax]
    cols='brgykcm'
##    if y[yargmax]<0.3:
    if doscatter:
        ##col=cols[randint(0,6)]
        plot(t,Y,'k:')
        ##plot(t,-lny,col+'--')
        ##scatter([c],[Y[len(t)/2]],c='b',faceted=False,alpha=0.5)
        ##scatter([maxpoint],[Y[yargmax]],c='r',faceted=False)
    return Y,maxpoint,entrop,Y[yargmax]

##for i in range(100):
##    pdf=sumProb(0,5,[-3,-1,1,1.1],[0.1,0.2,0.3,0.3])
##
##show()

def getGeneralizedMeanSd2(H0list,mean,sigma,sigm0=0.1,showpdf=0,mode=1):
    if H0list==[]:return None,None,None,None,None
    else:
        if mode==1:
            sum=0
            enum=0
            enum2=0
            for x in H0list:
                ##dataprec=1.0/(self.mydamplist[x[2]]**2-0.1**2)
                sum=sum+x[0]*x[1]
                enum=enum+x[1]
                #enum2=enum2+x[1]**2
            datamean=sum/enum
            norm=1.0/(enum/sigm0/sigm0+1.0/sigma/sigma)
            ##norm=1.0/(enum*dataprec+1.0/sigma/sigma)
            #norm2=1.0/(enum2/sigm0/sigm0+1.0/sigma/sigma)
            mean0=(sum/sigm0/sigm0+mean/sigma/sigma)*norm
            ##mean0=(sum/dataprec+mean/sigma/sigma)*norm
            varsum=((mean-mean0)**2)/sigma/sigma
            #varsum2=((mean-mean0)**2)/sigma/sigma
            datavarsum=0.0
            #datavarsum2=0.0
            for x in H0list:
                #varsum2=varsum2+((x[0]-mean0)**2)*x[1]*x[1]/sigm0/sigm0
                #datavarsum2=datavarsum2+((x[0]-datamean)**2)*x[1]*x[1]
                varsum=varsum+((x[0]-mean0)**2)*x[1]
                ##varsum=varsum+((x[0]-mean0)**2)*x[1]*dataprec
                datavarsum=datavarsum+((x[0]-datamean)**2)*x[1]
            #normvarsum=varsum*norm
            #return enum,mean0,normvarsum,datamean,datavarsum #(n,mean_averged,s**2_averaged)
            print 'mean0:',mean0
        if showpdf:
          if mode==1:
          ##pdf,pmean,entrop,hi=sumProb2(mean0,6,H0list,mean,sigma,sigm0)#;show()
            pdf,pmean,entrop,hi=sumProb1(mean,6,H0list,mean,sigma,sigm0)#;show()
            return pdf,pmean,1.0/hi*0.39894,hi,entrop#4.13273=sqrt(s*pi*e)
          elif mode==2:
            pdf,pmean,entrop,hi=sumProb2(mean,6,H0list,mean,sigma,sigm0,doscatter=1)#;show()
            return pdf,pmean,1.0/hi*0.39894,hi,entrop#4.13273=sqrt(s*pi*e)
        return enum,mean0,math.sqrt(norm),datamean,varsum #(n,mean_averged,s**2_averaged)

class ProbabilityDensityFunction:
    mean=-99999999
    sd=-99999999
    list=None

    def __init__(self,sh=None,sd=None,form='normal',wid=6.0,res=0.01):
        '''initializes a (normal) probability density function'''
	if sd==None:self.form='invalid'
	else:
	  print sh,sd,form
          t=arange(sh-wid,sh+wid+res,res)
          if form=='normal':
            self.form='normal'
            y0=normalP(t,sh,sd,1.0,norm=1)
            self.list=y0
            self.mean=sh
	    self.sd=sd
          elif form=='sumnormal':
            self.list=zeros(len(t))*1.0
            self.mean='sumnormal'
            self.mean=sh#???
            self.form='sumnormal'
            self.max=sh#OK?
	    self.states=sd
            for stat in sd:
		print stat
		shn,stdn,prob=stat
		yn=normalP(t,shn,stdn,1.0,norm=1)*prob
		self.list+=yn
	    ##plot(t,self.list);show()
          else:self.list=ones(len(t))*1.0
          self.width=wid
          self.resolution=res
	  self.prior=self.list[:]
          ##self.test()

    def __str__(self):
        return '%8s%6.2f%6.2f%8.4f%8.4f'%\
               (self.form,self.width,self.resolution,self.mean,self.sd)

    def getPosition(self,x):
        return int((x-self.mean+self.width)/self.resolution)

    def getRange(self):
        return linspace(self.mean-self.width,
                        self.mean+self.width+self.resolution,
                        len(self.list),
                        False)
        #TODO: find udaf hvorfor length mismatch kan forekomme
        
    def getValue(self,x):
        i=self.getPosition(x)
        ##print i,
        #overvej interpolation?
        return self.list[i]

    def integral(self,a=None,b=None):
        return sum(self.list)*self.resolution

    def calcIntNorm(self,mu,sd,verb=0):
        print self.form,
        if self.form=='invalid':print 'form==invalid';return None
        if self.form=='normal':
            enum=sd**2+self.sd**2
            dev2=(mu-self.mean)**2
	    earg=dev2/enum
            ##if verb:print self.mean,self.sd,mu,sd,earg
            print self.mean,self.sd,mu,sd,earg,
	    if earg>20.0:return 0.0#OK approx?
            return exp(-0.5*earg)/sqrt(enum)/SQRT_2PI
            #dette gaar ca. 10 gange saa hurtigt som nederst med res=0.1
        elif self.form=='sumnormal':
	  val=0.0
	  for stat in self.states:
	    shn,stdn,prob=stat
            enum=sd**2+stdn**2
            dev2=(mu-shn)**2
	    earg=dev2/enum
            if verb:print shn,stdn,mu,sd,earg,prob
            val+=(prob*exp(-0.5*earg)/sqrt(enum)/SQRT_2PI)
	  return val
        if verb:print self.max,mu,sd
        t=self.getRange()
        y0=normalP(t,mu,sd,1.0,norm=1)
        if verb:print len(y0),len(t),len(self.list)
        prod=y0*self.list
        return sum(prod)*self.resolution

    def getMoment(self,mu,n):
        I=self.getPosition(mu)
        Sum=0.0
        for i in range(len(self.list)):
            Sum+=self.list[i]*(i-I)**n
        return Sum*(self.resolution**(n+1))

    def test(self):
        print self.getValue(self.mean)
        print self.getValue(self.mean-self.sd)
        print self.getValue(self.mean+self.sd)
        print self.getValue(self.mean+2*self.sd)
        print self.integral()
        

class ResAssignment:
    def __init__(self,at):
	self.atom=at
        self.peakAssList=[]
        self.peakAssDBA={}
        self.avercount=0.0
        self.history=[[],[],None,at.fragment_id,at.name]
        self.mean=None
        self.sigma=None
        self.datamean=None
        self.datasigma=None
        self.truemean=None
	self.estVar=None
	self.pdf=None

    def putPDF(self,shift,std=None,form='normal'):
	wdct={'N':16,'C':10,'H':4,'HG':4}
	elem=self.atom.element
	width=wdct[elem]
        self.pdf=ProbabilityDensityFunction(sh=shift,sd=std,form=form,wid=width,res=0.01)
        self.max=self.pdf.mean
        print 'pdf initialized',form,self.max,shift,std
        
    def calcodds(self,shift,mean,sigma):
        delta=shift-mean
        odds0=math.exp(-0.5*(delta/sigma)**2)
        N=self.avercount
        #print self.datamean,shift
        D=(self.datamean-shift)
        S=self.datasigma
        #print N,D,S
        return odds0*(1.0+(D/S)**2)**(-N/2.0)
        #print x,odds

    def LikelihoodSum(self,Z,dampC,mean,sigma):
        #d=mean-x
        z=mean-Z
        sm=mean-self.datamean
        #print sm,z,self.datasigma,self.avercount
        def func(d):
            PriMu=math.exp(-0.5*(d/sigma)**2)/sigma
            Sharp=(1+((sm-d)/self.datasigma)**2)**(self.avercount)
            PosDev=math.exp(-0.5*((z-d)/dampC)**2)
            return PriMu*Sharp*PosDev
        return romb.romberg(func,(-10.0,10.0),accuracy=0.001)

    def normlize_exps(self,mat,normnums=[0]):
	dct={}
	for j in range(len(mat)):
            lst,fac=mat[j]
	    for i in range(len(lst)):
		c,sd,w,num=lst[i][0],lst[i][4],lst[i][1],int(lst[i][3])
		if not num in dct:dct[num]=[fac,[]]
		dct[num][0]+=w
		dct[num][1].append((j,i))
	for num in normnums:
	    if not num in dct:
              if self.atom.element=='H' and not self.atom.name in ['H','HN']:
		print 'WARNING no assigned peak for experiment',self.atom.name,self.atom.fragment_id,num
	    else:
		wtot=dct[num][0]
		print 'total sum for exp %d is %7.3f (not Normalizing)'%(num,wtot)
		##for j,i in dct[num][1]: mat[j][0][i]/=wtot

    def update(self,Show=1,Showpdf=0,normnums=[]):##0]):
        '''updates resonance assignments - edit method...'''
        print 'updating............',self.atom.fragment_id,self.atom.name,len(self.peakAssDBA.keys())
        XjsNew=[]
##        X2=[]
##        for i in range(len(self.peakAssList)):
##            x=self.peakAssList[i]
##            if Show:print x[:6]+x[7:]
##            X2.append(x[:6])
##        X2=array(X2)
        for dkey in self.peakAssDBA.keys():
            fac=self.peakAssDBA[dkey][1];print fac
            #fac pNonObservedPeak
            if True:##fac<0.9:
              expnum=dkey[1]
              X4d=self.peakAssDBA[dkey][0]
              X4=array(X4d)
              ksum=sum(X4[:,1])
	      if expnum in normnums or ksum>1.0-fac:
		print 'normalizing weights',expnum,ksum,dkey[0]
                X4[:,1]/=(ksum/(1-fac))#normalizing
		if len(X4)>1:print 'TJEK long X4',len(X4)
	      else:
		print 'not normalize',expnum,ksum,dkey[0]
              print X4
              print dkey,fac
              XjsNew.append([X4,fac])
        ##print XjsNew
        return self.getpdf(XjsNew,Show,Showpdf)

    def getpdf(self,XjsNew,Show,Showpdf,X2=None,add=0,thisUsed=None,showMarkers=True,pdfcol='k'):
        kms='so^>v<dph8'
        cols='bgrcmykkkyy'
        compdiff=None
        usedMean=self.pdf.mean
        if thisUsed<>None:usedMean=thisUsed
        numdat=0
        truemean=self.atom.shift[0]#the true mean
        sigma=self.atom.shift[1]
        if Show:print truemean,usedMean,sigma
        ##if Showpdf:scatter(X3[:,0],X3[:,1],c=X3[:,3]/8.0,s=20.0/X3[:,4],alpha=0.2,faceted=False)
        X2new=[]
        ##XjsNew2=[]
        for i in range(len(XjsNew)):
            XjsNew[i][1]+=add
            X4,fac=XjsNew[i]
##            maxi=max(X4[:,1])
##            print 'maxi:',maxi
            ##XjsNew2.append((X4,fac+add))
            if X2==None:
                for lst in X4:X2new.append(lst)
            numdat=numdat+len(X4)
            if Showpdf:
		##scatter(X2[:,0],X2[:,1],c=X2[:,3]/8.0,s=20.0/X2[:,4],alpha=0.75,faceted=False,marker=kms[Showpdf%10])
                if showMarkers:
		   ##scatter(X4[:,0],X4[:,1],c=cols[i%7],s=-60.0*log(fac-0.2),alpha=0.75,faceted=False,marker=kms[Showpdf%10])
		   print 'showing markers?'
        print numdat,'datapoints'
        if numdat>0:
	  self.normlize_exps(XjsNew) #already done above
          if X2==None:X2=array(X2new)
##        if len(X2)>0:
##            X2[:,1]=X2[:,5]
##            ##print X2
##            X3,Xjs,num=removeCorrelatedPeaks(X2)
##            oldfunclst,oldpmax,oldentrop,oldmaxhi=sumProb2(usedMean,wid,res,Xjs,truemean,sigma,doscatter=Showpdf)
          funclst,pmax,entrop,maxhi=sumProbNew(usedMean,self.pdf,XjsNew,truemean,sigma,pdfcol,doscatter=Showpdf,showMarkers=showMarkers)
          if funclst<>None:
            self.history[0].append(XjsNew)
            self.history[1].append(pmax)
            self.history[2]=usedMean
            psig=max(1.0/maxhi*0.39894,0.02)
            self.sigma=psig
            self.max=pmax
            self.pdf.max=pmax
            self.entrop=entrop
            self.pdf.list=funclst#overwrite list
            self.pdf.form='general'
            mom2=self.pdf.getMoment(pmax,2)
            #TODO: rykke intervallet i pdf'en...
            compdiff=pmax-truemean
            refdiff=usedMean-truemean
            refhi=self.pdf.getValue(truemean)
            relhi=refhi/maxhi
            self.relhi=relhi
            if len(X2)==0:oldpmax=pmax
##            if Show:print self.atom.fragment_id,self.atom.name,pmax,psig,\
##               entrop,compdiff,refdiff,relhi,mom2,oldpmax-truemean,numdat,self.atom.temp_factor,'aa'
            if Show:print 'updassign',self.atom.fragment_id,self.atom.name,pmax,psig,\
               entrop,compdiff,refdiff,relhi,mom2,numdat##,self.atom.temp_factor
        else:
	    print 'warning: no data points for atom',self.atom.fragment_id,self.atom.name
	    refdiff=None
        self.peakAssList=[]#klar til naeste iteration
        self.peakAssDBA={}#klar til naeste iteration
        return refdiff

def calcldvarsFromMinBound(ld,dmax):
    d=math.exp(ld)
    dmean=0.5*(d+dmax)
    ldmean=log(dmean)
    dgeomean2=d*dmax
    lsig2=ldmean-0.5*log(dgeomean2)
    return ldmean,max(lsig2,0.01)

def calcldvarsFromMinBound2(ld,dmax):
    ldmean=0.5*(ld+log(dmax))
    ldsig=0.4*(log(dmax)-ldmean)
    return ldmean,ldsig**2


class Connectivity:
    def __init__(self,matdim):
        self.d_updmat=numpy.zeros((matdim,matdim))
        self.s2_updmat=numpy.zeros((matdim,matdim))
        self.p_updmat=numpy.zeros((matdim,matdim))
        self.allAtomNHlst=[]

    def update(self,pts,gvc0,ldmean,ldsig2):
        self.p_updmat[pts[0],pts[1]]=gvc0
        self.d_updmat[pts[0],pts[1]]=ldmean
        self.s2_updmat[pts[0],pts[1]]=ldsig2
        self.p_updmat[pts[1],pts[0]]=gvc0
        self.d_updmat[pts[1],pts[0]]=ldmean
        self.s2_updmat[pts[1],pts[0]]=ldsig2


class AtomNeighbourhood:
    def __init__(self,at):
        self.fragid=string.atoi(at.fragment_id)
        self.name=at.name
        self.intra=[]
        self.seqUp=[]
        self.seqDown=[]
        self.other=[]

    def include(self,at,pt):
        fr=string.atoi(at.fragment_id)
        if fr==self.fragid:
            self.intra.append(pt)
        elif fr-self.fragid==-1:
            self.seqDown.append(pt)
        elif fr-self.fragid==1:
            self.seqUp.append(pt)
        else:self.other.append(pt)



def calcDist(at1,at2):
    return at1.calcDist(at2)


class _Atom(tuple):#asuming one chain!

    def __new__(cls,atom):
        atbase=atom.name[:2]
        names={'C':0,'N':0,'CA':1,'CB':2,'CG':3,'CD':4,'CE':5,'CZ':6,'CH':7}
        pt=99
        if atbase in names:pt=names[atbase]
	elif atom.name in ['H','HN']:pt=0
	else:
	  atn=atom.name
	  atn=atn.strip('H')
	  for dig in '123':atn=atn.strip(dig)
	  if atn=='':pt=7
	  elif atn in 'ABGDEZ':pt='ABGDEZ'.index(atn)+1
	  ##print 'using pointer for atom',pt,atom,atbase,atn
	elem=getelem(atom.name)
        return tuple.__new__(cls,(atom.parent.id[1]-1,elem,pt,atom.name))

    def __init__(self,atom):
        tuple.__init__(self)
	self.element=getelem(atom.name)
        self.fragment_id=str(atom.parent.id[1])
        self.name=atom.name
        self.res_name=atom.parent.resname
        self.pt=self.__getitem__(2)
	##print dir(atom);1/0
        self.position=atom.coord
        self.shift=atom.shift
        self.temp_factor=atom.bfactor
	try:
	    self.pointer =atom.pointer
	    self.pointer2=atom.pointer2
	except AttributeError:pass
        ##self.resAs=atom.resAs

    def calcDist(self,other):
        U=self.position
        V=other.position
        return math.sqrt(dot(U-V,U-V))

    def getAt(self,pdb):
        return pdb[0][self.fragment_id][self.name]

    def getPASDid(self):
        return self.res_name+self.fragment_id+self.name

    def isMultipleLongRange(atlst):
        lrcount=0
        for i in range(len(atlst)-1):
##            print atlst[i].__class__
##            print atlst[i]
            assert isinstance(atlst[i],_Atom) or isinstance(atlst[i],_Atom2) or isinstance(atlst[i],struc.Atom)
            rType=getType(atlst[i:i+2])
            if rType>4:lrcount+=1
            if lrcount>1:return True
        return False

    isMultipleLongRange = staticmethod(isMultipleLongRange)

    def getstd(at):
        B=at.temp_factor
        return 0.2+0.02*(min(40,max(5.0,B))-5.0)
##        std=0.2
##        if at.element=='N':std+=0.1
##        if at.name in ['C','CA','N']:std+=0.1
##        if at.fragment_id in ['1','2','6','7','12','70']:std+=0.1

    getstd = staticmethod(getstd)

    def getRefAt(resn,resi):
        if resn=='G':refat=resi['CA']
        elif resn=='A':refat=resi['CB']
        elif resn=='C':
            try:refat=resi['SG']
            except KeyError:
                print 'WARNING: no SG atom in CYS'
                refat=resi['CB']
        elif resn=='D':
            try:refat=resi['OD2']
            except KeyError:
                print 'WARNING: no OD2 atom in GLU'
                refat=resi['CG']
        elif resn=='E':
            try:refat=resi['OE2']
            except KeyError:
                print 'WARNING: no OE2 atom in ASP'
                refat=resi['CD']
        elif resn=='F':refat=resi['CZ']
        elif resn=='H':refat=resi['CE1']
        elif resn=='I':refat=resi['CD1']
        elif resn=='K':
            try:refat=resi['NZ']
            except KeyError:
                print 'WARNING: no NZ atom in LYS'
                refat=resi['CE']
        elif resn=='L':refat=resi['CD1']
        elif resn=='M':refat=resi['CE']
        elif resn=='N':
            try:refat=resi['ND2']
            except KeyError:
                print 'WARNING: no ND2 atom in ASN'
                refat=resi['CG']
        elif resn=='P':refat=resi['CD']
        elif resn=='Q':
            try:refat=resi['NE2']
            except KeyError:
                print 'WARNING: no NE2 atom in GLN'
                refat=resi['CD']
        elif resn=='R':refat=resi['CZ']
        elif resn=='S':
            try:refat=resi['OG']
            except KeyError:
                print 'WARNING: no OG atom in SER'
                refat=resi['CB']
        elif resn=='T':
            try:refat=resi['OG1']
            except KeyError:
                print 'WARNING: no OG1 atom in THR'
                refat=resi['CG2']
        elif resn=='V':refat=resi['CG1']
        elif resn=='W':refat=resi['CH2']
        elif resn=='Y':
            try:refat=resi['OH']
            except KeyError:
                print 'WARNING: no OH atom in TYR'
                refat=resi['CE2']
        else:refat=resi['CA']
        return _Atom(refat)

    getRefAt = staticmethod(getRefAt)

class _Atom2(_Atom):
    def __new__(cls,resn,fri,name,elem,pos=None,shift=None):
        atbase=name[:2]
        names={'C':0,'N':0,'CA':1,'CB':2,'CG':3,'CD':4,'CE':5,'CZ':6}
        if atbase in names:pt=names[atbase]
        else:pt=99
        return tuple.__new__(cls,(int(fri)-1,elem,pt,name))

    def __init__(self,resn,fri,name,elem,pos=None,shift=None):
        tuple.__init__(self)
        self.fragment_id=fri
        if len(elem)>1:self.element=elem[0]#can be HG! 
        else:self.element=elem
        self.name=name
        self.res_name=resn
        self.pt=self.__getitem__(2)
        self.position=pos
        self.shift=shift
        
class _Assignment(list):

    def __init__(self,*args):
        list.__init__(self,*args)
        ##self.sort()


def splitAtom(s):
    resn=s[0]
    for i in range(3,0,-1):
        fri=s[1:1+i]
        if fri.isdigit():break
    name=s[1+i:]
    return resn,fri,name

def splitAtom2(s):
    resn=s[0:3]
    for i in range(3,len(s)):
        fri=s[3:i+1]
        if not fri.isdigit():break
    fri=s[3:i]
    name=s[i:]
    return resn,fri,name

class _Assignment2(_Assignment):

    def __init__(self,s):
        ats=s.split('-')
        atlst=[]
        resn0,fri0,name0=splitAtom(ats[0])
        atlst.append(_Atom2(resn0,fri0,name0,name0[0]))
	resn_prev=resn0;fri_prev=fri0
        for j in range(1,len(ats)):
            if len(ats[j])<=2:
                namej=ats[j]
                resnj=resn_prev;frij=fri_prev
            elif len(ats[j])==3:
                if ats[j][1].isdigit():#as in M1C
                    resnj,frij,namej=splitAtom(ats[j])
		    resn_prev=resnj;fri_prev=frij
                else:
                    namej=ats[j]
                    resnj=resn_prev;frij=fri_prev
            else:
                resnj,frij,namej=splitAtom(ats[j])
		resn_prev=resnj;fri_prev=frij
            atlst.append(_Atom2(resnj,frij,namej,namej[0]))
            
        list.__init__(self,tuple(atlst))
        ##self.sort()


class LRProb:

    def __init__(self,val):
        self.prob=val

    def __str__(self):
        return '%9.6f'%self.prob

    def __lt__(self,other):
        return self.prob<other.prob
        
    def __gt__(self,other):
        return self.prob>other.prob
        
    def __eq__(self,other):
        return self.prob==other.prob

    def __iadd__(self,val):
        return LRProb(self.prob+val)
        
    def __idiv__(self,val):
        return LRProb(self.prob/val)


class ContactProbabilityMap:

    def __init__(self,A,mrdctmat,intradct,num,name,seq):
	##self.LRmap=[list(v) for v in A]
        self.LRmap=[]
        for i in range(len(A)):
            self.LRmap.append([])
            for j in range(len(A[i])):
                ##print i,j
                if 0<abs(j-i)<=4 and name<>'simple':
                    psum=0.0;nsum=0.0
                    mrdct=mrdctmat[min(i,j)][abs(i-j)-1]
                    for k in mrdct:
                        psum+=mrdct[k][0]
                        nsum+=1
                    self.LRmap[i].append(LRProb(psum/nsum))
                else:
                    self.LRmap[i].append(LRProb(exp(A[i,j])))
	self.MRmap=mrdctmat
	self.intraDict=intradct
	self.num=num
	self.name=name
	self.sequence=seq

    def setLRprob(self,pLR):
        for i in range(len(self.LRmap)):
	    for j in range(len(self.LRmap[i])):
                self.LRmap[i][j]=LRProb(pLR)

    def update(self,L,num):
	print 'updating contact probabilities with derived likelihoods...'
        print num,self
        const=CalcVolfacCum(log(4.5),0.15**2,self.rmin)/0.5
        #TODO: generaliser udtryk "(4.5)", "/0.5"...
	for i in range(len(self.LRmap)):
	    for j in range(len(self.LRmap[i])):
                I=min(i,j);J=max(i,j)
		print I,J,[self.LRmap[i][j]],self.LRmap[i][j].prob,L[I][J]*const
##		self.LRmap[i][j].prob=min(0.5,self.LRmap[i][j].prob*L[I][J]/(4-num))
		self.LRmap[i][j].prob=min(0.5,self.LRmap[i][j].prob*L[I][J]*const)
		##TODO: find bedre maade at skalere kortere mixingtider!...
		#prior * likelihood

    def update_from_other(self,other):
        const=CalcVolfacCum(log(4.5),0.15**2,self.rmin)/0.5
        #TODO: generaliser udtryk "(4.5)", "/0.5"...
	print 'updating contact probabilities with other cp'
        print self,other,const
	for i in range(len(self.LRmap)):
	    for j in range(len(self.LRmap[i])):
		self.LRmap[i][j].prob=other.LRmap[i][j].prob*const

    def deconvLR(self):
        lst=[]
        for i in range(len(self.LRmap)):
            lst.append([])
            for j in range(len(self.LRmap[i])):
                lst[i].append(self.LRmap[i][j].prob)
        return numpy.array(lst)

    def show(self,pdb,lim=0.0001,doimshow=True):
        ldlst=[]
        dlst=[]
        nuclst=[]
        A=numpy.zeros((70,70))
	dlst0=[]
	dlst1=[]
	ilst=[]
        for i in range(70):
            A[i][i]=0.5
        for i in range(70):
            for j in range(70):
                dmin=10
                for ati in pdb[0][i]:
                  if ati.element=='C':
                    for atj in pdb[0][j]:
                      if atj.element=='C':
                        d=AtomMath.calc_distance(ati,atj)
                        if d<dmin: dmin=d
##                        if abs(j-i)>=4:# and ati.name=='CA' and atj.name=='CA':print i,j,d
##                            if d<4.5:print i+1,j+1,abs(j-i),j+i,ati.name,atj.name,round(d,2)
                if dmin>8:ilum=0.0
                else:
		    ilum=(8.0-max(dmin,3.0))/5.0
		    dlst0.append(i+0.5)
		    dlst1.append(j+0.5)
		    dlst0.append(j+0.5)
		    dlst1.append(i+0.5)
		    ilst.append(ilum**2)
		    ilst.append(ilum**2)
                A[i][j]=ilum
                A[j][i]=ilum
        A=numpy.rot90(A)
##	B=numpy.array(self.LRmap)
	B=self.deconvLR()
	Bl=self.likRat
	Bl*=B
        Bl=numpy.rot90(Bl)
	Z=B<lim;B[Z]=lim
        B=numpy.rot90(B)
	##print [list(lst) for lst in B]
        if doimshow:
	    pass
	    ##subplot(121)
	    ##scatter(dlst0,dlst1,numpy.array(ilst)*100,'w',alpha=0.5,faceted=False)
            ##imshow(log(B),interpolation='nearest',cmap=pylab.cm.jet,vmax=-2.0)
	    ##pylab.colorbar()
	    ##pylab.title('prior')
	    ##subplot(122)
	    scatter(dlst0,dlst1,numpy.array(ilst)*100,'w',alpha=0.5,faceted=False)
            imshow(log(Bl),interpolation='nearest',cmap=pylab.cm.jet,vmax=log(120),vmin=-log(120))
	    pylab.colorbar()
	    pylab.title('likelihood product')
            show()

    def prob(self,*args,**kwargs):
	print args,
	atn0,atn1='CA','CA'
	if not (isinstance(args[0],int) or isinstance(args[0],str)) and len(args)==2:
	    at0=args[0];at1=args[1]
	    r0=int(at0.fragment_id)-1
	    r1=int(at1.fragment_id)-1
	    atn0=at0.name
	    atn1=at1.name
	elif len(args)==2:
	    r0,r1=args
	elif len(args)==3:
	    r0,atn0,atn1=args
	    r1=r0
	else:r0,r1,atn0,atn1=args
	if r0==r1 and atn0==atn1:
            ##print 'WARNING diagonal peak (NOTE: now returning 0.0 prob!)',r0,atn0
            ##return 1.0#diagonal peak
            return 0.0#diagonal peak
	##lst=[(r0,atn0),(r1,atn1)]
	lst=[(r0,at0),(r1,at1)]
	p0,p1=sorted(lst)
	print p0,p1
	if isinstance(r0,int) and abs(r1-r0)>4:
##	    return exp(self.LRmap[r0][r1])
	    return self.LRmap[r0][r1]
            #note LRProb class is returned!
	elif r0==r1:
	    ##if self.name=='simple':return LRProb(1.0)
	    if self.name=='simple':return Prob(1.0)
	    if isinstance(r0,str):rnam=r0
	    else:rnam=self.sequence[r0]
	    if 'pairdct' in kwargs:
		pdct=kwargs['pairdct']
		if len(pdct)>0:
		  bounds=kwargs['bounds']
		  ##print 'PairdctKeys',pdct.keys(),rnam,atn0,atn1
		  dist=pdct[rnam][atn0][atn1]
		  ##print 'using pairdct',dist,bounds
		  if dist>bounds[0] or dist<bounds[1]:
		    ##print 'NOTE: outside bounds, return 0.0',atn0,atn1,dist,bounds
		    return Prob(0.0)
	    try:
##                print len(self.intraDict[(rnam,atn0,atn1)])
##                print self.intraDict[(rnam,atn0,atn1)]
##                print self.num
                return self.intraDict[(rnam,atn0,atn1)][self.num+3]
	    except KeyError: return self.intraDict[(rnam,atn1,atn0)][self.num+3]
	else:
	    #medium range
	    print min(r0,r1),abs(r1-r0)-1
	    if self.name=='simple':
		if abs(r1-r0)==1:#seq
		  if p1[1].name=='HN':
		    print 'preceding atom to amide',p0,p1
		    return Prob(exp(-max(0,p0[1][2]-1)/3.0))
		  else:return Prob(self.MRmap.pseq)
		else:return Prob({2:0.3,3:0.15,4:0.2}[abs(r1-r0)])
	    print self.MRmap[min(r0,r1)][abs(r1-r0)-1].keys()
	    return self.MRmap[min(r0,r1)][abs(r1-r0)-1][(p0[1].name,p1[1].name)][0]


class ContactProbabilityMap2(ContactProbabilityMap):

    def __init__(self,rmin,seq,pdb):
        self.LRmap=[]
        self.rmin=rmin
        self.voldct={}
        self.sequence=seq
        for i in range(len(seq)):
            self.LRmap.append([])
            for j in range(len(seq)):
                self.LRmap[i].append(LRProb(0.0))
        for i in range(len(pdb[0])):
            for j in range(i,len(pdb[0])):
                self.voldct[(i,j)]={}
                for ati in pdb[0][i]:
                    if ati.element=='C':
                        atni=ati.name
                        for atj in pdb[0][j]:
                            atnj=atj.name
                            if atj.element=='C':
                                self.voldct[(i,j)][(atni,atnj)]=0.0
        print self.voldct[(0,1)]
##        for k in self.voldct:
##            for k2 in self.voldct[k]:
##                print k,k2,self.voldct[k][k2]

    def updateWithStructure(conpris,pdb,rmins,cmat,minapdct,pdbkey,dcut=7.0):
	pen=0
	sens=0
	falsepos=0
	missing=0
	senslst=[];misslst=[]
        for i in range(len(pdb[0])):
            cai=pdb[0][i]['CA']
            for j in range(i,len(pdb[0])):
                caj=pdb[0][j]['CA']
                count=0
                vsum=0.0
                cacadist=AtomMath.calc_distance(cai,caj)
		if (0<=i<7 or 11<=i<70) and (0<=j<7 or 11<=j<70) and j-i>5:
		  if (i,j) in minapdct:
		    minap=minapdct[(i,j)]
		    atmi=pdb[0][i][minap[0]]
		    atmj=pdb[0][j][minap[1]]
		    calcd=AtomMath.calc_distance(atmi,atmj)
		    refd=cmat[i,j]
		    #refd<dcut
		    if calcd<dcut-1 and refd<dcut:  sens+=1;senslst.append((i,j))
		    elif calcd>=dcut and refd<dcut-1:pen+=1;missing+=1;misslst.append((i,j))
		  else:
                    #refd>=dcut
		    calcd=AtomMath.calc_distance(cai,caj)
		    ##print i,j,calcd
		    if calcd<dcut-1:pen+=1;falsepos+=1
		    #else true false
		##mindist=cacadist
                if False:##cacadist<15:
                    for ati in pdb[0][i]:
                        atni=ati.name
                        if ati.element=='C':
                            for atj in pdb[0][j]:
                                atnj=atj.name
                                if atj.element=='C':
                                    dist=AtomMath.calc_distance(ati,atj)
				    ##mindist=min(dist,mindist)
                                    count+=1
                                    ld=log(dist)
                                    for rnum,r0 in enumerate(rmins):
                                        vol=CalcVolfacCum(ld,0.0,rmin=r0)
                                        ##print i,j,atni,atnj
                                        conpris[rnum].voldct[(i,j)][(atni,atnj)]+=vol
                                        vsum+=vol
    ##                                    conpris[rnum].LRmap[i][j]+=vol
                    conpris[rnum].LRmap[i][j]+=(vsum/count)
                    conpris[rnum].LRmap[j][i]+=(vsum/count)
	print 'penalty is:',pen
	print 'comparison:',pen-sens,pen,missing,falsepos,sens,pdbkey[0],pdbkey[1]
##	print senslst
##	print misslst
	return pen

    updateWithStructure = staticmethod(updateWithStructure)

    def scale(self,val):
        for i in range(len(self.LRmap)):
            for j in range(i,len(self.LRmap)):
                self.LRmap[i][j]/=val
                self.LRmap[j][i]/=val
                for ap in self.voldct[(i,j)]:
                    self.voldct[(i,j)][ap]/=val
                

class Dummy:

    def getRealDist(self,ats):
	if ats[0].name=='COM':
	    return ats[0].position 
        return AtomMath.calc_distance(ats[-2],ats[-1])


class Triplet(dict):

    def __init__(self,i,chain):
        dict.__init__(self)
	self.ires=i
	names={}
	self.resnames=[]
	for k in range(max(0,i-1),min(len(chain)-1,i+1)+1):
	    ##names[chain[k].res_name]={}
	    ##res=names[chain[k].res_name]
	    names[k]={}
	    res=names[k]
	    self.resnames.append(chain[k].res_name)
	    for at in chain[k].iter_atoms():
		elem=at.element
		if elem in 'CN':#'CNH'
		    if not elem in res:
			res[elem]=[]
		    res[elem].append(at.name)
	self.names=names

    def __iadd__(self,other):
	if abs(self.ires-other.ires)>2:return self
	#was updated in : "at.resAs.triplet[(expnum,i,k)]=as0"
	for k in other:
	    ask=other[k]
	    ats=ask.atoms
	    for n,at in enumerate(ats):
		if eval(at.fragment_id)-1 in self.names:
		    self[k]=ask
		    break
	return self

    def showtrip(self):
	print 'showing triplet:',self.ires
	for k in self:
	    ask=self[k]
	    ats=ask.atoms
	    ask.atoms=       [_Atom(at) for at in ats]
	    if len(set(ask.atoms))<len(ask.atoms):
		##print 'diagonal:',ask.atoms
		ask.pPost=0.0
            ask.orderedAtoms=[_Atom(at) for at in ask.orderedAtoms]
	    atstr=getAsFormat(ats)
	    coords=ask.coords+ask.shiftDiffSigned
	    ##print k,atstr,coords,ask.GiFi,ask.pPost,ask.likRat

    def writetrip(self,explst=None,resshift=0):
	##savepath='/opt/jakob/ubiquitin/tripletData/'
	savepath=''##/opt/jakob/ubiquitin/tripletData/'
	outfile=open(savepath+'trip'+str(self.ires+1)+'.txt','w')
	print 'writing triplet:',self.ires
	resks=self.names.keys()
	print resks
	if True:
	    for j in range(3):
	      for elem in 'NC':
	        namlst=self.names[resks[j]][elem]
		if 'C' in namlst:
		  namlst.remove('C')
		  namlst.append('C')
		for nam in namlst:
		  if nam in 'CN':
		    outfile.write('%4s   '%nam)
		  else:
		    outfile.write('%4s  '%nam)
	      ##outfile.write('| ')
	    outfile.write('pTot pSpace pShift exp peakn assn true\n')
	items=self.items()
	items.sort(lambda x, y: cmp(y[1].pPost,x[1].pPost))
	for item in items:
	  key,ask=item
	  if ask.pPost>=10.0**-6:
	    ats=ask.atoms
	    atns=[at.name for at in ats]
	    coords=ask.coords+ask.shiftDiffSigned
	    for j in range(3):#j=resnum_diff
	      for elem in 'NC':
	        namlst=self.names[resks[j]][elem]
		if 'C' in namlst:
		  namlst.remove('C')
		  namlst.append('C')
		for nam in namlst:
		  found=False
		  shdiff=0.0
		  for n,at in enumerate(ats):
		    if at.name==nam and eval(at.fragment_id)==self.ires+j+resshift:
		    #generaliseres for andre nummereringer
			found=True
			##if coords[n]>=100:
			if nam in 'CN':
			  outfile.write('%6.2f '%coords[n])
			else:
			  outfile.write('%5.2f '%coords[n])
			break
		  if not found:
			if nam in 'CN':
			  ##outfile.write('   .   ')
			  outfile.write('-999   ')
			else:
			  outfile.write('-999  ')
	      ##outfile.write('| ')
	    expnum,peaknum,assnum=key
            likFrac,Gpa=ask.getLikFrac(ret=2)
	    if explst<>None:
	      peak=explst[expnum].peakData.peaklist[peaknum]
	      trueAs=ask.isTrueAssignment(peak)
	    else:trueAs=0
	    ##print trueAs
	    outfile.write('%8.6f %7.5f %7.5f %1d %4d %3d %1d\n'%\
  		         (ask.pPost,ask.volFac,ask.shFac,expnum,peaknum,assnum,trueAs))
  	    ##outfile.write('%8.6f %8.6f %8.6f %8.6f %1d\n'%\
	   ## 	         (ask.pPost,ask.volFac,ask.shFac,likFrac,trueAs))
        outfile.close()


class SimpleMediumRangeProbs(list):

    def __init__(self,pintra=1.0,pseq=0.5,pmed=0.2,adv=True):
	list.__init__(self,(pintra,pseq,pmed))
	self.pintra=pintra
	self.pseq=pseq
	self.pmed=pmed
	self.adv=adv


class PseudoPDB(dict):

    def __init__(self,name):
	seq=initfil(name)[0][:-1]
	print seq
	data=[]
        dict.__init__(self)
	##self[0]={'A':data}
	self[0]=[data]
	for i,aa1 in enumerate(seq):
	  aa3=aa13dict[aa1]
	  pdbi=Bio.PDB.PDBParser().get_structure('res'+str(i),'aalib/'+string.lower(aa3)+'.pdb')
	  for resi in pdbi[0]['A']:pass
	  resi.id=('',i,'')
	  resi.fragment_id=str(i)
	  print resi,resi['N']
	  data.append(resi)
	

class Base:
    conmat=None
    
    def __init__(self,im,xnucdba,get=None,conpri=True):
      self.inpmap=im
      ##self.createStructureEnsemble()
      ##self.pdb=fileio.LoadStructure(file=im['structure'])
      if 'structure' in im:self.pdb= Bio.PDB.PDBParser().get_structure('pdb0',im['structure'])
      elif 'sequence' in im:self.pdb=PseudoPDB(im['sequence'])
      else:self.pdb=None
      if 'shiftfile' in im:
        self.setShiftData(im)
        self.putShiftsDBA(im)##,eval(im['fromdba']),True)
      if get=='restart':self.updateAllResAss()
      elif get<>None:self.showPDFs(get)

    def setShiftData(self,im):
      bmrbufi=initfil2(im['shiftfile'])
      self.molShifts={0:{}}
      if not 'shiftformat' in im or im['shiftformat']=='nmrstar':
	parser=Parser(bmrbufi)
	moldba=parser.findShiftData('A',verb=True)##,useallshifts=True)
      elif im['shiftformat']=='pine' or im['shiftformat']=='PINE':
	moldba=getpineShiftData(bmrbufi)
      elif im['shiftformat']=='sparky' or im['shiftformat']=='SPARKY':
	moldba=getShiftDBAsparky(bmrbufi)
      for res in moldba:
	if not res in self.molShifts[0]:self.molShifts[0][res]={}
	for at in moldba[res]:
	  dat=moldba[res][at]
	  ##self.molShifts[0][res][at]=(dat[0],dat[1])
	  self.molShifts[0][res][at]=(dat[0],dat[1],dat[2:])
      print self.molShifts
	
    def setSecStruct(self,im):
      if 'dsspname' in im:
	dsspname=im['dsspname']
	'''sets probabilistic dssp data
	   currently only APSSP2 format implemented'''
	print 'reading secondary structure (predictions) from',dsspname
	ssbuf=initfil2(dsspname)
	chain=self.pdb[0]#generalize!
	numres=len(chain)
	chain.numres=numres
	##chain.maxresid=max(self.allids)
	chain.maxresid=numres
	ssdict,pssdict=get_probdssp_dict_APSSP2(ssbuf,numres)
	pdsspData=DSSPData_probabilistic(ssdict,pssdict,chain)
	pdsspData.show()
	self.pdsspData=pdsspData
      else:
	self.pdsspData=None

    def getUpdatedAssignments(self):
      if not 'asdlst' in self.inpmap or\
         ('asdlst' in self.inpmap and self.inpmap['asdlst']=='get'):
          asdlst=AssignData.getASDlst(self.inpmap['resfile'])
          for asd in asdlst:
              asd.update(self.converged,self.pdb)
          save_dba2(asdlst,'asdlst.pck')
      else:
          print 'reading assignment data from: '+self.inpmap['asdlst']
          asdlst=read_dba(self.inpmap['asdlst'])
      fac=10000000;thr=0.00000000001;reject=thr;maxnum=50;show=1;show2=0;
      errmax=0.5;shtrunc=0.035;minmaxvolprob=0.0000000001
      for fac in [10000000]:##[10,50,100,500,1000,5000,10000,50000,100000]:
##      for minmaxvolprob in [0.001,0.0005,0.0001,0.00005,0.00001,0.000005,
##                            0.000001,0.0000005,0.0000001]:
          newasdlst=[]
          newAllassigns=[]
          for i,asd in enumerate(asdlst):
              newassigns=asd.chooseAssigns(fac,thr,reject,maxnum,show,show2,errmax,
                                           shtrunc,i,asd.coords,asd.bounds,minmaxvolprob)
##              newassigns=asd.assigns#for reference only!
              newAllassigns.append(newassigns)
              if asd.zeroNum>0:
                  newasdlst.append(asd)
                  print 'appended assignment data',i
          self.AllNotMat=numpy.ones((70,70))
          self.AllNotMatw=numpy.ones((len(self.pdb[0]),len(self.pdb[0])))
          B,D=self.visualize(newAllassigns,True)
          self.writeUpdatedAssignments(newasdlst)
    
    def writeUpdatedAssignments(self,asdlst):
        pasdfile=open('ubq_darr2d_ira_2it.peaks','w')
        pseudoshiftfile=open('sameshifts.shiftAssignments','w')
        restfile=open('ubq2itrestraints.txt','w')
        xplorfile=open('ubq_xplor.dat','w')
        resco=RestraintCounter()
        for i,asd in enumerate(asdlst):
            pspeak=PseudoPeak(asd)
            pspeak.writeRestraints(restfile,skipfile='none',name='DipCor'+str(i),
                                   pasdresfile=pasdfile,hasinfo=True,
                                   dmax=asd.bounds[0],resc=resco,xplorfile=xplorfile)
        pasdfile.close()
        restfile.close()
        pseudoshiftfile.close()
        resco.show(0.0)
        raise SystemExit,'done...'
    
    def createStructureEnsemble(self):
      self.converged={}
      if not 'asdlst' in self.inpmap:
        if 'ens' in self.inpmap:
          self.pdbdct={}
          num=eval(self.inpmap['numpdb'])
          pref=self.inpmap['ens']
          if 'converged' in self.inpmap:
              cbuf=initfil2(self.inpmap['converged'])
              for i in range(num):
                  lin=cbuf[i]
                  seed=lin[0];thisnum=lin[1]
                  endings=seed+'_'+thisnum+'.pdb'
                  name=pref+endings
                  try:
                      testfile=open(name)
                      print 'opened',name
                  except IOError:
                      print 'WARNING: failed opening',name
                      pref0=self.inpmap['grep']
                      namei=pref0+endings
                      os.system('grep ATOM '+namei+' | grep C>cpdb'+endings)
                  pdbi=fileio.LoadStructure(file=name)
                  self.converged[(seed,eval(thisnum))]=pdbi
	  else:
              seednums=self.inpmap['seednums']
              if len(seednums)==0:
                  seednums=[''];xcar=''
              else:xcar='_'
              ##else:xcar=''
              print 'loading',num,'structures:',pref+'*'
              for seed in seednums:
                for i in range(num):
                  name=pref+seed+xcar+str(i)+'.pdb'
		  try:
                      pdbi=fileio.LoadStructure(file=name)
                      self.pdbdct[(seed,i)]=pdbi
                      print 'opened',name
		  except IOError:
                      print 'WARNING: failed opening',name
              self.getConvergedStructures()

    def getConvergedStructures(self):
      asdlst=AssignData.getASDlst(self.inpmap['resfile'])
      results=self.getViolations(asdlst)
      for i in range(2,10):
            if 'resfile'+str(i) in self.inpmap:
                asdlsti=AssignData.getASDlst(self.inpmap['resfile'+str(i)])
                resultsi=self.getViolations(asdlsti)
                for j in range(len(results)):
                    results[j]+=resultsi[j][1:]
                    resj=results[j]
##                    results[j]+=[resj[3]+resj[5],resj[4]+resj[6]]
                    results[j]+=[resj[3]+resj[6],resj[4]+resj[7],resj[5]+resj[10]]
            else:break
      self.analyzeViolations(results)

    def analyzeViolations(self,results):
      ##print results
##      asort2(results,[9,10])
##      asort2(results,[9,1,10])
##      asort2(results,[3,1])
      ##print results
      results.sort()
      num=eval(self.inpmap['numpdb'])
      seednums=self.inpmap['seednums']
      self.converged={}
      print 'violation analysis:'
##      for i in range(max(int(len(results)*0.1),2)):
      for i in range(len(results)):
        pdbkey=results[i][0]
	seednum,n=pdbkey
	tup=tuple([seednum,n]+results[i][1:])
	print 'ANAL: %3s %3d %3d %6.3f %3d %6.3f %6.3f %3d %6.3f %3d %6.3f %6.3f %3d %6.3f %6.3f'%tup
	self.converged[pdbkey]=self.pdbdct[pdbkey]
      del self.pdbdct
              
    def getViolations(self,asdlst):
      dmaxs=[3.0,4.0,5.0,5.5,6.0,6.5]
      minviolsum=999999
      results=[]
      for pdbkey in self.pdbdct.keys():
          pdb=self.pdbdct[pdbkey]
          numviols=0
          violsum=0.0
          softviolsum=0.0
          lrnumviols=0
          lrviolsum=0.0
          for asd in asdlst:
              hviol=99999.0
              dmax=asd.bounds[0]
              for n in range(asd.numAccepted):
                  asn=asd.assigns[n][1]
                  violn=asn.getViolation(pdb,dmax+1.0)
                  print violn,
                  if violn<hviol:hviol=violn
              print hviol
              softviol=getsoftviol(math.sqrt(hviol),rSwitch=1.0,asympSlope=0.2)
              softviolsum+=softviol
              if 99999>hviol>0:
                  numviols+=1
                  violsum+=hviol
		  if asd.rType=='long':
                      lrnumviols+=1
                      lrviolsum+=hviol
	  violsum/=len(asdlst)
	  lrviolsum/=len(asdlst)
##	  softviolsum/=len(asdlst)
	  if violsum<minviolsum:
		minviolsum=violsum
		minpdb=pdb
	  results.append([pdbkey,numviols,violsum,lrnumviols,lrviolsum,softviolsum])
      ##print 'smallest sum of violations found in:',minpdb,minviolsum
      return results

    def getViolations0(self,exp0):
      dmaxs=[3.0,4.0,5.0,5.5,6.0]
      for i,pdb in enumerate(self.pdbdct):
          numviols=0
          violsum=0.0
          for peak in exp0.peakData.peaklist:
              hviol=99999.0
              dmax=dmaxs[peak.minexp]
              asd=peak.assignData
              for n in range(asd.numAccepted):
                  asn=asd.assigns[n][1]
                  violn=asn.getViolation(pdb,dmax+1.0)
                  if violn<hviol:hviol=violn
              if 99999>hviol>0:
                  numviols+=1
                  violsum+=hviol
          print 'VIOLSTAT: %3d%4d%7.3f'%(i,numviols,violsum/len(exp0.peakData.peaklist))

    def writeHbondRestraints(self):
      hblst=testUbiqConProb(None)
      print hblst
      pasdfile=open('testhb.restraint','w')
      pseudoshiftfile=open('pseudoshifts.shiftAssignments','w')
      restfile=open('junk.txt','w')
      xplorfile=open('testhb_xplor.tbl','w')
      resco=RestraintCounter()
      for i in range(1,70):
          for name in ['HN','O']:
	      print i,name
	      if self.pdb[0][i].res_name<>'PRO':
                  self.writePseudoShifts(_Atom(self.pdb[0][i][name]),pseudoshiftfile)
      print 'pseudo shifts written'
      for sec in hblst:
          for i,elem in enumerate(sec):
              resnum,firsteprob,respartners=elem
              res=self.pdb[0][resnum]
##              if i==0:
##                  nextnum=sec[-1][0]
##              elif i==len(sec)-1:
              if i==len(sec)-1:
                  nextnum=sec[0][0]
              else:
                  nextnum=resnum+1
              nextres=self.pdb[0][nextnum]
              asd=Dummy()
	      asd.entropy=0.0
              asd.sum_pPosts=1.0
              asd.numAccepted=len(respartners)*2
              asd.probNoise=1-firsteprob
              asd.assigns=[]
              for rp in respartners:
                  resnum2=rp[0]
                  res2=self.pdb[0][resnum2]
                  asn=Dummy()
                  asn.shFac=0.0
                  asn.volFac=0.0
                  asn.pPost=rp[1]/2.0/len(respartners)
                  ato=_Atom(res2['HN'])
                  asn.orderedAtoms=(_Atom(res['O']),ato)
                  asd.assigns.append((rp[1]/2.0,asn))
                  if abs(nextnum-resnum2)>2:
                      asn=Dummy()
                      asn.shFac=0.0
                      asn.volFac=0.0
                      asn.pPost=rp[1]/2.0/len(respartners)
                      ato=_Atom(res2['O'])
                      asn.orderedAtoms=(_Atom(nextres['HN']),ato)
                      asd.assigns.append((rp[1]/2.0,asn))
                  else:
                      asd.numAccepted-=1
                      print 'skipping',nextnum,resnum2
              pspeak=PseudoPeak(asd)
              pspeak.writeRestraints(restfile,skipfile='none',name='HB'+str(resnum),
                                     pasdresfile=pasdfile,hasinfo=True,dmax=2.0,resc=resco,
                                     xplorfile=xplorfile)
      pasdfile.close()
      restfile.close()
      pseudoshiftfile.close()

    def writeSolventExpRestraints(self):
      pasdfile=open('testSolvExp.peaks','w')
      restfile=open('junk.txt','w')
      xplorfile=open('testSE_xplor.tbl','w')
      pseudoshiftfile=open('seshifts.shiftAssignments','w')
      resco=RestraintCounter()
      buf=initfil2('selins.txt')
      #result:    OK  3 F E - +  0.122    9.19  0.000  0.000  0.000
      com=Dummy()
      com.fragment_id='100'
      com.name='COM'
      com.element='X'
      com.res_name='COM'
      com.shift=-99.0
      com.temp_factor=-99.0
      for lin in buf:
	  resnum=eval(lin[2])
	  prob0=eval(lin[7])
	  prob=max(prob0,1-prob0)
	  if prob>0.6:
	      if prob0>0.5:
		  dmax=20.82;dmin=10.41
	      else:
		  dmax=10.41;dmin=0.0
	      dist=eval(lin[8])
	      com.position=dist
      	      COM=_Atom(com)
	      resname=lin[3]
              res=self.pdb[0][resnum]
              at0=_Atom.getRefAt(resname,res)
              self.writePseudoShifts(at0,pseudoshiftfile)
              asd=Dummy()
	      asd.entropy=0.0
              asd.sum_pPosts=1.0
              asd.probNoise=1-prob
              asd.assigns=[]
	      if True:
                  asn=Dummy()
                  asn.shFac=0.0
                  asn.volFac=0.0
                  asn.pPost=prob
                  asn.orderedAtoms=(COM,at0)
                  asd.assigns.append((prob,asn))
                  asd.numAccepted=1
              pspeak=PseudoPeak(asd)
              pspeak.writeRestraints(restfile,skipfile='none',name='SE'+str(resnum),
                                     pasdresfile=pasdfile,hasinfo=True,dmax=dmax,dmin=dmin,resc=resco,
                                     xplorfile=xplorfile)
      pasdfile.close()
      restfile.close()

    def writePseudoShifts(self,at,csfilepasd):
      pasdid=at.getPASDid()
      shiftfinal=-999.0
      sides=['from','to']
      for s in sides:
          thisid=pasdid+s
          csfilepasd.write('shiftAssignment '+thisid+'\n')
          selstr='    -protonSelection (resid '+at.fragment_id+\
                  ' and resn '+at.res_name+\
                  ' and name '+at.name+')\n'
          csfilepasd.write(selstr)
          csfilepasd.write('    -'+s+'\n') #{'-from'|'-to'}
          csfilepasd.write('    -protonShift %6.2f\n'%shiftfinal)
          csfilepasd.write('end\n')
          csfilepasd.write('\n')


    def showPDFs(self,get):
      if isinstance(get,tuple) and len(get)==2:
        fr,nam=get
        self.gethistory(fr,nam)
      elif isinstance(get,str):
        cols='bgrcmyk'
        buf=initfil2(get)
        cnt=0
        for lin in buf:
            fr,nam=lin[2],lin[4]
            if nam[0]=='C':
                self.gethistory(fr,nam,'final',plotcol=cols[cnt%7])
                cnt+=1
      show()
      raise SystemExit

    def testConPriors(self):
      print 'TESTING PRIORS...'
      return -1
      print [self.conPriors[3].LRmap[4][66]]
      print self.conPriors[3].LRmap[4][66]
      print self.conPriors[2].MRmap[68][0][('CA','CA')]
      print self.conPriors[3].MRmap[68][0][('CA','CA')]
      cp3=self.conPriors[3]
      cp2=self.conPriors[2]
      print cp3.prob(4,66)
      print cp3.prob(3,68)
      print cp3.prob(68,3)
      print cp3.prob(3,68,'CA','CB')
      print cp3.prob(68,3,'CB','CA')
      print cp3.prob(3,5,'CA','CB')
      print cp3.prob(5,3,'CB','CA')
      print cp3.prob(3,5)
      print cp2.prob(3,5)
      print cp3.prob(68,71)
      print cp3.prob(68,71,'CA','CB')
      print cp3.prob(22,22,'C','CD1')
      print cp3.prob(22,'C','CD1')
      print cp3.prob('I','C','CD1')
      print cp2.prob(22,22,'C','CD1')
      ##print cp3.prob(68,71,'CA','CX')

    def putConPriors(self):
      self.conPriors=[]
      seq=''
      for res in self.pdb.iter_fragments():
          if res.is_amino_acid():seq+=aa31dict[res.res_name]
      ##seq='MQYKLILNGKTLKGETTTEAVDAATAEKVFKQYANDNGVDGEWTYDDATKTFTVTE' #GB1
      ##seq='MQIFVKTLTGKTITLEVEPSDTIENVKAKIQDKEGIPPDQQRLIFAGKQLEDGRTLSDYNIQKESTLHLVLRLRGG'#ubiquitin only!!!
      ##   12345678901234567890123456789012345678901234567890123456
      print 'seq is:',len(seq),seq
      if len(self.converged)==0:
	namlst=self.inpmap['priors']
	intradct=read_dba('intraProbs.pck')
	for i,name in enumerate(namlst):
	 if name=='simple':
	   print 'using simple priors'
	   A=zeros((len(seq),len(seq)))
	   smr=SimpleMediumRangeProbs()
	   self.conPriors.append(ContactProbabilityMap(A,smr,intradct,i,name,seq))
	   #MRmap will be None
	   #LRmap will be []
	 else:
	   self.conPriors.append
	   A,mrdctmat=read_dba(name)
	   self.conPriors.append(ContactProbabilityMap(A,mrdctmat,intradct,i,name,seq))
        ##self.conPriors[-1].show(self.pdb)
      else:
          cmat,minapdct=self.distanalyze(dcut=7.0,returnMat=True)
##          print cmat
          rmins=[2.2,3.2,3.6,4.0,4.4,4.7]
##          rmins=[4.5]
          for rmin in rmins:
              self.conPriors.append(ContactProbabilityMap2(rmin,seq,self.pdb))
          for pdbkey in sorted(self.converged.keys()):
              pdb=self.converged[pdbkey]
              print 'updating probabilities with structure',pdbkey
              pen=ContactProbabilityMap2.updateWithStructure(self.conPriors,pdb,rmins,cmat,minapdct,pdbkey)
          for cp in self.conPriors:
              cp.scale(1.0*len(self.converged))
              save_dba2(cp,'cp'+str(cp.rmin)+'.pck')
          self.conPriors[-1].show(self.pdb)

    def putPDFs(self):#deprecated?
      for pt in range(len(self.pointerList)):
          resAs=self.pointerList[pt]
          sh,sd=self.shiftVC.list[pt][0],self.shiftVC.stdlist[pt][0]
          #sh kan vaere perturberet
          if sd<>None:
              resAs.putPDF(sh,sd)
      print len(self.otherShiftVCs[('N','N')].list),len(self.pointerListN)
      for pt in range(len(self.pointerListN)):
          resAs=self.pointerListN[pt]
          sh,sd=self.otherShiftVCs[('N','N')].list[pt][0],self.shiftVC.stdlist[pt][0]
          #sh kan vaere perturberet
          if sd<>None:
              resAs.putPDF(sh,sd)

    def testDists(self,pt1,pt2):
      ati=self.shiftVC.atreflist[pt1]
      atj=self.shiftVC.atreflist[pt2]
      print ati
      print atj
      d=AtomMath.calc_distance(ati,atj)
      print d,log(d)
      print self.dbas.getlogDistMeanAndSd((ati,atj))

    def putPointers(self):
        count=0
        countN=0
	cntdct={}
        #nedenstaaende gennemloeb definerer en ordning
        for chainnum in range(len(self.pdb)):
         ch=self.pdb[chainnum]
	 for f in range(len(ch)):
           frag=ch[f]
           count2=0
           for a in range(len(frag)):
              at=frag[a]
     	      if at.element==self.mainnuc:#was C (solid state)
		if at.shift<>None:
                  at.pointer2=count2
                  count2=count2+1
                  at.pointer=count#er det vigtigt?...
                  count=count+1
     	      elif at.element in 'NHC':##['N']:
		if at.shift<>None:
		  if not at.element in cntdct:cntdct[at.element]=0
                  at.pointer=cntdct[at.element]#NOTE: may coninside with mainnuc pointers
                  ##was countN=countN+1
                  cntdct[at.element]+=1
	self.pointerList=[None for _ in range(count)]
        self.pointerLists={}
	print 'counts',count,cntdct
	for k in cntdct:
	  self.pointerLists[k]=[None for _ in range(cntdct[k])]
    
    def putShiftsDBA(self,im):##,DBA=None,allwaysFromdba=False):
     '''put the "true" shifts into the chain
	missing shift slots can be filled with data base values and large stds'''
     convdct=getconvdct(rev=True)##,fromxplor=True)
     metdct=getmethyls(1)
     ambdct=getambgroups(colnum=1)
     moldba=self.molShifts
     #initialize chemical shifts
     ##for chainnum in range(len(self.pdb)):
     chainnum=0#deprecated
     for ch in self.pdb[0]:
	 vals=[]
	 ##for f in range(len(ch)):
	 for frag in ch:
	   print '------------------------',frag,frag.id,frag.resname
           ##for a in range(len(frag)):
           for at in frag:
	      print at
	      aai=aa31dict[frag.resname]
	      elem=getelem(at.name)
	      print aai,elem
     	      if elem<>None and elem in 'CHN':
		    ##at.shift=[None,None]
		    at.shift=None
		    atkey=None
		    if at.name in ['1H','2H','3H']:atkey='H'
		    else:atkey=convdct[aai][at.name]
		    atkey0=atkey;atkey1=atkey
		    print at,atkey
		    fragment_id=str(frag.id[1])
		    if True:#not (f==0 and atkey in ['N','H','HN']):
		        ##print moldba[chainnum][at.fragment_id].keys()
		        if aai in metdct:
			  used=False
			  for trip in metdct[aai]:
			    ##print trip,metdct[aai][trip]
			    if atkey in trip:
			      atkey0=metdct[aai][trip]
			      if atkey==trip[0]:
				print 'found methyl group',aai,trip,atkey
				atkey1=metdct[aai][trip];used=True
			      else:
				print 'methyl proton - not using this one',trip,atkey
				if not used:atkey1=None
				#NOTE: remember to include all 3 methyl protons in restraint!
			if aai in ambdct:
			  used=False
			  for pair in ambdct[aai]:
			    if atkey==pair[0]:
				##atkey1=ambdct[aai][pair];used=True
				used=True
				print 'found amb. (methylene) group',aai,pair,atkey,ambdct[aai][pair]
			    elif atkey in pair:
                              try:sh0=moldba[chainnum][fragment_id][atkey][0]
			      except KeyError:sh0=None
                              try:sh1=moldba[chainnum][fragment_id][pair[0]][0]
			      except KeyError:sh1=None
			      if sh0==sh1:
				print 'ambigouos atom - not using this one',pair,atkey
				##if not used:atkey0=ambdct[aai][pair]
				if not used:atkey1=None
			      else:print 'using different methylene shifts anyway',atkey,pair,sh0,sh1
			print atkey,atkey0,atkey1
			if fragment_id in moldba[chainnum]:
			 if atkey0 in moldba[chainnum][fragment_id] and atkey1<>None:
                          avg,std,xtra=moldba[chainnum][fragment_id][atkey0]
			  if at.shift==None:at.shift=[None,None]
                          at.shift[0]=avg
                          at.shift[1]=std##math.sqrt(std**2+xstd**2)
			 else:print 'WARNING: atom (assignment) not found:',at,atkey0,atkey1

    def initvcs(self,im,xnucdba):
       '''define starting points for shift assignments
          "true" values are perturbed with "pert"'''
       ldict={'G':'H', 'H':'H', 'I':'H', 'E':'S', 'B':'-',
              '*':'-', 'T':'-', 'S':'-', '-':'-', '?':'-'}
       shlst=[]
       stdlst=[]
       atreflst=[]
       self.allAtomList=[]
       self.allTriplets={}
       if im.has_key('pert'):
           if im['pert']=='dba':stdpert='dba'
           elif im['pert']=='pine':stdpert='pine'
           else:stdpert=eval(im['pert'])
       else:stdpert=0
       if im.has_key('trunc'):trunc=eval(im['trunc'])
       else:trunc=1000##inf
       shavedct=read_dba('shiftAveStats.pck')
       moldba=self.molShifts
       ##-------------------------------------
       for ch in range(len(self.pdb)):#print mydata.pdb[ch]
         for i in range(len(self.pdb[ch])):
           triplet=Triplet(i,self.pdb[ch])
           for j in range(len(self.pdb[ch][i])):
             ##at=self.pdb[ch][i][j]
             at0=self.pdb[ch][i][j]
     	     if at0.element in 'H':##['C','H','N','HG']:##'CHN':
               ##at0.resAs=ResAssignment(at0)
	       print at0
               at=_Atom(at0)
               at.resAs=ResAssignment(at)
	       if 'usetriplets' in im and eval(im['usetriplets']):
                 at.resAs.triplet=triplet
                 self.allTriplets[i]=triplet
               sh=at.shift
	       print sh
               if sh==None:
               #print at
               ##shm=-999;stdev=None
                 at.resAs.truemean=-999
                 ##at.shift=(-999,1.0)
               else:
                 at.resAs.truemean=sh[0]
	       states=[]
               if sh<>None:##True:
                 self.allAtomList.append(at)
                 if stdpert=='dba':
                   print 'using starting shifts from data base'
		   if self.pdsspData<>None:
		     print 'using secondary structure (predictions) to refine starting shifts',
		     ssi =self.pdsspData.ssdict[str(i+1)]
		     gssi=ldict[ssi]
		     pssi=self.pdsspData.pssdict[str(i+1)][0]
		     #eprob,hprob,cprob=pssi
		     print pssi,
		     aa=aa31dict[at.res_name]
		     atn=at.name;elem=atn[0]
		     print aa,ssi,gssi,
		     avesh=0.0
		     for n in range(3):
			nprob=pssi[n]
			if nprob>0.0:
			  ssn='SH-'[n]
		     	  if not atn in shavedct[(aa,ssn)][elem]:
			    print 'WARNING: atom not found in shavedct',atn
                            shmn,stdn=self.dbaShifts[at.res_name][at.name]
			  else:
			    statsn=shavedct[(aa,ssn)][elem][atn]
			    shmn,stdn,numn=statsn
			  states.append((shmn,stdn,nprob))
			  avesh+=(shmn*nprob)
		     print states
                   shm,stdev=self.dbaShifts[at.res_name][at.name]
		   if self.pdsspData<>None:
		     shm=avesh
		     if len(states)==1:stdev=stdn#OK? stdn==ssi is the only defined stdn
                   print at.res_name,at.name,shm,stdev
                 elif stdpert=='pine':
                     print 'using starting (ambiguous) shifts from pine assignments'
		     aa=aa31dict[at.res_name]
		     atn=at.name;elem=atn[0]
                     if moldba[ch][at.fragment_id].has_key(at.name):
			stde={'H':0.05,'C':0.1,'N':0.1}[elem]
			totprob=0.0;avesh=0.0;xprob=0.0
              		avg0,prob0,xtra=moldba[ch][at.fragment_id][at.name]
			states=[(avg0,stde,prob0)]
			totprob+=prob0
			shm=avg0;stdev=stde
			if prob0<0.99:
			  print xtra
			  for I in range(len(xtra)/2):
			    avgi,probi=xtra[I*2],xtra[I*2+1]
			    totprob+=probi
			    print avgi,probi
			    avesh+=(avgi*probi)
			    states.append((avgi,stde,probi))
			print 'states:',states
			if totprob<0.99:
			    xprob=1-totprob
			    xavg,xstd=sh
			    ##wrong: avesh+=(avgi*probi)?
			    avesh+=(xavg*xprob)
			    states.append((xavg,xstd,xprob))#xstd>>stde
			print 'states:',states
			if xprob>=0.9:shm=xavg
			if xprob>0.05:stdev=xstd
			elif len(states)>1:
			  maxav=max([stati[0] for stati in states])
			  minav=min([stati[0] for stati in states])
			  stdev=(maxav-minav)/2
			  if prob0/totprob<0.5:shm=avesh
                     else:
			print 'WARNING: atom name not in moldba',atn
			shm,stdev=sh
                 else:
                   print 'using starting shifts perturbed from true value'
                   pert=0.0
                   if stdpert>0:
                     pert=10
                     while abs(pert)>trunc:pert=normalvariate(0.0,stdpert)
                   shm=sh[0]+pert;stdev=sh[1]
             ##print at,shm,stdev,pert
             #here this shifts are initialized with putPDF using shm and stdev
	         if len(states)<=1:##self.pdsspData==None:
		   at.resAs.putPDF(shm,stdev)
	         else:
		   at.resAs.putPDF(shm,states,form='sumnormal')
                 ##if at.element in ['C']:
     	         if at.element[0]==self.mainnuc:#was C (solid state)
                   shlst.append([shm])
                   stdlst.append([stdev])
                   atreflst.append(at)
                 for k in xnucdba.keys():
                   ##print at.element,xnuclst[k][0],at.name,xnuclst[k][1]
                   if at.element==k[0]:
                     if at.name==k[1] or k[1]==None:
                           print 'xnuc:',at,k
                           xnucdba[k][0].append([shm])
                           xnucdba[k][1].append([stdev])
                           xnucdba[k][2].append(at)
       shvc=VoxelContainer()
       shvc.dimension=1
       ##print len(shlst),'carbons in shlst'
       print len(shlst),'main atoms in shlst'
       print len(self.allAtomList),'atoms in total'
       #shvc.vwidth=self.cycle.shiftCutoff*2.01*numpy.ones(1)
       shvc.vwidth=[eval(im['vwidth'])]
       shvc.fillContainer(shlst,tag=0)
       self.shiftVC=ShiftVC(shvc,shlst,atreflst,stdlst)
       self.otherShiftVCs={}
       for k in xnucdba.keys():
           print k,len(xnucdba[k][0])
           xshvc=VoxelContainer()
           xshvc.dimension=1
           xshvc.vwidth=[xnucdba[k][3]*2.01]
           xshvc.fillContainer(xnucdba[k][0],tag=0)
           self.otherShiftVCs[k]=ShiftVC(xshvc,
                xnucdba[k][0],xnucdba[k][2],xnucdba[k][1])
       
    def basefindSpinSystems(self,damps,distthr=2.0):
	dct={}
	##tocsyconvdct=getconvdct(rev=True,fromxplor=True,tocsy=True)
	tocsyconvdct=getconvdct(rev=True,fromxplor=False,tocsy=True)
        for i in range(len(self.pdb[0])):
	    resi=self.pdb[0][i]
	    prev=self.pdb[0][i-1]
	    nat=resi['N']
	    aai=aa31dict[nat.res_name]
	    if aai<>'P' and i>0:
	      group=aai+str(i+1)
	      atns=['HA','N','H']
	      if aai=='G':atns[0]='1HA'
	      shiftlst=[]
	      for atn in atns:
		print atn,resi[atn].shift[0]
		shiftlst.append(resi[atn].shift[0][0])
	      print aai,i+1,shiftlst
	      peak=PeakSimple(shiftlst,'xxx')
	      ssi=SpinSystem(peak,group)
	      dct[group]=ssi
	      ssi.addTOCSYspins(resi,i,aai,tocsyconvdct)
	      ssi.addHetspins(resi,i,aai,prev)
	      ssi.addNOESYpeaks(resi,i,aai,self.pdb[0])
	      ssi.damps=damps
	print len(dct),'spin systems in total'
	return dct

    def getnearest(self,peak,exp0,mode):
        posasmat=[]
        print '*'*30
        sdamps=peak.getDampings(exp0)
        if isinstance(peak,BuildUpPeakVector):print peak.assSet
        print 'getting all assignment possibilities for',exp0.name,peak,sdamps,exp0.shnormcs
        for k in range(exp0.dimension):#k-dimensionelt experiment!
            shnormck=exp0.shnormcs[k]
            'find the correct vc...'
            nuc0=exp0.nucnames[k]
            at0= exp0.atnames[k]
            ##if at0<>None or nuc0<>'C':
            if at0<>None or nuc0<>self.mainnuc:
                vc0=self.otherShiftVCs[(nuc0,at0)]
            else:vc0=self.shiftVC
            #vc0 er den vc der skal bruges til at lave assignments
            print 'finding possible assignments for',exp0.name,peak,k,exp0.atnames[k]
            scut=exp0.shiftTols[k]
            print sdamps[k]
            posaslst,maxshfac=vc0.getnearestList(peak.shifts[k],scut,sdamps[k],mode,shnormck)
            posasmat.append(posaslst)
	    exp0.maxshfacs[k].append(maxshfac)
        ##print len(posasmat[0])*len(posasmat[1])
        ##------------------------------
        DistCuts=exp0.getDistCuts(peak)
        ResIncl =exp0.getResIncl(peak)
        ##if exp0.fromStructure:conPris=None
        if exp0.fromStructure or (not 1 in exp0.multi) or exp0.fixbonds<>None:conPris=None
        else:
	  conPris=exp0.getConPris(peak)
        print 'getting list of valid assignments',exp0.name,peak
        #elems are of type Assignment1D
        sideInds=peak.getSideInds()
        validassigns=exp0.getValidAssigns(posasmat,DistCuts,sideInds,conPris,ResIncl)
        valdba=getValidDBA(validassigns)
        minlen=len(sideInds)
        print 'minlen:',minlen
        if not SILENT:
          for tup in valdba.keys():
            print tup,valdba[tup]
            if len(valdba[tup][0])<minlen:
                del valdba[tup]
                print 'deleting:',tup
        print len(valdba),'valid assignments'
        peak.reshapeAssigns(valdba,posasmat)

    def calcVolfac(self,lr,lsig2):
        if lsig2==0:return 0.0
        n=2.0
        return exp(max(lr,log(3))+0.5*lsig2)**(-n)+0.00
        #return exp(n*(-max(lr,log(3))+0.5*n*lsig2))

    def calcVolfacCum(self,lr,lsig2,rmin=4.5,lsigint=1.0,n=6.0):
##        if lsig2==0:return 0.0
	z=(log(rmin)-lr)/(math.sqrt(lsig2+(lsigint/n)**2))
	return comStandardNormal(z)

    def calcVolfac2(self,lr,lsig2,rmin=4.5,lsigint=1.0,n=3.0):
        if lsig2==0:return 0.0
	z=(log(rmin)-lr)/(math.sqrt(lsig2+(lsigint/n)**2))
	return exp(-0.5*z**2)

    def getAssignments(self,exp0,mode,fractile=0.1):
##	self.checklist=[[],[],[]]
##	self.checklist2=[[],[],[]]
##	exp0.noisevols=[]
##	exp0.signalvols=[]
##	exp0.signalDists=[]
##	exp0.noiseIDs=[]
##	exp0.signalIDs=[]
        exp0.allassigns=[]
        if self.inpmap.has_key('xstd'):xstd=eval(self.inpmap['xstd'])
        else:xstd=0.0
        exp0.getNormCs(xstd)
	exp0.maxshfacs=[[]   for _ in range(exp0.dimension)]
	exp0.refshfacs=[None for _ in range(exp0.dimension)]
        initmat=[]
        ptten=[]
        for i in range(len(exp0.peakData.peaklist)):
            peak=exp0.peakData.peaklist[i]
            ##if not peak.strip:
            if exp0.acceptPeak(peak):
            ##if peak.hErrProb<0.95:
                print peak.shifts
                ##print peak.dba
                self.getnearest(peak,exp0,mode)#init. peak.initAssigns
            else:peak.initAssigns=[];peak.ptMat=[]
            peak.setAssignData(exp0,i)#AssignData is initialized...
            exp0.allassigns.append(peak.assignData)
	for k in range(exp0.dimension):
	    exp0.maxshfacs[k].sort()
	    refnum=int(round(len(exp0.peakData.peaklist)*fractile,0))
	    refval=exp0.maxshfacs[k][-refnum]
	    exp0.refshfacs[k]=refval
	    print 'reference value:',exp0.name,k,refval,exp0.maxshfacs[k][-1]
        print len(exp0.peakData.peaklist),'assignments initialized'

    def rankAllInitAssigns(self,exp0,updvec,mode='normal'):
        if self.inpmap.has_key('includeall'):inc=self.inpmap['includeall']
        else:inc=0
        for i in range(len(exp0.peakData.peaklist)):
            peak=exp0.peakData.peaklist[i]
            try: assSet=peak.assSet; maxAssSet=peak.maxAssSet
            except AttributeError:assSet=None;maxAssSet=None#empty
            peak.rankInitAssigns(i,exp0,inc,updvec,mode)#calculates Gi and Fi
            asd=peak.assignData
            asd.probNoise=peak.hErrProb
            asd.setAllNot()
            asd.setpPosts(exp0,assSet,Smax=maxAssSet)
            asd.setUse()
            asd.getProjectedAssignments(exp0)
        print 'assignments ranked'

    def getlinkdct(self,spindct,damppair):
	linkdct={}
	skeys=spindct.keys()
	def scmp(x,y):
	  ##if x[0]<>y[0]:return cmp(x[0],y[0])
	  return cmp(string.atoi(x[1:]),string.atoi(y[1:]))
	skeys.sort(scmp)
	print skeys
	for i in range(len(skeys)):
	  bestelink=999;bestprev=None
	  linkdct[i]={}
	  trueener=None
	  for j in range(len(skeys)):
	    if i<>j:
		kc=skeys[i];kp=skeys[j]
		elink=spindct[kc].calc_elink(spindct[kp])
		asdiff=string.atoi(kc[1:])-string.atoi(kp[1:])-1
		elinknoesy=spindct[kc].calc_elinknoesy(spindct[kp],damppair,asdiff)
		##elinkcomb=elink+elinknoesy
		elinkcomb=elinknoesy
		linkdct[i][j]=elinkcomb
		print 'linkresult %2d %2d %3s %3s %6.3f %6.3f %6.3f'%(i,j,kc,kp,elink,elinknoesy,elinkcomb)
		##if elink<bestelink:
		if elinkcomb<bestelink:
		  bestelink=elinkcomb
		  bestprev=j
		if asdiff==0:
		  print 'true assignment %2d %2d %3s %3s %6.3f %6.3f %6.3f'%\
			                 (i,j,kc,kp,elink,elinknoesy,elinkcomb)
		  trueener=elinkcomb
	  print 'bestlink: %2d %2d %3d %3s %3s %6.3f'%\
		           (i,bestprev,abs(i-bestprev-1),kc,skeys[bestprev],bestelink),
	  if trueener==None:print '  0.00000'
	  else:print ' %6.3f'%(trueener-bestelink)
	rev={}
	for i in linkdct:
	  rev[i]={}
	  for j in linkdct:
	    if i<>j:
	      rev[i][j]=linkdct[j][i]#reverse data structure
	return linkdct,skeys,rev
	
    def getentrop(self,linkdct,skeys,flag='',searchrange=None):
	if searchrange==None:searchrange=range(len(skeys))
	entrop=[]
	mat=[]
	for i in searchrange:
	  ##es=array([linkdct[i][j] for j in linkdct[i]]
	  its=linkdct[i].items()
	  its.sort(lambda x, y:cmp(x[1],y[1]))
	  mat.append(its)
	  es=array(its)[:,1]
	  ##es=linkdct[i].values()
	  ##es.sort()
	  ps=exp(-array(es))
	  ps=ps/sum(ps)
	  es1=-log(ps)
	  for n in range(len(ps)):
		if ps[n]==0.0:es1[n]=0.0
	  enti=sum(ps*es1)
	  ##print enti;print es1;print es;print ps
	  print '%sentrop %2d %3s %6.3f'%(flag,i,skeys[i],enti),es[:5]
	  entrop.append(enti)
	return entrop,mat
	
    def gettypedct(self,spindct,skeys):
	##tocsyconvdct=getconvdct(rev=True,fromxplor=True,tocsy=True)
	tocsyconvdct=getconvdct(rev=True,fromxplor=False,tocsy=True)
	dct={}
        allinds=[];alldiffs=[]
	for j in range(len(skeys)):
	  dct[j]={}
	  kj=skeys[j]
	  sj=spindct[kj]
	  besttyp=999
          for ch in range(len(self.pdb)):#print mydata.pdb[ch]
            for i in range(1,len(self.pdb[ch])):
	      resi=self.pdb[ch][i]
	      resp=self.pdb[ch][i-1]
	      etyp=sj.calc_etyp(resi,i,tocsyconvdct,resp)
	      dct[j][i]=etyp
	      if etyp<besttyp:
		besttyp=etyp;besti=i
	      if string.atoi(sj.group[1:])-i-1==0:
		trueval=etyp
	  lstj=dct[j].values()
	  lstj.sort()
	  trueind=lstj.index(trueval)
	  aveval=average(lstj)
	  avediff=aveval-trueval
	  print 'besttyp: %3s %2d %2d %6.3f  %6.3f %6.3f %2d %6.3f'%\
		 (sj.group,j,besti,besttyp,trueval-besttyp,trueval,trueind,avediff)
	  allinds.append(trueind*1.0)
	  alldiffs.append(avediff)
	print 'averesult: %7.4f %7.4f'%(average(allinds),average(alldiffs))
	rev={}
        for ch in range(len(self.pdb)):#print mydata.pdb[ch]
          for i in range(1,len(self.pdb[ch])):
	    rev[i]={}
	    for j in range(len(skeys)):
	      rev[i][j]=dct[j][i]#reverse data structure
	return dct,rev

    def getSpinSystems(self,explst,rootnum=0,noesynum=1):
	exp0=explst[rootnum]
	##spindct=exp0.peakData.findSpinSystems(exp0.shiftDamps)
	spindct=self.basefindSpinSystems(exp0.shiftDamps)
	exp1=explst[noesynum]
	##exp1.peakData.addNOESYspins(exp1.shiftDamps,spindct)
	linkdct,skeys,rev=self.getlinkdct(spindct,(exp0.shiftDamps[0],exp1.shiftDamps[2]))
	entropf,matf=self.getentrop(linkdct,skeys,flag='linkfwd')
	entropr,matr=self.getentrop(rev,skeys,flag='linkrev')
	typedct,revtyp=self.gettypedct(spindct,skeys)
        ress=[self.pdb[0][i] for i in range(len(self.pdb[0]))]
	typentropf,typmatf=self.getentrop(typedct,skeys,flag='typfwd')
	typentropr,typmatr=self.getentrop(revtyp,ress,flag='typrev',searchrange=range(1,len(self.pdb[0])))
	pckdata1=(spindct,linkdct,skeys,rev,typedct,revtyp)
	pckdata2=((entropf,entropr,typentropf,typentropr),(matf,matr,typmatf,typmatr))
	save_dba2(pckdata1,'enerdcts_ubiqnoC13b.pck')
	save_dba2(pckdata2,'entropies_ubiqnoC13b.pck')
	1/0

    def analyze(self,exp0,plotting=0):
        vollst=[]
        dlst=[]
        data=[]
        shd=[]
        shd1bs=[]
        logdata=[]
        sdlst=[]
        sdlst2=[]
        posslst=[]
        d1bs=[]
        d2bs=[]
        problst=[]
        for i in range(exp0.dimension):
            sdlst.append([]);sdlst2.append([]);posslst.append([])
        uar=Restraints()
        uar_skipped=Restraints()
        ar2=Restraints()
        print 'len(exp0.peakData.peaklist)',len(exp0.peakData.peaklist),len(exp0.allassigns)
        for i in range(len(exp0.allassigns)):
            asd=exp0.allassigns[i]
            rtype=-1
            if asd.assigns<>None:
                if asd.maxIndex<>None:
                    as0=asd.assigns[asd.maxIndex][1]
                else:
                    as0=asd.asMax2
                if as0<>None:
                    rtype=getType(as0.atoms,pair=(exp0.dimension-2,exp0.dimension-1))
            if asd.use==1:
                uar.update(rtype)
            elif asd.use==2:
                ar2.update(rtype)
            print rtype, '('+str(asd.use)+')',
            if asd.use in [1]:#[1,2]:
                vols=exp0.peakData.peaklist[i].volumes
		##print as0,vols,as0.distances[as0.maxPos][-1]
                pno=exp0.peakData.peaklist[i].probnoise
                poss=exp0.peakData.peaklist[i].shifts
                for i in range(exp0.dimension):
                    sdlst[i].append(as0.shiftDiffSigned[i])
                    for j in range(len(asd.assigns)):
                        assj=asd.assigns[j][1]
                        gvcj=asd.assigns[j][0]
                        if gvcj>0.1:
                            sdlst2[i].append((assj.shiftDiffSigned[i],gvcj))
                            posslst[i].append(poss[i])
                if rtype<=1 and asd.maxPpost>0.1:
                    dlst.append(as0.distances[as0.maxPos][-1])
                    problst.append(asd.maxPpost)
                    vollst.append(vols)
                    data.append((as0.distances[as0.maxPos][-1],vols))
                    logdata.append((log(as0.distances[as0.maxPos][-1]),log(vols)))
                    shd.append(abs(poss[-1]-poss[-2])/175.0)
                    if as0.distances[as0.maxPos][-1]>1.2 and as0.distances[as0.maxPos][-1]<1.8:
                        d1bs.append(log(vols))
			##print 'VOLS1',vols,as0.atoms[0].fragment_id,as0.atoms[0].name,as0.atoms[1].name,asd.maxPpost
                        shd1bs.append(abs(poss[-1]-poss[-2])/175.0)
                    elif as0.distances[as0.maxPos][-1]>2.0 and as0.distances[as0.maxPos][-1]<2.9:
                        d2bs.append(log(vols))
			##print 'VOLS2',vols,as0.atoms[0].fragment_id,as0.atoms[0].name,as0.atoms[1].name,asd.maxPpost
            elif asd.use==5:#3:
                uar_skipped.update(rtype)
        for i in range(exp0.dimension):
##            shm,shsd=calc_mean_and_stdd(sdlst[i])
##            print shm,shsd,'---'
            gmean=exp0.globShifts[i]
            gsigma=10.0 #inf
            ##gsigm0=exp0.shiftDamps[i]
            #TODO:put i inputfil
            tup=getGeneralizedMeanSd2(sdlst2[i],gmean,gsigma,sigm0=0.2)
            if tup[0]==None:print 'NODATA'
            else:
                updgs=tup[1]
                count,vari=tup[0],tup[-1]
                updsd=math.sqrt(vari/count)
                ##exp0.globShifts[i]=exp0.globShifts[i]-updgs
                ##exp0.shiftDamps[i]=updsd
                #TODO:bestemme hvornaar ovenst. skal goeres; include priors later+tol. update
                print updgs,updsd,'+++'
                print tup
        if len(posslst)>0:
            print len(logdata),'points used for calibration'
            uar.showres()
            ar2.showres()
            uar_skipped.showres()
        if 0:
            D=numpy.array(sdlst2)
            A=numpy.array((posslst[0],D[0,:,0]))
            A=A.transpose()
            print A[0]
            B=numpy.array((posslst[1],D[1,:,0]))
            B=B.transpose()
            if plotting:scatter(posslst[0],D[0,:,0],alpha=0.5,faceted=False,c='b')
            if plotting:scatter(posslst[1],D[1,:,0],alpha=0.5,faceted=False,c='r')
            fitvarslinA,chi2linA=leastSquaresFit(linlaw,(0.01,-0.0001),A)
            print fitvarslinA,chi2linA
            fitvarslinB,chi2linB=leastSquaresFit(linlaw,(-0.00001,0.00001),B)
            print fitvarslinB,chi2linB
            fitvarslin2,chi2lin2=leastSquaresFit(linlaw,(-0.00001,0.00001),A+B)
            print 'CORR?',fitvarslin2,chi2lin2
            t=numpy.arange(5,185,5)
            Alin=linlaw(fitvarslinA,t)
            Blin=linlaw(fitvarslinB,t)
            if plotting:plot(t,Alin)
            if plotting:plot(t,Blin,c='r')
            exp0.corrections=[fitvarslinA,fitvarslinB]
            if plotting:show()
            print exp0.globShifts
            fitvars,chi2=leastSquaresFit(powerlaw,(20.0,-1.0),data)
            fitvarslin,chi2lin=leastSquaresFit(linlaw,(-2.0,3.0),logdata)
            t=numpy.arange(2.3,7.0,0.1)
            #y=loglaw(fitvars,t)
            ypow=powerlaw(fitvars,t)
            ylin=linlaw(fitvarslin,log(t))
            print fitvars,chi2
            print fitvarslin,chi2lin
            #plot(log(t),ylin)
            ##plot(t,ylin)
            #plot(t,ypow)
            #scatter(log(numpy.array(dlst)),log(numpy.array(vollst)),
            #        alpha=0.5,faceted=False,c=numpy.array(shd))
            scatter(dlst,problst,alpha=0.5,faceted=False,c=numpy.array(shd))
            #scatter(numpy.array(shd),numpy.array(d1bs),alpha=0.5,faceted=False)
            #hist(d2bs,normed=1)
            print calc_mean_and_stdd(d1bs)
            print calc_mean_and_stdd(d2bs)
            show()
            self.cycle.dconst=fitvarslin[1]
            #return fitvarslin[0],shm

    def redoResAssignments_old(self,exp0,expnum,min_pFalse=0.1,usetrip=True):
        for i in range(len(exp0.allassigns)):
            asd=exp0.allassigns[i]
            if asd.use in [1,2,5]:
	      pnoise=exp0.peakData.peaklist[i].probnoise
              for k in range(len(asd.assigns)):
                as0=asd.assigns[k][1]
                gvc0=asd.assigns[k][0]
                if True:##likFrac>0.05:#TODO put i input-fil
                    ats=as0.atoms
                    askey=getAssignmentKey(ats)
                    frf=string.atoi(ats[-1].fragment_id)
                    fri=string.atoi(ats[-2].fragment_id)
                    frd=abs(frf-fri)
                    if frd<5:#TODO put i input-fil
                        pos=exp0.peakData.peaklist[i].shifts
                        ##ptlst=[]
                        ##for j in range(exp0.dimension):#generaliseres med 3d!!!!!!
                        ##    at=ats[j]
                        ##    pt=at.pointer
                        ##    ptlst.append(pt)
			print 'using assignment',ats,as0
                        for j in range(exp0.dimension):
                	    likFrac,Gpa=as0.getLikFrac(ret=2,dim=j,exp0=exp0)
                            at=ats[j]
			    if usetrip:at.resAs.triplet[(expnum,i,k)]=as0
			    ##if likFrac>0.05:
			    if Gpa>0.02:
			      sig=as0.peakDevs[j]
			      dba=at.resAs.peakAssDBA
			      key=(tuple(askey[:j]+askey[j+1:]),expnum)
			      if not dba.has_key(key):
                                G_expect=exp0.getGexpt(ats,j)
                                print 'G_expect',G_expect,ats,j
                                dba[key]=([],1-G_expect+min_pFalse)
                                #TODO aendre 0.1 - hvad med taet paa diag...
			      tup=(pos[j],Gpa,i,expnum,sig,likFrac)
			      dba[key][0].append(tup)
			      print 'ALL:',i,k,j,(pos[j],likFrac,i,expnum,sig,Gpa,askey[:j]+askey[j+1:]),exp0.name
##			    at.resAs.peakAssList.append((pos[j],likFrac,i,expnum,sig,Gpa,
##                                                         ats[:j]+ats[j+1:],askey[:j]+askey[j+1:]))
                            ##pt=at.pointer
                            ##sig=exp0.shiftDamps[j]
                            ##if at.element=='C':
                            ##    self.pointerList[pt].peakAssList.append(\
                            ##        (pos[j],likFrac,i,expnum,sig,ptlst[:j]+ptlst[j+1:],pos))
                            ##elif at.element=='N':
                            ##    self.pointerListN[pt].peakAssList.append(\

    def redoResAssignments(self,exp0,expnum,min_pFalse=0.1,usetrip=True):
        for i in range(len(exp0.allassigns)):
            asd=exp0.allassigns[i]
            if asd.use in [1,2,5]:
	      pnoise=exp0.peakData.peaklist[i].probnoise
              for j in range(exp0.dimension):
	  	asdpj=asd.projassigns[j]
                for k,asp in enumerate(asdpj):
                  likFrac,Gpa=asp.getLikFrac(ret=2)
                  ##if likFrac>0.05:#TODO put i input-fil
                  if Gpa>0.02:#TODO put i input-fil
                    ats=asp.atoms
                    at=ats[0]
                    pos=exp0.peakData.peaklist[i].shifts
		    sig=asp.peakDev
		    dba=at.resAs.peakAssDBA
		    key=(j,expnum)
		    if not dba.has_key(key):
		      G_expect=exp0.getGexpt(ats,j)
                      dba[key]=([],1-G_expect+min_pFalse)
		    tup=(pos[j],Gpa,i,expnum,sig,likFrac)
		    dba[key][0].append(tup)
		    print 'ALL:',i,k,j,(pos[j],likFrac,i,expnum,sig,Gpa),exp0.name,dba[key][1]

    def pickleData(self):
        Cmat=[];Nmat=[]
        for i in range(len(self.pointerList)):
            Cmat.append(self.pointerList[i].peakAssList)
        for i in range(len(self.pointerListN)):
            Nmat.append(self.pointerListN[i].peakAssList)
        mats=[Cmat,Nmat]
        save_dba2(mats,'CNmats.pck') 

    def regeneratePeakAssLists(self):
        Cmat,Nmat=read_dba('CNmats.pck')
        for i in range(len(self.pointerList)):
            self.pointerList[i].peakAssList=Cmat[i]
            ##at=self.shiftVC.atreflist[i]
            ##print at,Cmat[i]
        for i in range(len(self.pointerListN)):
            self.pointerListN[i].peakAssList=Nmat[i]

    def replaceShifts(self,showall=1):
        difflst=[]
        adifflst=[]
        relhilst=[]
        #TODO: finde iterator der passer til _Atom()
        ##atomlist=self.shiftVC.atreflist
        ##for k in self.otherShiftVCs.keys():
        ##    if k[0]<>'C':
        ##        atomlist+=self.otherShiftVCs[k].atreflist
        ##for at in atomlist:
	for at in self.allAtomList:
	##for at in self.pdb.iter_atoms():
	    ##if at.element=='C':
		if at.resAs.pdf.form<>'invalid':
                    if len(at.resAs.peakAssDBA.keys())>0:
                        diff=at.resAs.update(Show=1,Showpdf=showall)
			if diff<>None:
                          difflst.append(diff)
                          adifflst.append(abs(diff))
                          relhilst.append(at.resAs.relhi)
			  ##print 'tkeys:',at.resAs.triplet.keys()
        return difflst,adifflst,relhilst

    def summarizeTripletAssignments(self,explst=None):
	savepath=''##/opt/jakob/ubiquitin/tripletData/'
	for ires in range(len(self.pdb[0])):
	    trip=self.allTriplets[ires]
	    trip.showtrip()
	    save_dba2(trip,savepath+'trip'+str(ires+1)+'.pck')
	for ires in range(1,len(self.pdb[0])-1):
	    trip=    read_dba(savepath+'trip'+str(ires+1)+'.pck')
	    prevtrip=read_dba(savepath+'trip'+str(ires+1-1)+'.pck')
	    posttrip=read_dba(savepath+'trip'+str(ires+1+1)+'.pck')
	    prevtrip.showtrip()
	    posttrip.showtrip()
	    trip+=prevtrip
	    trip+=posttrip
	    trip.showtrip()
	    trip.writetrip(explst)#TAKEBACK resshift
	    save_dba2(trip,savepath+'trip'+str(ires+1)+'all.pck')
	##del self.allTriplets[len(self.pdb[0])-1]
	##del self.allTriplets[0]
	##save_dba2(self.allTriplets,'allTriplets.pck')

    def replaceShifts_old(self):
        difflst=[]
        adifflst=[]
        for i in range(len(self.shiftVC.atreflist)):
            showpdf=1
            pt=self.shiftVC.atreflist[i].pointer
            shs=self.shiftVC.atreflist[pt].shift
	    if shs<>None and shs[1]>1:
              print '------------------------',i
              print self.shiftVC.atreflist[i]
              showpdf=1
            resAs=self.pointerList[pt]
            ##diff=resAs.update(self.shiftVC,pt,Show=1,Showpdf=showpdf)
            diff=resAs.update(self.shiftVC,pt,Show=1,Showpdf=showpdf)
            if diff<>None:
                difflst.append(diff)
                adifflst.append(abs(diff))
            if resAs.mean<>None:
                #print resAs.mean,pt
                self.shiftVC.list[i]=[resAs.mean]
                self.shiftVC.stdlist[i]=[resAs.sigma]
		print 'NEW',self.shiftVC.stdlist[i]
        #show()
        newvc=VoxelContainer()
        newvc.vwidth=self.shiftVC.vc.vwidth
        newvc.fillContainer(self.shiftVC.list,tag=0)
        self.shiftVC.vc=newvc#overskriver den gamle
        return difflst,adifflst

    def distanalyze(self,showfile=None,doimshow=False,cors=None,dcut=8.0,returnMat=False,cmat=None):
        ldlst=[]
        dlst=[]
        nuclst=[]
        A=numpy.zeros((70,70))
        Cmat=numpy.ones((70,70))*dcut
	dlst0=[]
	dlst1=[]
	ilst=[]
	minapdct={}
	if cors<>None:
            G=cors[0];G1=cors[1]
            pfile=open('longrange.txt','w')
        for i in range(70):
            A[i][i]=1.0
        for i in range(70):
            for j in range(70):
                dmin=10
                for ati in self.pdb[0][i]:
                  if ati.element=='C':
                  ##if ati.element=='C' and ati.name=='C':
                    for atj in self.pdb[0][j]:
                      if atj.element=='C':
                      ##if atj.element=='H' and atj.name=='H':
                        d=AtomMath.calc_distance(ati,atj)
                        if d<dmin:
                            dmin=d
			    minap=(ati.name,atj.name)
                        if abs(j-i)>=4:# and ati.name=='CA' and atj.name=='CA':print i,j,d
                            if d<6.0:
                                if i>j and ati.shift<>None and atj.shift<>None:
                                    print i+1,j+1,abs(j-i),j+i,ati.name,atj.name,round(d,2),
                                    if cors<>None:
                                        shi=ati.shift[0];shj=atj.shift[0]
                                        ci=shi-G[0]-G1[0]*shi
                                        cj=shj-G[0]-G1[0]*shj
                                        print ci,cj
                                        if cors<>None:
                                            pstr=getAsFormat((ati,atj))
                                            pstrm=getAsFormat((atj,ati))
                                            if ci<75:pfile.write('%14s%8.3f%8.3f%6.3f\n'%(pstr,ci,cj,d))
                                            if cj<75:pfile.write('%14s%8.3f%8.3f%6.3f\n'%(pstrm,cj,ci,d))
                                    else:print
                if dmin>dcut:ilum=0.0
                else:
		    Cmat[i,j]=dmin
		    minapdct[(i,j)]=minap
		    ilum=(dcut-max(dmin,3.0))/5.0
		    dlst0.append(i+0.5)
		    dlst1.append(j+0.5)
		    dlst0.append(j+0.5)
		    dlst1.append(i+0.5)
		    ilst.append(ilum**2)
		    ilst.append(ilum**2)
		    if cmat<>None:
                      if j>i+3:
                        print 'TJEK COUNT:',i,j,dmin,int(cmat[i,j])
                if cmat<>None and cmat[i,j]<1 and j>i+3:
                    print 'ZERO',i,j,dmin
                A[i][j]=ilum
                A[j][i]=ilum
        if cmat<>None:
            cvec=[0,0,0,0]
            for I in range(len(cmat)):
                for J in range(I+3,len(cmat[I])):
                    if cmat[I,J]<4:
                        cvec[int(cmat[I,J])]+=1
            print 'count analysis:',cvec
	if returnMat:return Cmat,minapdct
        A=rot90(A)
	scatter(dlst0,dlst1,numpy.array(ilst)*100,'w',alpha=0.5,faceted=False)
	if showfile<>None:
            C=numpy.zeros((70,70))*1.0
            buf=initfil2(showfile)
            for i in range(len(buf)):
               I,J,val=eval(buf[i][0]),eval(buf[i][1]),eval(buf[i][2])
               if I<70 and J<70:
		    CAats=(self.pdb[0][I]['CA'],self.pdb[0][J]['CA'])
        	    prild,prisig=self.dbas.getlogDistMeanAndSd(CAats)
		    priContProb=self.calcVolfacCum(prild,prisig**2,rmin=4.5,n=6.0)
                    C[I][J]=min(val*priContProb*5.0,1.0)
                    ##C[I][J]=min(val*0.1,1.0)
                    ##if C[I][J]>1:print I,J,C[I][J]
            B=rot90(C)
            if doimshow:
                imshow(A,interpolation='nearest',cmap=cm.jet)
                imshow(B,interpolation='nearest',cmap=cm.jet)
                show()
        return 0
        for ati in self.shiftVC.atreflist:
            nuci=string.atoi(ati.fragment_id)
            for atj in self.shiftVC.atreflist:
                nucj=string.atoi(atj.fragment_id)
                if abs(nuci-nucj)>1:
                    d=AtomMath.calc_distance(ati,atj)
                    lr=log(d)
                    ldlst.append(lr)
                    dlst.append(d)
                    nuclst.append(abs(nuci-nucj))
        m,sd=calc_mean_and_stdd(ldlst)
        print m,sd
        hist(dlst,bins=25,normed=1)
        t=numpy.arange(1.6,35,0.2)
        g=exp(-0.5*((t-m)/sd)**2)/sd*math.sqrt(0.5/pi)
        h=logNormal(t,m,sd*1.4)
        plot(t,h,'k--')
        #scatter(nuclst,dlst,alpha=0.05,faceted=False)
        show();return 0

    def calcerr(self,errs):
        lds=[0,0]
	ersum=0.0
	ernum=0
        for j in range(len(errs)):
            er=errs[j][0] #generaliseres med flere mixt.!
	    if er<>2000:
		ersum=ersum+er
		ernum=ernum+1
	return ernum/ersum

    def calcdist(self,vols):
        lds=[0,0]
        for j in range(len(vols)):
            v=vols[j][0] #generaliseres med flere mixt.!
            if v>0:
                ld=(self.cycle.dconst-log(v))/self.cycle.dpow
                lds[j]=min(max(ld,log(3)),log(7))
                #print exp(ld),
        return max(lds)

    def getConlst(self,exp0,silent=1):
        #jeg taenker paa NCACX skal det vaere mere generelt?
        #der skal komme noget med exp0.atnames[k]==None:...
        #TODO: generalisere metode
        conmat=[]
        for I in range(len(self.pdb[0])):
            #TODO:generaliser til andre proteiner med flere chains
            conmat.append([])
            if not silent:print '---------',I,'--------------'
            for i in range(len(self.pdb[0][I])):
                if self.pdb[0][I][i].element==exp0.nucnames[-1]:
                    if not silent:print self.pdb[0][I][i].name
                    if self.pdb[0][I][i].shift==None:
                        'nucleus is not assigned'
		        conmat[I].append([[1.0,None]])
		        #en dim. mere for at tillade flere peaks vedr. samme assignment
##		    elif self.pdb[0][I][i].name==exp0.atnames[-2]:
##		        conmat[I].append([1.0,None])
		    else:
                        shvec=[]
                        skip=0
                        for k in range(exp0.dimension-1):
                            atname=exp0.atnames[k]
                            if atname<>None:
                                ##print I,atname
                                shs=self.pdb[0][I][atname].shift
                                if shs==None:skip=1;break
                                else: shvec.append(shs[0])
                            else:pass #then what?...
                        ati=self.pdb[0][I][i]
                        shvec.append(ati.shift[0])
                        atn1=exp0.atnames[exp0.dimension-2]
                        at1=self.pdb[0][I][atn1]
                        dist=AtomMath.calc_distance(ati,at1)
                        pF=1-self.calcVolfacCum(log(dist),0.05**2,rmin=exp0.rmins[-1],n=6.0)
                        pnot=1-(1-pF)*(1-exp0.pFN)
                        if not silent:print shvec
                        if not silent:print 'skip=',skip,
                        if not skip:skip=exp0.get_pFalseNeg(shvec)
                        if skip==1.0: conmat[I].append([[1.0,None]])
                        else:         conmat[I].append([[pnot,None]])
                        if not silent:print skip
##                      if exp0.atnames[-1]==None:
##		        conmat[I].append([0.75,None])
##		      else:
##                        print self.pdb[0][I][i].name,exp0.atnames[-1]
##                        if self.pdb[0][I][i].name==exp0.atnames[-1]:
##                            conmat[I].append([0.75,None])
##                        else:
##                            conmat[I].append([1.0,None])
		        #TODO:generaliser, hvis det bliver relevant
	    conmat[I].append({})#place-holder for inter-res. peaks
	    ##print I,conmat[I]
        'now put in assignments...'
        for i in range(len(exp0.allassigns)):
            asd=exp0.allassigns[i]
            if asd.use in [1,2,5]:
              ##print i,'*****************'
              for k in range(len(asd.assigns)):
                as0=asd.assigns[k][1]
                gvc0=asd.assigns[k][0]
                #TODO gen. nedenst.
                if gvc0>0:
                    at3=as0.atoms[-1]
                    at2=as0.atoms[-2]
                    fr3=eval(at3.fragment_id)-1
                    fr2=eval(at2.fragment_id)-1
                    frd=abs(fr3-fr2)
                    pt3=at3.pointer2
                    if frd==0:
                        pnot=conmat[fr3][pt3][0][0]
                        thisk=conmat[fr3][pt3][0][1]
                        if thisk==None:
                            asd.assigns[k][1].pnot=pnot
                            conmat[fr3][pt3][0][0]=asd
                            conmat[fr3][pt3][0][1]=k
                        else:
                            print 'TJEK1',thisk,fr3,pt3,asd.assigns[k][1].coords
                            pnot=conmat[fr3][pt3][0][0].assigns[thisk][1].pnot
                            asd.assigns[k][1].pnot=pnot
                            conmat[fr3][pt3].append([asd,k])#NYT
                            #kan give fejl?
                    else:
                        key=(fr2,at2.name,at3.name)
                        conmat[fr3][-1][key]=(asd,k)
        if not silent:
            'now do testing:'
            for I in range(len(self.pdb[0])):
                ci=-1
                obs=0
                print '-----------',I+1,self.pdb[0][I][0].res_name,'-----------'
                ##print conmat[I]
                for i in range(len(self.pdb[0][I])):
                    if self.pdb[0][I][i].element==exp0.nucnames[-1]:
                        ci=ci+1
                        ##print ci
                        thisk=conmat[I][ci][1]
                        if thisk<>None:
                            obs=1
                            asd=conmat[I][ci][0]
                            this_as=asd.assigns[thisk]
                            print this_as[0],this_as[1].shFac,this_as[1].atoms[-1].name
                        else:
                            if conmat[I][ci][0]<>1.0:
                                print ci,'not'
                print I+1,obs
        return conmat

    def getConlst2d(self,exp0,silent=1):
        #virker til et experiment med 2 generelle og ingen ikke-generelle kerner
        #TODO?: generalisere
        conmat=[]
        for I in range(len(self.pdb[0])):
            #TODO:generaliser til andre proteiner med flere chains
            conmat.append([])
            for i in range(len(self.pdb[0][I])):
                if self.pdb[0][I][i].element==exp0.nucnames[-2]:
                    #TODO:generaliseres til det 1. k som har atname[k]==None
                    conmat[I].append([])
                    ##for j in range(i+1,len(self.pdb[0][I])):
                    for j in range(len(self.pdb[0][I])):
                     ##print I,i,j
                     ##print self.pdb[0][I][j].shift,self.pdb[0][I][i].shift
                     if self.pdb[0][I][j].element==exp0.nucnames[-1]:
            	      conmat[I][-1].append([[1.0,None],[1.0,None]])
		      if self.pdb[0][I][j].shift<>None and self.pdb[0][I][i].shift<>None:
                        if i<>j:
                          ati=self.pdb[0][I][i]
                          atj=self.pdb[0][I][j]
                          dist=AtomMath.calc_distance(ati,atj)
                          pF=1-self.calcVolfacCum(log(dist),0.05**2,rmin=exp0.rmins[-1],n=6.0)
                          for k in range(2):
                              if k==0:shvec=[ati.shift[0],atj.shift[0]]
                              else:shvec=[atj.shift[0],ati.shift[0]]
                              pFNS=exp0.get_pFalseNeg(shvec)
                              pnot=1-(1-pF)*(1-exp0.pFN)*(1-pFNS)
                              conmat[I][-1][-1][k][0]=pnot
	    conmat[I].append({})#place-holder for inter-res. peaks
	    print conmat[I]
	    print '+++++++++++++++++'
	#this means that k==0 <=> at[0].pt2<at[1].pt2?
        'now put in assignments...'
        for i in range(len(exp0.allassigns)):
            asd=exp0.allassigns[i]
            if asd.use in [1,2,5]:
              ##print i,'*****************'
              for k in range(len(asd.assigns)):
                as0=asd.assigns[k][1]
                gvc0=asd.assigns[k][0]
                #TODO gen. nedenst.
                if gvc0>0:
                    at3=as0.atoms[-1]
                    at2=as0.atoms[-2]
                    fr3=eval(at3.fragment_id)-1
                    fr2=eval(at2.fragment_id)-1
                    frd=abs(fr3-fr2)
                    pt3=at3.pointer2
                    pt2=at2.pointer2
		    if pt2<pt3:side=0##;Pt2=pt2;Pt3=pt3
		    else:side=1##;Pt2=pt3;Pt3=pt2
		    ##FOER: altid over "pt-diagonalen"
		    ##NU begge steder
                    if frd==0:
			##print 'PTS:',fr2,pt2,pt3
##			print fr3,Pt2,Pt3,conmat[fr3][Pt2][Pt3]
##                        conmat[fr3][Pt2][Pt3][side][0]=asd
##                        conmat[fr3][Pt2][Pt3][side][1]=k
                        pnot=conmat[fr3][pt3][pt2][side][0]
                        thisk=conmat[fr3][pt3][pt2][side][1]
                        if thisk==None:
                            asd.assigns[k][1].pnot=pnot
                        else:
                            print 'TJEK',thisk,fr3,pt3,pt2
                            pnot=conmat[fr3][pt2][pt3][side][0].assigns[thisk][1].pnot
                            asd.assigns[k][1].pnot=pnot
                        conmat[fr3][pt2][pt3][side][0]=asd
                        conmat[fr3][pt3][pt2][side][0]=asd
                        conmat[fr3][pt2][pt3][side][1]=k
                        conmat[fr3][pt3][pt2][side][1]=k
                    ##else:
                    if frd<=1:
##                        if fr3>fr2:
##                            fI=(fr3,fr2);key0=(at3.name,at2.name)
##                            key1=((fr2,at2.name),(fr3,at3.name))
##                        else:
##                            fI=(fr2,fr3);key0=(at2.name,at3.name)
##                            key1=((fr3,at3.name),(fr2,at2.name))
                        fI=(fr3,fr2);key0=(at3.name,at2.name)
                        key1=((fr2,at2.name),(fr3,at3.name))
                        for ki in range(2):
                            if not conmat[fI[ki]][-1].has_key(key0[ki]):
                                conmat[fI[ki]][-1][key0[ki]]={}
                            if not conmat[fI[ki]][-1][key0[ki]].has_key(key1[ki]):
                                #first observation of atom pair
                                conmat[fI[ki]][-1][key0[ki]][key1[ki]]=[[],[]]
                            conmat[fI[ki]][-1][key0[ki]][key1[ki]][ki].append((asd,k))
			    print 'inter',fI,key0[ki],key1[ki][0],key1[ki][1]
##                        if not conmat[fr3][-1].has_key(at3.name):
##                            conmat[fr3][-1][at3.name]={}
##                        conmat[fr3][-1][at3.name][(fr2,at2.name)]=(asd,k)
##			print 'inter',fr3,at3.name,fr2,at2.name
        if not silent:
            'now do testing - skipped:'
            pass
        return conmat

    def analyzeConten(self,conten,explst):
        problst=numpy.ones(len(self.pdb[0]))
	vollst=[]
	vollst2=[]
	vollst3=[]
	volmat=[vollst,vollst2,vollst3]
        for I in range(len(self.pdb[0])):
            print '-----------',I+1,self.pdb[0][I][0].res_name,'-----------'
            ci=-1
            obs=0
            ##print conmat[I]
            for i in range(len(self.pdb[0][I])):
##               if self.pdb[0][I][i].element==explst[n].nucnames[-1]:
               ati=self.pdb[0][I][i]
               if ati.element=='C':#carbon-detected!
                 pti=ati.pointer
                 ci=ci+1
                 print '+++',ci
		 shlst=[]
                 cj=-1
                 for j in range(len(self.pdb[0][I])):
		   if self.pdb[0][I][j].element=='C':
		     cj=cj+1
                     for n in range(len(conten)):
                      conmat=conten[n][0]
                      if conten[n][1]==1:#1 general atom (carbon, CX?) in the experiment
		       if cj==0:
                        ##print ci
                        for K in range(len(conmat[I][ci])):
                          thisk=conmat[I][ci][K][1]
                          if thisk<>None:
                            obs=1
                            asd=conmat[I][ci][K][0]
                            this_as=asd.assigns[thisk]
                            ass=this_as[1]
                            allnot=ass.allNot/(1-ass.GiFi)
                            Gpa=ass.volFac*allnot;panot=1-allnot
                            likRat=Gpa/panot
                            likFrac=Gpa/(Gpa+panot)
                            ##print this_as[1].pnot
                            prob=1-(min(1,1-this_as[0]))*(1-ass.pnot)
			    coords=ass.coords+ass.shiftDiffSigned
                            print prob,likFrac,ass.shFac,ass.atoms[-1].name,\
                                  len(asd.assigns),coords,coords[-1],explst[n].name
                            problst[I]=problst[I]*prob
                            self.pointerList[pti].peakAssList.append((coords[-1],likFrac,n,asd.identifier))
			    shlst.append(coords[-1])
                          else:
                            prob=conmat[I][ci][K][0]
                            if prob<>1.0:
                                print prob,ci,'not'
                                problst[I]=problst[I]*prob
                      elif conten[n][1]==2:
                                kvec=conmat[I][ci][cj][0][1],conmat[I][ci][cj][1][1]
				prob=0
                                for K in range(2):
                                  thisk=kvec[K]
                                  if thisk<>None:
                                    asd=conmat[I][ci][cj][K][0]
                                    this_as=asd.assigns[thisk]
				    ##if this_as[1].pnot<0.6:#one-bond (set parameter)
##				    inds=[[0,2],[2,3],[0,1]]
##				    #if ci==0 and cj==2:#CA,CB
##				    for m in range(len(inds)):
##				     if ci==inds[m][0] and cj==inds[m][1]:
##				      if len(asd.assigns)==1:
##					ide=asd.identifier
##					vol=explst[n].peakData.peaklist[ide].volumes
##					volmat[m].append(vol)
##				      else:volmat[m].append(1.0)
                                    ##print this_as[1].pnot,'thisispnot'
				    newprob=1-(min(1,1-this_as[0]))*(1-this_as[1].pnot)
				    if newprob>prob:max_as=this_as;prob=newprob;maxK=K
				if prob==0:
				    if conmat[I][ci][cj][0][0]<1.0 or conmat[I][ci][cj][1][0]<1.0:
				       if conmat[I][ci][cj][0][0]==1.0 or conmat[I][ci][cj][1][0]==1.0:
                                          pnot=min(conmat[I][ci][cj][0][0],conmat[I][ci][cj][1][0])
				       else:
                                          pnot=max(conmat[I][ci][cj][0][0],conmat[I][ci][cj][1][0])
                                       print pnot,cj,'not',explst[n].name,n
                                       if j>i:problst[I]=problst[I]*pnot
				else:
			      	    if j>i:side=maxK
				    else:side=1-maxK
				    coords=max_as[1].coords+this_as[1].shiftDiffSigned
                                    print prob,max_as[1].shFac,max_as[1].atoms[-1].name,\
                                          max_as[1].atoms[-2].name,len(asd.assigns),\
					  coords,coords[side], explst[n].name,n
				    shlst.append(coords[side])
                                    if j>i:problst[I]=problst[I]*prob
		 print shlst
		 if len(shlst)>10:print 'RMS',len(shlst),calc_mean_and_stdd(shlst)
		 count=0
		 amap={}
                 for n in range(len(conten)):
                    if conten[n][1]==2:
                        key=self.pdb[0][I][i].name
                        if conten[n][0][I][-1].has_key(key):
                            map=conten[n][0][I][-1][key]
                            ##print map
                            for m in map.keys():
                              if not amap.has_key(m):amap[m]={}
                              amap[m][n]=map[m]
##                              for ki in range(2):
##                                tup=map[m][ki]
##                                ##print tup
##                                if tup<>[]:
##                                  for p in range(len(tup)):
##                                    asd,k=tup[p]
##                                    this_as=asd.assigns[k]
##                                    ass=this_as[1]
##                                    likRat=ass.pE_H/ass.pE
##                                    if likRat>1.1:
##                                        count=count+1
##                                        coords=this_as[1].coords+this_as[1].shiftDiffSigned
##                                        print 'INTER',count,p,likRat,this_as[0],ass.shFac,asd.ambgvc,m,\
##                                              len(asd.assigns),coords,coords[1-ki],explst[n].name,n
                 for m in amap.keys():
                     for n in amap[m].keys():
                         for ki in range(2):
                            tup=amap[m][n][ki]
                            if tup<>[]:
                              for p in range(len(tup)):
                                asd,k=tup[p]
                                this_as=asd.assigns[k]
                                ass=this_as[1]
                                allnot=ass.allNot/(1-ass.GiFi)
                                Gpa=ass.volFac*allnot;panot=1-allnot
                                likRat=Gpa/panot
                                likFrac=Gpa/(Gpa+panot)
                                if likFrac>0.03:
                                    count=count+1
                                    coords=ass.coords+ass.shiftDiffSigned
                                    print 'INTER',count,p,likFrac,likRat,Gpa,panot,this_as[0],round(ass.shFac,3),m,\
                                          len(asd.assigns),coords,round(coords[1-ki],3),n
                                    self.pointerList[pti].peakAssList.append((coords[1-ki],likFrac,n,asd.identifier))
            print I,problst[I]

        for I in range(len(self.pdb[0])):
            print I+1,self.pdb[0][I][0].res_name,problst[I]
        ##scatter(range(len(self.pdb[0])),log(problst))
        ##scatter(range(len(self.pdb[0])),log(vollst))
        ##scatter(range(len(self.pdb[0])),log(vollst2),c='r')
        ##scatter(range(len(self.pdb[0])),log(vollst3),c='g')
        ##show()
                              
    def getConmat(self,exp0,silent=1):
        conmat=[]
        for I in range(len(self.pdb[0])):
            #TODO:generaliser til andre proteiner med flere chains
            conmat.append([])
            for J in range(len(self.pdb[0])):
              conmat[I].append([])
              if I<=J-3:
                #only long-range con. are interesting
                for i in range(len(self.pdb[0][I])):
                  if self.pdb[0][I][i].element==exp0.nucnames[0]:
                    conmat[I][J].append([])
                    for j in range(len(self.pdb[0][J])):
                     ##print I,J,i,j
                     ##print self.pdb[0][J][j].shift,self.pdb[0][I][i].shift
                     if self.pdb[0][J][j].element==exp0.nucnames[1]:
		      if self.pdb[0][J][j].shift==None or self.pdb[0][I][i].shift==None:
                       print 'nucleus is not assigned'
		       if exp0.dimension==2:
		          conmat[I][J][-1].append([[1.0,None]])
		       else:
		          conmat[I][J][-1].append(None)
		      else:
                          if exp0.dimension==2:
                              p_not=exp0.pFN
                              conmat[I][J][-1].append([[p_not,None]])
                              ##for k in range(exp0.dimension):
			      shvec=(self.pdb[0][I][i].shift[0],self.pdb[0][J][j].shift[0])
                              pFalseNeg=exp0.get_pFalseNeg(shvec,symm=True)
                              pFN=max(pFalseNeg,exp0.pFN)
			      print shvec,pFN
			      conmat[I][J][-1][-1][0][0]=pFN
                          else:#3d!!!!!!!!
                              conmat[I][J][-1].append([[],[]])
                              for K in range(2):#TODO generaliser?
                                if K==0:X=I;x=i;Y=J;y=j
                                else:X=J;x=j;Y=I;y=i
                                #HUSK I<=J-3 gaelder
                                for k in range(len(self.pdb[0][X])):
                                    atk=self.pdb[0][X][k]
                                    if atk.element==exp0.nucnames[2]:
                                      if atk.shift<>None and k<>x:
                                        atx=self.pdb[0][X][x]
                                        shvec=[0,0,0]
                                        aty=self.pdb[0][Y][y]
                                        shvec[0]=atk.shift[0]
                                        shvec[1]=atx.shift[0]
                                        shvec[2]=aty.shift[0]
                                        intraDist=AtomMath.calc_distance(atx,atk)
                                        #TODO: tag afstande fra database
                                        pFalseNeg=exp0.get_pFalseNeg(shvec)
                                        pFN=max(pFalseNeg,exp0.pFN)
                                        if intraDist<1.8:
                                            pnot=1-exp0.pE1_H*(1-pFN)#1 bond
                                        elif intraDist<2.8:
                                            pnot=1-exp0.pE2_H*(1-pFN)#2 bonds
                                        else:
                                            pnot=1-exp0.pEn_H*(1-pFN)#more bonds
                                        conmat[I][J][-1][-1][K].append([pnot,None])
                                      else: conmat[I][J][-1][-1][K].append([1.0,None])
                                conmat[I][J][-1][-1][K].append([1.0,None])
                                #above for inter-res F1-F2 connectivity
                      ##print I,J,i,j,len(conmat[I][J][-1][-1][0]),len(conmat[I][J][-1][-1][1])
        print 'conmat is now prepared for the specific protein and spectrum' #(via. exclude)
        #------------------------------------------------------------------
        'now put in assignments...'
        for i in range(len(exp0.allassigns)):
            asd=exp0.allassigns[i]
            if asd.use in [1,2,5]:
              print i,'-----------------'
              for k in range(len(asd.assigns)):
                as0=asd.assigns[k][1]
                gvc0=asd.assigns[k][0]
                ats=as0.orderedAtoms
                frvec=[];ptvec=[]
                for n in range(exp0.dimension):
                    frvec.append(eval(ats[n].fragment_id)-1)
                    ptvec.append(ats[n].pointer2)
                if abs(frvec[-2]-frvec[-1])>=3:
                  if exp0.dimension==2 or frvec[0]==frvec[1]:
                    print frvec,ptvec
                    if frvec[-2]<frvec[-1]:
                        Fr1=frvec[-2];Fr2=frvec[-1];Pt1=ptvec[-2];Pt2=ptvec[-1]
                        SI=0
                    else:
                        Fr1=frvec[-1];Fr2=frvec[-2];Pt1=ptvec[-1];Pt2=ptvec[-2]
                        SI=1
                    print Fr1,Fr2,Pt1,Pt2
                    ask=asd.assigns[k][1]
                    if exp0.dimension==2:
                        asdslst=conmat[Fr1][Fr2][Pt1][Pt2]
                    else:
                        asdslst=conmat[Fr1][Fr2][Pt1][Pt2][SI]
                    if exp0.dimension==2:thisPos=0
                    else:thisPos=ptvec[0]
                    if asdslst[thisPos][1]==None:
                        asdslst[thisPos][0]=ask
                        asdslst[thisPos][1]=k#unoedvendig nu
                    else:
                        print thisPos,asdslst[thisPos][1]
                        print 'NEW',thisPos,Fr1,Fr2,k
                        asdslst.append((ask,k))
        if not silent:
            'analyze again:'
            for I in range(len(self.pdb[0])):
                for J in range(len(self.pdb[0])):
                  if I<=J-3:
                    ci=-1
                    for i in range(len(self.pdb[0][I])):
                      if self.pdb[0][I][i].element==exp0.nucnames[0]:
                        ci=ci+1
                        cj=-1
                        for j in range(len(self.pdb[0][J])):
                         if self.pdb[0][J][j].element==exp0.nucnames[1]:
                             cj=cj+1
                             print 'TJEK',I,J,ci,cj,\
                                   len(conmat[I][J][ci][cj][0]),len(conmat[I][J][ci][cj][1])
        return conmat

    def getLikRat(self,asds,exp0,condH0=0):
	if asds==None:return 1.0, 0
        filled=False
        dim=exp0.dimension
        print condH0
        likProd=1.0
	if exp0.dimension==2:
            for n in range(len(asds)):
              if asds[n][1]<>None:
		filled=True
	      else:
		likProd*=asds[n][0]
	else:
	  for k in range(2):
            for n in range(len(asds)):
              if asds[k][n][1]<>None:
		##filled=True;break
		likProd*=asds[k][n][0].likRat
	      else:
		likProd*=asds[k][n][0]
	if filled:
	    print 'correlation is observed' 
	    return asds[0][0].likRat,1
	else:
	    if likProd==1.0: return likProd, 0
	    elif likProd>1.0:return likProd, 1
            if condH0:pnot=exp0.pH_H0
            else:pnot=exp0.pE_H
            LikRat=1-pnot*(1-likProd)*(1-exp0.pFNcorr)
	    print 'correlation is not observed' 
	    return LikRat, 0

    def getLikRat_old(self,asds,exp0,condH0=0):
##         if asds[0][0][0] in [1.0,None] and asds[1][0][0] in [1.0,None]:
         filled=[0,0]
         dim=exp0.dimension
         for side in range(2):#generaliseres?
             for n in range(len(asds[side])):
                 if asds[side][n][1]<>None:filled[side]=1;break
         if filled==[0,0]:
            print 'no side is observed'
##            if asds[0][0][0]==1.0 and asds[1][0][0]==1.0: LikRat=1.0
##            elif asds[0][0][0]==1.0 or asds[1][0][0]==1.0:
##            oldLikRat=1-(1-0.5)*(1-0.30)*(1-0.4)
##            oldLikRat2=1-(1-0.5)*(1-0.30**2)*(1-0.4)
            if condH0:pnot=exp0.pH_H0
            else:pnot=exp0.pE_H
            pAllNot=1.0
            for side in range(2):
                for n in range(len(asds[side])):
                    pAllNot=pAllNot*asds[side][n][0]
            LikRat=1-pnot*(1-pAllNot)*(1-exp0.pFNcorr)
##            print 'compare',LikRat,oldLikRat,oldLikRat2
            return LikRat,0
##         elif asds[0][0][0] in [1.0,None] or asds[1][0][0] in [1.0,None]:
         elif filled in [[1,0],[0,1]] or dim==3:
            if dim<3:
                print 'one side is observed'
                if filled==[1,0]:
                    print len(asds[0][0][0].pointerMat)
                    sides=[0]
                else:
                    print len(asds[1][0][0].pointerMat)
                    sides=[1]
            else:
                print str(dim)+'d data'
                filled=[1,1];sides=[0,1]
            ##if not asds[0][0][0] in [1.0,None]:
            finalLikRat=1.0
            pAllE_H=1.0
            pAllE=1.0
            for side in sides:
                print 'COUNT',len(asds[side])
                for n in range(len(asds[side])):
                    this_k=asds[side][n][1]
                    if dim==3:print 'DIM3',n,this_k,len(asds[side]),len(asds[1-side])
                    if this_k==None:
                        LikRat=asds[side][n][0]
                        pAllE_H=pAllE_H*LikRat
                    else:
                        this_as=asds[side][n][0].assigns[this_k]
                        if asds[1-side][0][0]==1.0 or dim==3:
                            '3d data or 2d+ carbonyl only on one side'
                            ##LikRat=this_as[0]/this_as[1].volFac
                            LikRat=this_as[1].pE_H/this_as[1].pE
                            pAllE_H=pAllE_H*this_as[1].pE_H
                            pAllE=pAllE*this_as[1].pE
                            #beregnes paa den "gamle maade"
                        else:
                            ide=asds[side][n][0].identifier
                            lvol=log(exp0.peakData.peaklist[ide].volumes)
                            DIFFPROB=comStandardNormal(-lvol/2)
                            pE_HnoN=1-(1-this_as[1].pE_H)/(1-exp0.pN)
                            pEnoN=1-(1-this_as[1].pE)/(1-exp0.pN)
                            pD_H=1-(1-DIFFPROB)*(1-exp0.pE_H)
                            pD=1.0
                            for m in range(len(asds[side][n][0].assigns)):
                                asm=asds[side][n][0].assigns[m][1]
                                GmFm=asm.GiFi
                                pD=pD*(1-(1-DIFFPROB)*GmFm)
                                if m<>this_k:
                                    pD_H=pD_H*(1-(1-DIFFPROB)*GmFm)
                            pU_H=(1-(1-pD_H)*(1-exp0.pDiff))
                            pU=(1-(1-pD)*(1-exp0.pDiff))
                            pEf=1-(1-pEnoN*pU)*(1-exp0.pC)
                            pE_Hf=1-(1-pE_HnoN*pU_H)*(1-exp0.pC)
                            ##pE_Hf=1-(1-pE_Hf)*(1-0.5*pEf)
                            LikRat=pE_Hf/pEf
                    finalLikRat=finalLikRat*LikRat
            if dim==3:
                finalLikRat=(1-(1-exp0.pH_H0*pAllE_H)*(1-(1-exp0.pH_H0)*pAllE))/\
                             pAllE
            print 'LikRat1',finalLikRat
            return finalLikRat,1
         else:
            print 'both sides are observed'
            print 'COUNT2',len(asds[0]),len(asds[1])
            #det ser ud til at man kun for een peak paa hver side
            #ret flg. betingelse i rankInitAssigns(...) for at aendre paa det:
            #if stdsize>dampsize or ide==dlst[0][2]:...
            lst0=asds[0][0][0].pointerMat
            lst1=asds[1][0][0].pointerMat
            k0=asds[0][0][1]
            k1=asds[1][0][1]
            alllst=[]
            for m in range(len(lst0)):
                ip=lst0[m]
                if ip in lst1:
                    m2=lst1.index(ip)
                    alllst.append((m,m2))
            print alllst
            FH0=asds[0][0][0].assigns[k0][1].shFac
            FH1=asds[1][0][0].assigns[k1][1].shFac
            pE_H2not=1-0.01**2*(1-exp0.pE_H*FH0*FH1)
            pE2not=1-0.01**2
            for mp in alllst:
               Gi=asds[0][0][0].assigns[mp[0]][1].volFac
               #Gi=asds[1][0].assigns[mp[1]][1].volFac
               #above should be the same..
               Fi0=asds[0][0][0].assigns[mp[0]][1].shFac
               Fi1=asds[1][0][0].assigns[mp[1]][1].shFac
               if mp[0]<>k0 and mp[1]<>k1:
                   pE_H2not=pE_H2not*(1-Gi*Fi0*Fi1)
               pE2not=pE2not*(1-Gi*Fi0*Fi1)
            pE_H0=asds[0][0][0].assigns[k0][1].pE_H
            pE0=  asds[0][0][0].assigns[k0][1].pE
            pE_H1=asds[1][0][0].assigns[k1][1].pE_H
            pE1=  asds[1][0][0].assigns[k1][1].pE
            pE_H_final=1-pE_H2not*(1-pE_H2not*pE_H0*pE_H1)
            pE_final=1-pE2not*(1-pE2not*pE0*pE1)
            LikRat=pE_H_final/pE_final
            print 'LikRat',LikRat
            print len(alllst),len(lst0),len(lst1),len(asds[0][0][0].assigns),len(asds[1][0][0].assigns)
            return LikRat,2

    def fillAssignDBA(self,explst):
        updtyp='max' #default
        if 'updatemethod' in self.inpmap:
            updtyp=self.inpmap['updatemethod']
	print explst
        exp0=explst[0]
        if exp0.rangeType=='intra':
            conmat=[]
            for i in range(len(explst)):
                #getConlst kan bruges for NCACX, NCACO og NCOCX
                #der er kun eet k med explst[i].atnames[k]==None (nemlig i F3)
                numGenNuc=0
                k1=-1
                for k in range(explst[i].dimension):
                    if explst[i].atnames[k]==None:
                        numGenNuc=numGenNuc+1
                        if k1==-1:k1=k
                        #skal det bruges til noget?
                if numGenNuc in[0,1]:
                    conlst=self.getConlst(explst[i])
                    conmat.append((conlst,1))
                elif numGenNuc==2:
                    conlst2d=self.getConlst2d(explst[i])
                    conmat.append((conlst2d,2))
                print explst[i].name
            self.analyzeConten(conmat,explst)
        #------------------------------------------------------------
        elif exp0.rangeType=='long-range':
            print 'updating contact probabilities...'
            ##DIFFPROB=0.5
    	    T_0=time.time()
	    obsmat=[[],[]]
            if len(explst)>1:
                conmat1=self.getConmat(explst[1],silent=1)
		print 'timechek:3d conmat produced in:',time.time()-T_0
            C=numpy.zeros((len(self.pdb[0]),len(self.pdb[0])))
            conmat=self.getConmat(exp0,silent=1)
	    print 'timechek:2d conmat produced in:',time.time()-T_0
            for I in range(70):
                for J in range(I+3,70):
                    if abs(I-J)>=3 and not I in [7,8,9,10] and not J in [7,8,9,10]:
			obscount=0
                        if updtyp=='NB':#"Naive Bayes": product of likelihoods
                            thisprod=1.0
                        else: thisprod=0.1
                        for i in range(len(conmat[I][J])):
                            thisprodi=1.0
                            for j in range(len(conmat[I][J][i])):
                                 print '----------',I,J,i,j
                                 asds=conmat[I][J][i][j]
				 print 'asds',asds
                                 LikRat,obs=self.getLikRat(asds,exp0)
				 print LikRat,obs
				 obscount+=obs
                                 if obs>0 and len(explst)>1:
                                     asds3d=conmat1[I][J][i][j]
				     print 'asds3d',asds3d
                                     LikRat3D,obs3D=self.getLikRat(asds3d,explst[1],condH0=1)
                                     print 'LikRat3D',LikRat3D, obs3D
                                 else:LikRat3D=1.0
                                 #print conmat[I][J][i][j],
                                 thisprodi=thisprodi*LikRat*LikRat3D
                                 if updtyp=='max':thisprod=max(thisprod,LikRat*LikRat3D)
                            if updtyp=='NB':thisprod*=max(0.75,thisprodi)
                        cai=self.pdb[0][I]['CA']
			caj=self.pdb[0][J]['CA']
                        cadist=AtomMath.calc_distance(cai,caj)
                        ##prild,prisig=self.dbas.getlogDistMeanAndSd(CAats)
                        ##priContProb=self.calcVolfacCum(prild,prisig**2,rmin=4.5,n=6.0)
                        print 'IJ:',I,J,thisprod,cadist,obscount
			obsmat[min(obscount,1)].append(cadist)
                        ##C[I][J]=min(thisprod*priContProb*5.0,1.0)
                        C[I][J]=thisprod
                        ##print 'postH',I,J,C[I][J]
            print 'I,J=',I,J
            ##B=rot90(C)
            ##imshow(B,interpolation='nearest');show()
##            subplot(121)
##            title('non-observed contacts')
##	    hist(numpy.array(obsmat[0]))
##            subplot(122)
##            title('observed contacts')
##	    hist(numpy.array(obsmat[1]))
##	    pylab.show()
	print 'timechek:likelihood matrix produced in:',time.time()-T_0
	mat=[list(v) for v in C]
	print '_-'*20
	print mat
	print '_-'*20
	return C

    def final_analyze(self,maxshort=6):
        ldifflst=[]
        ldsum=0.0
        ldsum2=0.0
        estdba={}
        truedba={}
        pos=0.0 #(s,s)
        falsepos=0.0 #(l,s)
        posfalses=0.0 #(l,l)
        falses=0.0 #(s,l)
        totnum=len(self.checklist[0])
        for i in range(totnum):
            trued=self.checklist[0][i]
            estd=self.checklist[1][i]
            if trued<=maxshort and estd<=maxshort:
                pos=pos+1
            elif trued<=maxshort and estd>maxshort:
                falses=falses+1
            elif trued>maxshort and estd>maxshort:
                posfalses=posfalses+1
            elif trued>maxshort and estd<=maxshort:
                falsepos=falsepos+1
            eind=int(log(estd)*5)
            if not estdba.has_key(eind):estdba[eind]=[[],[]]
            estdba[eind][0].append(trued)
            estdba[eind][1].append(estd)
            tind=int(log(trued)*5)
            if not truedba.has_key(tind):truedba[tind]=[[],[]]
            truedba[tind][0].append(trued)
            truedba[tind][1].append(estd)
            ldiff=log(trued)-log(estd)
            ldifflst.append(ldiff)
            ldsum=ldsum+abs(ldiff)
            ldsum2=ldsum2+ldiff**2
        save_dba2(estdba,'estdba')
        save_dba2(truedba,'truedba')
        print '---------'
        print pos/totnum,posfalses/totnum,falses/totnum,falsepos/totnum
        ldm,ldsd=calc_mean_and_stdd(ldifflst)
        print ldm,ldsd
        print ldsum/len(self.checklist[0]),math.sqrt(ldsum2/len(self.checklist[0]))
        scatter(log(numpy.array(self.checklist[0])),
                log(numpy.array(self.checklist[1])),
                s=25*numpy.ones(len(self.checklist[0])),
                alpha=0.2, faceted=False)
        scatter(log(numpy.array(self.checklist[0])),
                log(numpy.array(self.checklist[2])),
                s=25*numpy.ones(len(self.checklist[0])),c='r',
                alpha=0.2, faceted=False)
        t=numpy.arange(1.6,35,5)
        plot(t,t,'k--')
            #print self.checklist2[0][i],self.checklist2[1][i],ldiff
        print truedba.keys()
        if 0:
            scatter(diffmat[0],diffmat[-1])
            t=numpy.arange(-0.3,0.3,0.02)
            plot(t,t,'k--')
            c1,c2,c3=0.0,0.0,0.0
            for d in adifflst:
                if d<0.1:c1=c1+1
                if d<0.2:c2=c2+1
                if d<0.3:c3=c3+1
            print c1/len(adifflst),c2/len(adifflst),c3/len(adifflst)
        #print len(adiffmat[0]),len(adiffmat[-1])
        #hist(numpy.array(adiffmat[0])-numpy.array(adiffmat[-1]),bins=20)
        #hist(adifflst,bins=20,normed=1)
        #hist(adiffmat[0],bins=20,normed=1,width=0.015)
        #show()
        save_dba2(self.checklist,'checklist')

    def visualize(self,newAllassigns,show=1,cmat=None):
##        A=numpy.ones((70,70))
##        C=numpy.ones((len(self.pdb[0]),len(self.pdb[0])))
        C1=numpy.ones((len(self.pdb[0]),len(self.pdb[0])))
        A=self.AllNotMat
        C=self.AllNotMatw
        for i in range(len(newAllassigns)):
          if newAllassigns[i]<>None:
            num=len(newAllassigns[i])
	    ##fac=min(newAllassigns[i][1].sum_pPosts,1.0)
	    sum=0.0
	    for j in range(num):sum+=newAllassigns[i][j][1].pPost
	    fac=min(sum,1.0)
	    if fac==0.0:print 'fac==0.0!'
	    else:
	      print 'fac is:',fac
              for j in range(num):
                ass=newAllassigns[i][j][1]
                atoms=ass.atoms
                rtype=getType(atoms)
                ##print rtype,
                if rtype>0:
                    fr0=string.atoi(atoms[0].fragment_id)-1
                    fr1=string.atoi(atoms[1].fragment_id)-1
                    ##print fr0,fr1,(1-1.0/num),A[fr0][fr1]
##                    at0=atoms[0].name
##                    at1=atoms[1].name
##                    res0=atoms[0].res_name
##                    res1=atoms[1].res_name
                    thisgvc=ass.pPost
                    A[fr0][fr1]*=(1-1.0/num)
                    A[fr1][fr0]*=(1-1.0/num)
                    C[fr0][fr1]*=((1-thisgvc/fac))
                    C[fr1][fr0]*=((1-thisgvc/fac))
                    if num==1 and rtype>4:
                        print 'single:',fr0,fr1,thisgvc,A[fr0][fr1]
                        ass.show_assign(False)
        B=numpy.ones((70,70))-A
        D=C1-C
##        for i in range(70):
####                B[i][i]=1.0
####                D[i][i]=1.0
##                for j in range(70):#truncation:
####                    if B[i,j]>=0.2:B[i,j]=0.2
##                    if D[i,j]>=0.2:D[i,j]=0.2
        if show:
            D=rot90(D)
            B=rot90(B)
            if cmat==None:
                subplot(121)
                title('probability weighted - no truncation')
                self.distanalyze()
                imshow(D,interpolation='nearest')
                subplot(122)
                title('frequency weighted')
                self.distanalyze()
                imshow(B,interpolation='nearest')
            else:
                E=rot90(cmat)
##                subplot(221)
##                title('probability weighted - no truncation')
##                self.distanalyze()
##                imshow(D,interpolation='nearest')
##                subplot(222)
##                title('frequency weighted')
##                self.distanalyze()
##                imshow(B,interpolation='nearest')
##                subplot(223)
##                title('count matrix')
##                self.distanalyze(cmat=cmat)
##                imshow(E,interpolation='nearest')
                title('count matrix')
                self.distanalyze(cmat=cmat)
                imshow(E,interpolation='nearest')
            pylab.show()
	return B,D

    def output(self,csname,talosname,asname=None,
               pasdshiftname='ubq_darr2d_pass1.shiftAssignments'):
        csfile=open(csname,'w')
        csfilepasd=open(pasdshiftname,'w')
        asfile=open(asname,'w')
        talosfile=open(talosname,'w')
        count=0
        for at in self.allAtomList:
              pasdid=at.res_name+at.fragment_id+at.name
              count+=1
              shiftfinal=at.resAs.max
              stdfinal  =at.resAs.sigma
              if stdfinal==None:stdfinal=0.1
              tup=(count,at.fragment_id,at.res_name,at.name,
                   at.element,shiftfinal,stdfinal)
              print tup
              tuptal=(at.fragment_id,aa31dict[at.res_name],at.name,shiftfinal)
              print tuptal
              #print tuptal
              if stdfinal<>None:
                  lintal='%4s %1s    %4s     %8.2f'%tuptal
                  lin='%4d%4s%5s%5s%3s%8.2f%8.3f 1'%tup
                  #print lin
                  #print shlst[i],stdlst[i],at.shift
                  csfile.write(lin+'\n')
                  talosfile.write(lintal+'\n')
              #----------now the PASD file---------
              ##pasdid=at.res_name+at.fragment_id+at.name
              pasdid=at.getPASDid()
              if shiftfinal<80:sides=['from','to']
              else:sides=['to']
              for s in sides:
                  thisid=pasdid+s
                  csfilepasd.write('shiftAssignment '+thisid+'\n')
                  selstr='    -protonSelection (resid '+at.fragment_id+\
                          ' and resn '+at.res_name+\
                          ' and name '+at.name+')\n'
                  csfilepasd.write(selstr)
                  csfilepasd.write('    -'+s+'\n') #{'-from'|'-to'}
                  csfilepasd.write('    -protonShift %6.2f\n'%shiftfinal)
                  csfilepasd.write('end\n')
                  csfilepasd.write('\n')
              #----------end of PASD stuff---------
        csfile.close()
        talosfile.close()
        csfilepasd.close()
        return 0
        writemat=[]
        for i in range(len(newAllassigns)):
            aslst=newAllassigns[i]
            if aslst<>[]:
                vols=self.peakData.peaklist[i].volumes
                writemat.append((len(aslst),aslst,vols))
        writemat.sort()
        prevlen=1
        sumc=0.0
        sumLRc=0.0
        sumAllLRc=0.0
        gensumAllLRc=0.0
        sumviol=0.0
        for i in range(len(writemat)):
            LR=0
            length,aslst,vols=writemat[i]
            tup0=(i,length,newvol(vols[0][0]),newvol(vols[1][0]))
            lin='%4d%3d%9.3f%9.3f'%tup0
            thissumviol=0.0
            thisLR=0.0
            for j in range(len(aslst)):
                ats=aslst[j][1].atoms
                dist=AtomMath.calc_distance(ats[0],ats[1])
                tupj=(ats[0].fragment_id,ats[0].name,
                      ats[1].fragment_id,ats[1].name,dist)#,aslst[j][0])
                linj='%6s%4s%4s%4s%5.1f'%tupj
                lin=lin+linj
                diff=abs(eval(ats[0].fragment_id)-eval(ats[1].fragment_id))
                if diff>4:
                    LR=1
                    thisLR=thisLR+1
                    if dist>6.5:
                        thissumviol=thissumviol+1
            if thisLR>0:sumviol=sumviol+thissumviol/thisLR
            asfile.write(lin+'\n')
            if length>prevlen:
                #print prevlen,sumLRc,sumc
                sumAllLRc=sumAllLRc+sumLRc
                gensumAllLRc=gensumAllLRc+sumLRc/prevlen
                sumLRc=0.0
                prevlen=length
                sumc=1.0
            if LR:sumLRc=sumLRc+1
            sumc=sumc+1
        print sumAllLRc,gensumAllLRc,sumviol/len(writemat),len(writemat)

    def write_history(self):
        ##for at in self.pdb.iter_atoms():
	for at in self.allAtomList:
            if at.resAs.pdf.form<>'invalid':
              his=at.resAs.history
              usedMean=his[2]
              if usedMean<>None:
                name=his[3]+his[4]+'.dba'
                save_dba2((his[0],his[1],his[2]),name)

    def updateAllResAss(self):
        for at in self.allAtomList:
            name=at.fragment_id+at.name+'.dba'
	    if 'usestdev' in self.inpmap:
                if self.inpmap['usestdev']<>'auto':
                    usestd=eval(self.inpmap['usestdev'])
                else:
                    usestd=_Atom.getstd(at)
	        at.shift=(at.shift[0],usestd)
            try:
                ##his=read_dba('refined/'+name)
                his=read_dba(name)
                usedMean=his[2]
		newsh=at.shift[0]
        	print newsh,usedMean
##                at.resAs.getpdf(his[0][-1],1,0,add=0.0,thisUsed=usedMean,showMarkers=False)
                at.resAs.getpdf(his[0][-1],1,0,add=0.0,thisUsed=newsh,showMarkers=False)
            except IOError:
                print 'WARNING: no data for',at.fragment_id+at.name

    def gethistory(self,fr,nam,showits='all',plotcol='k'):
        ##at=self.pdb[0][fr][nam]
        for at in self.allAtomList:
            if at.fragment_id==fr and at.name==nam:break
        name=fr+nam+'.dba'
        his=read_dba(name)
        usedMean=his[2]
	newsh=at.shift[0]
	if 'usestdev' in self.inpmap:
            if self.inpmap['usestdev']<>'auto':
                usestd=eval(self.inpmap['usestdev'])
            else:
                usestd=_Atom.getstd(at)
	    at.shift=(at.shift[0],usestd)
        print newsh,usedMean
        print his[1]
        if showits=='all':     inds=range(len(his[0]))
        elif showits=='final': inds=[len(his[0])-1]
        elif showits=='first': inds=[0]
	plotcols='bgmckwyr'
	##inds=inds[:-1]
        for i in inds:
##            at.resAs.getpdf(his[0][i],i+1,i+1,add=0.0,thisUsed=usedMean,showMarkers=True,pdfcol=plotcol)
##            at.resAs.getpdf(his[0][i],i+1,False,add=0.0,thisUsed=usedMean,showMarkers=False,pdfcol=plotcol)
            at.resAs.getpdf(his[0][i],i+1,True,add=0.0,thisUsed=usedMean,showMarkers=i==inds[-1],pdfcol=plotcols[i])

    def testDistances(self):
	dct={}
	for i in range(7)+range(10,70):
	    carbi=self.pdb[0][i]['C']
	    cdct={}
	    for at in self.pdb.iter_atoms():
		if at.element=='C' and at.name<>'C':
		    if not at.name in cdct:cdct[at.name]=0
            	    d=AtomMath.calc_distance(carbi,at)
		    ##if at.name=='CB' and d<5:print i+1,eval(at.fragment_id),d
		    if d<=3.5:
		        ##print i,at.fragment_id,at.name,d
			cdct[at.name]+=1
	    for atn in cdct:
		if not atn in dct:dct[atn]={}
		cnt=cdct[atn]
		if not cnt in dct[atn]: dct[atn][cnt]=0
		dct[atn][cnt]+=1
	print dct
	##for nam in ['CA','CB','CG','CG1','CG2']:
	for nam in ['CB']:
	  dct={}
	  for i in [6,11,13,21,54,65]:
           try:
	    refati=self.pdb[0][i][nam]
	    c=0
	    for at in self.pdb.iter_atoms():
		if at.element=='C':# and at.name=='C':
            	    d=AtomMath.calc_distance(refati,at)
		    if d<=4.5:
			if abs(eval(at.fragment_id)-i-1)>2:
			  print i+1,at.fragment_id,at.name,d,at.shift
			c+=1
	    if not c in dct: dct[c]=0
	    dct[c]+=1
	   except KeyError:pass
	  print nam,dct
	1/0
	    

def getinpmap(inname,type=None):
    buf=initializer(inname)
    im={}
    for x in buf:
        x=string.split(x,sep=',')
        for y in x:
          if len(y)>1:
            ##print string.split(y,sep='=')
            var,val=string.split(y,sep='=')
            var=string.strip(var)
            val=string.strip(val)
            if var[:8]=='specvars':val=getinpmap(val);im[var]=val#;print im[var]
	    elif val[0]=='(' and val[-1]==')':
		val=string.split(val[1:-1])
	        if type=='float':val=[string.atof(vi) for vi in val]
		print 'list value:',var,val
	    elif type=='float':val=string.atof(val)
            im[var]=val
    if 'state' in im:state=im['state']
    else:state=None
    if state<>None:
      if state[:2] in ['ss','SS']:defaults=getinpmap('vsdefaults_ssnmr.inp')
      elif state[:3] in ['liq','Liq']:defaults=getinpmap('vsdefaults_liqnmr.inp')
      elif state[:3] in ['liq','Liq']:defaults=getinpmap('vsdefaults_liqnmr.inp')
      for i in range(1,99):
	if 'specvars'+str(i) in im:
	  im['specvars'+str(i)].update(defaults)
	  print i,im['specvars'+str(i)]
	else:break
    return im

def getExperiment(spv):
    s=spv['spectype']
    if s in ['DipCor3Ali','Dipcor3Ali','dipcor3ali','Dipcor3ali','DipCor3dAli']:
        return NMR3DcccDipCorAli()
    elif s in ['darr','DARR','Darr','Dipcor','dipcor','DipCor','Dipcor2','DipCor2']:
        return NMRccDipCor()
    elif s in ['NOESY','1H1HNOESY']:
        return NOESY()
    elif s in ['HNOESY','HNNOESY']:
        return HNOESY()
    elif len(s)>4 and 'occ7' in s:
	print 'using special vars for occ7',s#TODO generalize
        expm=NMRccDipCor()
	expm.exclude=[((75,200),(0,200)),((75,170),(0,200)),((0,200),(75,200))]# exclude also carbonyl
	if s=='occ71':expm.diagCut=5.0
	return expm
    elif s in ['NCACX','ncacx','NCaCx']:
        return ncacx()
    elif s in ['NCOCX','ncocx','NCOCx']:
        return ncocx()
    elif s in ['NCACO','ncaco','NCaCO','NCaCo']:
        return ncaco()
    elif s in ['NCACON','ncacon','NCaCON','NCaCoN']:
        return ncacon()
    elif s in ['NCACBCON','ncacbcon','NCaCbCON','NCaCbCoN']:
        return ncacbcon()
    elif s in ['CACONCACO','caconcaco','CaCONCaCO','CaCoNCaCo']:
        return caconcaco()
    elif s in ['CACBCONCACBCO','cacbconcacbco','CaCbCONCaCCbO','CaCbCoNCaCbCo']:
        return cacbconcacbco()
    elif s in ['NcaCX','n(ca)cx','N(CA)CX']:
        return NcaCX()
    elif s in ['CANCO','canco','CaNCO','CANC']:
        return canco()
    elif s in ['CONCA','conca','CONCa','CNCA']:
        return conca()
    elif s in ['CONCX','concx','CNCX']:
        return concx()
    elif s in ['CANCX','cancx','CaNCX']:
        return cancx()
    elif s in ['NCACB','ncacb','NCaCb']:
        return ncacb()
    elif s in ['NCApCX','ncapcx']:
        return ncapcx()
    elif s in ['NCOiCX','ncoicx']:
        return ncoicx()
    elif s in ['HNCA','hnca']:
        return hnca()
    elif s in ['HNCAp','hncap']:
        return hncap()
    elif s in ['HNCAc','hncac']:
        return hncac()
    elif s in ['HNCACB','HNCB','hncacb','hncb']:
        return hncb()
    elif s in ['HNCACBp','HNCBp','hncacbp','hncbp']:
        return hncbp()
    elif s in ['HNCACBc','HNCBc','hncacbc','hncbc']:
        return hncbc()
    elif s in ['N15HSQC','15NHSQC']:
        return n15hsqc()
    elif s in ['HNCO','HNCo']:
        return hnco()
    elif s in ['HCO','HCo']:
        return hco()
    elif s in ['CAHN','CaHN']:
        return cahn()
    elif s in ['C13HSQC','13CHSQC']:
        return c13hsqc()
    elif s in ['C13HSQCb2','13CHSQCb2','C13HSQCB2','13CHSQCB2']:
        return c13hsqcb2()
    elif s in ['N15HSQC_TOCSY','15NHSQC_TOCSY']:
        return n15hsqc_tocsy()
    elif s in ['N15HSQC_NOESY','15NHSQC_NOESY']:
        return n15hsqc_noesy()
    elif s in ['C13HSQC_NOESY','13CHSQC_NOESY']:
        return c13hsqc_noesy()
    else:
	print 'getting generic experiment',s
	expm=Generic_Experiment()
	expm.setVars(spv)
	return expm
         
def getdata(buf,noeval=0):
    if not noeval:del buf[0]
    peaklst=[];peaklst2=[]
    tmap={};tmap2={}
    coords=[]#for checking...
    uis=[]
    for i in range(len(buf)):# buf -> tmap, peaklst, uis, (coords)
        if not noeval:buf[i]=eval(buf[i])
        coords.append(array(buf[i][6]))
        if buf[i][3] in [7,8] and len(buf[i])>5:#TODO: generaliser index for 3d CCC
            p=array(buf[i][6])
            p=sort(p)
            pts=array(buf[i][5])
            pts=sort(pts)
            tkey=tuple(pts)
            pd=pts[1]-pts[0]
            if not tmap.has_key(tkey):tmap[tkey]=[]
            peaklst.append((p[0],p[1],p[2],pd,buf[i][4],i))
            tmap[tkey]=tmap[tkey]+[len(peaklst)-1]
        elif buf[i][3] in [3,4,5,6] and len(buf[i])>5:#TODO: generaliser index for buildup
            p=array(buf[i][6])#shifts
            tkey=buf[i][5][0]#pointer
            peaklst2.append((p[0],p[1],tkey,buf[i][4],buf[i][3],i))
            if not tmap2.has_key(tkey):tmap2[tkey]={}
            mkey=buf[i][3]
            if not tmap2[tkey].has_key(mkey):tmap2[tkey][mkey]=[[],[]]
            if p[0]>p[1]:
                tmap2[tkey][mkey][0].append(len(peaklst2)-1)
            else:
                tmap2[tkey][mkey][1].append(len(peaklst2)-1)
        else:uis.append(i)
        buf[i]=buf[i][:5]
    return tmap,tmap2,peaklst,peaklst2,uis,coords

def allignData(P,buf,tmap,uis,coords,doscatter=0,dCluster=0.75,dStrip=1.0):
    #allign 3d Data
    kms='so^>v<dph8'
    ##for tkey in [(65,67)]:#tmap.keys():
    for tkey in tmap.keys():
          inds=array(tmap[tkey])
          print inds
          if len(inds)<2:
            I=int(P[inds[0]][-1])
            uis.append(I)
          else:
            data1=take(P,inds)
            V=take(data1,arange(3),axis=1)
            C=take(data1,[3],axis=1)
            print len(V)
            vc=VoxelContainer()
            vc.vwidth=[1.0,1.0,1.0]
            vc.fillContainer(V)
            ##print V[0]
            if doscatter:scatter(V[:,0],V[:,1],c=V[:,2],s=ones(len(V))*100,alpha=0.75,faceted=False)#,marker=kms[n%len(kms)])
            ##if doscatter:scatter(V[:,1],V[:,0],c=cols[j],alpha=0.75,faceted=False,marker='s')
            pcc=getClusters(vc,distMetricVec,None,dCluster)
            pcc.stripAll(dStrip,distMetricVec,None)
            keys=pcc.c2i.keys()
            allis=[]
            maxlst=[]
            print len(keys),'clusters'
            for i in range(len(keys)):
                k=keys[i]
                ilst=pcc.c2i[k]
                allis=allis+ilst
                T=take(V,ilst)
                if doscatter:scatter(T[:,0],T[:,1],c='k',alpha=0.95,faceted=False,marker=kms[i%10])
                blst=[]
                for j in range(len(T)):
                    I=int(P[inds[ilst[j]]][-1])
                    ##print j,I
                    blst.append((buf[I][1],I))
                blst.sort()
                ##print blst
                maxi=blst[-1][1]
                maxlst.append(maxi)
                shs=coords[maxi]
                shs=sort(shs)
                if doscatter:scatter([shs[0]],[shs[1]],c='r',alpha=0.95,faceted=False,marker=kms[i%10])
            singles=[]
            for i in range(len(V)):
                if not i in allis:
                    singles.append(V[i])
                    I=int(P[inds[i]][-1])
                    maxlst.append(I)
                    ##shs=coords[I]
                    ##shs=sort(shs)
                    ##scatter([shs[0]],[shs[1]],c='c',alpha=0.95,faceted=False,marker='s')
            S=array(singles)
            if doscatter and len(S)>0:scatter(S[:,0],S[:,1],c='w',alpha=0.75,faceted=False,marker='8')
            print maxlst
            uis=uis+maxlst
    return uis

def plotDataFromSuperClusterContainer(scc,buf,coords,Vmat,uis,doscatter=0):
    kms='so^>v<dph8'
    cols='bcrk'
    keys=scc.c2i.keys()
    allis=[]
    maxlst=[]
    ##print Vmat
    char=scc.getCharacteristics(distMetricVec,None)
    for c in char:print c
    for i in range(len(keys)):
          blst=[]
          k=keys[i]
          print '------------key:',k
          xlst=scc.getElems(k)
          T=[]
          ##print xlst
          for lst in xlst:
              for x in lst:
                  print x
                  side=x[2]
                  pk=Vmat[x[1]][x[2]][x[0]]
                  #pk=scc.cclst[x[1]].vcs[x[2]].list[x[0]]
                  print pk,
                  if scc.countmap==None:listInd=x[1]
                  else:
                      ks=scc.countmap.keys()
                      ##print ks,x[1]
                      listInd=ks.index(x[1])
                  vals=scc.cclst[listInd].vcs[x[2]].vallst[x[0]]
                  I=int(vals[0])
                  print buf[I]
                  blst.append((buf[I][1],I))
                  if side==0:T.append(pk)
                  else:T.append((take(pk,[1,0])))
          T=array(T)
          if doscatter:scatter(T[:,0],T[:,1],s=ones(len(T))*35,c='y',alpha=1.00,faceted=False,marker=kms[i%10])
          blst.sort()
          ##print blst
          maxi=blst[-1][1]
          maxlst.append(maxi)
          cos=char[i][0]
          if abs(buf[maxi][0]-cos[0])<1.0:buf[maxi][0]=cos[0]
          else:buf[maxi][0]=cos[1]
          ##TODO: find bedre loesning...
          shs=coords[maxi]
          shs=sort(shs)
          if doscatter:scatter([shs[1]],[shs[0]],s=ones(1)*20,c='k',alpha=0.95,faceted=False,marker=kms[i%10])
##            singles=[]
##            for i in range(len(V)):
##                if not i in allis:
##                    singles.append(V[i])
##                    I=int(P[inds[i]][-1])
##                    maxlst.append(I)
##                    ##shs=coords[I]
##                    ##shs=sort(shs)
##                    ##scatter([shs[0]],[shs[1]],c='c',alpha=0.95,faceted=False,marker='s')
##            S=array(singles)
##            if doscatter:scatter(S[:,0],S[:,1],c='w',alpha=0.75,faceted=False,marker='8')
          print 'maxlst',maxlst
    uis=uis+maxlst
    print 'UIS',uis
    return uis

def allignData2(P,buf,tmap,uis,coords,doscatter=0):
    part=Partition(half,2,highest)
    kms='so^>v<dph8'
    cols='bcrk'
    for tkey in tmap.keys():
    ##for tkey in [38]:#tmap.keys():
      print 'tkey:',tkey
      map=tmap[tkey]
      print map
      keys=map.keys()
      cclst=[]
      Vmat=[]
      for j in range(len(keys)):
          cclst.append(None)
          Vmat.append(None)
      startIndex=0
      for j in range(len(keys)):
          inds=array(map[keys[j]][0]),array(map[keys[j]][1])#2dim
          print inds
          print '-------',j,keys[j]
          sumlen=0
          side=None
          for n in range(2):
              sumlen=sumlen+len(inds[n])
              if len(inds[n])>0:side=n
          print 'sumlen',sumlen
##          if sumlen<2:
##            ##I=int(P[inds[n][0]][-1])
##            ##uis.append(I)
##            data1=take(P,inds[side])
##            V=take(data1,arange(2),axis=1)
##            Va[side]=V
##            #Va[1-side] is still None
##          else:
          if 1:
            vclst=[None,None]
            Va=[None,None]
            for n in range(2):
                vc=VoxelContainer()#er tom nu
                vc.partition=part#forsoeges...
                print 'inds[n]:',inds[n]
                if len(inds[n])>0:
                    data1=take(P,inds[n])
                    print 'DATA1',data1
                    V=take(data1,arange(2),axis=1)
                    vals=take(data1,[5,3],axis=1)
                    Va[n]=V
                    ##print len(V)
                    vc.vwidth=[1.0,1.0]
                    vc.fillContainer(V)
                    vc.vallst=vals
                    ##print vc.list
                    if doscatter:scatter(V[:,n],V[:,1-n],s=ones(len(V))*175,c=cols[keys[j]-3],alpha=0.75,faceted=False,marker=kms[n])
                vclst[n]=vc
            pcc=getClusters2(vclst,distMetricVec,None,0.35,sym=1,tag=j,Index0=startIndex)#pair cross-diagonal peaks
            cclst[j]=pcc
            Vmat[j]=Va
            print pcc.i2c
            startIndex=pcc.index
      ##show()
      scc=combineClusters(cclst,distMetricVec,None,0.4)
      for c in scc.c2i.keys():
          print c,scc.c2i[c]
      ##print scc.i2c
      scc.stripAll2(0.5,distMetricVec,None)
      plotDataFromSuperClusterContainer(scc,buf,coords,Vmat,uis,doscatter=doscatter)
      print '********************************************'
    ##show()
    return uis

def removeCorrelatedPeaks(X2,thr=0.25,d0=0.2,dStrip=0.3,dAgg=0.7):
    #TODO: overvej defaults...
    V=take(X2,[0],axis=1)
    inds=X2[:,4]<thr
    indsc=X2[:,4]>=thr
    vlst=[];ilst=[]
    vlstc=[];ilstc=[]
    for i in range(len(X2)):
        if X2[i][4]<thr:vlst.append(X2[i][:1]);ilst.append(i)
        else:vlstc.append(X2[i][:1]);ilstc.append(i)
    vc=VoxelContainer()
    vc.vwidth=[d0*2.01]
    ##vc.fillContainer(V[inds])
    vc.fillContainer(vlst)
    print len(vlstc),'broad peaks'
    print len(vlst),'anchor peaks'
    if len(vlst)==0:
        return removeCorrelatedPeaks(X2,thr=thr*1.1,dStrip=dStrip*1.1)
        ##return X2, [(X2,1.0)],0
    vcc=VoxelContainer()
    vcc.vwidth=[d0*2.01]
    ##vcc.fillContainer(V[indsc])
    vcc.fillContainer(vlstc)
    
    for k in vc.container.keys():
        print k,len(vc.container[k]),vc.container[k]
    #-----------------------------------
    
    pcc=getClusters1d(vc,distMetricVec,None,d0)
    pcc.stripAll(dStrip,distMetricVec,None)
    pcc.appendSingles()
    keys=pcc.c2i.keys()
    print len(keys),'clusters'
    char=pcc.getCharacteristics(distMetricVec,None)
    pcca=pcc.getAggregates(vcc,distMetricVec,None,dAgg)
    for ch in char:print ch

    Xjs=[]
    maxlst=[]
    for j in range(len(keys)):
        k=keys[j]
        print pcc.c2i[k]
        print pcca.c2i[k]
        blst=[]
        for i in pcc.c2i[k]:
            I=ilst[i]
            blst.append((X2[I][1],I))
        for i in pcca.c2i[k]:
            I=ilstc[i]
            blst.append((X2[I][1],I))
        blst.sort()
        print blst[len(blst)-6:]
        print len(blst)
        print blst[0]
        maxi=blst[-1][1]
        maxlst.append(maxi)
        ##X2[maxi][0]=char[j][0]
        Xj=[]
        for i in range(len(blst)):Xj.append(X2[blst[i][1]])
        Xjs.append((Xj,blst[-1][0]))
    print maxlst
    return take(X2,maxlst),Xjs,len(keys)
    

def stripData(buf):
    tmap,tmap2,peaklst,peaklst2,uis,coords=getdata(buf,noeval=1)
    if len(buf)<=1:
	print 'only',len(buf),'peak to strip'
	return buf,[(buf,1.0)],1
    X=array(buf)
    P=array(peaklst);P2=array(peaklst2)
    uis=allignData(P,buf,tmap,uis,coords,doscatter=0)
    uis=allignData2(P2,X,tmap2,uis,coords,doscatter=0)
    X2=take(X,uis)
    print len(X2),'peaks after allignment'
    if len(X2)<=1:
	print 'only',len(X2),'peak to strip after allignment'
        return X2,[(X2,1.0)],1
    X3,Xjs,num=removeCorrelatedPeaks(X2)
    return X3,Xjs,num
  
def testPDF(filename,Marker):
    buf=initfil(filename)
    vec=string.split(buf[0])
    xr=eval(vec[0]);x0=eval(vec[1]);lw=eval(vec[2])
    #initialize data
    tmap,tmap2,peaklst,peaklst2,uis,coords=getdata(buf)
    X=array(buf)
    ##for key in tmap.keys():print key,len(tmap[key])
    for key in tmap2.keys():print key,len(tmap2[key]),tmap2[key]
    kms='so^>v<dph8'
    mlst=[]
    P=array(peaklst);P2=array(peaklst2)
    ##scatter(P[:,0],P[:,1],c=(P[:,2]-40)%4,s=25/P[:,4],alpha=0.5,faceted=False)
    ##scatter(P2[:,0],P2[:,1],c=(P[:,2]-40)%4,s=25/P[:,4],alpha=0.5,faceted=False)
    ##show()
    #allign data
    uis=allignData(P,buf,tmap,uis,coords,doscatter=0)
    ##show()
    ##uis=allignData2(P2,buf,tmap2,uis,coords,doscatter=1)
    uis=allignData2(P2,X,tmap2,uis,coords,doscatter=1)
    ##show()
    ##return 0
    ##show()
    ##for i in range(len(buf)-1,-1,-1):
    ##    if buf[i][1]<0.12:del buf[i]
    X2=take(X,uis)
    ##show()
    X3,Xjs,num=removeCorrelatedPeaks(X2)
    ##pdf,pmean,psig,hi,entrop=getGeneralizedMeanSd2(X2,x0,lw,showpdf=1)
    ##pdf,pmean,psig,hi,entrop=getGeneralizedMeanSd2(X3,x0,lw,showpdf=1)
    pdf,pmean,psig,hi,entrop=getGeneralizedMeanSd2(Xjs,x0,lw,showpdf=1,mode=2)
    print pmean,xr,psig,hi,entrop
    ##scatter(X[:,0],X[:,1],c=X[:,3]/8.0,s=12.0/X[:,4],alpha=0.5,faceted=False,marker=Marker)
    scatter(X2[:,0],X2[:,1],c=X2[:,3]/8.0,s=12.0/X2[:,4],alpha=0.6,faceted=False,marker=Marker)
    scatter(X3[:,0],X3[:,1],c=X3[:,3]/8.0,s=40.0/X3[:,4],alpha=0.2,faceted=False,marker=Marker)
    ##show()

def stripAllPeaks(explst):
    explst[0].stripPeaks([explst[5]],pair=(1,2),tol=0.7)
    explst[1].stripPeaks([explst[5]],pair=(1,2),tol=0.7)
    explst[2].stripPeaks([explst[5]],pair=(1,2),tol=0.7)
    explst[3].stripPeaks([explst[4]])
    explst[4].stripPeaks([explst[5]])
    explst[5].stripPeaks([explst[6]])
    explst[7].stripPeaks([explst[8]],pair=(0,1,2),sym=1,tol=0.7)#no sym
    explst[7].stripPeaks([explst[3]],pair=(0,1),tol=0.7)
    explst[7].stripPeaks([explst[4]],pair=(1,2),tol=0.7)
    explst[8].stripPeaks([explst[5]],pair=(0,1),tol=0.7)
    explst[8].stripPeaks([explst[5]],pair=(1,2),tol=0.7)


def getAssignSet(modelAsslst):
    asslst=[]
    keylst=[]
    for s in modelAsslst:
        ass=_Assignment2(s)#unsorted!
        ##print ass
        asslst.append(ass)
        key=ass[:]
        key.sort()
        keylst.append(tuple(key))
    return asslst,Set(keylst)

def getAssignSet2(assigns):
    asslst=[]
    keylst=[]
    for i in range(len(assigns)):
        ass=assigns[i][1]
        key=ass.atoms[:]
        asslst.append(key)
        key.sort()
        keylst.append(tuple(key))
    return asslst,Set(keylst)

def getAssignSet3(assigns):
    if assigns==None:return [],Set([])
    asslst=[]
    keylst=[]
    for i in range(len(assigns)):
        key=list(assigns[i])
        asslst.append(key)
        key.sort()
        keylst.append(tuple(key))
    return asslst,Set(keylst)

def isTrueAlign(s,t,specname=None):
    for sp in s:
      for sat in sp:
	for tp in t:
	  for tat in tp:
	    if sat==tat:
	      if specname==None:return True,sat
	      elif sat.name==specname:return True,sat
    return False,None

def allIntersect(sets):
    if len(sets)==1:return sets[0]
    elif len(sets)==0:return set([])#empty
    if isinstance(sets[0],set):
      S=set(sets[0])
    elif isinstance(sets[0],Set):
      S=Set(sets[0])
    else:S=sets[0]
    for i in range(1,len(sets)):
	S&=sets[i]
    return S

def allUnion(sets):
    if len(sets)==1:return sets[0]
    elif len(sets)==0:return set([])#empty
    if isinstance(sets[0],set):
      S=set(sets[0])
    elif isinstance(sets[0],Set):
      S=Set(sets[0])
    else:S=sets[0]
    for i in range(1,len(sets)):
	S|=sets[i]
    return S

class BuildUpPeakVector(Peak_sparky):

    def __init__(self,c):
      self.dba=c[-1]
      self.stripNonConsecutive(c[-2])
      self.shifts=c[0]
      self.meand=c[1]
      self.rmsd=c[2]
      self.maxd=c[3]
      self.minexp=min(self.dba.keys())
      self.maxexp=max(self.dba.keys())
      self.shiftsSTD=self.getShiftsSTD()
##      self.shifts,self.sumdDivli,self.rmsd,self.maxd,minc,
##      maxc,mini,maxi,self.numPeaks,self.dba=c

    def stripNonConsecutive(self,defnum):
      print self.dba
      mnums=self.dba.keys()
      mnums.sort()
      mnums.reverse()
      print mnums
      prevmnum=mnums[0]
##      self.numPeaks=0
      self.numPeaks=defnum
      if len(mnums)>1:
          if mnums[-2]-mnums[-1]>1:
              if len(self.dba[mnums[-1]].keys())==1:
                  del self.dba[mnums[-1]]
                  print 'deleting mnum:',mnums[-1]
                  self.numPeaks-=1
##          for i,mnum in enumerate(mnums[1:]):
##              if prevmnum-mnum>1:
##                  print 'deleting mnum:',mnum
##                  del self.dba[mnum]
##              else:
##                  prevmnum=mnum
##                  for side in self.dba[mnum]:
##                      for I in self.dba[mnum][side]:
##                          self.numPeaks+=1

    def putError(self,sumh,pNs,pLR=0.001):
        noiseHeight=sqrt(0.1**2+sumh**2)
        minerr=min(take(pNs,self.dba.keys()))*0.1
        ##print minerr
        if noiseHeight>3:hErr=0.9;print 'probably an error'
        else:hErr=comStandardNormal(log(noiseHeight)*1.6)
	print 'esimating error using:',hErr
        self.hErrProb=min((1.0-(1.0-hErr)*(1.0-minerr))**self.numPeaks+pLR,0.5)
        print 'old errors:',
        allErr=1.0
        for peaklet in self.oldPeaks:
            print peaklet.hErrProb,
            allErr*=peaklet.hErrProb
        self.hErrProb=min(allErr+pLR,0.5)
        print

    def getShiftsSTD(self):
        li=self.numPeaks
        if li==1:return self.defaultShiftSTD
        elif li<4:
            return self.rmsd
        else:
            return sqrt((li-1.0)/(li-3.0)/li*self.rmsd**2)

    def getVolume_and_oldPeaks(self,explst,scc):
      allAssPoslst=[]
      self.maxAssSet=Set([])
      vols=[]
      self.oldPeaks=[]
      for mnumi in self.dba:
        dbai=self.dba[mnumi]  
        for side in dbai.keys():
          for I in dbai[side]:
              oldi=scc.cclst[mnumi].vcs[side].vallst[I]
              peaklet=explst[mnumi].peakData.peaklist[oldi]
              self.oldPeaks.append(peaklet)
	      peaklet.mixnum=mnumi
      mnum=self.minexp
      dba0=self.dba[mnum]
      for side in dba0.keys():
          for I in dba0[side]:
              oldi=scc.cclst[mnum].vcs[side].vallst[I]
	      print len(explst[mnum].peakData.peaklist),mnum,oldi
              peaklet=explst[mnum].peakData.peaklist[oldi]
              vols.append(peaklet.volumes)
              try:
                  if peaklet.names<>None:
                      self.maxAssSet|=getAssignSet3([peaklet.names])[1]
              except AttributeError:pass
              if isinstance(peaklet,Peak_sparky_Virtual):
                  if peaklet.allAssigns<>None:
		    asslst,assigns=getAssignSet3(peaklet.allAssigns)
		    ##peaklet.allAssigns is a list of tuple of _Atom instances
		    allAssPoslst.append(assigns)
              else:
                try:
                  if len(peaklet.modelAsslst)>0:
		    asslst,assigns=getAssignSet(peaklet.modelAsslst)
		    allAssPoslst.append(assigns)
		except AttributeError:pass
      self.volumes=average(array(vols))
      self.assSet=allIntersect(allAssPoslst)
      if isinstance(self,BuildUpPeakVector2D):
          print 'TJEK:',self.shifts,self.assSet
      single=False
      print 'tjeknum',self.numPeaks,len(self.oldPeaks),self.maxAssSet,peaklet.names
      if self.numPeaks==1:
	##TODO: hvad med 3D?
	single=True
	if isinstance(self,BuildUpPeakVector2D):
	    sh0=self.oldPeaks[0].shifts
	    if explst[mnum].get_pFalseNeg((sh0[1],sh0[0])):single=False
      if single:
        lvol=log(vols[0])
        self.pDiff=comStandardNormal(-lvol/2)#gamle DIFFPROB
      else: self.pDiff=None

    def showPeaks(self,explst,scc):
      print self.dba
      r=scc.cclst[0].vcs[0].partition.repr
      allAssPoslst=[]
      mind=999.0
      for mnum in self.dba.keys():
          for side in self.dba[mnum].keys():
              for I in self.dba[mnum][side]:
                  pk=scc.cclst[mnum].vcs[side].list[I]
                  oldi=scc.cclst[mnum].vcs[side].vallst[I]
                  peaklet=explst[mnum].peakData.peaklist[oldi]
		  if DEBUG:
                      if peaklet.mindist<mind:mind=peaklet.mindist
                  if isinstance(peaklet,Peak_sparky_Virtual):
                      print peaklet.allAssigns
                      if peaklet.allAssigns<>None:
                        asslst,assigns=getAssignSet3(peaklet.allAssigns)
                        ##peaklet.allAssigns is a list of tuple of _Atom instances
                        allAssPoslst.append(assigns)
                  else:#Peak_sparky
                    try:
                      print peaklet.modelAsslst
                      if len(peaklet.modelAsslst)>0:
                        asslst,assigns=getAssignSet(peaklet.modelAsslst)
                        allAssPoslst.append(assigns)
                      else:asslst=[None]
        	    except AttributeError:asslst=[None]
                  if distMetricVec(pk,r(peaklet.shifts),None)>0.001:
                      print 'WARNING:matching old peaks could be an error!'
                      print pk,
                      print mnum,side,I,oldi,
                      print r(peaklet.shifts)
      allAssPos=allIntersect(allAssPoslst)
      if DEBUG:
          print 'STAT:',self.rmsd,self.numPeaks,len(allAssPoslst),mind,
          print allAssPos
          if len(allAssPoslst)==0:
              return 0,[None],'NOISE'
          elif len(allAssPos)==0:
            print 'EMPTY INTERSECTION!!!',self.rmsd
            return 0,asslst,'EMPTY INTERSECTION'
          else:return len(allAssPos),asslst,allAssPos
      else:return 0,[],None
      
    def stripTest(self,exp0,vc,peaklist,tol,pairlist,accepts,pdb=None,**kwargs):
        #TODO: generalize...
        shs=self.shifts
        xErr=0.0
	##print 'stripping peak',self,shs,self.dba#,kwargs
	if 'reverse' in kwargs:rev=kwargs['reverse']
	else:rev=False
        if len(self.dba)==1 or rev:
            k=self.dba.keys()[0]
            ##print 'single mixing time:',k,
            if len(self.dba[k])==1 or rev:
                side=self.dba[k].keys()[0]
                ##print 'only one side:',side
                shs=reOrder(shs,side)
                if 'scc' in kwargs:
                    return Peak_sparky.stripTest(self,exp0,vc,peaklist\
                        ,tol,pairlist,accepts,pdb=pdb,shifts=shs,scc=kwargs['scc'],explst=kwargs['explst'],rev=rev)
                else:
                    return Peak_sparky.stripTest(self,exp0,vc,peaklist\
                        ,tol,pairlist,accepts,pdb=pdb,shifts=shs,rev=rev)
	return False

    def getPeakDevs(self,exp0):
        print self.numPeaks,'peak(s) in peak cluster'
        return sqrt(self.shiftsSTD**2+self.mergeSTD**2)*ones(exp0.dimension)

    def getRmin(self,exp0):
        I=self.minexp
        self.vnormc=exp0.vnormc[I]
        return exp0.rminMat[I]

    def getDampings(self,exp0=None,dim=None):
        if exp0<>None:
            return sqrt(self.shiftsSTD**2+self.mergeSTD**2)*ones(exp0.dimension)
        elif dim<>None:
            return sqrt(self.shiftsSTD**2+self.mergeSTD**2)*ones(dim)
        else:
            return sqrt(self.shiftsSTD**2+self.mergeSTD**2)
        
    def writeRestraints(self,file,skipfile=None,pasdresfile=None,name='',resc=None,
                        hasinfo=False,dmax=None,dmin=0.0,includeLowProbs=False,xplorfile=None):
        dmaxs=[3.0,4.0,5.0,5.5,6.0,6.5]
        minrtyp=1000
        mindist=1000
        if dmax==None:dmax=dmaxs[self.minexp]
        dest=['from','to']
        if not hasinfo:hasinfo=self.restraintContainInformation()
	print 'vols',
	hdct={}
        for peaklet in self.oldPeaks:
	    print peaklet.height,peaklet.mixnum
	    if not peaklet.mixnum in hdct:
		hdct[peaklet.mixnum]=peaklet.height
	    else:hdct[peaklet.mixnum]=(hdct[peaklet.mixnum]+peaklet.height)/2#kun for 2d!
	print hdct
        asd=self.assignData
        ##sumpp=min(asd.sum_pPosts,1.0)
        sumpp=min(asd.sum_pPosts+asd.probNoise,1.0)
	print 'sumpp =',sumpp
	if includeLowProbs:
            maxNum=asd.zeroNum
	else:
            maxNum=asd.numAccepted
        for n in range(maxNum):
            asn=asd.assigns[n][1]
            ##ats=asn.atoms
            ats=asn.orderedAtoms
            rType=getType(ats,pair=(-2,-1))
            minrtyp=min(minrtyp,rType)
##        if asd.numAccepted>0 and minrtyp>4:
        thisentropy=0.0
        for n in range(maxNum):
            asn=asd.assigns[n][1]
            thispp=asn.pPost/sumpp
            if thispp>0:thisentropy-=(thispp*log(thispp))
        if maxNum==0:
            print 'no accepted assignments'
            hasinfo=False
        elif sumpp<=0.0:
            print 'no assignments with good chemical shift agreement'
            hasinfo=False
	if skipfile==None or hasinfo:
            outfile=file
        else:outfile=skipfile
        if outfile<>'none':##hasinfo or skipfile<>None:
            minshfac=0.0
            lastvol=0.0
            if len(asd.assigns)>0:
                minshfac=asd.assigns[0][1].shFac
                try:lastvol=hdct[2]/hdct[0]
		except KeyError:lastvol=-1.0
                ##try:lastvol=self.oldPeaks[-1].volumes
		##except AttributeError:lastvol=1.0
            lin='%4d%7.3f%7.3f%7.2f%7.2f%6.3f%3d%8.3f'%\
                 (maxNum,1-asd.probNoise,exp(thisentropy),
                  self.shifts[0],self.shifts[1],minshfac,
                  self.numPeaks,lastvol)
            lin2=''
            print lin
            if hasinfo and skipfile<>None:
##                pasdresfile.write('restraint '+name+'\n')
                pasdresfile.write('peak '+name+'\n')
##                pasdresfile.write('    -bounds %4.2f %4.2f\n'%(dmin,dmax))
                pasdresfile.write('    -bounds %4.2f %4.2f\n'%(dmax,dmin))
                pasdresfile.write('    -fromProtonShift %6.2f\n'%self.shifts[-2])
                pasdresfile.write('    -toProtonShift   %6.2f\n'%self.shifts[-1])
                pasdresfile.write('    -note degeneracy %2s\n'%maxNum)
                pasdresfile.write('    -note noiseprob, totprob = %6.4f %6.4f\n'%\
                                  (asd.probNoise,sumpp))
            for n in range(maxNum):
                asn=asd.assigns[n][1]
                ##ats=asn.atoms
                ats=asn.orderedAtoms
                rType=getType(ats,pair=(-2,-1))
                minrtyp=min(minrtyp,rType)
##                dist=AtomMath.calc_distance(ats[-2],ats[-1])
                dist=asn.getRealDist(ats)
                mindist=min(dist,mindist)
                asstr=getAsFormat(ats)
                lin2+='%20s%6.2f%8.5f'%(asstr,dist,asn.pPost/sumpp)
            lrlin=' %2d '%minrtyp
            outfile.write(lin+lrlin+lin2+'\n')
            if hasinfo and skipfile<>None:
                if xplorfile<>None:xplorfile.write('assign ')
                tnote=get_tnote(minrtyp)
                if minrtyp>4:
                    if dmin-1<mindist<dmax+1.0:
                        resc.numgoodlr+=1
                        resc.lst.append(asd)
                    else:
                        resc.numbadlr+=1
                if dmin-1<mindist<dmax+1.0:
                    pasdresfile.write('    -note Good (total) %5.2f\n'%mindist)
                else:
                    pasdresfile.write('    -note Bad (total)  %5.2f\n'%mindist)
                pasdresfile.write('    -note '+tnote+'\n')
                selpairstrlst=[]
                for n in range(maxNum):#asd.assigns[n] is sorted
                    asn=asd.assigns[n][1]
                    ats=asn.orderedAtoms
                    rType=getType(ats,pair=(-2,-1))
                    tnote=get_tnote(rType)
##                    dist=AtomMath.calc_distance(ats[-2],ats[-1])
                    dist=asn.getRealDist(ats)
                    pp=asn.pPost
                    likelihood=pp/sumpp##*(1-asd.probNoise)
##                    asid='    assign '+name+'_'+str(n)
                    asid='    peakAssign '+name+'_'+str(n)
                    for i in [-2,-1]:
                        pasdid=ats[i].getPASDid()+dest[i]
                        asid+=(' '+pasdid)
                    pasdresfile.write(asid+'\n')
                    if dmin-1<dist<dmax+1.0:
                        pasdresfile.write(' '*8+'-good\n')
                        pasdresfile.write(' '*8+'-note Good %5.2f\n'%dist)
                        if rType>5:
                            resc.numgoodlrw+=likelihood
                            resc.numgoodlrwf+=1.0/maxNum
                            resc.lst2.append((likelihood,asn))
                            if minrtyp>5:
                                resc.numgoodlrwr+=likelihood
                                resc.lst2r.append((likelihood,asn))
                    else:
                        pasdresfile.write(' '*8+'-note Bad  %5.2f\n'%dist)
                        if rType>5:
                            resc.numbadlrw +=likelihood
                            resc.numbadlrwf+=1.0/maxNum
                            resc.lst2bad.append(asn)
                            if minrtyp>5:
                                resc.numbadlrwr +=likelihood
                                resc.lst2badr.append((likelihood,asn))
                    pasdresfile.write(' '*8+'-note '+tnote+'\n')
                    #TODO: TJEK er ovenst. OK?
                    pasdresfile.write(' '*8+'-likelihood %10.8f\n'%likelihood)
                    pasdresfile.write(' '*8+'-note shiftprob %10.8f\n'%asn.shFac)
                    pasdresfile.write(' '*8+'-note volprob %10.8f\n'%asn.volFac)
                    pasdresfile.write(' '*4+'end\n')
                    #xplorfile stuf.......#
                    strn=''
                    for i in range(len(ats)):
                            strn+=' (resid %1s and name %4s)'%(ats[i].fragment_id,ats[i].name)
                    selpairstrlst.append(strn)
                if xplorfile<>None:
                    xplorfile.write(selpairstrlst[0])
##                    xplorfile.write(' 4.00 2.00 2.00')
                    xplorfile.write(' %4.2f %4.2f %4.2f'%\
                                    ((dmax+1)/2,(dmax+1)/2,(dmax+1)/2))#upper bound dmax
##                    for strn in selpairstrlst[1:]:xplorfile.write(' OR'+strn)
                    for strn in selpairstrlst[1:]:
			xplorfile.write(' OR\n')
			xplorfile.write(strn)
                    xplorfile.write('\n')
                pasdresfile.write('end\n')
                pasdresfile.write('\n')


class PseudoPeak(BuildUpPeakVector):

    def __init__(self,asd):
        self.assignData=asd
        self.shifts=(-999.0,-999.0)
        if isinstance(asd,_AssignData):
            self.shifts=asd.coords
        self.dimension=2
        self.numPeaks=1

##    def restraintContainInformation(self):
##        return False

        
def get_tnote(minrtyp):
    if minrtyp==0:tnote='intraresidue'
    elif minrtyp==1:tnote='sequential'
    elif minrtyp<=5:tnote='medium range'
    else:tnote='long range'
    return tnote


class BuildUpPeakVector2D(BuildUpPeakVector):
    defaultShiftSTD=0.20
    mergeSTD=0.1

    def restraintContainInformation(self,printcount=True,errmax=0.5):##long-range information
	return True
##        return self.assignData.probNoise<=errmax
        #aendret igen!!!!!!
        asd=self.assignData
        if asd.probNoise>errmax:
            print 'noise', asd.probNoise
            return False
        if len(asd.assigns)==0 or asd.use>2:
            print 'no assignments',self.shifts,asd.probNoise,self.minexp
            return False
        if printcount:print 'asd.numAccepted:',asd.numAccepted
	if self.minexp<=1:
            print 'small dist',self.minexp
            return False
##        if self.oldPeaks[-1].volumes>1000.0:
##            print 'high volume:',self.oldPeaks[-1].volumes
##            return False
	asm=asd.asMax
	if asm==None:
	   print 'no asMax!',asd.identifier
	   ##return False#noise is asMax...
	   return True#noise is asMax...
	ats=asm.atoms
        d=AtomMath.calc_distance(ats[0],ats[1])
        if d<3.0 and asm.pPost>0.25:
            print 'small dist',ats[0],ats[1]
            return False
	return True


class BuildUpPeakVector3D(BuildUpPeakVector):
    defaultShiftSTD=0.3
    mergeSTD=0.2

    def getSideInds(self):
        sides=self.dba[self.minexp].keys()
        inds=[]
        for i in range(len(sides)):
            inds.append(getOrder(sides[i]))
        return inds

    def restraintContainInformation(self):##long-range information
        if (self.minexp==0 and self.numPeaks>4) or \
           self.assignData.probNoise>0.5:return False
	asm=self.assignData.asMax
	if asm==None:
	   print 'no asMax!',self.assignData.identifier
	   ##return False#noise is asMax...
	   return True#noise is asMax...
	ats=asm.orderedAtoms
        rtype1=getType(ats,pair=(0,1))
        rtype2=getType(ats,pair=(1,2))
	if rtype1+rtype2==0:
           d1=AtomMath.calc_distance(ats[0],ats[1])
           d2=AtomMath.calc_distance(ats[1],ats[2])
           if d1<2.0 and d2<3.0 and asm.pPost>0.25:return False
	return True


class BuildUpData(PeakData):

    def __init__(self,scc,explst,dim):
        self.oldexplst=explst
        self.numexp=len(explst)
        self.scc=scc
        self.dimension=dim
        print explst[0].shiftDamps*2.01
	print 'total of:',len(scc.meanvc.list),'peaks in peakvc before rescaling'
        ##self.peakvc=scc.meanvc.getCopy(vwidth=explst[0].shiftDamps*2.01)
        self.peakvc=scc.meanvc.getCopy(vwidth=explst[0].shiftDamps*12.01)
	#TODO:find rigtige vwidth
	print 'total of:',len(self.peakvc.list),'peaks in peakvc'
        self.peaklist=[]
        ##print self.peakvc.list
        print 'initializing...'
        keys=scc.c2i.keys()
	print 'total of:',len(keys),'peaks in cluster'
        for i in range(len(keys)):
          k=keys[i]
          ##print '------------key:',k
          ##xlst=scc.getElems(k)
          ##print xlst
          c=scc.char[i]
          if self.dimension==2:peakVec=BuildUpPeakVector2D(c)
          elif self.dimension==3:peakVec=BuildUpPeakVector3D(c)
          ##print peakVec.shifts,peakVec.shiftsSTD
          peakVec.getVolume_and_oldPeaks(explst,scc)#from smallest mixingtime
          ##peakVec.showPeaks(explst,scc)
          self.peaklist.append(peakVec)
        print 'total of:',len(self.peaklist),'peaks in buildup'

    def estimateErrorProbs(self,pNs,pLR=0.0):
      print 'estimating error probabilities for peak vector...'
      for i in range(len(self.peaklist)):
        peakVec=self.peaklist[i]
        print peakVec.shifts,peakVec.maxexp,self.numexp,peakVec.numPeaks
	skip=0
	errormin=pLR
        if peakVec.maxexp<self.numexp-1:
            skip=1
            if peakVec.numPeaks==1:
              ediff=self.numexp-1-peakVec.maxexp
              peakVec.hErrProb=1-0.1**ediff
	      print 'skipping peak - only one mixing time:',peakVec.maxexp
              ##peakVec.hErrProb+=0.1
              #TODO: findes der noget bedre at goere...?
	      if DEBUG:
                  peaklet=peakVec.oldPeaks[0]
                  S0=peakVec.assSet
                  print 'peakVec Error?:',peaklet.names,
            elif peakVec.numPeaks==2:
                peakVec.hErrProb=0.5
                print 'missing peaks above mixing time:',peakVec.maxexp
	    else:
                skip=0
                S0=Set(None)
                if peakVec.numPeaks<5:
                    errormin=max(0.5**(peakVec.numPeaks-1),pLR)
        else:S0=Set(None)
	if (not skip) and peakVec.numPeaks==1:
            if peakVec.oldPeaks[0].badlyIntegrated:
                peakVec.hErrProb=0.5
                print 'could be an error since peak was not properly integrated'
                skip=1
        if 1:
          dlst=self.peakvc.findNearestFromCoord(\
              peakVec.shifts,distMetricVec,None,maxd=1.0)[1:]
	  if not skip:
            sumh=0.0
            for x in dlst:
              xvol=self.peaklist[x[2]].volumes
              sumh=sumh+(exp(-0.5*(x[0]/0.4)**2)*xvol)
            sumh/=peakVec.volumes
            print 'sumh is:',sumh
            if sumh<0.5:
		print 'low sumh:',sumh
		sumh=0.0
            peakVec.putError(sumh,pNs,errormin)
        print i,peakVec.hErrProb,peakVec.numPeaks,len(dlst),
        if DEBUG:
            if len(dlst)>0:
              di,ci,ni=dlst[0]
              npeaklet=self.peaklist[ni].oldPeaks[0]
              Sn=getAssignSet3(npeaklet.allAssigns)[1]
              print di,ni,npeaklet.names,
            else:Sn=Set([]);print 'real error?',
            if len(S0)==0 and len(Sn)==0:print 'empty',
            else:print 'common',
            try:
                   allass=peakVec.oldPeaks[0].modelAsslst
                   print len(allass),allass
            except AttributeError:print 0,[]
        else:print

    def showErrorProbs(self):
      kms='so^>v<dph8'
      lsts=[[[],[],[],[]] for i in range(7)]
      for peakvec in self.peaklist:
          err=peakvec.hErrProb
          np=min(peakvec.numPeaks-1,6)
          vol=peakvec.volumes
          shs=peakvec.shifts
          lsts[np][0].append(shs[0])
          lsts[np][1].append(shs[1])
          lsts[np][2].append(10*(log(vol)**2))
          lsts[np][3].append(log(err))
      for i in range(len(lsts)):
          scatter(lsts[i][0],lsts[i][1],s=lsts[i][2],c=lsts[i][3],faceted=False,marker=kms[i])
      show()          
        

class BuildUp(Experiment):

    def __init__(self,explst,scc):
##        if explst[0].dimension==2:
##            self.damp0=0.1*ones(explst[0].dimension)
        self.explst=explst
        self.transfer_attributes(explst[-1])
        self.transfer_multiple_attributes(explst)
        self.peakData=BuildUpData(scc,explst,self.dimension)
        ##self.peakData.estimateErrorProbs(self.pNs,self.pLR)
##        self.peakData.showErrorProbs()
        self.setGexpect()

    def transfer_multiple_attributes(self,explst):
        self.rminMat=[]
        self.distCutMat=[]
        self.resInclMat=[]
        self.shiftTolMat=[]
        self.pNs=[]
        self.conPriors=[]
        for exp0 in explst:
            try:self.conPriors+=exp0.conPriors
	    except AttributeError: pass
            self.rminMat.append(exp0.rmins)
            self.shiftTolMat.append(exp0.shiftTols)
            self.distCutMat.append(exp0.distCut)
            self.resInclMat.append(exp0.resInclude)
            self.pNs.append(exp0.pN)

    def transfer_attributes(self,exp0):
        self.hypType  =exp0.hypType
        self.rangeType=exp0.rangeType
        self.dimension=exp0.dimension
	self.nucnames =exp0.nucnames
	self.atnames  =exp0.atnames
	self.multi    =exp0.multi
	self.exclude  =exp0.exclude
	self.diagCut  =exp0.diagCut
	self.structure=exp0.structure
	self.shiftTols=exp0.shiftTols
	self.fromStructure=exp0.fromStructure
        self.globShifts=zeros(exp0.dimension)
        self.resInclude=exp0.resInclude
        self.pLR       =exp0.pLR
        self.name      =exp0.name
        self.noArom    =exp0.noArom
        self.order     =exp0.order
        self.exporder  =exp0.exporder
        self.lineWidth =exp0.lineWidth
        try:
            self.pE1_H =exp0.pE1_H
            self.pE2_H =exp0.pE2_H
            self.pEn_H =exp0.pEn_H
        except AttributeError:
            print 'WARNING: cannot set pEn_H attributes'
        self.restrictpairs=exp0.restrictpairs
        self.ccpairdct =exp0.ccpairdct
        self.bounds    =exp0.bounds
        self.shiftDamps=exp0.shiftDamps
        self.shiftstds =exp0.shiftstds

    def setGexpect(self):
        dkBonds=(1.7,2.6,3.4)
        Gs=ones((self.dimension-1,3))*1.0
        for n in range(self.dimension-1):
          if self.multi[n]<>0:
            for k in range(len(dkBonds)):
                Gs[n,k]=CalcVolfacCum(log(dkBonds[k]),0.15,
                                      rmin=self.rminMat[-1][n],n=6.0)
        print 'Gs are:',Gs
        self.G_expects=Gs

    def getNormCs(self,xstd,mindist=1.53,elr=0.15):
	shnormc=1.0##((self.mergeSTD**2+xstd**2)*2*pi)**(-0.5*self.dimension)
        for k in range(self.dimension):
            shnormc=shnormc/math.sqrt(2*pi*self.shiftDamps[k]**2+xstd**2+self.mergeSTD**2)##+\
                                   ##(self.shiftstds[k]*minstd)**2)#hvad for hypType==2?
                                   ##(self.shiftstds[k]*1.0   )**2)#hvad for hypType==2?
	print 'shnormc',shnormc
	vnormc=ones(len(self.explst))*1.0
	for n in range(len(self.explst)):
            for k in range(self.dimension-1):
                if self.multi[k]>0:
                    vnormc[n]=vnormc[n]*CalcVolfacCum\
                              (log(mindist),elr**2,rmin=self.explst[n].rmins[k],
                               lsigint=1.0,n=6.0)
        self.shnormc=shnormc
        self.vnormc=vnormc

    def getGexpt(self,ats,j):
        G=1.0
        Gs=self.G_expects
        ds=[]
        comblst=comb4(len(ats))
        for pair in comblst:
            if self.multi[pair[0]]==1 or pair[1]-pair[0]>1:
                d=AtomMath.calc_distance(ats[pair[0]],ats[pair[1]])
                ds.append(d)
        if prod(self.multi)>0:
            ds.sort()
            ds=ds[:len(ats)-1]
        else:
            #TODO: implementer metode faerdig...
            print 'WARNING: method not implemented for buildUp with through-bond transfers yet'
        for i in range(len(ds)):#mindste foerste
            d=ds[i]
            if 1.0<d<=2.1:G*=Gs[i,0]#1-bond
            elif 2.1<d<=2.7:G*=Gs[i,1]#2-bond
            elif 3.0<d<=3.7:G*=Gs[i,2]#3-bond
            else:G*=CalcVolfacCum(log(d),0.15,self.rminMat[-1][i],n=6.0)
        return G

    def getDistCuts(self,peak):
        #peak is class BuildUpPeakVector2D
        return self.distCutMat[peak.minexp]

    def getResIncl(self,peak):
        #peak is class BuildUpPeakVector2D
	print 'getting ResIncl for BuildUpPeakVector',peak.__class__,peak,peak.minexp,self.resInclMat
        return self.resInclMat[peak.minexp]

    def getMaxRmin(self):
        return self.rminMat[-1]

    def getConPris(self,peak):
        #peak is class BuildUpPeakVector2D
	rdim=self.dimension-1
	##print  'self.conPrior indexes:',peak.minexp,rdim
        ##print self.conPriors
	return self.conPriors[peak.minexp*rdim:peak.minexp*rdim+rdim]

    def writeRestraints(self,thr=0.002,includeLowProbs=False):
	lst=[]
	resc=RestraintCounter()
	file1=open('testrestraint'+str(self.dimension)+'.txt','w')
	file2=open('testrestraint_unsorted'+str(self.dimension)+'.txt','w')
	skipfile1=open('skipped_restraints'+str(self.dimension)+'.txt','w')
	skipfile2=open('skipped_restraints_unsorted'+str(self.dimension)+'.txt','w')
	pasdresfile=open('ubq_darr2d_ira.peaks','w')
	xplorfil=open('amb.dat','w')
	if self.name=='':self.name='darr'+str(self.dimension)+'d'
        for i,peak in enumerate(self.peakData.peaklist):
	    asd=peak.assignData
	    lst.append((asd.numAccepted,i))
	    peak.writeRestraints(file2,includeLowProbs=includeLowProbs)##,skipfile2)
	for nac,i in sorted(lst):
	    peak=self.peakData.peaklist[i]
	    peak.writeRestraints(file1,skipfile1,pasdresfile,self.name+'_'+str(i),resc,
                                 includeLowProbs=includeLowProbs,xplorfile=xplorfil)
	file1.close()
	file2.close()
	skipfile1.close()
	skipfile2.close()
	pasdresfile.close()
	resc.show(thr)
	

class RestraintCounter:
    numgoodlr=0
    numgoodlrw=0.0
    numgoodlrwf=0.0
    numbadlrw=0.0
    numbadlrwf=0.0
    numgoodlrwr=0.0
    numbadlrwr=0.0
    numbadlr=0
    
    def __init__(self):
        self.lst=[]
        self.lst2=[]
        self.lst2bad=[]
        self.lst2r=[]
        self.lst2badr=[]

    def show(self,thr):
	return None
        print 'Summary of long-range contacts using threshold: %8.6f'%thr
        print 'Number of good long-range contacts: %2d'%self.numgoodlr
        print 'Number of bad long-range contacts:  %2d'%self.numbadlr
        print 'Good long-range score: %5.2f (%5.2f)'%(self.numgoodlrw,self.numgoodlrwr)
        print 'Bad long-range score:  %5.2f (%5.2f)'%(self.numbadlrw,self.numbadlrwr)
        print 'Fraction of Bad long-range:  %5.3f (%5.3f) [%5.3f]'%\
              (self.numbadlrw/(self.numgoodlrw+self.numbadlrw),
               self.numbadlrwr/(self.numgoodlrwr+self.numbadlrwr),
               self.numbadlrwf/(self.numgoodlrwf+self.numbadlrwf))
        print 'Factor of Bad to long-range:  %5.3f (%5.3f) [%5.3f]'%\
              (self.numbadlrw/self.numgoodlrw,
               self.numbadlrwr/self.numgoodlrwr,
               self.numbadlrwf/self.numgoodlrwf)
        print 'Number of good long-range "hits": %3d (%3d)'%(len(self.lst2),len(self.lst2r))
        print 'Number of bad long-range "hits":  %3d (%3d)'%(len(self.lst2bad),len(self.lst2badr))
        print 'Factor of Bad to long-range using no weight: %5.3f (%5.3f)'%\
              (len(self.lst2bad)*1.0/len(self.lst2),
               len(self.lst2badr)*1.0/len(self.lst2r))
	print 'all good hits within long range restraints:'
	for tup in self.lst2r:
	    lik,asn=tup
            ats=asn.orderedAtoms
            rType=getType(ats,pair=(-2,-1))
##            dist=AtomMath.calc_distance(ats[-2],ats[-1])
            dist=asn.getRealDist(ats)
            asstr=getAsFormat(ats)
            lin='%20s%6.2f%8.5f%8.5f%8.5f%3d'%(asstr,dist,lik,asn.shFac,asn.volFac,rType)
	    print lin
	print 'all good hits:'
	for tup in self.lst2:
	    lik,asn=tup
            ats=asn.orderedAtoms
            rType=getType(ats,pair=(-2,-1))
##            dist=AtomMath.calc_distance(ats[-2],ats[-1])
            dist=asn.getRealDist(ats)
            asstr=getAsFormat(ats)
            lin='%20s%6.2f%8.5f%8.5f%8.5f%3d'%(asstr,dist,lik,asn.shFac,asn.volFac,rType)
	    print lin
        

class BuildUp2D(BuildUp):
    mergeSTD=0.1#TODO:i inputfil


class BuildUp3D(BuildUp):
    mergeSTD=0.2#TODO:i inputfil
    
    def getNormCs(self,xstd,mindist=1.53,elr=0.15):
	shnormc=1.0##((self.mergeSTD**2+xstd**2)*2*pi)**(-0.5*self.dimension)
        for k in range(self.dimension):
            shnormc=shnormc/math.sqrt(2*pi*self.shiftDamps[k]**2+xstd**2+self.mergeSTD**2+\
                                   ##(self.shiftstds[k]*minstd)**2)#hvad for hypType==2?
                                     (self.shiftstds[k]*1.0   )**2)#hvad for hypType==2?
	print 'shnormc',shnormc
	##vnormc=ones(len(self.explst))*1.0
	vnormc=ones((len(self.explst),self.dimension-1))*1.0
	for n in range(len(self.explst)):
            for k in range(self.dimension-1):
                if self.multi[k]>0:
                    ##vnormc[n]=vnormc[n]*CalcVolfacCum\
                    vnormc[n,k]*=CalcVolfacCum\
                              (log(mindist),elr**2,rmin=self.explst[n].rmins[k],
                               lsigint=1.0,n=6.0)
        self.shnormc=shnormc
        self.vnormc=vnormc

    def getVolFac(self,ass,Rmins,vnormc):
        ##TODO: tjek metode...
        print Rmins
        maxvolfac=0.0
        maxpos=None
        minvolfac=9.0
        minpos=None
	maxprodpos=None
	maxprod=0.0
	print vnormc
	print len(ass.distances)
	dct={}
	ass.showDistanceCombinations()
        for k in range(len(ass.distances)):
            distsk=ass.distances[k]
	    print distsk
	    prodk=1.0
            for i in range(self.dimension-1):
		atskey=ass.getAtomPairKey(k,i)
                if self.fromStructure:
                    modelLD=log(distsk[i])
                    modelSig=0.15
                    volfaci=CalcVolfacCum(modelLD,modelSig**2,rmin=Rmins[i],n=6.0)
                else:
                    if isinstance(distsk[i],LRProb):
                        volfaci=distsk[i].prob
                    else:
                        volfaci=distsk[i]#a probability
		volfaci=min(volfaci/vnormc[i],1.0)
		prodk*=volfaci
                if volfaci>maxvolfac:maxvolfac=volfaci;maxpos=(k,i,atskey)
                if volfaci<minvolfac:minvolfac=volfaci;minpos=(k,i,atskey)
		if atskey in dct and volfaci>dct[atskey]:pass
		else:dct[atskey]=volfaci
	    if prodk>=maxprod:maxprod=prodk;maxprodpos=k
	finalvolfac=1.0
	for ak in dct:finalvolfac*=dct[ak]
        print maxpos,maxvolfac,minpos,minvolfac
        ##ass.setVolFac(maxvolfac,maxvolfacs,maxpos)
        ass.setVolFac(min(1.0,finalvolfac),(maxvolfac,minvolfac),maxprodpos)
        ass.setOrderAtoms()


def plotDataFromClusterContainer(cc,inc=0):
    kms='so^>v<dph8'
    cols='bcrk'
    keys=cc.c2i.keys()
    subplot(222)
    for i in range(len(keys)):
        T=[]
        k=keys[i]
        for x in cc.c2i[k]:
            print x
            side=x[2]#;listInd=x[1]
            pk=cc.vcs[x[2]].list[x[0]]
            T.append(pk)
        T=array(T)
        if len(T)>0:scatter(T[:,0]*-1.0,T[:,1]*-1.0,s=ones(len(T))*40,
                           c='y',alpha=0.9,faceted=False,marker=kms[(i+inc)%10])
    
def plotDataFromSuperClusterContainer2(scc,doscatter=1,inc=0,dim=0):
    kms='so^>v<dph8'
    cols='bcrk'
    keys=scc.c2i.keys()
    allis=[]
    maxlst=[]
    ##print Vmat
    ##for c in scc.char:print c
    if dim==3:subplot(221)
    for i in range(len(keys)):
          blst=[]
          k=keys[i]
          ##print '------------key:',k
          xlst=scc.getElems(k)
          T=[]
          ##print xlst
          for lst in xlst:
              for x in lst:
                  ##print x
                  side=x[2]
                  if scc.countmap==None:
                      listInd=x[1]
                  else:
                      ks=scc.countmap.keys()
                      ##print ks,x[1]
                      listInd=ks.index(x[1])
                  pk=scc.cclst[listInd].vcs[x[2]].list[x[0]]
                  ##print pk,
                  ##vals=scc.cclst[listInd].vcs[x[2]].vallst[x[0]]
                  ##I=int(vals[0])
##                  if side==0:T.append(pk)
##                  else:T.append((take(pk,[1,0])))
                  T.append(pk)
          T=array(T)
          if doscatter and len(T)>0:scatter(T[:,0]*-1.0,T[:,1]*-1.0,s=ones(len(T))*60,
                               c='y',alpha=1.00,faceted=False,marker=kms[(i+inc)%10])
def box_old(mat):
    def thisbox(v):
        for i in range(len(v)):#dimensionen
            Bool=False
            for j in range(len(mat)):
                Bool=(Bool or mat[j][0]<v[i]<mat[j][1])
            if Bool==False:
                return False
        #still here
        return True    
    return thisbox

def box(mat):
    def thisbox(v):
        fullst=[]
        for i in range(len(v)):fullst.append(range(len(v)))
        comblst=comb2(fullst)
        for lst in comblst:
            Bool=True
            for i in range(len(v)):#dimensionen
                Bool=(Bool and mat[lst[i]][0]<v[i]<mat[lst[i]][1])
            if Bool==True:return True
        #stil here
        return False
    return thisbox

def half(v):
    if v[1]>v[0]:return 0
    else: return 1

def highest(v):
    if v[1]>v[0]:return v
    else: return take(v,[1,0])

def prism(v):
    if   v[2]>v[1]>v[0]:return 0#012
    elif v[2]>v[0]>v[1]:return 1#102
    elif v[1]>v[2]>v[0]:return 2
    elif v[0]>v[1]>v[2]:return 3
    elif v[0]>v[2]>v[1]:return 4
    elif v[1]>v[0]>v[2]:return 5

def reOrder(v,p):#p is prism-index
    #TODO:test funktion
    if p==0:return v
    elif p==1:inds=[1,0,2]
    elif p==2:inds=[0,2,1]
    elif p==3:inds=[2,1,0]
    elif p==4:inds=[1,2,0]
    elif p==5:inds=[2,0,1]
    return repos(v,inds,dtype=float)

def reorder(v,p):#p is prism-index
    1/0
    #TODO:test funktion
    print p
    if p==0:return v
    elif p==1:inds=[1,0,2]
    elif p==2:inds=[0,2,1]
    elif p==3:inds=[2,1,0]
    elif p==4:inds=[1,2,0]
    elif p==5:inds=[2,0,1]
    return take(v,inds)

def getOrder(p):#p is prism-index
    if   p==0: return [0,1,2]
    elif p==1: return [1,0,2]
    elif p==2: return [0,2,1]
    elif p==3: return [2,1,0]
    elif p==4: return [1,2,0]
    elif p==5: return [2,0,1]

def ascending(v):
    lst=[]
    for i in range(len(v)):
        lst.append(v[i])
    lst.sort()
    return array(lst)

class Partition:

    def __init__(self,f,numparts,r):
        self.classifyer=f
        self.numParts=numparts
        self.repr=r
        self.comblst=[]
        for i in range(numparts-1):
            for j in range(i+1,numparts):
                self.comblst.append((i,j))
        print self.comblst


def allignBuildUp(explst,dget,dcomb,dstrip,doplot=0):
    cols='bgrkmc'
    kms='so^>v<dph8'
    cclst=[]
    if explst[0].dimension==2:part=Partition(half,2,highest)
    else:part=Partition(prism,6,ascending)
    for i in range(len(explst)):
        vci=explst[i].peakData.peakvc
        vci.partition=part
        ##print vci.partition
        ##vci.divide(choose=box(((55,65),(16.5,20.2))),vwidth=[1.0,1.0],doarray=1)
        ##vci.divide(choose=box(((59.4,60.1),(17.9,18.3))),vwidth=[1.0,1.0],doarray=1)
        ##vci.divide(choose=box(((55.0,60.0),(15.0,20.0))),vwidth=[1.0,1.0],doarray=1)
        ##vci.divide(choose=box(((50.0,65.0),(15.0,30.0))),vwidth=[1.0,1.0],doarray=1)
        if explst[i].dimension==2:
            Box=((15.0,20.0),(40.0,45.0))
            ##Box=((5.0,75.0),(5.0,190.0))
            ##vci.divide(choose=box(Box),vwidth=[1.0,1.0],doarray=1)
            ##vci.divide(choose=box(Box),vwidth=[1.0,1.0],doarray=1)
            vci.divide(vwidth=[1.0,1.0],doarray=1)
            print len(vci.vcs[0].list),len(vci.vcs[1].list)
        else:
            Box=((8,12.0),(15.0,75.0),(15.0,75.0))
            Box=((26.0,34.0),(49.0,51.5),(167,185.0))
            Box=((5.0,75.0),(5.0,75.0),(5.0,185.0))
            Box=((16.0,19.0),(26.0,30.0),(40.0,45.0))
            ##Box=((5.0,75.0),(5.0,75.0),(5.0,185.0))
            ##vci.divide(choose=box(Box),vwidth=[1.4,1.4,1.4],doarray=1)
            vci.divide(vwidth=[1.4,1.4,1.4],doarray=1)
    for i in range(len(explst)):
        vci=explst[i].peakData.peakvc
        if doplot:
          for n in range(vci.partition.numParts):
            V=array(vci.vcs[n].list)*-1.0
            if len(V)>0:
                if explst[i].dimension==2:
                    scatter(V[:,0],V[:,1],s=ones(len(V))*175,c=cols[i],alpha=0.75,faceted=False,marker=kms[n])
                else:
                    subplot(221)
                    scatter(V[:,0],V[:,1],s=ones(len(V))*120.0*(i*1.8+1),
                            vmin=Box[2][0],vmax=Box[2][1],c=V[:,2]*-1.0,
                            alpha=0.75,faceted=False,marker=kms[n])
                    title('(0,1)')
                    subplot(222)
                    scatter(V[:,0],V[:,1],s=ones(len(V))*120.0*(i*1.8+1),
                            vmin=Box[2][0],vmax=Box[2][1],c=V[:,2]*-1.0,
                            alpha=0.75,faceted=False,marker=kms[n])
                    title('(0,1)')
                    subplot(223)
                    scatter(V[:,0],V[:,2],s=ones(len(V))*120.0*(i*1.8+1),
                            vmin=Box[1][0],vmax=Box[1][1],c=V[:,1]*-1.0,
                            alpha=0.75,faceted=False,marker=kms[n])
                    title('(0,2)')
                    subplot(224)
                    scatter(V[:,1],V[:,2],s=ones(len(V))*120.0*(i*1.8+1),
                            vmin=Box[0][0],vmax=Box[0][1],c=V[:,0]*-1.0,
                            alpha=0.75,faceted=False,marker=kms[n])
                    title('(1,2)')
        pcc=getClusters2(vci.vcs,distMetricVec,None,dget,sym=1,tag=i)#pair cross-diagonal peak
        ##plotDataFromClusterContainer(pcc)
        cclst.append(pcc)
        ##for k in pcc.c2i.keys():print k,len(pcc.c2i[k]),pcc.c2i[k]
    scc=combineClusters(cclst,distMetricVec,None,dcomb)
    print 'Number of clusters:',len(scc.c2i.keys())
    ##for c in scc.c2i.keys():
        ##print c,scc.c2i[c]
        ##print '    ',
        ##for ccnum,cI in scc.c2i[c]:
        ##    print scc.cclst[ccnum].c2i[cI],
        ##print
    scc.stripAll2(dstrip,distMetricVec,None)
    if explst[0].dimension==2:mdiff=0.3
    else:mdiff=0.4
    scc.mergeClusters(distMetricVec,None,mdiff=mdiff)
    if doplot:plotDataFromSuperClusterContainer2(scc,doscatter=1,dim=explst[0].dimension)
    if doplot:show()
    ##return scc.score
    return scc

def get_explst_and_xnucdba(inpmap):
    explst=[]
    xnucdba={}
    elemuse=False
    evdct={('N','N'):4.0, ('C','CB'):2.5, ('C','CA'):2.0, ('H','HN'):0.7, ('C',None):2.5, ('H',None):0.7}
    if 'useelemvwidths' in inpmap:elemuse=eval(inpmap['useelemvwidths'])
    for i in range(eval(inpmap['numexperiments'])):
	exp0=getExperiment(inpmap['specvars'+str(i+1)])
        exp0.putVars(inpmap,suffix=str(i+1))
        print exp0.distCut
        explst.append(exp0)
        xnuclst=[]
        for n in range(exp0.dimension):
          ##if exp0.atnames[n]<>None:
	      key=(exp0.nucnames[n],exp0.atnames[n])
              xnuclst.append(key)#NOTE can be appended more than once
              ##scut=exp0.shiftTols[n]*exp0.shiftstds[n]
              scut=exp0.shiftTols[n]*exp0.shiftDamps[n]
	      if elemuse:scut=max(scut,exp0.shiftTols[n]*evdct[key])
              ##if not key in xnucdba:xnucdba[key]=[],[],[],scut
              if not key in xnucdba:xnucdba[key]=[[],[],[],scut]
	      else:
		##print key,xnucdba[key]
		if scut>xnucdba[key][-1]:xnucdba[key][-1]=scut
        print 'xnucdba',xnucdba
    return explst,xnucdba

def getPeakData(explst,inpmap,thispdb):
    virt=False;seed=None
    for i in range(1,len(explst)+1):
      if 'infile'+str(i) in inpmap:
        pksname=inpmap['infile'+str(i)]
        if pksname[:4] in ['virt','Virt','VIRT']:
            virt=True;break
      else:virt=True
    if virt:seed=randNorm(0.0,1.0,1000)
    for i in range(len(explst)):
        exp0=explst[i]
        exp0.seed=seed
        print exp0.name
        exp0.peakData=PeakData(inpmap,exp0,suffix=str(i+1),pdb=thispdb)
        print 'numpeaks:',i,len(exp0.peakData.peaklist)
        ##exp0.peakData.estimateErrorProbs(exp0)
        exp0.expnum=i
	##if i==0: show();1/0
##    rats=testDuplicates2(explst)
##    return rats
##    explst[1].showProj(0,2)
##    explst[2].showProj(0,1,col='k')

##def getBuildUplst(explst,b3d=(7,9),b2d=(3,7),save=1,read=0):
def getBuildUplst(explst,im,save=1,read=0):
    buildUplst=[]
    if read==1:
        if 'b3d' in im:scc3=read_dba('scc3.pck')
        if 'b2d' in im:scc2=read_dba('scc2.pck')
        if 'b1ds' in im:scc1lst=read_dba('scc1lst.pck')
    else:
        if 'b3d' in im:
	    i0,i1=[eval(x) for x in im['b3d']];i0-=1
	    lw=explst[i0].lineWidth
	    ##scc3=allignBuildUp(explst[i0:i1],0.65,0.7,0.8)#,doplot=1)
	    scc3=allignBuildUp(explst[i0:i1],lw+0.15,lw+0.2,lw+0.3)#,doplot=1)
        if 'b2d' in im:
	    i0,i1=[eval(x) for x in im['b2d']];i0-=1
	    lw=explst[i0].lineWidth
	    ##scc2=allignBuildUp(explst[i0:i1],0.35,0.4,0.5)##,doplot=1)
	    scc2=allignBuildUp(explst[i0:i1],lw-0.15,lw-0.1,lw)##,doplot=1)
        if 'b1ds' in im:
	  scc1lst=[]
	  for bi in im['b1ds']:
	    i1=eval(bi);i0=i1-1
	    scc1=allignBuildUp(explst[i0:i1],0.35,0.4,0.5)
	    scc1lst.append(scc1)
    #TODO: generaliser... konstanter
    cnt=0
    if 'b1ds' in im:
        if save:save_dba2(scc1lst,'scc1lst.pck')
	for ni,bi in enumerate(im['b1ds']):
	    i1=eval(bi);i0=i1-1
            buildUp=BuildUp2D(explst[i0:i1],scc1lst[ni])
            buildUplst.append(buildUp)
            buildUp.expnum=len(explst)+cnt
	    cnt+=1
    if 'b2d' in im:
	i0,i1=[eval(x) for x in im['b2d']];i0-=1
	print 'indexes:',i0,i1
        if save:save_dba2(scc2,'scc2.pck')
        buildUp=BuildUp2D(explst[i0:i1],scc2)
        buildUp.expnum=len(explst)+1+cnt
        buildUplst.append(buildUp)
    if 'b3d' in im:
	i0,i1=[eval(x) for x in im['b3d']];i0-=1
        if save:save_dba2(scc3,'scc3.pck')
        buildUp=BuildUp3D(explst[i0:i1],scc3)
        buildUp.expnum=len(explst)+cnt
        buildUplst.append(buildUp)
    return buildUplst

def main(get=None,read=0,dosave=False,conpri=True,linew=None):
    T_0=time.time()
    inpmap=getinpmap('vs.inp')
    if linew<>None:inpmap['overridelw']=str(linew)
    print inpmap
    'prepare experiments'
    explst,xnucdba=get_explst_and_xnucdba(inpmap)
    'initialize data'
    base=Base(inpmap,xnucdba,get,conpri)
##    base.conPriors[-1].show(base.pdb)
##    G =explst[5].globShifts
##    G1=explst[5].gsh1s#1st order corrections
##    base.distanalyze(cors=(G,G1))
##    base.output('cstbl2.txt','ubiqshifts.tab','assignments.txt')
    if read==0:
        'get peakData'
	##base.getSpinSystems(explst)
        getPeakData(explst,inpmap,base.pdb)
	return explst
	allndats=[]
	for exp0 in explst:
	  if 'N' in exp0.nucnames:##exp0.name in ['NCACX','NCOCX']:
	    ndat=exp0.peakData.getNshiftData(exp0)
	    allndats.append(ndat)
	  else:allndats.append({})
	exp0.peakData.analNshiftDatas(allndats,explst)
        if dosave:save_dba2(explst,'explst.pck')
        if conpri:Experiment.putConPriors(explst,base.conPriors,inpmap)
##        del base.pdb
##        if dosave:save_dba2(base,'base.pck')
        'get buildup data'
        buildUplst=getBuildUplst(explst,inpmap)
        buildUplst[0].writePeakList2('_combined')
        buildUplst[1].writePeakList2('_combined')
    elif read==1:
        print [base.conPriors[3].LRmap[4][66]]
        print base.conPriors[3].LRmap[4][66]
        explst=read_dba('explst.pck')
        Experiment.putConPriors(explst,base.conPriors,inpmap)
        ##for peak in explst[6].peakData.peaklist:peak.TEST()
        'get buildup data'
        buildUplst=getBuildUplst(explst,inpmap,read=1)
    elif read in [2,3]:
##        pdb=fileio.LoadStructure(file=inpmap['structure'])
##        base=read_dba('base.pck')
##        base.pdb=pdb
        explst=read_dba('explst.pck')
	newexplst=read_dba('newexplst.pck')
##	trip=read_dba('trip27VKAc.pck')
##	trip.showtrip()
##	trip.writetrip(newexplst)
##	1/0
    elif read>=7:
	newexplst=read_dba('newexplst_assigned2.pck')
	##newexplst=[read_dba('exp_2.pck'),read_dba('exp_4.pck')]#NCACX,NCOCX
	##for ne,exp0 in enumerate(newexplst):
	##    save_dba2(exp0,'exp_'+str(ne)+'.pck')
    elif read>=4:
	newexplst=read_dba('newexplst_assigned.pck')
        ##Experiment.putConPriors(explst,base.conPriors,inpmap)
    ##if read>0:base=Base(inpmap,xnucdba,get)
    if read<2:
        ##newexplst=buildUplst##+explst[:3]#(2d foer 3d i buildUplst)
        ##newexplst=buildUplst+explst##[:5]#(2d foer 3d i buildUplst)
        newexplst=explst[:-2]+buildUplst
	#TODO: generalize numbers!
        if dosave:save_dba2(newexplst,'newexplst.pck')
    if read<3:
        if len(newexplst)>1:
	    pass
	    #4: NCACX
	    #5: NCACO
	    #6: NCOCX
	    #7: CANCO
	    #8: NCACB
            ##newexplst[1].peakData.stripPeaks(newexplst[0],tol=1.0,reverse=True,pairlist=[(0,1)])
            ##newexplst[3].peakData.stripPeaks(newexplst[2],pdb=base.pdb,tol=0.70,accepts=[3],pairlist=[(0,1)])
            ##newexplst[3].peakData.stripPeaks(newexplst[2],pdb=base.pdb,tol=0.70,accepts=[3],pairlist=[(0,2)])
            ##newexplst[3].peakData.stripPeaks(newexplst[2],pdb=base.pdb,tol=0.70,accepts=[3],pairlist=[(1,2)])
            ##newexplst[4].peakData.stripPeaks(newexplst[2],pdb=base.pdb,tol=0.60,accepts=[3],pairlist=[(1,2)])
            ##newexplst[5].peakData.stripPeaks(newexplst[2],pdb=base.pdb,tol=0.60,accepts=[3],pairlist=[(1,2)])
            ##newexplst[6].peakData.stripPeaks(newexplst[2],pdb=base.pdb,tol=0.60,accepts=[3],pairlist=[(1,2)])
            ##newexplst[7].peakData.stripPeaks(newexplst[2],pdb=base.pdb,tol=0.80,accepts=[3],pairlist=[(0,2)])
            ##newexplst[8].peakData.stripPeaks(newexplst[0],pdb=base.pdb,tol=0.80,accepts=[0],pairlist=[(1,2)])
            ##newexplst[8].peakData.stripPeaks(newexplst[2],pdb=base.pdb,tol=0.80,accepts=[0],pairlist=[(1,2)])
	##TODO: ovenst. i input fil
        ##if dosave:save_dba2(newexplst,'newexplst.pck')
	##pass
    if len(newexplst)>1:
        pass##newexplst[3].norm1=newexplst[2].G_expects[0,0]

    ##base.distanalyze()#showfile='tmp')
    print 'data initialized'

    if read==4:
        ##for j in [1]:
        if len(newexplst)>2:newexplst[3].norm1=newexplst[2].G_expects[0,0]
        for j in range(len(newexplst)):
        ##for j in [0]:##range(len(newexplst)):
            exp0=newexplst[j]
            exp0.hypType=2
            for i in range(len(exp0.peakData.peaklist)):
                peak=exp0.peakData.peaklist[i]
                try:
                    assSet=peak.assSet
                    maxAssSet=peak.maxAssSet
                except AttributeError:assSet=None;maxAssSet=None#empty
                asd=peak.assignData
                asd.setpPosts(exp0,assSet,Smax=maxAssSet)
                asd.setUse()
            base.analyze(exp0)
        raise SystemExit, 'Done analyzing'

    elif read==5:
        likmat=base.fillAssignDBA(newexplst)
        base.testConPriors()
	for i,cp in enumerate(base.conPriors):
	    cp.update(likmat,i)
	    if i==3:
                pass
                ##cp.show(base.pdb)
        base.testConPriors()
##        return 0
##	del base.pdb
##        if dosave:
##            save_dba2(base,'base.pck')
##            save_dba2(newexplst,'newexplst_assigned.pck')
##	return 0
##
##    elif read==6:
##        base.conPriors[3].show(base.pdb)
        for i in range(1):
            for j in range(len(newexplst)):
                exp0=newexplst[j]
                exp0.hypType=2
                updvec=[True,False]#volumes,shifts
                base.rankAllInitAssigns(exp0,updvec)
                base.analyze(exp0)
        if dosave:
            del base.pdb
            save_dba2(base,'base.pck')
            save_dba2(newexplst,'newexplst_assigned2.pck')
        base.AllNotMat=numpy.ones((70,70))
        base.AllNotMatw=numpy.ones((len(base.pdb[0]),len(base.pdb[0])))
        ##del newexplst[1]
        for i,exp0 in enumerate(newexplst):
            newAllassigns=exp0.chooseAssigns(fac=250)
            B,D=base.visualize(newAllassigns,i==len(newexplst)-1)
            ##B,D=base.visualize(newAllassigns,0)
            print [list(v) for v in B]
            print [list(v) for v in D]
        show()
	return 0

    elif read==7:
        base.AllNotMat=numpy.ones((70,70))
        base.AllNotMatw=numpy.ones((len(base.pdb[0]),len(base.pdb[0])))
        ##del newexplst[1]
        ##thrs=[0.1*exp(-i*log(2)/2.0) for i in range(30)]
	thrs=[0.01,0.005,0.001,0.0005,0.0001,0.00005,0.00001,0.000005,0.000001,0.0000005,0.0000001,0.00000005,0.00000001]
	thrs=[0.1,0.05,0.025,0.01,0.005,0.0025,0.001]
	thrs.reverse()
	thrs=[0.00]
##	thrs=[0.00000001]
##        thrs=[0.0000001]
##        facs=[1.0*exp(i*log(2)/2.0) for i in range(30)]
        facs=[5,10,50,100,500,1000,10000]
        ##facs=[100]
##        errmaxs=[exp(-i*log(2)/2.0)/2 for i in range(20)]
        errmaxs=[0.5]
##	shtruncs=[0.005,0.01,0.02,0.03,0.04,0.05,0.07,0.1]
	shtruncs=[0.07]
##        for thr in thrs:
##        for errm in errmaxs:
#        for thr in [0.002]:
        sh0=eval(inpmap['shtrunc'])
##        for shtr in shtruncs:
        incall=eval(inpmap['includelowprob'])
##        for fac in facs:
##        for sh0 in shtruncs:
        for mmvp in [0.005]:##[0.01,0.005,0.0025,0.0001,0.00005]:
##	for fac in [10]:##facs:
            for i,exp0 in enumerate(newexplst):
                newAllassigns,Mat=exp0.chooseAssigns(50,reject=0.0,errmax=0.5,\
                    shtrunc=sh0,minmaxvolprob=mmvp)
                ##B,D=base.visualize(newAllassigns,i==len(newexplst)-1)
                doshow=False##(mmvp==0.0025)##(i==len(newexplst)-1 or len(newexplst)==1)
                B,D=base.visualize(newAllassigns,doshow)##,cmat=Mat)
                ##base.getViolations(exp0)
                exp0.writeRestraints(mmvp,includeLowProbs=incall)
                print 'done writing restraints'
                print [list(v) for v in B]
                print [list(v) for v in D]
	if doshow: show()
        base.output('cstbl2.txt','ubiqshifts.tab','assignments.txt')
	return 0


    elif read==9:
        for j in range(len(newexplst)):
            exp0=newexplst[j]
            ##base.redoResAssignments(exp0,j,min_pFalse=0.25)
            base.redoResAssignments(exp0,j,min_pFalse=0.10)
            print 'TIME:',time.time()-T_0,'peaklist assigned','expnum ==',j;T_0=time.time()
        base.summarizeTripletAssignments(explst)
	1/0

    if True:##elif read==8:
        for exp0 in newexplst:
	    if False:##exp0.name=='NCACO':
        	exp0.atnames=['N','CA',None]#through space NCACO
            exp0.peakData.peakvc.putSlices(exp0.lineWidth*4)##vwidth=1.0)
            ##for asd in exp0.allassigns:asd.setEntrop()
	outfile=open('allaligns.txt','w')
	allneardct={}
	nallneardct={}
	hasncaco='NCACO' in [expk.name for expk in newexplst]
	allatnords=[]
        for i in range(len(newexplst)):
	    newexplst[i].setatnords()
	    allatnords.append(newexplst[i].atnords)
        for i in range(len(newexplst)):
	    exp0=newexplst[i]
	    print exp0.name
            ##for exp1 in newexplst[i:]:
            for j in range(i,len(newexplst)):
                exp1=newexplst[j]
		data=exp0.getAlignedPeaks(exp1,outfile,hasncaco,allatnords,delta=0)
		if data<>None:
		  ##nearest,nearestrev,labels=data
		  ##allneardct[(exp0.expnum,exp1.expnum)]=data
		  exp0.modify_data(exp1,data)
		  for ax0 in data:
		    ##allneardct[(exp0.expnum,exp1.expnum,ax0)]=data
		    allneardct[(i,j,ax0)]=data[ax0]
		ndata=exp0.getAlignedPeaks(exp1,outfile,hasncaco,allatnords,delta=1)
		if ndata<>None:
		  ##for ax0 in data:
		  for ax0 in ndata:
		    nallneardct[(i,j,ax0)]=ndata[ax0]
	save_dba2(allneardct,  'allneardct.pck')
	save_dba2(nallneardct,'nallneardct.pck')
	1/0
##    for exp0 in newexplst:
##        Experiment.putConPriors(exp0.explst,base.conPriors,inpmap)
##    if conpri:Experiment.putConPriors(explst,base.conPriors,inpmap)
        
    ##print base.conPriors[0].rmin
    print 'processing data for resonance/restraint assignments'
    print 'TIME:',time.time()-T_0,'data initialized and alligned';T_0=time.time()
    if get=='restart':mode='pdf'
    else:mode='normal'
    mode='pdf'#pine assignments
    #NOTE: mode=='normal' means that shfac is calculated using calcIntNorm
    # mode=='pdf' means that it is reused
##    for i in ran#generaliseres med 3d!!!!!!ge(2):
    usetrip=False
    if 'usetriplets' in inpmap:usetrip=eval(inpmap['usetriplets'])
    for i in range(5):
        for j in range(len(newexplst)):
            exp0=newexplst[j]
            exp0.hypType=2
            exp0.itnum=i
            ##base.analyze(exp0)
            if i==0:
                base.getAssignments(exp0,mode)
                updvec=[True,True]#volumes,shifts
                print 'TIME:',time.time()-T_0,'assignments set, #it ==',i,'expnum ==',j;T_0=time.time()
            else:
		##updvec=[False,True]
                updvec=[False,True]
		if i==1:exp0.hypType=2
		mode='update'
            base.rankAllInitAssigns(exp0,updvec,mode)
            print 'TIME:',time.time()-T_0,'peaklist assigned, #it ==',i,'expnum ==',j;T_0=time.time()
            ##base.analyze(exp0)
            ##if i==4 and exp0.dimension==2:
	    if False:##True:
                exp0.writePeakListAssigned(str(j)+str(exp0.hypType)+'.test')
            base.redoResAssignments(exp0,j,min_pFalse=exp0.pFN,usetrip=usetrip)#also updates triplets - needs read<=3!
        difflst,adifflst,relhilst=base.replaceShifts(showall=0)
	print 'TIME:',time.time()-T_0,'resonances assigned, #it ==',i;T_0=time.time()
        if not explst[0].fromStructure:base.testConPriors()
	if i==0:##1:
            if dosave:save_dba2(newexplst,'newexplst_assigned.pck')
        if False:##not explst[0].fromStructure:
            ##likmat=base.fillAssignDBA(newexplst[:1])
            ##TODO: ret problem: NB og max giver samme output i restraint fil
            likmat=base.fillAssignDBA(newexplst)
            ##for k,cp in enumerate(base.conPriors):
            base.conPriors[-1].update(likmat,len(base.conPriors))
##            base.conPriors[-1].show(base.pdb)
            for cp in base.conPriors[:-1]:
                cp.update_from_other(base.conPriors[-1])
            if dosave:save_dba2(newexplst,'newexplst_assigned0.pck')

    if False:
      base.AllNotMat=numpy.ones((70,70))
      base.AllNotMatw=numpy.ones((len(base.pdb[0]),len(base.pdb[0])))
      ##del newexplst[1]
      for i,exp0 in enumerate(newexplst):
        newAllassigns=exp0.chooseAssigns(fac=10)
        doshow=(i==len(newexplst)-1 or len(newexplst)==1)
        B,D=base.visualize(newAllassigns,doshow)
        print [list(v) for v in B]
        print [list(v) for v in D]
        exp0.writeRestraints(10,includeLowProbs=False)
        print 'done writing restraints'
      if doshow:show()
                ##cp.show(base.pdb) 
        ##base.fillAssignDBA(explst)
        ##base.pickleData()
    if False:
        for exp0 in newexplst:
	    if exp0.name=='NCACO':
        	exp0.atnames=['N','CA',None]#through space NCACO
            exp0.peakData.peakvc.putSlices(vwidth=1.0)
            for asd in exp0.allassigns:asd.setEntrop()
	outfile=open('allalignsb.txt','w')
        for i in range(len(newexplst)):
	    exp0=newexplst[i]
	    print exp0.name
            for exp1 in newexplst[i:]:
		exp0.getAlignedPeaks(exp1,outfile)
    print 'summarizing triplets'
    if usetrip:base.summarizeTripletAssignments(newexplst)

    if dosave:save_dba2(newexplst,'newexplst_assigned2.pck')
    base.write_history()    
    del base.pdb
    if dosave:save_dba2(base,'base.pck')
    base.output('cstbl2.txt','hmbd.tab','assignments.txt')
    ##clf()
    ##hist(adifflst,bins=20)
    ##show()
    #base.output('cstbl2.txt','ubiqshifts.tab','assignments.txt',newAllassgns)
    ##base.final_analyze()

##for n in [8]:##range(2,8):
##    buf=initfil2('enerlog'+str(n)+'.txt')
##    parser=Parser(buf)
##    fil=open('enerlst'+str(n),'w')
##    etot=1
##    while etot<>None:
##        num,etot,hbdb,se=getTot(parser,getambhb=True,getSE=True)
##        if etot<>None:fil.write('%3d%10.1f%8.2f%7.2f\n'%(num,etot,hbdb,se))
##    fil.close()


##main(read=3,dosave=False,conpri=True)##,get='restart')
##main(get='newatoms.txt')
##show()

def visualizeTripletAssignmentPossibilities():
    buf=initfil2('trip27.txt')
    numas=len(buf)-1
    numnuc=len(buf[1])-7
    A=zeros((numas,numnuc))
    del buf[0]
    lencol=200
    numcol=5
    tvals=[[] for i in range(numcol)]
    vals=[[] for i in range(numnuc)]
    lst=[]
    for i in range(numas):
	lst.append([eval(val) for val in buf[i][numnuc:numnuc+3]])
    ##a=array(lst).transpose()
    ##scatter(log(clip(a[1]*a[2],0.000001,1.0)),log(clip(a[0],0.000001,1.0)),\
    ##        alpha=0.5,faceted=False)
    ##plot([log(0.000001),log(0.45)],[log(0.000001),log(0.45)],'k--')
    ##show()
    for i in range(numnuc):
            for j in range(numas):
                if buf[j][i]<>'-999':
                    vals[i].append(eval(buf[j][i]))
		    if buf[j][-1]=='1':
		      if j<lencol*numcol:
			tvals[j/lencol].append((j%lencol+0.5,i+0.5))
    res=[]
    tvals=[array(tval).transpose() for tval in tvals]
    for i in range(len(vals)):
        if len(vals[i])>0:
            res.append((average(vals[i]),std(vals[i])))
        else:res.append((0,0))
    for i in range(numnuc):
            for j in range(numas):
                if buf[j][i]<>'-999':
                    ##A[j][i]=(eval(buf[j][i])-res[i][0])/res[i][1]
                    A[j][i]=eval(buf[j][i])
    print A[0]
    print A[1]
    1/0
    tbuf=initfil2('trues22h2')
    T=zeros((len(tbuf),numnuc))
    for i in range(numnuc):
            for j in range(len(tbuf)):
                if tbuf[j][i]<>'-999':
                    T[j][i]=(eval(tbuf[j][i])-res[i][0])/res[i][1]
    subplot((numcol+1)*100+11)
    ##imshow(T,interpolation='nearest',cmap=cm.jet,vmin=-1.0,vmax=1.0)
    imshow(rot90(T),interpolation='nearest',cmap=cm.jet,vmin=-2.0,vmax=2.0)
    for i in range(numcol):
      subplot((numcol+1)*100+12+i)
      B=A[lencol*i:lencol*(i+1),:]
      imshow(rot90(B),interpolation='nearest',cmap=cm.jet,vmin=-2.0,vmax=2.0)
      scatter(tvals[i][0],tvals[i][1],ones(len(tvals[0]))*100,'w',alpha=0.5,faceted=False)
      pylab.axis((0,lencol,0,numnuc))

    show()
##visualizeTripletAssignmentPossibilities()

def b1b2overlap():
    buf=initfil2('b2x')
    colst=[array((eval(lin[0]),eval(lin[1]))) for lin in buf]
    namlst=[lin[2] for lin in buf]
    vc=VoxelContainer()
    vc.vwidth=2.01*ones(2)
    vc.fillContainer(colst)
    buf2=initfil2('b1')
    for lin in buf2:
      co=array((eval(lin[0]),eval(lin[1])))
      if co[0]>co[1]:
	nam=lin[2]
        dlst=vc.findNearestFromCoord(co,distMetricVec,None,maxd=1.0)
	if len(dlst)>0:
	  ##print co,nam,dlst[0][0],dlst[0][1],namlst[dlst[0][2]],colst[dlst[0][2]]
	  print '%12s %5.2f %12s'%(nam,dlst[0][0],namlst[dlst[0][2]])
	
def normcols(X):
    return X/sum(X,0)

def normrows(X):
    return transpose(normcols(transpose(X)))

def normSym(X,iter=10):
    for i in range(iter):
	X=normcols(X)
	X=normrows(X)
    return X

def testnormSym():
    A=arange(1.0,10.0,1.0).reshape((3,3))
    X=A+A.transpose()
    print X
    print normSym(X)

def analyzeAllAlignments():
    buf=initfil2('allaligns.txt')
    dct={}
    conv={'10':0,'9':1,'0':2,'1':3,'2':4}
    #values of conv must be range(n)
    tvals=[]
    for lin in buf:
      for k in [0,1]:
	key=conv[lin[0+k]]
	val=eval(lin[2+k])
	if not key in dct:dct[key]=-1
	if val>dct[key]:dct[key]=val+1
    num=sum(dct.values())
    cnumlst=cumsum(dct.values())
    print cnumlst
    mat=zeros((num+1,num+1))
    emat=zeros((num+1,num+1))
    tmat=zeros((num+1,num+1))
    for lin in buf:
	keys=[conv[lin[0+k]] for k in [0,1]]
	vals=[eval(lin[2+k]) for k in [0,1]]
	inds=[cnumlst[keys[k]]-dct[keys[k]]+vals[k] for k in [0,1]]
	prob=eval(lin[5])
	entp=eval(lin[6])
	true=eval(lin[7])
	if true:
	  tvals.append((inds[0]+0.5,inds[1]+0.5))
	  tvals.append((inds[1]+0.5,inds[0]+0.5))
	  tmat[inds[0],inds[1]]=1
	  tmat[inds[1],inds[0]]=1
	try:
	  mat[inds[0],inds[1]]=prob
	  mat[inds[1],inds[0]]=prob
	  emat[inds[0],inds[1]]=entp
	  emat[inds[1],inds[0]]=entp
	except IndexError:print 'WARNING IndexError',inds,prob
    tvals=array(tvals).transpose()
    mat=normSym(mat)
    outfile=open('analaligns.txt','w')
    for i in range(len(mat)):
        for j in range(i+1,len(mat[i])):
	  prob=mat[i,j]
	  if prob>0.00001:
	    true=tmat[i,j]
	    entp=emat[i,j]
	    outfile.write('%4d %4d %7.3f %7.4f %1d\n'%(i,j,entp,prob,true))
    outfile.close()
    1/0
    ##M=log(1-mat)
    ##imshow(rot90(M),interpolation='nearest',cmap=cm.jet,vmin=-0.05,vmax=-0.001)
    imshow(rot90(mat),interpolation='nearest',cmap=cm.jet,vmax=0.03,vmin=0.002)
    scatter(tvals[0],tvals[1],ones(len(tvals))*25,'w',alpha=0.5,faceted=False)
    show()
  
def analyzeAllAlignments2():
    tbuf=initfil2('truealigns')
    fbuf=initfil2('falsealigns')
    tentps=array([eval(lin[2]) for lin in tbuf])
    fentps=array([eval(lin[2]) for lin in fbuf])
    tprobs=array([eval(lin[3]) for lin in tbuf])
    fprobs=array([eval(lin[3]) for lin in fbuf])
    tle=tentps<20
    fle=fentps<20
    ##scatter(fentps,log(fprobs),ones(len(fprobs))*10,'r',alpha=0.12,faceted=False)
    ##scatter(tentps,log(tprobs),ones(len(tprobs))*10,'b',alpha=0.12,faceted=False)
    hist(log(fprobs[fle]),bins=arange(-7.5,-2.5,0.25),normed=0,fc='r',alpha=0.5)
    hist(log(tprobs[tle]),bins=arange(-7.5,-2.5,0.25),normed=0,fc='b',alpha=0.5)
    show()

##analyzeAllAlignments2()
 

def clusterAlligns():
    buf=initfil2('bestaligns.txt')
    entps=array([eval(lin[2]) for lin in buf])
    probs=array([eval(lin[3]) for lin in buf])
    trues=array([eval(lin[4]) for lin in buf])
    co0s =array([eval(lin[0]) for lin in buf])
    co1s =array([eval(lin[1]) for lin in buf])

    dct={}    
    tdct={}    
    for i in range(len(buf)):
      if not co0s[i] in dct:dct[co0s[i]]={}
      if not co1s[i] in dct:dct[co1s[i]]={}
      dct[co0s[i]][co1s[i]]=probs[i]
      dct[co1s[i]][co0s[i]]=probs[i]
      if not co0s[i] in tdct:tdct[co0s[i]]={}
      if not co1s[i] in tdct:tdct[co1s[i]]={}
      tdct[co0s[i]][co1s[i]]=trues[i]
      tdct[co1s[i]][co0s[i]]=trues[i]

    for k in dct:
        print k,len(dct[k]),dct[k].keys()
      

    cc=ClusterContainer()
    tripsum=0
    tripdct={}
    besttrips={}
    for i in range(len(buf)):
      i0=co0s[i]
      i1=co1s[i]
      dct0=dct[i0]
      dct1=dct[i1]
      Si=set(dct0.keys())&set(dct1.keys())
      prod0=probs[i]
      ##print Si
      maxprob=0.0
      maxelem=None
      for j in Si:
	elem=[i0,i1,j]
	elem.sort()
	tripdct[tuple(elem)]=1
	prod=(dct[i0][j]*dct[i1][j]*prod0)**(1.0/3)
	if prod>maxprob:
	    maxprob=prod
	    maxelem=elem
      if maxelem<>None:besttrips[tuple(maxelem)]=maxprob,trues[i]*tdct[i0][j]*tdct[i1][j]
      tripsum+=len(Si)
      if probs[i]>0.03:
        cc.pair(co0s[i],co1s[i])
    for k in besttrips:print '%4d %4d %4d'%k,'%7.4f %1d'%besttrips[k]
    print tripsum
    print len(tripdct)

##clusterAlligns()

def analyzeTriplets():
    buf=initfil2('besttriplets.txt')
    ts=[];fs=[]
    for lin in buf:
        if lin[-1]=='1':
	    ts.append(eval(lin[-2]))
        else:
	    fs.append(eval(lin[-2]))
    hist(log(fs),bins=arange(-5.0,-2.0,0.125),normed=1,fc='r',alpha=0.5)
    hist(log(ts),bins=arange(-5.0,-2.0,0.125),normed=1,fc='b',alpha=0.5)
    show()
##analyzeTriplets()

##main(read=0,dosave=True,conpri=True)##,get='restart')
##main(dosave=True,conpri=False)##,get='restart')
##main(read=0,dosave=True,conpri=True)##,get='restart')
##main(read=3,dosave=False,conpri=True)##,get='restart')
if len(sys.argv)>1:
  resi=sys.argv[1]
  atn =sys.argv[2]
  get=(resi,atn)
  title(resi+' '+atn)
else:get=None

explst=main(read=0,dosave=True,conpri=True)

